#! /usr/bin/tcsh -f
#
# HISTORY:
# 1999.09.06-SW: default.mk is not necessary anymore.
# 1999.09.23-SW: the spec/proc tree was missing!!
# 1999.12.17-SW: the scripts directory added.
# 2000.03.02-AM: added the "h" option to tar to follow links to the repository
#
#------------------------------------------------------------------------------
#
# Check parameters
#
set TARFILE = $1
if ($1 == '') then
  echo ' '
  echo 'PROGRAM: CreaRelease                      09.07.99-SW'
  echo 'Creates a UVES pipeline release for delivery to'
  echo 'DFO and instrument team UVES. It can be started from'
  echo 'any directory. '
  echo ' '
  echo 'SYNTAX: CreaRelease <archive-name> [vV]'
  echo ' '
  echo '<archive-name> archive name or release number.'
  echo '               in case of a release number (like 0.8.2) a standard'
  echo '               archive name will be created using this number.'
  echo 'v (or V)       prints additional information on the terminal.'
  exit
else if ($1 =~ [a-z]) then
  set TARFILE = $1
else
  set TARFILE = uves-$1.tar
endif
set PIP_VERS = $1
echo $TARFILE
#
# Change directory to pipeline development area
#
echo "Create the archive: $TARFILE."

set lpath = `pwd`

# !!!!!!!!!!!!!!
#
# 2000.03.02-AM:
#
# BEFORE WAS REFERRING to /midas/${MIDVERS}/pipeline
# as root directory
#
# cd /midas/${MIDVERS}/pipeline
#
# NOW WE REFER TO ${HOME}/DfsPrivatePipelines/
#
# This to reflect the new location (in Garching) of the UVES pipeline in
# a common repository. Please veryfy if it works only on dfs3
# due to a problem connected to cross/mounting user permissions.
#
# NB:
# May be on Paranal they have a different directory structure
# so you should check if there you have to specify a different
# root directory  (proabably) you have to use
#
# /midas/${MIDVERS}/pipeline
#
# to take what present there.

# The following setting may be updated: it refers to the location
# where it is stored the pipeline-release version.
# cd ${HOME}/DfsPrivatePipelines
cd ${PIPE_HOME}/uves
echo `pwd`

#
# Create and fill the archive with the UVES CONTEXT files ...
#
echo "Fill the archive with the UVES CONTEXT files ..."
tar -ch$2f $TARFILE uves/uves/etc/uves.ctx
tar -uh$2f $TARFILE uves/uves/etc/makefile
tar -uh$2f $TARFILE uves/uves/proc/*.prg
tar -uh$2f $TARFILE uves/uves/proc/makefile
#
# Fill the archive with the UVES HELP files ..."
#
echo "Fill the archive with the UVES HELP files ..."
tar -uh$2f $TARFILE uves/help/*.hlq
tar -uh$2f $TARFILE uves/help/makefile
#
# Fill the archive with the UVES RECIPES ...
#
echo "Fill the archive with the RECIPES ..."
tar -uh$2f $TARFILE uves/rec/proc/uves_*.prg
tar -uh$2f $TARFILE uves/rec/proc/makefile
#
# Fill the archive with the UVES PIPELINE PROCEDURES ...
#
echo "Fill the archive with the UVES PIPELINE PROCEDURES ..."
tar -uh$2f $TARFILE uves/pipe/proc/*.prg
tar -uh$2f $TARFILE uves/pipe/proc/makefile
tar -uh$2f $TARFILE uves/pipe/src/*.c
tar -uh$2f $TARFILE uves/pipe/src/makefile
tar -uh$2f $TARFILE uves/pipe/libsrc/*.c
tar -uh$2f $TARFILE uves/pipe/libsrc/*.for
tar -uh$2f $TARFILE uves/pipe/libsrc/makefile
tar -uh$2f $TARFILE uves/pipe/incl/*.h
#
# Fill the archive with the UVES GASGANO scripts
#
echo "Fill the archive with the UVES SCRIPTS ..."
tar -uh$2f $TARFILE uves/scripts/*.sh
#
# Fill the archive with the UVES ECHELLE PROCEDURES ...
#
echo "Fill the archive with the UVES ECHELLE PROCEDURES ..."
tar -uh$2f $TARFILE uves/echelle/proc/*.prg
tar -uh$2f $TARFILE uves/echelle/proc/makefile
#tar -uh$2f $TARFILE uves/echelle/proc/default.mk
#tar -uh$2f $TARFILE uves/echelle/src/*.c
tar -uh$2f $TARFILE uves/echelle/src/*.for
tar -uh$2f $TARFILE uves/echelle/incl/*.inc
tar -uh$2f $TARFILE uves/echelle/src/makefile
tar -uh$2f $TARFILE uves/spec/proc/*.prg
tar -uh$2f $TARFILE uves/spec/proc/makefile
tar -uh$2f $TARFILE uves/spec/src/*.c
#tar -uh$2f $TARFILE uves/spec/src/*.for
tar -uh$2f $TARFILE uves/spec/src/makefile
#
# Fill the archive with the UVES DOCUMENTATION ...
#
# echo "Fill the archive with the UVES DOCUMENTATION ..."
# tar -uh$2f $TARFILE uves/doc/*.tex
# tar -uh$2f $TARFILE uves/doc/*.sty
# tar -uh$2f $TARFILE uves/doc/*.eps
# tar -uh$2f $TARFILE uves/doc/chunk_pos.ps
# tar -uh$2f $TARFILE uves/doc/chunk_fwhm.ps
# tar -uh$2f $TARFILE uves/doc/*.fig
# tar -uh$2f $TARFILE uves/doc/cookbook.txt
# tar -uh$2f $TARFILE uves/doc/uves_context_cookbook.tex
#
# Fill the archive with the UVES CalibDb Files ...
#
# rules
echo "Fill the archive with the rules Files..."
tar -uh$2f $TARFILE uves/calibDB/rul/uves.akd
tar -uh$2f $TARFILE uves/calibDB/rul/uves.bdd
tar -uh$2f $TARFILE uves/calibDB/rul/uves.dpr
tar -uh$2f $TARFILE uves/calibDB/rul/uves.ehr
tar -uh$2f $TARFILE uves/calibDB/rul/uves.pdd
tar -uh$2f $TARFILE uves/calibDB/rul/load_ima.rrd
tar -uh$2f $TARFILE uves/calibDB/rul/uves_*.rrd

# recipes
echo "Fill the archive with the recipes Files..."
tar -uh$2f $TARFILE uves/calibDB/ech/rec/*.prg
tar -uh$2f $TARFILE uves/calibDB/ech/rec/*.rri
#
# The system default.mk has to be included also, in case the pipeline tree
# is not created in the standard MIDAS environment!
#
#cp ${MIDASHOME}/${MIDVERS}/local/default.mk uves/
#tar -uh$2f $TARFILE uves/default.mk
#rm uves/default.mk
#
#
# Copy the Installation makefile into the root (= uves/) directory so that
# after unpacking the archive the makefile will be immediatly on the top, i.e
# in the current directory. Then you just have to type make at the end.
#
tar -uh$2f $TARFILE uves/Install/makefile
tar -uh$2f $TARFILE uves/Install/ReadMe
tar -uh$2f $TARFILE uves/Install/ReleaseNotes
tar -uh$2f $TARFILE uves/Install/setup
tar -uh$2f $TARFILE uves/Install/Disclaimer.txt
tar -uh$2f $TARFILE uves/Install/HowToInstallForUser.txt
tar -uh$2f $TARFILE uves/Install/purify-quantify.mk
tar -uh$2f $TARFILE uves/Install/INSTALL.PARANAL
tar -uh$2f $TARFILE uves/Install/UvesCreaRelease.sh
#
cp uves/Install/makefile ./
cp uves/Install/ReadMe ./
cp uves/Install/ReleaseNotes ./
cp uves/Install/setup ./
cp uves/Install/Disclaimer.txt ./
cp uves/Install/HowToInstallForUser.txt ./
cp uves/Install/INSTALL.PARANAL ./
tar -uh$2f $TARFILE makefile
tar -uh$2f $TARFILE ReleaseNotes
tar -uh$2f $TARFILE ReadMe
tar -uh$2f $TARFILE setup
tar -uh$2f $TARFILE Disclaimer.txt
tar -uh$2f $TARFILE HowToInstallForUser.txt
tar -uh$2f $TARFILE INSTALL.PARANAL
\rm -f makefile ReadMe ReleaseNotes setup Disclaimer.txt HowToInstallForUser.txt INSTALL.PARANAL
#
# The context directory has not to be archived as there are only links to
# the context files: echelle.ctx and uves.ctx which have to be created by
# INSTALLATION script!
#
mv $TARFILE $lpath
cd $lpath

echo "Add uves/ path on top of tree structure to be compliant with MIDAS ..."
# add the uves/ directory on top of the structure
mkdir -p tmp_pipe_release/uves-$PIP_VERS/Dictionary
mv $TARFILE tmp_pipe_release/uves-$PIP_VERS
cd tmp_pipe_release/uves-$PIP_VERS
#dfs4 specific path
#cp -r ${HOME}/DfsPrivatePipelines/dic/ESO-DFS-DIC.UVES_QC ./Dictionary
cp -r ${PIPE_HOME}/uves/Dictionary/ESO-DFS-DIC.UVES_QC ./Dictionary

tar -xf $TARFILE
rm $TARFILE
cd ..
tar -cf $TARFILE uves-$PIP_VERS
mv $TARFILE $lpath
cd $lpath
rm -rf tmp_pipe_release

# echo "Compress the archive (gzip) ..."
gzip $TARFILE
echo "FINISH."




















