//$Id: uves.dpr,v 1.17 2002/08/16 15:24:01 amodigli Exp $
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT Data Flow System
//AUTHOR:    S.Wolf - ESO/DMD/DPG
//SUBSYSTEM: Pipeline
//PURPOSE:   Definition of reduction rules for UT2/UVES
//
//$Name:  $
//$Revision: 1.17 $
//$Log: uves.dpr,v $
//Revision 1.17  2002/08/16 15:24:01  amodigli
//Correct FIB_ORDEF_x into FIB_ORDEF_TABLE_x
//
//Revision 1.16  2002/05/30 06:33:21  amodigli
//Added support of point,extended,multiple sources
//
//Revision 1.15  2002/04/30 16:59:35  amodigli
//Supported new DRS, Guess order, line tab for
//Fiber mode
//
//Revision 1.14  2002/04/16 09:13:43  amodigli
//Simplified rules
//
//Revision 1.13  2002/04/11 15:31:55  amodigli
//version working for FLAMES
//
//Revision 1.12  2001/05/02 15:14:07  amodigli
//Added some cases to support FIBER mode
//
//Revision 1.11  2001/03/09 12:21:08  amodigli
//Takem out some commented commands
//
//Revision 1.10  2001/03/02 14:16:11  amodigli
//Now is case sensitive
//
//Revision 1.9  2001/01/04 11:09:40  amodigli
//Added Id, Name, Revision, Log conf. control info
//
//Revision 1.8  2000/04/03 13:08:02  oboitqui
//stability check added by Andreas KAUFER
//
//Revision 1.7  2000/03/31 16:22:30  oboitqui
//Paranal 2 release
//
//Revision 1.6  2000/03/06 18:01:36  ddorigo
//*** empty log message ***
//
//Revision 1.5  2000/03/06 17:23:44  ddorigo
//*** empty log message ***
//
//Revision 1.4  2000/03/03 14:49:24  oboitqui
//*** empty log message ***
//
//Revision 1.3  2000/03/03 12:15:08  oboitqui
//Release for Paronal including slicer and efficiency corrections.
//
//Revision 1.2  2000/02/16 14:23:00  oboitqui
//Corrected typo
//
//Revision 1.1  2000/02/16 14:05:12  oboitqui
//First release:
//files taken from quality/calibDB area
//
//Revision 1.21  1999/06/09 08:56:32  quality
//ntt update
//
//Revision 1.19  1999/05/14 12:56:13  quality
//ntt update
//
//Revision 1.18  1999/05/04 15:26:21  quality
//ntt update
//
//Revision 1.17  1999/03/29 13:31:19  quality
//ntt update
//
//Revision 1.11  1999/03/29 12:01:44  quality
//ntt update
//
//Revision 1.9  1999/03/26 17:21:57  quality
//ntt update
//
//Revision 1.3  1999/03/16 09:39:43  quality
//ntt update
//
//Revision 1.2  1999/03/10 17:38:40  quality
//ntt update
//
//Revision 1.1  1999/03/09 12:51:51  quality
//Initial revision
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
instrument: "UVES";
name:       "$Id: uves.dpr,v 1.17 2002/08/16 15:24:01 amodigli Exp $";
version:    "$Revision: 1.17 $";



//
// Reduction rules for signal_newFrame
// ===================================
//

UVES_MASTER_BIAS_B(REF_IMAGE){
      execute uves_cal_mkmaster(REF_IMAGE);
}

UVES_MASTER_BIAS_R(REF_IMAGE){
      execute uves_cal_mkmaster(REF_IMAGE);
}

UVES_MASTER_DARK_B(REF_IMAGE){
  if (ref_bias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_BLUE,
                         MIN_RET=1, MAX_RET=1))
      execute uves_cal_mkmaster(REF_IMAGE,ref_bias);
}


UVES_MASTER_DARK_R(REF_IMAGE){
  if (ref_bias_l=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_u=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDU,
                         MIN_RET=1, MAX_RET=1))
      execute uves_cal_mkmaster(REF_IMAGE,ref_bias_l,ref_bias_u);
}


UVES_MASTER_FLAT_B(REF_IMAGE){
  if (ref_bias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_BLUE,
                         MIN_RET=0, MAX_RET=1) &&
      ref_dark=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK_BLUE,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1))
      execute uves_cal_mkmaster(REF_IMAGE,ref_bias,ref_dark,ref_drs,ref_order,ref_back);
}


UVES_MASTER_FLAT_R(REF_IMAGE){
  if (ref_bias_l=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_dark_l=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_l=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_l=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_l=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_u=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_dark_u=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_u=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_u=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1))
      execute uves_cal_mkmaster(REF_IMAGE,ref_bias_l,ref_dark_l,ref_drs_l,ref_order_l,ref_back_l,ref_bias_u,ref_dark_u,ref_drs_u,ref_order_u,ref_back_u);
}

UVES_CAL_PREDICT_R(REF_IMAGE){
  if (ref_line=gen_query(REF_IMAGE, DO_CATG=LINE_REFER_TABLE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_form_l=gen_query(REF_IMAGE, DO_CATG=MASTER_FORM_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_form_u=gen_query(REF_IMAGE, DO_CATG=MASTER_FORM_REDU,
                         MIN_RET=0, MAX_RET=1))
    execute uves_cal_predict(REF_IMAGE,ref_line,ref_form_l,ref_form_u);
}

UVES_CAL_PREDICT_B(REF_IMAGE){
  if (ref_line=gen_query(REF_IMAGE, DO_CATG=LINE_REFER_TABLE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_form=gen_query(REF_IMAGE, DO_CATG=MASTER_FORM_BLUE,
                         MIN_RET=0, MAX_RET=1))
    execute uves_cal_predict(REF_IMAGE,ref_line,ref_form);
}


UVES_CAL_ORDERPOS_B(REF_IMAGE){
  if (ref_drs=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_BLUE,
                         MIN_RET=0, MAX_RET=1))
	execute uves_cal_orderpos(REF_IMAGE,ref_drs);
}


UVES_CAL_ORDERPOS_R(REF_IMAGE){
  if (ref_drs_l=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDU,
                         MIN_RET=0, MAX_RET=1))
	execute uves_cal_orderpos(REF_IMAGE,ref_drs_l,ref_drs_u);
}


UVES_CAL_WAVECAL_B(REF_IMAGE){
  if (ref_drs=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_line=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE,
                         MIN_RET=0, MAX_RET=1) &&
      ref_lref=gen_query(REF_IMAGE, DO_CATG=LINE_REFER_TABLE,
                         MIN_RET=1, MAX_RET=1))
	execute uves_cal_wavecal(REF_IMAGE,ref_drs,ref_order,ref_line,ref_lref);
}

UVES_CAL_WAVECAL_R(REF_IMAGE){
  if (ref_drs_l=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_l=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_line_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_u=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_line_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_lref=gen_query(REF_IMAGE, DO_CATG=LINE_REFER_TABLE,
                         MIN_RET=1, MAX_RET=1))
	execute uves_cal_wavecal(REF_IMAGE,ref_drs_l,ref_order_l,ref_line_l,ref_drs_u,ref_order_u,ref_line_u,ref_lref);
}


UVES_CAL_RESPONSE_B(REF_IMAGE){
  if (ref_drs=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_BLUE,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_flux=gen_query(REF_IMAGE, DO_CATG=FLUX_STD_TABLE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_extc=gen_query(REF_IMAGE, DO_CATG=EXTCOEFF_TABLE,
                         MIN_RET=1, MAX_RET=1))
	execute uves_cal_response(REF_IMAGE,ref_drs,ref_order,ref_back,ref_lin1,ref_lin2,ref_lin3,ref_bias,ref_flat,ref_flux,ref_extc);
}

UVES_CAL_RESPONSE_R(REF_IMAGE){
  if (ref_drs_l=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_l=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_l=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_l=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_l=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_u=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_u=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_u=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_u=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_flux=gen_query(REF_IMAGE, DO_CATG=FLUX_STD_TABLE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_extc=gen_query(REF_IMAGE, DO_CATG=EXTCOEFF_TABLE,
                         MIN_RET=1, MAX_RET=1))
	execute uves_cal_response(REF_IMAGE,ref_drs_l,ref_order_l,ref_back_l,ref_lin1_l,ref_lin2_l,ref_lin3_l,ref_bias_l,ref_flat_l,ref_drs_u,ref_order_u,ref_back_u,ref_lin1_u,ref_lin2_u,ref_lin3_u,ref_bias_u,ref_flat_u,ref_flux,ref_extc);
}


UVES_OBS_EXTRORD_B(REF_IMAGE){
  if (ref_drs=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_ord=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1))
      execute uves_obs_extrord(REF_IMAGE,ref_drs,ref_ord);
}

UVES_OBS_EXTRORD_R(REF_IMAGE){
  if (ref_drs_l=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_ord_l=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_ord_u=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1))
      execute uves_obs_extrord(REF_IMAGE,ref_drs_l,ref_drs_u,ref_ord_l,ref_ord_u);
}

UVES_OBS_SCIRED_B(REF_IMAGE){
  if (ref_drs=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_BLUE,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_BLUE,
                         MIN_RET=0, MAX_RET=1))
	execute uves_obs_scired(REF_IMAGE,ref_drs,ref_order,ref_back,ref_lin1,ref_lin2,ref_lin3,ref_bias,ref_flat,ref_resp);
}

UVES_OBS_SCIRED_R(REF_IMAGE){
  if (ref_drs_l=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_l=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_l=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_l=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_l=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp_l=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_u=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_u=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_u=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_u=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp_u=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_REDU,
                         MIN_RET=0, MAX_RET=1))
	execute uves_obs_scired(REF_IMAGE,ref_drs_l,ref_order_l,ref_back_l,ref_lin1_l,ref_lin2_l,ref_lin3_l,ref_bias_l,ref_flat_l,ref_resp_l,ref_drs_u,ref_order_u,ref_back_u,ref_lin1_u,ref_lin2_u,ref_lin3_u,ref_bias_u,ref_flat_u,ref_resp_u);
}



UVES_OBS_EXTENDED_B(REF_IMAGE){
  if (ref_drs=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_BLUE,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_BLUE,
                         MIN_RET=0, MAX_RET=1))
	execute uves_obs_spatred(REF_IMAGE,ref_drs,ref_order,ref_back,ref_lin1,ref_lin2,ref_lin3,ref_bias,ref_flat,ref_resp);
}

UVES_OBS_EXTENDED_R(REF_IMAGE){
  if (ref_drs_l=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_l=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_l=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_l=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_l=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp_l=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_u=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_u=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_u=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_u=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp_u=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_REDU,
                         MIN_RET=0, MAX_RET=1))
	execute uves_obs_spatred(REF_IMAGE,ref_drs_l,ref_order_l,ref_back_l,ref_lin1_l,ref_lin2_l,ref_lin3_l,ref_bias_l,ref_flat_l,ref_resp_l,ref_drs_u,ref_order_u,ref_back_u,ref_lin1_u,ref_lin2_u,ref_lin3_u,ref_bias_u,ref_flat_u,ref_resp_u);
}




UVES_OBS_MULTIPLE_B(REF_IMAGE){
  if (ref_drs=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_BLUE,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_BLUE,
                         MIN_RET=0, MAX_RET=1))
	execute uves_obs_spatred(REF_IMAGE,ref_drs,ref_order,ref_back,ref_lin1,ref_lin2,ref_lin3,ref_bias,ref_flat,ref_resp);
}

UVES_OBS_MULTIPLE_R(REF_IMAGE){
  if (ref_drs_l=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_l=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_l=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_l=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_l=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp_l=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_u=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_u=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_u=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_u=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp_u=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_REDU,
                         MIN_RET=0, MAX_RET=1))
	execute uves_obs_spatred(REF_IMAGE,ref_drs_l,ref_order_l,ref_back_l,ref_lin1_l,ref_lin2_l,ref_lin3_l,ref_bias_l,ref_flat_l,ref_resp_l,ref_drs_u,ref_order_u,ref_back_u,ref_lin1_u,ref_lin2_u,ref_lin3_u,ref_bias_u,ref_flat_u,ref_resp_u);
}



UVES_OBS_POINT_B(REF_IMAGE){
  if (ref_drs=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_BLUE,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_BLUE,
                         MIN_RET=0, MAX_RET=1))
	execute uves_obs_scired(REF_IMAGE,ref_drs,ref_order,ref_back,ref_lin1,ref_lin2,ref_lin3,ref_bias,ref_flat,ref_resp);
}

UVES_OBS_POINT_R(REF_IMAGE){
  if (ref_drs_l=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_l=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_l=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_l=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_l=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp_l=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_u=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_u=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_u=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_u=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp_u=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_REDU,
                         MIN_RET=0, MAX_RET=1))
	execute uves_obs_scired(REF_IMAGE,ref_drs_l,ref_order_l,ref_back_l,ref_lin1_l,ref_lin2_l,ref_lin3_l,ref_bias_l,ref_flat_l,ref_resp_l,ref_drs_u,ref_order_u,ref_back_u,ref_lin1_u,ref_lin2_u,ref_lin3_u,ref_bias_u,ref_flat_u,ref_resp_u);
}

UVES_CCDTEST(REF_IMAGE){
      execute uves_cal_ccdtest(REF_IMAGE);
}

// Image slicer

UVES_OBS_SLICER_SCIRED_B(REF_IMAGE){
  if (ref_drs=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_BLUE3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_BLUE,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_BLUE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_BLUE,
                         MIN_RET=0, MAX_RET=1))
	execute uves_obs_slicer_scired(REF_IMAGE,ref_drs,ref_order,ref_back,ref_lin1,ref_lin2,ref_lin3,ref_bias,ref_flat,ref_resp);
}

UVES_OBS_SLICER_SCIRED_R(REF_IMAGE){
  if (ref_drs_l=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_l=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_l=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_l=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDL3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_l=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_l=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp_l=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_u=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_u=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin1_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU1,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin2_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU2,
                         MIN_RET=1, MAX_RET=1) &&
      ref_lin3_u=gen_query(REF_IMAGE, DO_CATG=LINE_TABLE_REDU3,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_u=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_flat_u=gen_query(REF_IMAGE, DO_CATG=MASTER_FLAT_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_resp_u=gen_query(REF_IMAGE, DO_CATG=INSTR_RESPONSE_REDU,
                         MIN_RET=0, MAX_RET=1))
	execute uves_obs_slicer_scired(REF_IMAGE,ref_drs_l,ref_order_l,ref_back_l,ref_lin1_l,ref_lin2_l,ref_lin3_l,ref_bias_l,ref_flat_l,ref_resp_l,ref_drs_u,ref_order_u,ref_back_u,ref_lin1_u,ref_lin2_u,ref_lin3_u,ref_bias_u,ref_flat_u,ref_resp_u);
}

//
// FIBER MODE
// ==========
//

FLAMES_CAL_PREDICT_R(REF_IMAGE){
  if (ref_line=gen_query(REF_IMAGE, DO_CATG=LINE_REFER_TABLE,
                         MIN_RET=1, MAX_RET=1) &&
      ref_form_l=gen_query(REF_IMAGE, DO_CATG=MASTER_FORM_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_form_u=gen_query(REF_IMAGE, DO_CATG=MASTER_FORM_REDU,
                         MIN_RET=0, MAX_RET=1))
	execute flames_cal_predict(REF_IMAGE,ref_line,ref_form_l,ref_form_u);
}


FLAMES_CAL_ORDERPOS_R(REF_IMAGE){
 if (ref_drs_l=gen_query(REF_IMAGE, DO_CATG=FIB_DRS_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=FIB_DRS_REDU,
                         MIN_RET=1, MAX_RET=1))
	execute flames_cal_orderpos(REF_IMAGE,ref_drs_l,ref_drs_u);	
	
}

FLAMES_CAL_FIBORDPOS_R(REF_IMAGE){
 if (ref_mbia_l=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDL,
                         MIN_RET=0, MAX_RET=1) &&
     ref_mbia_u=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDU,
                         MIN_RET=0, MAX_RET=1) &&
     ref_order_l=gen_query(REF_IMAGE, DO_CATG=FIB_ORD_GUE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
     ref_order_u=gen_query(REF_IMAGE, DO_CATG=FIB_ORD_GUE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
     ref_sflat_l=gen_query(REF_IMAGE, DO_CATG=SFLAT_REDL,
                         MIN_RET=1, MAX_RET=5) &&
     ref_sflat_u=gen_query(REF_IMAGE, DO_CATG=SFLAT_REDU,
                         MIN_RET=1, MAX_RET=5))
	execute flames_cal_prep_sff_ofpos(REF_IMAGE,ref_mbia_l,ref_mbia_u,ref_order_l,ref_order_u,ref_sflat_l,ref_slat_u);
}



FLAMES_CAL_WAVECAL_R(REF_IMAGE){
  if (ref_lref=gen_query(REF_IMAGE, DO_CATG=LINE_REFER_TABLE,
                         MIN_RET=1, MAX_RET=1)
      ref_drs_l=gen_query(REF_IMAGE, DO_CATG=FIB_DRS_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_line_l=gen_query(REF_IMAGE, DO_CATG=FIB_LIN_GUE_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=FIB_DRS_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_line_u=gen_query(REF_IMAGE, DO_CATG=FIB_LIN_GUE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_l=gen_query(REF_IMAGE, DO_CATG=FIB_ORDEF_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&   
      ref_order_u=gen_query(REF_IMAGE, DO_CATG=FIB_ORDEF_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1))
	execute flames_cal_wavecal(REF_IMAGE,ref_der_l,ref_order_l,ref_line_l,ref_der_u,ref_order_u,ref_line_u,ref_lref);
}



FLAMES_OBS_SCIRED_R(REF_IMAGE){
 if (ref_mbias_l=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_mbias_u=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDU,
                         MIN_RET=1, MAX_RET=1) &&                        
      ref_line_l=gen_query(REF_IMAGE, DO_CATG=FIB_LINE_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_line_u=gen_query(REF_IMAGE, DO_CATG=FIB_LINE_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&                         
      ref_sff_com_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_COM_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_sff_nor_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_NOR_REDL,
                         MIN_RET=1, MAX_RET=1) && 
      ref_sff_com_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_COM_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_sff_nor_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_NOR_REDU,
                         MIN_RET=1, MAX_RET=1) && 
      ref_order_l=gen_query(REF_IMAGE, DO_CATG=FIB_ORDEF_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&                         
      ref_order_u=gen_query(REF_IMAGE, DO_CATG=FIB_ORDEF_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&           
      ref_sff_d01_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_D01_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_sff_s01_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_S01_REDL,
                         MIN_RET=1, MAX_RET=1) && 
      ref_sff_bp1_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BP1_REDL,
                         MIN_RET=1, MAX_RET=1) && 
      ref_sff_bnd1_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BND1_REDL,
                         MIN_RET=1, MAX_RET=1) && 
      ref_sff_d01_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_D01_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_sff_s01_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_S01_REDU,
                         MIN_RET=1, MAX_RET=1) && 
      ref_sff_bp1_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BP1_REDU,
                         MIN_RET=1, MAX_RET=1) && 
      ref_sff_bnd1_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BND1_REDU,
                         MIN_RET=1, MAX_RET=1) &&   
      ref_sff_d02_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_D02_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_sff_s02_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_S02_REDL,
                         MIN_RET=0, MAX_RET=1) && 
      ref_sff_bp2_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BP2_REDL,
                         MIN_RET=0, MAX_RET=1) && 
      ref_sff_bnd2_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BND2_REDL,
                         MIN_RET=0, MAX_RET=1) && 
      ref_sff_d02_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_D02_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_sff_s02_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_S02_REDU,
                         MIN_RET=0, MAX_RET=1) && 
      ref_sff_bp2_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BP2_REDU,
                         MIN_RET=0, MAX_RET=1) && 
      ref_sff_bnd2_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BND2_REDU,
                         MIN_RET=0, MAX_RET=1) &&  
      ref_sff_d03_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_D03_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_sff_s03_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_S03_REDL,
                         MIN_RET=0, MAX_RET=1) && 
      ref_sff_bp3_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BP3_REDL,
                         MIN_RET=0, MAX_RET=1) && 
      ref_sff_bnd3_l=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BND3_REDL,
                         MIN_RET=0, MAX_RET=1) && 
      ref_sff_d03_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_D03_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_sff_s03_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_S03_REDU,
                         MIN_RET=0, MAX_RET=1) && 
      ref_sff_bp3_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BP3_REDU,
                         MIN_RET=0, MAX_RET=1) && 
      ref_sff_bnd3_u=gen_query(REF_IMAGE, DO_CATG=SLIT_FF_BND3_REDU,
                         MIN_RET=0, MAX_RET=1) &&                                                                                                               
      ref_fff_com_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_COM_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_fff_nor_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_NOR_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_fff_nsi_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_NSIGMA_REDL,
                         MIN_RET=1, MAX_RET=1) && 
      ref_fff_com_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_COM_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_fff_nor_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_NOR_REDU,
                         MIN_RET=1, MAX_RET=1) && 
      ref_fff_nsi_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_NSIGMA_REDU,
                         MIN_RET=1, MAX_RET=1) &&                                 
      ref_fff_d01_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_D01_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_fff_s01_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_S01_REDL,
                         MIN_RET=1, MAX_RET=1) && 
      ref_fff_bp1_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_BP1_REDL,
                         MIN_RET=1, MAX_RET=1) && 
      ref_fff_d01_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_D01_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_fff_s01_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_S01_REDU,
                         MIN_RET=1, MAX_RET=1) && 
      ref_fff_bp1_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_BP1_REDU,
                         MIN_RET=1, MAX_RET=1) &&      
      ref_fff_d02_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_D02_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_fff_s02_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_S02_REDL,
                         MIN_RET=0, MAX_RET=1) && 
      ref_fff_bp2_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_BP2_REDL,
                         MIN_RET=0, MAX_RET=1) && 
      ref_fff_d02_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_D02_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_fff_s02_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_S02_REDU,
                         MIN_RET=0, MAX_RET=1) && 
      ref_fff_bp2_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_BP2_REDU,
                         MIN_RET=0, MAX_RET=1) &&                     
      ref_fff_d03_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_D03_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_fff_s03_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_S03_REDL,
                         MIN_RET=0, MAX_RET=1) && 
      ref_fff_bp3_l=gen_query(REF_IMAGE, DO_CATG=FIB_FF_BP3_REDL,
                         MIN_RET=0, MAX_RET=1) && 
      ref_fff_d03_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_D03_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_fff_s03_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_S03_REDU,
                         MIN_RET=0, MAX_RET=1) && 
      ref_fff_bp3_u=gen_query(REF_IMAGE, DO_CATG=FIB_FF_BP3_REDU,
                         MIN_RET=0, MAX_RET=1))
execute flames_obs_scired(REF_IMAGE,ref_mbias_l,ref_order_l,ref_line_l,ref_mbias_u,ref_order_u,ref_line_u,ref_sff_com_l,ref_sff_nor_l,ref_sff_d01_l,ref_sff_d02_l,ref_sff_d03_l,ref_sff_s01_l,ref_sff_s02_l,ref_sff_s03_l,ref_sff_bp1_l,ref_sff_bp2_l,ref_sff_bp3_l,ref_sff_bnd1_l,ref_sff_bnd2_l,ref_sff_bnd3_l,ref_sff_com_u,ref_sff_nor_u,ref_sff_d01_u,ref_sff_d02_u,ref_sff_d03_u,ref_sff_s01_u,ref_sff_s02_u,ref_sff_s03_u,ref_sff_bp1_u,ref_sff_bp2_u,ref_sff_bp3_u,ref_sff_bnd1_u,ref_sff_bnd2_u,ref_sff_bnd3_u,ref_fff_com_l,ref_fff_nor_l,ref_fff_nsi_l,ref_fff_d01_l,ref_fff_d02_l,ref_fff_d03_l,ref_fff_s01_l,ref_fff_s02_l,ref_fff_s03_l,ref_fff_bp1_l,ref_fff_bp2_l,ref_fff_bp3_l,ref_fff_com_u,ref_fff_nor_u,ref_fff_nsi_u,ref_fff_d01_u,ref_fff_d02_u,ref_fff_d03_u,ref_fff_s01_u,ref_fff_s02_u,ref_fff_s03_u,ref_fff_bp1_u,ref_fff_bp2_u,ref_fff_bp3_u);

}


FLAMES_MASTER_FLAT_R(REF_IMAGE){
  if (ref_bias_l=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_dark_l=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK_REDL,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_l=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_l=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_l=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDL,
                         MIN_RET=1, MAX_RET=1) &&
      ref_bias_u=gen_query(REF_IMAGE, DO_CATG=MASTER_BIAS_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_dark_u=gen_query(REF_IMAGE, DO_CATG=MASTER_DARK_REDU,
                         MIN_RET=0, MAX_RET=1) &&
      ref_drs_u=gen_query(REF_IMAGE, DO_CATG=DRS_SETUP_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_order_u=gen_query(REF_IMAGE, DO_CATG=ORDER_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1) &&
      ref_back_u=gen_query(REF_IMAGE, DO_CATG=BACKGR_TABLE_REDU,
                         MIN_RET=1, MAX_RET=1))
      execute flames_cal_mkmaster(REF_IMAGE,ref_bias_l,ref_dark_l,ref_drs_l,ref_order_l,ref_back_l,ref_bias_u,ref_dark_u,ref_drs_u,ref_order_u,ref_back_u);
}





