C $Id: calc_i.for,v 1.3 2001/11/01 09:30:16 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)calc_i.for	2.4 (ESO-DFS) 23/10/01 12:26:19
C derived from necopt.for
C
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  subroutine  calc_i.for  version 1.0 011023 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C To do a weighted mean of the x-order profile
C 
C.SYNTAX
C
C.HISTORY:
C  2001.10.23-AM Created
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.3 $
C $Log: calc_i.for,v $
C Revision 1.3  2001/11/01 09:30:16  amodigli
C Eliminated several warnings from ftnchek
C
C Revision 1.2  2001/11/01 08:17:50  amodigli
C Eliminated some warning indicated by ftncheck
C
C Revision 1.1  2001/11/01 07:20:54  amodigli
C First version of necopt with splitted data
C
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     23/10/01       Created                           all
C------------------------------------------------------------------------------
C  Role: to do a weighted mean of the FF signal using the weight image coming
C       from the object extraction
C------------------------------------------------------------------------------
c here extracts the profiles and the variances doing a weighted mean of the 
c profile and its variance
C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.
      SUBROUTINE CALC_I(IN,VARI,PROF,YVAL,OUT,VOUT
     1     ,NPIXA1,NPIXA2,NB1,NB2,DIMP,IXMIN,IXMAX,IWIDTH,INDX)
      
      IMPLICIT NONE

      INTEGER IXMIN                         !index corresponding to Xmin pos
      INTEGER IXMAX                         !index corresponding to Xmax pos
      INTEGER IWIDTH                        !Y size of order
      INTEGER INDX                          !current Y position
      INTEGER DIMP                          !x-order size of profile
      INTEGER NPIXA1                        !x size of input frm
      INTEGER NPIXA2                        !y size of input frm
      INTEGER NB1                           !x size of output frm
      INTEGER NB2                           !y size of output frm

      REAL IN(NPIXA1,NPIXA2)                !input raw ima 
      REAL VARI(NPIXA1,NPIXA2)              !input variance ima
      REAL OUT(NB1,NB2)                     !out ima
      REAL VOUT(NB1,NB2)                    !output variance ima
      REAL PROF(NPIXA1,DIMP)                !profile (weights)

      DOUBLE PRECISION YVAL(NPIXA1)         !Y order position(X) 

C
C     Local variables
C
      INTEGER I                             !Loop variable
      INTEGER J                             !Loop variable
      INTEGER K                             !current x-order pos index
      INTEGER KMAX                          !max value of x-order point
      INTEGER L                             !Aux var for Y point

      REAL PART                !Fraction of pix holder

      DOUBLE PRECISION SUM1                 !Image sum along x-order dir
      DOUBLE PRECISION SUM2                 !weight sum along x-order dir
      DOUBLE PRECISION SUM3                 !var ima sum along x-order dir


 
          
      DO I = IXMIN,IXMAX
         SUM3 = 0
         SUM2 = 0               !do the extraction ...
         SUM1 = 0               !... sum over spatial coordinate
      IF (IXMIN.EQ.IXMAX) GOTO 10 !skip if the order is just 1 point

c change?
         K = INT(YVAL(I)-DBLE(IWIDTH/2.0))
               
c     am added to skip problems if outside the detector        
          IF ( (K .LT. 1) .OR. (K .GT. 4096) )  GOTO 10 
          
         
         
                 
c     am: added to trace an error     
c         print *,'We are inside necopt.for, subroutine CALC_I 4b'                   
         KMAX = K+DIMP-1


C
C  Do the extraction
C

         IF (NINT(YVAL(I)).EQ.INT(YVAL(I))) THEN
            PART = SNGL(NINT(YVAL(I)-0.5D0)-YVAL(I)+0.5D0)
            SUM1 = DBLE(PROF(I,1)*IN(I,K)*PART)
            SUM2 = DBLE(PROF(I,1))
            SUM3 = DBLE(PROF(I,1)*PROF(I,1)*VARI(I,K)*PART)
            DO J=K+1,KMAX-2
               L = J-K+1
               SUM1 = SUM1 + DBLE(PROF(I,L)*IN(I,J))
               SUM2 = SUM2 + DBLE(PROF(I,L))
               SUM3 = SUM3 + DBLE(PROF(I,L)*PROF(I,L)*VARI(I,J))
            ENDDO
            J = KMAX-1
            PART = SNGL(0.5D0+YVAL(I)-NINT(YVAL(I)-0.5D0))
            SUM1 = SUM1+DBLE(PROF(I,DIMP-1)*IN(I,J)*PART)
            SUM2 = SUM2+DBLE(PROF(I,DIMP-1))
            SUM3 = SUM3+DBLE(PROF(I,DIMP-1)*PROF(I,DIMP-1)*
     +                       VARI(I,J)*PART)
         ELSE
            PART = SNGL(1-(YVAL(I)-0.5D0-NINT(YVAL(I)-0.5D0)))
            SUM1 = DBLE(PROF(I,2)*IN(I,K+1)*PART)
            SUM2 = DBLE(PROF(I,2))
            SUM3 = DBLE(PROF(I,2)*PROF(I,2)*VARI(I,K+1)*PART)
            DO J= K+2,KMAX-1
               L = J-K+1
               SUM1 = SUM1 + DBLE(PROF(I,L)*IN(I,J))
               SUM2 = SUM2 + DBLE(PROF(I,L))
               SUM3 = SUM3 + DBLE(PROF(I,L)*PROF(I,L)*VARI(I,J))
            ENDDO
            J = KMAX
            PART = SNGL(YVAL(I)-0.5D0-NINT(YVAL(I)-0.5D0))
            SUM1 = SUM1 + DBLE(PROF(I,DIMP)*IN(I,J)*PART)
            SUM2 = SUM2 + DBLE(PROF(I,DIMP))
            SUM3 = SUM3 + DBLE(PROF(I,DIMP)*PROF(I,DIMP)*VARI(I,J)*PART)
         ENDIF
         



c     am added CONTINUE to be able to exit from the loop
c        when K<1 or K>4096
10    CONTINUE


         IF (SUM2.NE.0) THEN
c            OUT(I,INDX)  = SUM1/SUM2/(DIMP-2)               !extracted spect. 
c            VOUT(I,INDX) = SUM3/SUM2/SUM2/(DIMP-2)/(DIMP-2) !and its variance
            OUT(I,INDX)  = SNGL(SUM1/SUM2)                         !extracted spect. 
            VOUT(I,INDX) = SNGL(SUM3/SUM2/SUM2)                    !and its variance
         ELSE
            OUT(I,INDX)  = 1.0
            VOUT(I,INDX) = 1.0
         ENDIF            

c To be consistent the variance has to be devided by the square of the
c number of extracted lines as the extracted data are averged data!
c THIS HAS BEEN CHANGED NOW (07.12.99-SW): extracted data are NOT averaged!


      ENDDO
      RETURN
      END
C ===================================================================================
