C $Id: calc_p.for,v 1.3 2001/11/01 09:30:17 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)calc_p.for	2.4 (ESO-DFS) 23/10/01 12:26:19
C derived from necopt.for
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  subroutine  CALC_P  version 1.0 910120 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C To do a weighted mean of the x-order profile
C
C.SYNTAX
C
C.HISTORY:
C  2001.10.23-AM Created
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.3 $
C $Log: calc_p.for,v $
C Revision 1.3  2001/11/01 09:30:17  amodigli
C Eliminated several warnings from ftnchek
C
C Revision 1.2  2001/11/01 08:17:51  amodigli
C Eliminated some warning indicated by ftncheck
C
C Revision 1.1  2001/11/01 07:20:56  amodigli
C First version of necopt with splitted data
C
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     23/10/01       Created                           all
C------------------------------------------------------------------------------
C  Role: to do a weighted mean of the FF signal using the weight image coming
C       from the object extraction
C------------------------------------------------------------------------------
c here extracts the profiles and the variances doing a weighted mean of the 
c profile and its variance
      SUBROUTINE CALC_P(IN,VARI,PROF,YVAL,OUT,VOUT
     $     ,NPIXA1,NPIXA2,NB1,NB2,DIMP,IXMIN,IXMAX,IWIDTH,INDX)
      

      IMPLICIT NONE
      INTEGER IXMIN                       !index corresponding to Xmin pos
      INTEGER IXMAX                       !index corresponding to Xmax pos
      INTEGER IWIDTH                      !Y size of order
      INTEGER INDX                        !current Y position
      INTEGER DIMP                        !x-order size of profile
      INTEGER NPIXA1                      !x size of input frm
      INTEGER NPIXA2                      !y size of input frm
      INTEGER NB1                         !x size of output frm
      INTEGER NB2                         !y size of output frm

      REAL IN(NPIXA1,NPIXA2)              !input raw ima
      REAL VARI(NPIXA1,NPIXA2)            !input variance ima
      REAL OUT(NB1,NB2)                   !out ima
      REAL VOUT(NB1,NB2)                  !output variance ima
      REAL PROF(NPIXA1,DIMP)              !profile (weights)

      DOUBLE PRECISION YVAL(NPIXA1)       !Y order position(X) 

C
C     Local variables
C
      INTEGER I                           !Loop variable
      INTEGER J                           !Loop variable
      INTEGER K                           !current x-order pos index
      INTEGER KMAX                        !max value of x-order point
      INTEGER L                           !Aux var for Y point
      INTEGER IWIDTH2                     !index Y position of order center

      REAL PART                           !Fraction of pix holder

      DOUBLE PRECISION SUM1               !Image sum along x-order dir
      DOUBLE PRECISION SUM2               !weight sum along x-order dir
      DOUBLE PRECISION SUM3               !var ima sum along x-order dir

      IWIDTH2 = IWIDTH/2
      DO I = IXMIN,IXMAX
c change?
         K = INT(YVAL(I)-IWIDTH2)
         
c        am: added IF check to skip prtoblems if K<1 
        
         IF (K .LT. 1) GOTO 10
         KMAX = K+DIMP-1

C
C  Do the extraction
C
         PART = SNGL(INT(YVAL(I))+1-YVAL(I))
         SUM1 = DBLE(PROF(I,1)*IN(I,K)*PART)
         SUM2 = DBLE(PROF(I,1))
         SUM3 = DBLE(PROF(I,1)*PROF(I,1)*VARI(I,K)*PART)

         DO J=K+1,KMAX-1
            L = J-K+1
            SUM1 = SUM1 + DBLE(PROF(I,L)*IN(I,J))
            SUM2 = SUM2 + DBLE(PROF(I,L))
            SUM3 = SUM3 + DBLE(PROF(I,L)*PROF(I,L)*VARI(I,J))
         ENDDO

         J = K+DIMP-1
         PART = SNGL(YVAL(I)-INT(YVAL(I)))
         SUM1 = SUM1 + DBLE(PROF(I,DIMP)*IN(I,J)*PART)
         SUM2 = SUM2 + DBLE(PROF(I,DIMP))
         SUM3 = SUM3 + DBLE(PROF(I,DIMP)*PROF(I,DIMP)*VARI(I,J)*PART)

c        am: if K< 1 this CONTINUE allow to skip problems
10       CONTINUE

         IF (SUM2.NE.0) THEN
c            OUT(I,INDX)  = SUM1/SUM2/(DIMP-1)              !extracted spect. 
c            VOUT(I,INDX) = SUM3/SUM2/SUM2/(DIMP-1)/(DIMP-1)!and its variance
            OUT(I,INDX)  = SNGL(SUM1/SUM2)                        !extracted spect. 
            VOUT(I,INDX) = SNGL(SUM3/SUM2/SUM2)                   !and its variance
         ELSE
         
c     am: added to trace an error     
C          print *,'Warning: Filling OUT and VOUT with 0s !' 
                  
            OUT(I,INDX)  = 0.0
            VOUT(I,INDX) = 0.0
         ENDIF            

c To be consistent the variance has to be devided by the square of the
c number of extracted lines as the extracted data are averaged data!
c THIS HAS BEEN CHANGED NOW (07.12.99-SW): extracted data are NOT averaged!


      ENDDO
      RETURN
      END
C ================================================================================
