C $Id: caprof.for,v 1.1 2002/08/28 09:41:20 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)caprof.for	1.0 (ESO-DMD) 28/08/02 12:26:19
C+
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  subroutine  CAPROF  version 1.0 011023 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C Collapse the order in chunks, get its Gaussian center and FWHM, fit such point distribution.
C
C CALLED BY: extobj.for
C
C CALLING: g_prof.for
C          gauss_fitx (gauss_fit.c)
C          qcotr1.for
C          myavrg(my_fun.for)
C          mysigm(my_fun.for)
C          polfit(gauss_fit.c)
C          qcotro.for
C          MIDAS
C.SYNTAX
C
C.HISTORY:
C  2002.08.28-AM Created
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.1 $
C $Log: caprof.for,v $
C Revision 1.1  2002/08/28 09:41:20  amodigli
C Scorporated caprof.for and g_prof.for from necopt.for
C
C Revision 1.2  2002/08/28 09:30:41  amodigli
C Scoproprated by necopt.for
C
C
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     28/08/02       Created                           all
C==================================================================================                    
C==================================================================================
C=====SUBROUTINE TO GET ORDER POS and FWHM DISTRIBUTIONS===========================
C==================================================================================
C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C
C  AM     03/07/01       portability-D1NAME,D2NAME are     CAPROF
C                        now char*N with N multiple of 8
C
C  AM     24/05/01       BINX and BINY must not be         CAPROF
C                        initialized to 0 more than once!
C
C  AM     08/05/01       clean-comment                     CAPROF
C------------------------------------------------------------------------------
C  Role: determines the order and FWHM distribution along dispersion coordinate
C        First divide the order in subsections, collapse the signal in each 
C        subsection over dispersion direction obtaining a series of x-order
C        light distribution profiles. To these are done gaussian fits so that
C        one gets a series of order center positions and FWHMs. These two
C        distribution are interpolated with a parabola (or if the parabola
C        coefficients are not proper a linear or uniform) approximation.
C------------------------------------------------------------------------------
C Pseudocode:
C        Prepare tables to log info on obj pos/fwhm and x-order chunk profile
C        Check:
C        if an order has one point only
C           Output Variance=Flux =1
C           PROF=1
C           PBUF=VBUG=1
C           FWHM=S/N=OBJ_POS=0
C           RETURN
C        endif
C        DO K=1,NUM  :Loops over all chunks
C           DO J=1,DIM : Loop over x-order direction
C              DO=start_pix(chunk),end_pix(chunk)
C                if not masked collapse intensity (COLUMN=sum(FBVAL)*MASK)
C                if not masked collapse variance  (SIGMA=sum(VBVAL)*MASK)
C              ENDDO
C              get max of x-order profile and its pos
C              fill x-order table with x-order profile values found
C           ENDDO
C           call GAUSS_FITX to fit the x-order profile with a Gaussian+linear
C           save Gauss parameters in array PBUF,VBUF,PMSK
C           check: counts the masked points
C
C           copy PBUF,VBUF,PMSK into POSVAL,SIGVAL,POSMSK and set POSWAV
C           for later fit along order
C        ENDDO
C        if the number of masked point is less than 10 issue a warning
C        do a pre k-sigma clip on POS,FWHM values found and mask out bad points
C        do a polynomial fit of POS and FWHM unmasked point distributions
C        save results in order_table
C        calculates chi2 between fit and actual POS,FWHM distributions
C        if both the chi2(POS & FWHM) .lt. 2e-2 => good fit =>stop (goto 96)
C        if chi2(POS).gt.0.5 check coeff polyn and if bad lower degree to 1
C        else (diff tresh)   check coeff polyn and if bad lower degree to 1
C
C        if chi2(FWHM).gt.1  check coeff polyn and if bad lower degree to 1
C        else (diff tresh)   check coeff polyn and if bad lower degree to 1
C        do a polynomial fit of POS and FWHM unmasked point distributions
C        if pol degree=1 check coeff polyn(POS)  and if bad lower degree to 0
C        if pol degree=1 check coeff polyn(FWHM) and if bad lower degree to 0
C        logs the polynomial deg in the order_table
C        do a k-sigma clipping to remove bad point from POS & FWHM distribution
C           do a polynomial fit (POS)
C           do a polynomial fit (FWHM)
C           get the points which scatter the most from POS & FWHM distribution
C           mask it out
C        enddo (k-sigma)
C
C 96:    The found distributions are now Ok (end):
C
C        DO =IXMIN,IXMAX (loop along the order)
C           PBUF(2,I)=position from polynomial fit of POS  distribution
C           PBUF(3,I)=position from polynomial fit of FWHM distribution
C        ENDDO
C        Stores results in order_table
C        RETURN
C
C------------------------------------------------------------------------------
      SUBROUTINE CAPROF(PROF,VBVAL,FBVAL,MASK,PBUF,VBUF,PMSK
     $     ,COLUM,SIGMA,XPOS,OUT,VOUT,NB1,NB2,NPIXA1,IXWIND,IXMIN,IXMAX
     $     ,DIM,DIMP,PDIM,RON,CONAD,THRESH,BINSIZE,WIDTH,INDX,OBJSET
     +     ,SKYBKG,VARSKY) 

      IMPLICIT NONE
      INTEGER MINPOI
      INTEGER POLORD                     !Order of polynomial
      INTEGER NGPAR                      !No of par of Gauss+linear
      INTEGER ORDFIT                     !Order of polynomial fit
      INTEGER N_MAX_ORD                  !Expected max number of orders
      INTEGER MAX_DIM                    !Expected max X ima size
      INTEGER ND1COL
      INTEGER ND2COL
      INTEGER NQC0                       !No of QC par dumped at order 0
      PARAMETER (N_MAX_ORD=50)           !Max No. of Orders to be changed 
                                         !accordingly in many places
      PARAMETER (MAX_DIM=4096,ND1COL=18,ND2COL=4)
      REAL KAPPA                         !constant for k-sigma clipping
      PARAMETER (MINPOI=15,KAPPA=6.0,POLORD=3,ORDFIT=5,NGPAR=5,NQC0=12)
C
C     Interface variables  
C   
      INTEGER DIM                        !Y dimension of mask
      INTEGER DIMP                       !Dimention of profile
      INTEGER PDIM                       !X dimention of mask
      INTEGER NPIXA1                     !Y dimention of mask
      INTEGER NB1                        !X dim of ima
      INTEGER NB2                        !Y dim of ima
      INTEGER INDX
      INTEGER*2 MASK(NPIXA1,DIM)         !Mask array
      INTEGER*2 PMSK(PDIM,NPIXA1)        !profile mask
      INTEGER IXMIN                      !pix position of XMIN
      INTEGER IXMAX                      !pix position of XMAX
      INTEGER IXWIND                     !size of window (chunk size)
      INTEGER BINSIZE                    !Detector Bin Size
      INTEGER DTID1                      !Debug table Id.
      INTEGER DTID2                      !Id of debug table  (MIDAS env)
      INTEGER STOCH                      !current raw position in tab 1
      INTEGER STOCH2                     !current raw position in tab 2
    
      REAL SKYBKG(NB1,NB2)      !Sky bkg image  
      REAL VARSKY(NB1,NB2)      !var Sky bkg image  
      REAL MID_S_N_CENT(N_MAX_ORD)       !av. S/N at the center of the order
      REAL OBJ_POS_CENT(N_MAX_ORD)       !av. obj pos at order center
      REAL FWHM(N_MAX_ORD)               !average FWHM at the center of order

      REAL VBVAL(NPIXA1,DIM)             !variance ima
      REAL FBVAL(NPIXA1,DIM)             !raw data
      REAL PROF(NPIXA1,DIMP)             !profile
      REAL PBUF(PDIM,NPIXA1)             !Profile buffer
      REAL VBUF(PDIM,NPIXA1)             !Variance buffer
      REAL OUT(NB1,NB2)                  !Output image
      REAL VOUT(NB1,NB2)                 !Output variance
      REAL RON                           !Read Out Noise
      REAL CONAD                         !Analog To Digital conversion factor
      REAL THRESH                        !threshold??
      
      REAL XPOS(DIM)
      REAL COLUM(DIM)
      REAL SIGMA(DIM)                    
      REAL OBJSET                        !position of object

      DOUBLE PRECISION WIDTH             !Slit width

      
C
C     Local variables
C

     
      INTEGER IPOI                       !counter variable
      INTEGER I                          !loop variable
      INTEGER J                          !loop variable
      INTEGER IXWMAX                     !IXWMAX=IXMIN+(IXMAX-IXMIN)/IXWIND-1  
      INTEGER*2 IMASK                    !mask: 1=valid pixel, 0:invalid
      INTEGER ISTAT                      !Status of operation (MIDAS env)
      INTEGER JWIN                       !JWIN=2*BINSIZE
      INTEGER LP(NGPAR)                  !data structure to swich on/off fit 
                                         !coeff variation
      INTEGER MFIT                       !No. of fit coeff
      INTEGER IMASK_COUNTER              !counts how many chunks survives.
      
      INTEGER IAPOS(ORDFIT)              !auxilliary array to dis/activate fit 
                                         !of coeff in position
      INTEGER IAFWHM(ORDFIT)             !auxilliary array to dis/activate 
                                         !fit of coeff in FWHM
      INTEGER NCLEAN                     !counter
      INTEGER IDPCH                      !Id of table column (MIDAS env)
      INTEGER IDPF0                      !Id of table column (MIDAS env)
      INTEGER IDPF1                      !Id of table column (MIDAS env)
      INTEGER IDPER0                     !Id of table column (MIDAS env)
      INTEGER IDS0                       !Id of table column (MIDAS env)
      INTEGER IDS1                       !Id of table column (MIDAS env)
      INTEGER IDICH                      !Id of table column (MIDAS env)
      INTEGER IDMSK0                     !Id of table column (MIDAS env)
      INTEGER IDXCH0                     !Id of table column (MIDAS env)
      INTEGER IDORD                      !Id of table column (MIDAS env)
      INTEGER IDFWHM                     !Id of table column (MIDAS env)
      INTEGER IDFWER                     !Id of table column (MIDAS env)
      INTEGER ICURP                      !current raw position in table
      
      INTEGER IT                         !IT=ORDFIT=fit degree
      INTEGER KFIT                       !number of fit params to be modified
      INTEGER KP(ORDFIT)                 !array to spec parms to be modified 
      INTEGER IDDEG                      !Id of table column (MIDAS env)
      INTEGER IDCHI0                     !Id of table column (MIDAS env)
      INTEGER IDKS                       !Id of table column (MIDAS env)
      INTEGER BINX                       !Bin X size
      INTEGER BINY                       !Bin Y size
      INTEGER NFIT                       !counter to get chi2red
      
      
      INTEGER D1COL(ND1COL)              !Descriptor id (MIDAS env)
      INTEGER D2COL(ND2COL)              !Descriptor id (MIDAS env)
      INTEGER D1ROW                      !current raw
      INTEGER D2ROW                      !current raw
      INTEGER PDEG                       !Polynomial degree
      INTEGER SDEG                       !Polynomial degree
      INTEGER N_CURR_ORD                 !current order position

      INTEGER STATUS                     !return status from MIDAS call 


      INTEGER I1                         !Aux var
      INTEGER I2                         !aux var
      INTEGER ICURP2                     !aux var
      INTEGER IDBKGE                     !column no.
      INTEGER IDBKGV                     !column no.
      INTEGER IDMSK1                     !column no.
      INTEGER IMAX                       !aux var
      INTEGER J1                         !aux var
      INTEGER JMAX                       !aux var
      INTEGER NUM                        !aux var
      INTEGER IORDER                     !current order position
      INTEGER K                          !Loop variable
      INTEGER KK                         !Loop variable
      INTEGER kkmax                      !aux variable
      INTEGER IDQC0(NQC0)                  !aux variable to hold tab col ids
      INTEGER IFAIL                      !Variable to check if gaussj fail

      REAL CHISQ                         !chi square
      REAL EPOS                          !error on pos
      REAL AVERR                         !average error
      REAL PARAM(NGPAR)                  !parameters of Gaussian profile
      REAL VPARAM(NGPAR)                 !variance parameters
      REAL CHI2(NGPAR)                   !chi square
      REAL AMIN1                         !funct to get the min of 2 real values
      REAL LOW                           !min thresh on FWHM to mask out points
      REAL HGH                           !max thresh on FWHM to mask out points
      REAL PCOF1(POLORD)                 !profile coefficients
      REAL PCOF2(POLORD)                 !profile coefficients
      REAL POLYN3                        !polynomial of degree 3
      REAL X1                            !tmp holder for X position
      REAL X1MAX                         !tmp holder for X position of max func
      REAL XSTP                          !X step= size of chunk
      REAL KAPPAMAX                      !maximum value of k in k-sigma clip

   
      REAL TCOEF(ORDFIT)                 !true coefficient
      REAL COVAR(ORDFIT,ORDFIT)          !covariance array
      REAL POSWAV(MAX_DIM)               !wavelength of pos
      REAL POSVAL(MAX_DIM)               !value of position
      REAL POSERR(MAX_DIM)               !error on position
      REAL BKGVAL(MAX_DIM)               !value of position
      REAL BKGERR(MAX_DIM)               !error on position
      REAL POSMSK(MAX_DIM)               !mask on position
      REAL SIGVAL(MAX_DIM)               !sigma of value
      REAL SIGERR(MAX_DIM)               !sigma of error
      REAL SIGMSK(MAX_DIM)               !sigma of mask
      REAL CI                            !current index along the order
      REAL CHIFIT                        !chi-square value of fit to chunk pos
      REAL AFWHM                         !Average FWHM
      REAL EFWHM                         !Error on FWHM  
      REAL AVPOS                         !Average position
      REAL AVFWHM                        !Average FWHM

      REAL SIFWHM                        !Sigma FWHM
      REAL SIPOS                         !Sigma FWHM
      REAL SUM                           !Aux var to store sum of chi2
      REAL XFIT                          !value for chi of fit
      REAL YMAX                          !Aux variable
      REAL YMIN                          !aux variable

      DOUBLE PRECISION D2VAL(ND2COL)     !Descriptor value
       
    
      CHARACTER*64 D1NAME(ND1COL)        !Descriptor name
      CHARACTER*64 D2NAME(ND2COL)        !Descriptor name
      CHARACTER*96 LINE                  !Buffer to display informations
      CHARACTER    DBG_SW                !debug switch

      EXTERNAL POLYN                     !Polynomial
      EXTERNAL POLYN3                    !Polynomial of degree 3
      EXTERNAL GAUSS_FITX                !Gaussian Fit function
      EXTERNAL POLY_FITX                 !Polynomial Fit function
      EXTERNAL POLFIT                    !Polynomial Fit function
      EXTERNAL LSFIT                     !Least Square Fit function
      EXTERNAL MYAVRG                    !average function
      EXTERNAL MYSIGM                    !rms function
      EXTERNAL G_PROF                    !subroutine to extract profile

     
C      COMMON /DBGORT/DTID1,DTID2         !to reduce par in subroutine call
C      COMMON /QCF0/IDORD,IDPCH,IDICH,IDPER0,IDFWHM,IDFWER,IDBKGV,
C     +               IDBKGE,IDMSK0,IDXCH0,IDPF0,IDS0
C      COMMON /QCF1/IDPF1,IDMSK1,IDS1 
C      COMMON /QCEXT/IDDEG,IDCHI0,IDKS      
      
      COMMON /DBGPOS/STOCH,STOCH2        !to reduce par in subroutine call 
          

 
      COMMON /FIT/BINX,BINY              !Necessary to set tresholds
                                         !over polynomial fit slope and 
                                         !curvature BINX,BINY dependent

      COMMON /DBG/IORDER                 !Used for debugging purposes?
      COMMON /QC0NM/D1NAME,D2NAME,D1COL,D2COL,IDQC0
      
      INCLUDE 'MID_INCLUDE:ST_DEF.INC'   !to get the definitions 
                                         !on the ST interfaces
      INCLUDE '../incl/common_qc_log.inc'    
      INCLUDE '../incl/common_dbgort.inc' 
      INCLUDE '../incl/common_qc.inc'  
      INCLUDE '../incl/common_qcf1.inc'  
      INCLUDE '../incl/common_vmr.inc'   
                                      
      INCLUDE 'MID_INCLUDE:ST_DAT.INC'   !to get data values for 
                                         !definitions above
                                  
                                         
      DATA LP/1,2,3,4,0/                 !To dis/activate parameter variation
      DATA MFIT/4/                       !Fit degree
      
      DATA KP/1,2,3,0,0/                 !To dis/activate parameter variation
      DATA KFIT/3/                       !Fit degree
   
C
C     Initialization of local variables: 
C     DATA variables are initialized with DATA statement
C

C     Integers
      IFAIL=0
      IPOI =0  
      I    =0  
      J    =0  
      IXWMAX=0   
      IMASK =0 
      ISTAT =0 
      JWIN  =0


      IMASK_COUNTER=0
     

      DO I=1,ORDFIT
         IAPOS(I)=0
         IAFWHM(I)=0 
      ENDDO

      NCLEAN=0

      ICURP =0            
      
      IT    =0
      KFIT  =0

        
      NFIT  =0
      K=0
      KK=0
      kkmax=0

      

      
      D1ROW =0
      D2ROW =0
      PDEG  =0
      SDEG  =0


      I1=0             
      I2=0             
      ICURP2=0         
      IMAX=0           
      J1=0             
      JMAX=0          
      NUM=0           



C     Reals

      CHISQ=0.0
      EPOS =0.0
      AVERR=0.0
      DO I=1,NGPAR
         PARAM(I)=0.0 
         VPARAM(I)=0.0
         CHI2(I)=0.0  
      ENDDO

      LOW  =0.0
      HGH  =0.0
      DO I=1,POLORD
         PCOF1(I)=0.0
         PCOF2(I)=0.0
      ENDDO

C      POLYN3               !this is a function
      X1    =0.0  
      XSTP  =0.0  
      KAPPAMAX=0.0
     
     
      
      DO I=1,ORDFIT
      TCOEF(I)=0.0           
         DO J=1,ORDFIT
            COVAR(I,J)=0.0   
         ENDDO
      ENDDO

      DO I=1,MAX_DIM
         POSWAV(I)=0.0
         POSVAL(I)=0.0
         POSERR(I)=0.0
         POSMSK(I)=0.0
         SIGVAL(I)=0.0
         SIGERR(I)=0.0
         SIGMSK(I)=0.0
         BKGVAL(I)=0.0
         BKGERR(I)=0.0
      ENDDO
      CI=0.0
      CHIFIT=0.0
      AFWHM=0.0 
      EFWHM=0.0 
  
      AVPOS=0.0 
      AVFWHM=0.0

      SIFWHM=0.0   
      SIPOS=0.0    
      SUM=0.0     
      XFIT=0.0     
      YMAX=0.0     
      YMIN=0.0   

C     Doubles

      DO I=1,ND2COL
         D2VAL(I)=0.0D0 
      ENDDO
C     Chars
      LINE(1:) = ' '    
  
c ================================================================
c >>>>>>>> OPTIMAL EXTRACTION DEBUG TABLES COLUMN ID SEARCH  <<<<<
c ================================================================      
c      CALL QCTSR(D1NAME,ND1COL,D2NAME,ND2COL,D1COL,D2COL,IDQC0)
c ==============================================================

      ICURP=STOCH    
      ICURP2=STOCH2
c
c     Chop the order (FBVAL) into equaly spaced windows (IXWIND) in
c     wavelength space.
c     Do an averaging in the chunks to get averaged order profiles
c     Fit a gaussian to each of these averaged profiles.
c     Save the parameters of the gaussian in the first four rows
c     of FBVAL.
c     These parameters will finally be fitted in wavelength space,
c     this will be done by G_PROF().
c
      IMASK = 1                 !mask: 1=valid pixel, 0:invalid

      LOW = 1.1                 !these are empirical data !!!
      HGH = SNGL(BINSIZE*WIDTH/3.0D0) !.. and should be replaced by measurements

      NUM = (IXMAX - IXMIN) / IXWIND    !dimension in pix of each chunk
      STOCH=STOCH+NUM
      STOCH2=STOCH2+NUM*DIM

C      if NUM is 0 means that it has identified one order containing only
C      one point. So we have to fill basic arrays with something and skip
C      the order
      IF (NUM .EQ. 0) THEN 
               
               
         DO I=1,NPIXA1
            DO J=1,DIM
               VBVAL(I,J)=1.0
               FBVAL(I,J)=1.0
            ENDDO
            DO J=1,DIMP
               PROF(I,J)=1.0
            ENDDO
            DO J=1,PDIM
               PBUF(J,I)=1.0
               VBUF(J,I)=1.0
            ENDDO   
         ENDDO  
         FWHM(N_CURR_ORD)=0
         MID_S_N_CENT(N_CURR_ORD)=0
         OBJ_POS_CENT(N_CURR_ORD)=0
         GOTO 100
      ENDIF     
             
      I1 = IXMIN
      I2 = I1 + IXWIND - 1
      CI = I1-REAL(IXWIND/2)

      kk=1
      
      IMASK_COUNTER = 0
      

      DO K=1,NUM
         YMIN = 1E33
         YMAX = -YMIN
         J1 = 1

         DO J = 1,DIM           !Do the averaging within one window
            XPOS(J) = -1.0
            COLUM(J1) = 0.0
            SIGMA(J1) = 0.0
            IPOI  = 0
C
C           collapses the chunk            
C
            DO I=I1,I2

               COLUM(J1) = COLUM(J1) + FBVAL(I,J)*MASK(I,J)
               SIGMA(J1) = SIGMA(J1) + VBVAL(I,J)*MASK(I,J)
               IF(MASK(I,J).GT.0) IPOI = IPOI+1
            ENDDO

C
C ??? is this check ok!? we have to prevent that sigma is getting .le.0.0 !!
C
            IF (SIGMA(J1) .GT. 1.E-9) THEN  
               COLUM(J1) = COLUM(J1) / IPOI
               SIGMA(J1) = SQRT( SIGMA(J1) / IPOI / IPOI )
c               SIGMA(J1) = 1.0

c               SIGMA(J) = (RON + MAX(0.0,COLUM(J))/CONAD) / IXWIND 
               YMIN = AMIN1(COLUM(J1),YMIN) !get the minimum
               IF (COLUM(J1) .GT. YMAX) THEN !get maximum
                  JMAX = J1
                  YMAX = COLUM(JMAX)
               ENDIF
               XPOS(J1) = FLOAT(J)  !Here set the x pos to its real value
                                    !taking in count of the masking
           
C ==================================================================         
               IF (DBG_SW .EQ. "Y") THEN
                  D2VAL(1) = N_CURR_ORD
                  D2ROW=ICURP2+(K-1)*DIM+J1
                  D2VAL(2)=K 
                  D2VAL(3)=DBLE(XPOS(J1))
                  D2VAL(4)=DBLE(COLUM(J1))
                  CALL TBRWRD(DTID2,D2ROW,4,D2COL,D2VAL,ISTAT)
               ENDIF
C =================================================================
               J1 = J1+1
            ENDIF
         ENDDO

c        so finds Ymax and Ymin


C     Determine the initial parameters for gaussfit
         JWIN = 2 * BINSIZE
         PARAM(4) = (YMAX - YMIN) * 0.01    !base gaussian: constant of linear 
         PARAM(1) = YMAX - PARAM(4)         !gaussian amplitude
         PARAM(2) = DIM/2.+OBJSET           !gaussian center
C
C         IF (JMAX.LT.JWIN .OR. DIM-JMAX.LT.JWIN) GOTO 99
C
         PARAM(3) = 2.                      !gaussian sigma
         PARAM(5) = 0.                      !slope of linear term
         LP(5) = 0
         I = 5


         CALL GAUSS_FITX(XPOS,COLUM,SIGMA,J1-1,PARAM,I,LP,MFIT,VPARAM
     $        ,CHI2(1),ISTAT)

C  Check whether the output parameters are reasonable
C if not put IMASK = 0 

         IF (ISTAT.LT.0 .OR.
     $        PARAM(3).LT.LOW .OR. PARAM(3).GT.HGH .OR.
     $        PARAM(2).LT.1. .OR. PARAM(2).GT. FLOAT(DIM)) THEN 
            IMASK = 0
         ENDIF

         I = IXMIN+K-1


c     Save parameters for later fit of the parameters -> G_PROF()!

         PBUF(1,I) = 1.0
        
         DO J=2,5
            PBUF(J,I) = PARAM(J)
            VBUF(J,I) = VPARAM(J)
            PMSK(J,I) = IMASK   !mask out if parameters are BAD
                                !(PMSK may be a 1dim array, future changes)
         ENDDO
         
         IF(IMASK .GT. 0) THEN
            IMASK_COUNTER=IMASK_COUNTER+1
         ENDIF
         


         
         IMASK = 1
         I1 = I2 + 1
         I2 = I1 + IXWIND - 1          !in this way we go to the next chunk
         CI = CI + IXWIND



         IF (PBUF(2,I) .LT. 0 .OR. PBUF(2,I) .GT. DIM .OR.
     +       VBUF(2,I) .LT. 0 .OR. VBUF(2,I) .GT. DIM .OR.
     +       PMSK(2,I) .EQ. 0 ) THEN 
            POSVAL(kk)=DIM/2.+OBJSET
            SIGVAL(kk)=2.0
            POSERR(kk) = 1.0e6
            SIGERR(kk) = 1.0e6
            BKGVAL(kk)=PBUF(4,I)
            BKGERR(kk)=VBUF(4,I)
            POSMSK(kk)=PMSK(2,I)
            SIGMSK(kk)=PMSK(2,I)
            POSWAV(kk)=CI
            kk=kk+1 

         ELSE 

            POSVAL(kk)=PBUF(2,I) 
            POSERR(kk)=VBUF(2,I)
            SIGVAL(kk)=PBUF(3,I)
            SIGERR(kk)=VBUF(3,I)
            BKGVAL(kk)=PBUF(4,I)
            BKGERR(kk)=VBUF(4,I)
            POSMSK(kk)=PMSK(2,I)
            SIGMSK(kk)=PMSK(2,I)
            POSWAV(kk)=CI
            kk=kk+1 

         ENDIF

      ENDDO


C     ====================================================================  
      kkmax=kk-1

      IF ( IMASK_COUNTER .LT. 10 ) THEN
            WRITE(LINE,*) 'Warning: IMASK_COUNTER LESS THAN 10'
            CALL STTPUT(LINE,STATUS)      
      ENDIF
      
      TCOEF(1)=DIM/2.+OBJSET
      TCOEF(2)=.01
      TCOEF(3)=.0
      TCOEF(4)=.0
      TCOEF(5)=.0
        
      IT = ORDFIT
c       KP(2) = 0
c       KP(3) = 0
        KP(4) = 0
        KP(5) = 0
 
C     
C     calculates mean and std dev of errors on position   
C
      NCLEAN=0
70    NCLEAN=NCLEAN+1   

      CALL MYAVRG(PBUF,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,2,AVPOS)
      CALL MYAVRG(PBUF,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,3,AVFWHM)
      
      CALL MYSIGM(PBUF,AVPOS,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,2,SIPOS)
      CALL MYSIGM(PBUF,AVFWHM,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,3,SIFWHM)

      CALL MYAVRG(VBUF,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,2,AVERR)
      CALL MYAVRG(VBUF,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,3,AFWHM)
      
      CALL MYSIGM(VBUF,AVERR,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,2,EPOS)
      CALL MYSIGM(VBUF,AFWHM,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,3,EFWHM)

c     mask out the point which have too big error  
c     these points are generated from GAUSS_FITX
c     the idea is that if a point has associated a big error
c     has not to be trusted    

      DO I = IXMIN,IXMIN+NUM-1
         IF ( 
     +        (VBUF(2,I)-AVERR).GT.(2*EPOS).OR.
     +        (VBUF(3,I)-AFWHM).GT.(2*EFWHM).OR.
     +    ABS((PBUF(2,I)-AVPOS)).GT.(2*SIPOS).OR.
     +    ABS((PBUF(3,I)-AVFWHM)).GT.(2*SIFWHM).OR.
     +        PBUF(3,I).LE.0.) THEN
            PMSK(2,I)=0
            POSMSK(I)=0
            SIGMSK(I)=0  
            POSERR(I)=1e6
            SIGERR(I)=1e6   
         ENDIF      
      ENDDO
      IF (NCLEAN.LE.1) GOTO 70  !Repeat if necessary 
                                !to have a better estimate of AVERR and EPOS
      
c =========================================================
      IXWMAX = IXMIN + (IXMAX - IXMIN) / IXWIND - 1
c =========================================================
c     Monitoring the values after LSFIT
      I1 = IXMIN+IXWIND/2            !central point in each chunk
      XSTP = IXWIND
      X1 = I1
      
      PCOF1(1)=AVPOS
      PCOF1(2)=.01
      PCOF1(3)=.0
      
      PCOF2(1)=AVFWHM
      PCOF2(2)=.01
      PCOF2(3)=.0

      IAPOS(1)=1
      IAPOS(2)=1
      IAPOS(3)=1

      IAFWHM(1)=1
      IAFWHM(2)=1
      IAFWHM(3)=1

      CALL POLFIT(POSWAV,POSVAL,POSERR,POSMSK,kkmax,PCOF1
     $               ,IAPOS,POLORD,COVAR,POLORD,CHISQ,POLYN,IFAIL)

      CALL POLFIT(POSWAV,SIGVAL,SIGERR,SIGMSK,kkmax,PCOF2
     $               ,IAFWHM,POLORD,COVAR,POLORD,CHISQ,POLYN,IFAIL)
  

C     Dump QC Info on order-trace pos, fwhm, bkg values and errors for each chunk     
      
      CALL QCOTR0(I1,XSTP,IXMIN,NUM,PBUF,PDIM,NPIXA1,DIM,ICURP,
     .                  N_CURR_ORD,VBUF,BKGVAL,BKGERR,MAX_DIM,PMSK,
     +                  PCOF1,PCOF2,POLORD,IDQC0) 
      
      
      CALL MYAVRG(PBUF,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,2,AVPOS)
      CALL MYAVRG(PBUF,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,3,AVFWHM)     

c     I chech also the chi-square deviation of fit from points
      NFIT=0
      CHIFIT=0.0
      XFIT=0.0
      X1 = I1 - XSTP
      DO I = IXMIN,IXMIN+NUM-1
         X1 = X1 + XSTP
         IF (PMSK(2,I).GT.0) THEN
            CHIFIT = CHIFIT + (PBUF(2,I)-POLYN3(X1,PCOF1))**2     !mean squares
            XFIT   = XFIT   + (PBUF(3,I)-POLYN3(X1,PCOF2))**2     !mean squares
            NFIT=NFIT+1
         ENDIF
      ENDDO
      CHIFIT=CHIFIT/(NFIT-3.0)
      XFIT  =XFIT/(NFIT-3.0)
c     check if first fit on "good" points give too strange coefficients
c     the curvature and the inclination cannot be too strange
c     If yes, it change the polinomial fit to a linear one
      
c     max allowed error:  0.5 pix incl err over 1024 pix in X dir for obj pos
c                         0.5 pix curvature over 1024 pix in X dir for obj pos
c     max allowed error:  3 pix incl err over 1024 pix in X dir for obj pos
c                         3 pix curvature over 1024 pix in X dir for obj pos
c     If chi-square < 1
c     max allowed error:  0.5 pix incl err over 1024 pix in X dir for obj fwhm
c                         0.5 pix curvature over 1024 pix in X dir for obj fwhm
c     max allowed error:  2 pix incl err over 1024 pix in X dir for obj fwhm
c                         2 pix curvature over 1024 pix in X dir for obj fwhm
c     remember that BINX and BINY are swapped on the detector
c     if very good fit no k-sigma clipping
      PDEG=2
      SDEG=2
      IF (CHIFIT.LT.2.0e-2.AND.XFIT.LT.2.0e-2) THEN
       D1ROW=ICURP+1
          CALL TBRWRR(DTID1,D1ROW,1,IDDEG,
     +                FLOAT(PDEG),ISTAT)
          CALL TBRWRR(DTID1,D1ROW,1,IDCHI0,
     +                CHIFIT,ISTAT)
          CALL TBRWRR(DTID1,D1ROW,1,IDKS,
     +            FLOAT(0),ISTAT)
          GOTO 96
      ENDIF

      IF (IFAIL.EQ.1) THEN
              IAPOS(3)=0
              PDEG=1
              PCOF1(1)=AVPOS
              PCOF1(2)=0.0
              PCOF1(3)=0.0

              IAFWHM(3)=0
              SDEG=1
              PCOF2(1)=AVFWHM
              PCOF2(2)=0.0
              PCOF2(3)=0.0
              GOTO 96
      ENDIF 
      IF (CHIFIT.GT.0.5) THEN
          IF ( ABS(PCOF1(2)).GT.(0.5/1024*BINY/BINX).OR.    
     +         ABS(PCOF1(3)).GT.(0.5/1024/1024*BINY*BINY/BINX/BINX) 
     +        ) THEN
              IAPOS(3)=0
              PDEG=1
              PCOF1(1)=AVPOS
              PCOF1(2)=0.0
              PCOF1(3)=0.0
          ENDIF
      ELSE
          IF ( ABS(PCOF1(2)).GT.(3.0/1024*BINY/BINX).OR.    
     +    ABS(PCOF1(3)).GT.(3.0/1024/1024*BINY*BINY/BINX/BINX) 
     +        ) THEN
              IAPOS(3)=0
              PDEG=1
              PCOF1(1)=AVPOS
              PCOF1(2)=0.0
              PCOF1(3)=0.0
          ENDIF
      ENDIF
      
      IF (XFIT.GT.1) THEN
          IF ( ABS(PCOF2(2)).GT.(0.5/1024*BINY/BINX).OR.    
     +         ABS(PCOF2(3)).GT.(0.5/1024/1024*BINY*BINY/BINX/BINX) 
     +        ) THEN
              IAFWHM(3)=0
              SDEG=1
              PCOF2(1)=AVFWHM
              PCOF2(2)=0.0
              PCOF2(3)=0.0
          ENDIF
      ELSE
          IF ( ABS(PCOF2(2)).GT.(2.0/1024*BINY/BINX).OR.    
     +    ABS(PCOF2(3)).GT.(2.0/1024/1024*BINY*BINY/BINX/BINX) 
     +        ) THEN
              IAFWHM(3)=0
              SDEG=1
              PCOF2(1)=AVFWHM
              PCOF2(2)=0.0
              PCOF2(3)=0.0
          ENDIF
      ENDIF
  
    
c     before starting real k-sigma clipping calculates again the
c     right coefficients
      
      CALL POLFIT(POSWAV,POSVAL,POSERR,POSMSK,kkmax,PCOF1
     $               ,IAPOS,PDEG+1,COVAR,PDEG+1,CHISQ,POLYN,IFAIL)
      CALL POLFIT(POSWAV,SIGVAL,SIGERR,SIGMSK,kkmax,PCOF2
     $               ,IAFWHM,SDEG+1,COVAR,SDEG+1,CHISQ,POLYN,IFAIL)
     
     
      IF (PCOF1(3).EQ.0) THEN
          IF ( ABS(PCOF1(2)).GT.(0.5/1024*BINY/BINX)) THEN
              IAPOS(3)=0
              IAPOS(2)=0
              PDEG=0
              PCOF1(1)=AVPOS
              PCOF1(2)=0.0
              PCOF1(3)=0.0
          ENDIF
      ENDIF

      
      IF (PCOF2(3).EQ.0) THEN
          IF ( ABS(PCOF2(2)).GT.(0.25/1024*BINY/BINX)) THEN
              IAFWHM(3)=0
              IAFWHM(2)=0
              SDEG=0
              PCOF2(1)=AVFWHM
              PCOF2(2)=0.0
              PCOF2(3)=0.0
          ENDIF
      ENDIF


      D1ROW=ICURP+1
      CALL TBRWRR(DTID1,D1ROW,1,IDDEG,
     +            FLOAT(PDEG),ISTAT)
      CALL TBRWRR(DTID1,D1ROW,1,IDCHI0,
     +            CHIFIT,ISTAT)
      CALL TBRWRR(DTID1,D1ROW,1,IDKS,
     +            FLOAT(1),ISTAT)
        
   
c  ==============================================================

c     Do a linear least squares fit on the position and the sigma
c     parameter of the gaussians

 95   I1 = IXMIN+IXWIND/2            !central point in each chunk
      XSTP = IXWIND
      X1 = I1
       
      IF(PDEG.GT.0) THEN
      CALL POLFIT(POSWAV,POSVAL,POSERR,POSMSK,kkmax,PCOF1
     $               ,IAPOS,PDEG+1,COVAR,PDEG+1,CHISQ,POLYN,IFAIL)
      ENDIF
      IF(SDEG.GT.0) THEN
      CALL POLFIT(POSWAV,SIGVAL,SIGERR,SIGMSK,kkmax,PCOF2
     $               ,IAFWHM,SDEG+1,COVAR,SDEG+1,CHISQ,POLYN,IFAIL)
      ENDIF
 
         
c     PCOF(1) is the y constant coeff position of the spectrum within the order
c     PCOF(2) is the y slope    coeff position of the spectrum within the order

C     It is safer to assume that the sigma is constant along the order      
c      WRITE(*,*) 'FITPOS',PCOF1
c =========================================================================
c     Now do a kappa sigma clipping on the profiles to get
c     best positions and sigma of the gaussians
c =========================================================================
      KAPPAMAX = 2*KAPPA**2
      IMAX = 0
      IPOI = 0
      X1 = I1 - XSTP
      DO I = IXMIN,IXMIN+NUM-1
         X1 = X1 + XSTP
         IF (PMSK(2,I).GT.0) THEN
            IPOI = IPOI+1
            CHI2(2) = (PBUF(2,I)-POLYN3(X1,PCOF1))**2/VBUF(2,I)
            CHI2(3) = (PBUF(3,I)-POLYN3(X1,PCOF2))**2/VBUF(3,I)
            SUM = CHI2(2) + CHI2(3)
            IF (SUM.GT.KAPPAMAX) THEN
               KAPPAMAX = SUM         !determines maximum discrepancy 
               IMAX = I               !of position and FWHM and its index
               X1MAX = X1
            ENDIF
         ENDIF
      ENDDO
      
      
      IF (IMAX.GT.0) THEN
         DO J=1,4
            PMSK(J,IMAX) = 0
         ENDDO
         POSMSK(IMAX) = 0
         POSERR(IMAX) = 1e6
         SIGMSK(IMAX) = 0
         SIGERR(IMAX) = 1e6

C        if less than some (15) chunks the k-sigma clipping is repeated 
C        else goes on
C         
         IF (IPOI.LE.MINPOI) GOTO 96
         GOTO 95                !re-do the least squares fit
      ENDIF
c     Now determine the position and sigma of the gaussian at
c     each column ( = each wavelength).
 

 96      DO I =  IXMIN,IXMAX
         PBUF(2,I) = POLYN3(FLOAT(I),PCOF1)    !position of maximum

c         PBUF(2,I) = POLYN5(FLOAT(I),TCOEF)    !position of maximum
         PBUF(3,I) = POLYN3(FLOAT(I),PCOF2)   !sigma of gaussian
      ENDDO

c  ============================================================
c   Monitoring the values after LSFIT
      CALL  QCOTR1(I1,XSTP,IXMIN,NUM,ICURP,X1,PCOF1,PMSK,PCOF2,
     +                  PDIM,NPIXA1,POLORD)
    
      
c     Now determine background and amplitude for fixed 
c     sigma and position of maximum (result from least squares fit).
c     Determine the gaussian profils at each wavelength. At the beginning
c     the gaussians have a zero background and an amplitude of 1.0.

      CALL G_PROF(PROF,VBVAL,FBVAL,MASK,PBUF,VBUF,COLUM,SIGMA,OUT
     $     ,VOUT,NB1,NB2,NPIXA1,IXMIN,IXMAX,DIM,DIMP,PDIM
     +     ,CONAD,THRESH,INDX,OBJSET,SKYBKG,VARSKY)  
         
100   CONTINUE     

      RETURN
      END
      
C----------------------------------------------------------------------------------




