C $Id $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)cent.for	1.0 (ESO-DFS) 23/10/01 12:26:19
C+ derived from necopt.for
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  subroutine CENT version 1.0 011023 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE 
C See Role below
C
C.SYNTAX
C
C.HISTORY:
C  2001.10.23-AM
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.4 $
C $Log: cent.for,v $
C Revision 1.4  2002/06/03 09:36:15  amodigli
C added 2 comments
C
C Revision 1.3  2002/02/03 08:04:24  amodigli
C Eliminated misleading info
C
C Revision 1.2  2001/11/01 09:30:17  amodigli
C Eliminated several warnings from ftnchek
C
C Revision 1.1  2001/11/01 07:18:35  amodigli
C First version of necopt with splitted data
C
C
C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************
C*******************************************************************      
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     CENT
C------------------------------------------------------------------------------
C  Role: get the centres of the orders at each wavelength
C------------------------------------------------------------------------------
      SUBROUTINE CENT(YVAL,NPIXA1,DPAR,ND,K,L,YV,IWIDTH,
     +                SCAN,OFFSET,IXMIN,IXMAX,STARTA,STEPA)
C
C     Get the centres of the orders at each wavelength
C
      IMPLICIT NONE
      INTEGER NWAV                     !No of wavelength points
      INTEGER NAXIMA                   !No of axes of image
      PARAMETER (NWAV=50,NAXIMA=2)           

      INTEGER K                       !max value of loop variable
      INTEGER L                       !max value of loop variable
      INTEGER ND                      !number of polynomial coefficients
      INTEGER NPIXA1                  !X ima size
      INTEGER IWIDTH                  !size in pix of slit
      INTEGER IXMIN                   !pix position of X min
      INTEGER IXMAX                   !pix position of X max
      INTEGER SCAN(NAXIMA)            !Scan ima parameter (X, Y)

      REAL OFFSET                     !Offset of object

      DOUBLE PRECISION YVAL(NPIXA1)   !Y position in world
      DOUBLE PRECISION DPAR(ND)       !order polynomial coeff
      DOUBLE PRECISION YV             !actual order number
      DOUBLE PRECISION STARTA(NAXIMA) !Start value of ima (X,Y)
      DOUBLE PRECISION STEPA(NAXIMA)  !Step of ima (X,Y)



C
C     Local variables
C

      INTEGER IX1                   !loop variable
      INTEGER L1                    !loop variable
      INTEGER K1                    !loop variable
      INTEGER IP                    !tmp variable
      INTEGER ITEST                 !switch

      INTEGER I                     !loop variable

      DOUBLE PRECISION AX           !X position in pix
      DOUBLE PRECISION XVAL         !X position in world
      DOUBLE PRECISION DC           !tmp variable
      DOUBLE PRECISION WW(NWAV)       !wav position


      DOUBLE PRECISION XI_R8R8      !MIDAS function to convert from 
                                    !world to pix coordinate
      DOUBLE PRECISION IX_R8R8      !MIDAS function to convert from 
                                    !pix to world coordinate
C
C     Local variables initialization
C

C     Integers

      IX1=0
      L1=0
      K1=0   
      IP=0   
      ITEST=0
      I=0

C     Reals

C     Double precision
      AX=0.0D0
      XVAL=0.0D0  
      DC=0.0D0    
      DO I=1,NWAV
         WW(I)=0.0D0  
      ENDDO






          AX = 1
          IXMIN = 1
          IXMAX = NPIXA1
          ITEST = 0
          DO 50 IX1 = 1,NPIXA1
              IP     = 0
              DC     = 1.D0
              YVAL(IX1)  = 0.D0
              XVAL   = IX_R8R8(AX, STARTA(1), STEPA(1))
              DO 30 L1 = 0,L
                  IP     = IP + 1
                  WW(IP) = DC
                  YVAL(IX1)  = YVAL(IX1)+ WW(IP)*DPAR(IP)
                  DO 20 K1 = 1,K
                      IP     = IP + 1
                      WW(IP) = WW(IP-1)*XVAL
                      YVAL(IX1)  = YVAL(IX1) + WW(IP)*DPAR(IP)
   20             CONTINUE
                  DC     = DC*YV
   30         CONTINUE
              YVAL(IX1) = XI_R8R8(YVAL(IX1),STARTA(2),STEPA(2)) + 
     !                            DBLE(OFFSET)
              AX = AX+1
  
          IF ( INT(YVAL(IX1)-DBLE(IWIDTH/2.0)-1).LE.SCAN(1)) THEN
           !here take the last point of the sample such that the order is cut (bottom image)
           !no test variable
              IXMIN = IX1
          ELSE IF ( INT(YVAL(IX1)+DBLE(IWIDTH/2.0)+2).GE.SCAN(2).AND.
     +         ITEST.EQ.0) THEN
           !here take the 1st point of the sample such that the order is cut (bottom image)
           !test variable is necessary
              IXMAX = IX1
              ITEST = 1
          ENDIF

   50     CONTINUE
 
      RETURN
      END
C ===============================================================================    
