C $Id: extff.for,v 1.4 2002/02/11 09:32:23 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)extff.for	1.0 (ESO-DFS) 23/10/01 12:26:19
C+
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  subroutine EXTFF version 1.0 011023 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C See Role below
C
C.SYNTAX
C
C.HISTORY:
C  2001.10.23-AM Created
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.4 $
C $Log: extff.for,v $
C Revision 1.4  2002/02/11 09:32:23  amodigli
C Fixed arroneous checkin
C
C Revision 1.3  2002/02/03 08:05:05  amodigli
C Added EXTERNALs to reduce ftnchk warnings
C
C Revision 1.2  2001/11/01 09:30:18  amodigli
C Eliminated several warnings from ftnchek
C
C Revision 1.1  2001/11/01 07:18:36  amodigli
C First version of necopt with splitted data
C
C
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     23/10/01       created                           all
C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where


C  AM     07/06/01       Corrected some typo and some not  EXTFF
C                        proper initializations to fix 
C                        portability problems to PC Linux
C  AM     29/05/01       Corrected typo in IKIND info      EXTFF
C  AM     08/05/01       clean-comment                     EXTFF
C------------------------------------------------------------------------------
C  Role: Extract the FF
C        creates and maps vdummx
C        call CENT to get the centres of the orders at each wavelength
C        call CALC_I/CALC_P to to a weigted mean of the FF signal
C------------------------------------------------------------------------------
C Pseudocode:
C        some initial setting for the extraction constants
C        crea/maps frame vdummx
C        call CENT to get order centers
C        initializes out ima to 0
C        call CALC_I or CALC_P if IKIND is 0 or 1
C        close every open frame
C        RETURN
C-----------------------------------------------------------------------------
      SUBROUTINE EXTFF(IN,VARI,NPIXA1,NPIXA2
     $     ,OUT,VOUT,NB1,NB2,WGT,NC1,NC2,INDX,NPAR,PAR
     $     ,ND,DPAR,SCAN,OFFSET,STARTA,STEPA,IKIND,IWIDTH,DIMP)
C
C  Do an immediate extract using the weights of a previous optimal extraction
C
C   ARGUMENTS
C IN     REAL*4  INPUT IMAGE

C VARI   REAL*4  VARIANCE IMAGE
C NPIXA1 INTG*4  NO. OF SAMPLES
C NPIXA2 INTG*4  NO. OF LINES
C OUT    REAL*4  EXTRACTED ORDERS
C VOUT   REAL*4  VARIANCE OF EXTRACTED ORDERS
C NB1    INTG*4  NO. OF SAMPLES
C NB2    INTG*4  NO. OF ORDERS
C WGT    REAL*4  WEIGHT MAP
C NC1    INTG*4  NO. OF SAMPLES
C NC2    INTG*4  NO. OF ORDERS * THICKNESS OF ORDERS (profil dimension)
C INDX   INTG*4  SEQUENTIAL NO. OF THE ORDER
C NPAR   INTG*4  NO. OF EXTRACTION PARAMETERS
C PAR    REAL*4  EXTRACTION PARAMETERS
C ND     INTG*4  NO. OF COEFFS.
C DPAR   REAL*8  COEFFS. TO DEFINE ORDER POSITION
C IKIND  INTG*4  EVEN/ODD FLAG: 1: EVEN, 0: ODD SLIT WIDTH SIZE
C IWIDTH INTG*4  SLIT WIDTH  (size perpendicular to dispersion in pixel)
C DIMP   INTG*4  DIMENSION OF THE PROFILE OF THE ORDERS
C
      IMPLICIT NONE
C
C     Variable from interface (defined and initialized elsewere)
C
      INTEGER  NAXIMA
      PARAMETER (NAXIMA=2)              !Number of ima axes
      INTEGER  NPIXA1                   !Number of X pix of ima
      INTEGER  NPIXA2                   !Number of Y pix of ima
      INTEGER  NB1                      !Number of X pix of ima
      INTEGER  NB2                      !Number of Y pix of ima
      INTEGER  NC1                      !Number of X pix of ima
      INTEGER  NC2                      !Number of Y pix of ima
      INTEGER  NPAR                     !Number of parameters
      INTEGER  ND                       !Number of polynomial coefficients
      INTEGER  IKIND                    !switch for even/odd pix No. slit width
      INTEGER  IWIDTH                   !Dimension of order
      INTEGER  DIMP                     !Dim (No of points) of x-order profile
      INTEGER  INDX
      INTEGER  SCAN(NAXIMA)
      
      REAL WGT(NC1,NC2)                 !Weight ima
      REAL OUT(NB1,NB2)                 !Output ima
      REAL VOUT(NB1,NB2)                !output variance ima
      
      REAL IN(NPIXA1,NPIXA2)            !Input ima
      REAL VARI(NPIXA1,NPIXA2)          !input variance ima
      REAL OFFSET                       !offset
      REAL PAR(NPAR)                    !array holding extraction par
      
      DOUBLE PRECISION DPAR(ND)         !Array with polynomial coefficients
      DOUBLE PRECISION STARTA(NAXIMA)   !Ima starting point (X,Y)
      DOUBLE PRECISION STEPA(NAXIMA)    !Ima steps (X,Y)

C
C     Local variables
C

      INTEGER          K
      INTEGER          L
      INTEGER          IX
      INTEGER          NBY1             !Number of X points of ima
      INTEGER*8        PT1              !Pointer id (MIDAS env)
      INTEGER          ISTAT            !Status of operation (MIDAS env)
      INTEGER          MADRID           !Memory Address Id (MIDAS env)
      INTEGER          IAC              !Actual No of mapped points (MIDAS env)
      INTEGER          FID1             !File Id (MIDAS env)
      INTEGER          IXMIN            !Pixel id corresponting to XMIN
      INTEGER          IXMAX            !Pixel Id corresponding to XMAX
      INTEGER          IOFF             !offset of pixel point      


      DOUBLE PRECISION YV               !Current order
      EXTERNAL CENT                     !get the centres of the orders at each wavelength
      EXTERNAL CALC_I                   !do a weighted mean of the FF signal using 
                                        !the weight image coming from the obj extrac
      EXTERNAL CALC_P                   !do a weighted mean of the FF signal using 
                                        !the weight image coming from the obj extrac
 
      INCLUDE 'MID_INCLUDE:ST_DEF.INC'  !to get the definitions
                                        !on the ST interfaces
                                        
      COMMON /VMR/MADRID(1)             !COMMON block for the 
                                        !pointer emulation in F77
                                        
      INCLUDE 'MID_INCLUDE:ST_DAT.INC'  !to get data values for 
                                        !definitions above
      
C
C     Initialization of variables
C               

C     Integers
      K  =0
      L  =0
      IX =0
      NBY1=0  
      PT1 =0  
      ISTAT=0 
C      MADRID           !Memory Address Id (MIDAS env)
      IAC=0   
      FID1=0 
      IXMIN=0
      IXMAX=0
      IOFF=0
C     Reals

C     Doubles

      YV=0.0D0       
C     Characters


C
C ... get ordinate values
C
      K      = INT(PAR(9))           !Dimension of the coefficient
      L      = INT(PAR(10))          !matrix to get order pos.
      YV     = INT(PAR(11))          !Current order
C
C ... get offset within weight buffer for corresponding order
C
      IOFF   = (INDX - 1) * DIMP + 1
C
C ... loop over the samples
C
      NBY1 = NPIXA1 
      
      
C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.
C     Module STFMAP (f77): Map a new data frame into memory 
C                          (just get virtual memory (iomode=F_X_MODE))            
      CALL STFCRE('vdummx',D_R8_FORMAT,F_X_MODE,1,NBY1,FID1,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTFF: Error creating vdummx frm')
      ENDIF

      CALL STFMAP(FID1,F_X_MODE,1,NBY1,IAC,PT1,ISTAT) 
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTFF: Error mapping vdummx frm')
      ENDIF
c
c get the centers of the orders: stored in yval[] == madrid(pt1)
c
      CALL CENT(MADRID(PT1),NPIXA1,DPAR,ND,K,L,YV,IWIDTH,SCAN,OFFSET,
     1          IXMIN,IXMAX,STARTA,STEPA)

          
      DO IX=1,NPIXA1
         OUT(IX,INDX) = 0
      ENDDO


      IF (IKIND.EQ.0) THEN !switch for even/odd pix No. slit width
                
         CALL CALC_I(IN,VARI,WGT(1,IOFF),MADRID(PT1),OUT,VOUT
     $        ,NPIXA1,NPIXA2,NB1,NB2,DIMP,IXMIN,IXMAX,IWIDTH,INDX)
     
     
      ELSE                 
                       
         CALL CALC_P(IN,VARI,WGT(1,IOFF),MADRID(PT1),OUT,VOUT
     $        ,NPIXA1,NPIXA2,NB1,NB2,DIMP,IXMIN,IXMAX,IWIDTH,INDX)


      ENDIF
             
C     Module STFCLO (f77): close a frame           
      CALL STFCLO(FID1,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'EXTFF: Error closing FID1')
     

      END

C ================================================================================
