C $Id: extobj.for,v 1.7 2002/02/11 09:24:50 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)extobj.for	1.0 (ESO-IPG) 23/10/01 12:26:19
C+
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  program  ECHOPT  version 1.0 011023 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C 
C See Role and Pseudocode below
C
C.SYNTAX
C
C.HISTORY:
C  2001.10.23-AM
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.7 $
C $Log: extobj.for,v $
C Revision 1.7  2002/02/11 09:24:50  amodigli
C Added var image to sky image
C
C Revision 1.6  2002/02/03 08:14:09  amodigli
C reduced warnings from ftncheck
C
C Revision 1.5  2002/02/02 16:59:08  amodigli
C Eliminated checks on return status as
C suggested from Klaus, as MIDAS has its own
C error handling system
C
C Revision 1.4  2001/12/21 13:41:33  amodigli
C Added sky image
C
C Revision 1.3  2001/11/02 12:26:20  amodigli
C Eliminated unnecessary interface parameters
C
C Revision 1.2  2001/11/01 09:30:18  amodigli
C Eliminated several warnings from ftnchek
C
C Revision 1.1  2001/11/01 07:18:37  amodigli
C First version of necopt with splitted data
C

C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************
          
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     23/10/01       created                           all
C------------------------------------------------------------------------------
C  Role: Extract the Object
C        creates and maps vdummx, vdummy, mask, vari, pmsk, pbuf, ebuf, column
C                         sigma, xpos
C        call CENT to get the centres of the orders at each wavelength
C        call VSAMPLE to do a virtual sampling of the obj and variance
C        call CAPROF to determine the object and FWHMs distributions along 
C                    dispersion direction
C-----------------------------------------------------------------------------
C Pseudocode:
C        some initial setting for the extraction constants
C        crea/maps frame vdummx
C        call CENT to get order centers
C        crea/maps vdummy,vari,mask,pbuf,ebuf,pmsk,colum,sigma,xpos
C        initializes out ima to 0
C        call VSAMPL for variance and input ima
C        call CAPROF to calculate x-order prof, weights, and extract the obj
C        close every open frame
C        RETURN
C-----------------------------------------------------------------------------
      SUBROUTINE EXTOBJ(IN,VARI,NPIXA1,NPIXA2,OUT,VOUT,NB1,NB2,WGT
     $     ,NC1,NC2,INDX,NPAR,PAR,ND,DPAR,SCAN,OFFSET,BINSIZE,STARTA
     +     ,STEPA,IKIND,IWIDTH,DIMP,OBJSET,SKYBKG,VARSKY)
C
C
C   ARGUMENTS
C IN     REAL*4  INPUT IMAGE
C VARI   REAL*4  VARIANCE IMAGE
C NPIXA1 INTG*4  NO. OF SAMPLES
C NPIXA2 INTG*4  NO. OF LINES
C OUT    REAL*4  EXTRACTED ORDERS
C VOUT   REAL*4  VARIANCE OF EXTRACTED ORDERS
C NB1    INTG*4  NO. OF SAMPLES
C NB2    INTG*4  NO. OF ORDERS
C WGT    REAL*4  WEIGHT MAP
C NC1    INTG*4  NO. OF SAMPLES
C NC2    INTG*4  NO. OF ORDERS * THICKNESS OF ORDERS (profil dimension)
C INDX   INTG*4  SEQUENTIAL NO. OF THE ORDER
C NPAR   INTG*4  NO. OF EXTRACTION PARAMETERS
C PAR    REAL*4  EXTRACTION PARAMETERS
C ND     INTG*4  NO. OF COEFFS.
C DPAR   REAL*8  COEFFS. TO DEFINE ORDER POSITION
C XMAX   REAL*4  MAXIMUMVALUE
C IKIND  INTG*4  EVEN/ODD FLAG: 1: EVEN, 0: ODD SLIT WIDTH SIZE
C IWIDTH INTG*4  SLIT WIDTH  (size perpendicular to dispersion in pixel)
C DIMP   INTG*4  DIMENSION OF THE PROFILE OF THE ORDERS
C
      IMPLICIT NONE
      INTEGER  NAXIMA
      PARAMETER (NAXIMA=2)         !Max No. of elements in STEP START
                                   !accordingly in many places
C
C Interface variable (defined and initialized elsewere)
C
      INTEGER  NPIXA1              !1st dimension (X) of input ima
      INTEGER  NPIXA2              !2nd dimension (Y) of input ima
      INTEGER  NB1                 !1st dimension (X) of output ima
      INTEGER  NB2                 !2nd dimension (Y) of output ima
      INTEGER  NC1                 !1st dimension (X) of weight ima
      INTEGER  NC2                 !2nd dimension (Y) of weight ima
      INTEGER  NPAR                !Number of parameters
      INTEGER  ND                  !Dimension of array with order polyn coeff
      INTEGER  IKIND               !switch for even/odd pix No. slit width
      INTEGER  IWIDTH              !dimention slit width in pix
      INTEGER  DIMP                !Dimension (No of points) of x-order profile
       
      INTEGER INDX                 !current chunk?
      INTEGER BINSIZE              !Size of bin (for virtual resampling)
      INTEGER SCAN(NAXIMA)         !Scan ima parameter (MIDAS)
      
      REAL WGT(NC1,NC2)            !Array for weight ima
      REAL OUT(NB1,NB2)            !Array for out ima
      REAL VOUT(NB1,NB2)           !Array for varance
      REAL IN(NPIXA1,NPIXA2)       !Array for in ima
      REAL VARI(NPIXA1,NPIXA2)     !Array for variance
      REAL PAR(NPAR)               !Array parameter with Thresh,CONAD,RON
      REAL OFFSET                  !Offset of object (initial value constant)
      REAL OBJSET                  !Offset of object (variable)
      REAL SKYBKG(NB1,NB2)         !Sky bkg image
      REAL VARSKY(NB1,NB2)         !var Sky bkg image
      
      DOUBLE PRECISION DPAR(ND)    !Parameters for polynomial coefficients
      DOUBLE PRECISION STARTA(NAXIMA)!Start point of image (X,Y)
      DOUBLE PRECISION STEPA(NAXIMA) !Step of image        (X,Y)
      


C
C     Local variables
C

     
           
      INTEGER          K           !K=PAR(9)   !Dimension of the coefficient
      INTEGER          L           !L=PAR(10) !matrix to get order pos.
      INTEGER          IX          !Loop variable
      INTEGER          NBY1        !Ysize of out ima
      INTEGER          ISTAT       !Status of operation (MIDAS env)
      INTEGER          MADRID      !Memory Adders Id (MIDAS env)
      INTEGER          DIM         !Size of profile
      
      INTEGER          NBY2        !Ysize of out ima
      INTEGER          IAC         !Actual number of mapped pix (MIDAS env)
      INTEGER          NBY4        !Size of the buffer (MIDAS env)
      
      INTEGER          FID1        !file Id (MIDAS env)
      INTEGER          FID2        !file Id (MIDAS env)
      INTEGER          FID2V       !file Id (MIDAS env)
      INTEGER          FID3        !file Id (MIDAS env)
      INTEGER          FID4        !file Id (MIDAS env)
      INTEGER          FID4E       !file Id (MIDAS env)
      INTEGER          FID4M       !file Id (MIDAS env)
      
      INTEGER          FID5        !file Id (MIDAS env)
      INTEGER          FID6        !file Id (MIDAS env)
      INTEGER          FID7        !file Id (MIDAS env)
      
      INTEGER          IR(NAXIMA) 
      INTEGER          IXMIN       !pix id correxpondent to XMIN
      INTEGER          IXMAX       !pix id correxpondent to XMAX
      
      INTEGER          IXWIND      !size of order chunk to get obj pos 
                                   !and FHHM ditrib
      INTEGER          IOFF        !offset pixel point
      INTEGER          I           !Loop variable
 
      INTEGER*8        PT1         !Pointer id (MIDAS env)
      INTEGER*8        PT2         !Pointer id (MIDAS env)
      INTEGER*8        PT2V        !Pointer id (MIDAS env)
      INTEGER*8        PT3         !Pointer id (MIDAS env)
      INTEGER*8        PT4         !Pointer id (MIDAS env)
      INTEGER*8        PT4E        !Pointer id (MIDAS env)
      INTEGER*8        PT4M        !Pointer id (MIDAS env)
      INTEGER*8        PT5         !Pointer id (MIDAS env)
      INTEGER*8        PT6         !Pointer id (MIDAS env)
      INTEGER*8        PT7         !Pointer id (MIDAS env)
      

      REAL RON                     !Read Out Noise
      REAL CONAD                   !Conversion Analog to Digital
      REAL THRESH                  !Treshold for cosmic ray rejection
      
      DOUBLE PRECISION WIDTH       !Slit width in pix   

      DOUBLE PRECISION YV          !YV=PAR(11)!Current order

      EXTERNAL CENT                !get the centres of the orders at each wavelength
      EXTERNAL CAPROF              !determines the order and FWHM distribution 
                                   !along dispersion coordinate
      EXTERNAL VSAMPL              !Does the sampling in the virtual space


      INCLUDE 'MID_INCLUDE:ST_DEF.INC'  !to get the definitions
                                        !on the ST interfaces
                                        
      COMMON /VMR/MADRID(1)             !COMMON block for the 
                                        !pointer emulation in F77
                                        
      INCLUDE 'MID_INCLUDE:ST_DAT.INC'  !to get data values for 
                                        !definitions above
                                        


C
C     Local variables initialization. 
C     NB DATA variables initialized with DATA statement
C

C     Integers


      K    =0
      L    =0
      IX   =0
      NBY1 =0
      ISTAT=0 
C      MADRID     !Memory Adders Id (MIDAS env)
      DIM  =0
      
      NBY2 =0
      IAC  =0
      NBY4 =0
      
      FID1 =0
      FID2 =0
      FID2V=0
      FID3 =0
      FID4 =0
      FID4E=0
      FID4M=0
      
      FID5 =0
      FID6 =0
      FID7 =0
      
      DO I=1,NAXIMA
         IR(I)=0 
      ENDDO
      IXMIN=0  
      IXMAX=0
      
      IXWIND=0 
      IOFF=0
C      Better to not initialize pointers to not have portability
C      problems on PC-Linux (this applies PT1-7)    

      

C     Reals
      RON=0.0   
      CONAD=0.0 
      THRESH=0.0       
     


C     Doubles
      WIDTH =0.0D0      

      YV    =0.0D0   
C     Characters
C      CUNIT(1:) = ' '   !CUNIT is initialized from a DATA statement

     
      
      

     
C
C ... define extraction constants
C
      WIDTH  = IWIDTH                          ! slit width in pix units
   
      
            
          
                
C
C ... get RON and CONAD
C
      THRESH = PAR(2)           !cosmic ray rejection threshold
      CONAD  = PAR(3)           !inverse GAIN: CONAD [e-/ADU]
      RON    = PAR(4)           !readout-noise [e-]
      RON    = (RON / CONAD)**2 !-> converted to ADU^2 (variance!)
      
      
      

C
C ... get ordinate values
C
      K      = INT(PAR(9))           !Dimension of the coefficient
      L      = INT(PAR(10))          !matrix to get order pos.
      YV     = INT(PAR(11))          !Current order
      
      
      
                
C
C ... get offset within weight buffer for corresponding order
C
      IOFF   = (INDX - 1) * DIMP + 1
             

C
C ... loop over the samples
C

      NBY1 = NPIXA1                !number of samples
      
C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.      
      CALL STFCRE('vdummx',D_R8_FORMAT,F_X_MODE,1,NBY1,FID1,ISTAT)

C     Module STFMAP (f77): Map a new data frame into memory 
C                         (just get virtual memory (iomode=F_X_MODE))
      CALL STFMAP(FID1,F_X_MODE,1,NBY1,IAC,PT1,ISTAT) 


C     Gets centers of the orders at each wavelength     
      CALL CENT(MADRID(PT1),NPIXA1,DPAR,ND,K,L,YV,IWIDTH,SCAN,OFFSET,
     1          IXMIN,IXMAX,STARTA,STEPA)
     

           

      DIM = DIMP                !same size as the profil, why not !?
                                !DIM = ((IWIDTH/2)*2+2)*BINSIZE+1

      IR(1) = NPIXA1
      IR(2) = DIM
      NBY2  = NPIXA1 * DIM
      

C     creates and maps new frames to store output results from this routine
C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.
C     Module STFMAP (f77): Map a new data frame into memory 
C                          (just get virtual memory (iomode=F_X_MODE))                
      CALL STFCRE('vdummy',D_R4_FORMAT,F_X_MODE,1,NBY2,FID2,ISTAT)
      CALL STFMAP(FID2,F_X_MODE,1,NBY2,IAC,PT2,ISTAT)
      CALL STFCRE('vari',D_R4_FORMAT,F_X_MODE,1,NBY2,FID2V,ISTAT)
      CALL STFMAP(FID2V,F_X_MODE,1,NBY2,IAC,PT2V,ISTAT)


C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.
C     Module STFMAP (f77): Map a new data frame into memory 
C                          (just get virtual memory (iomode=F_X_MODE))       
      CALL STFCRE('mask',D_I2_FORMAT,F_X_MODE,1,NBY2,FID3,ISTAT)
      CALL STFMAP(FID3,F_X_MODE,1,NBY2,IAC,PT3,ISTAT)


      IR(2) = 5                 !parameter, error and mask buffer
      NBY4 = IR(1) * IR(2)      !size of the buffer



C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.
C     Module STFMAP (f77): Map a new data frame into memory 
C                          (just get virtual memory (iomode=F_X_MODE))  
      CALL STFCRE('pbuf',D_R4_FORMAT,F_X_MODE,1,NBY4,FID4,ISTAT)
      CALL STFMAP(FID4,F_X_MODE,1,NBY4,IAC,PT4,ISTAT)
      CALL STFCRE('ebuf',D_R4_FORMAT,F_X_MODE,1,NBY4,FID4E,ISTAT)
      CALL STFMAP(FID4E,F_X_MODE,1,NBY4,IAC,PT4E,ISTAT)
      CALL STFCRE('pmsk',D_I2_FORMAT,F_X_MODE,1,NBY4,FID4M,ISTAT)
      CALL STFMAP(FID4M,F_X_MODE,1,NBY4,IAC,PT4M,ISTAT)

      IR(1) = DIM
      IR(2) = 1                 !parameter buffer
      NBY4 = IR(1) * IR(2)      !size of the buffer
      
 
C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.
C     Module STFMAP (f77): Map a new data frame into memory 
C                          (just get virtual memory (iomode=F_X_MODE))        
      CALL STFCRE('colum',D_R4_FORMAT,F_X_MODE,1,NBY4,FID5,ISTAT)
      CALL STFMAP(FID5,F_X_MODE,1,NBY4,IAC,PT5,ISTAT)
      CALL STFCRE('sigma',D_R4_FORMAT,F_X_MODE,1,NBY4,FID6,ISTAT)
      CALL STFMAP(FID6,F_X_MODE,1,NBY4,IAC,PT6,ISTAT)
      CALL STFCRE('xpos',D_R4_FORMAT,F_X_MODE,1,NBY4,FID7,ISTAT)
      CALL STFMAP(FID7,F_X_MODE,1,NBY4,IAC,PT7,ISTAT)

      DO IX=1,NPIXA1
         OUT(IX,INDX) = 0
      ENDDO
      
                 
      CALL VSAMPL(VARI,WGT(1,IOFF),MADRID(PT1),MADRID(PT2V),
     1     MADRID(PT3),DIMP,IWIDTH,BINSIZE,NPIXA1,IXMIN,
     2     IXMAX,NPIXA2,DIM,IKIND)
     


    
      CALL VSAMPL(IN,WGT(1,IOFF),MADRID(PT1),MADRID(PT2),
     1     MADRID(PT3),DIMP,IWIDTH,BINSIZE,NPIXA1,IXMIN,
     2     IXMAX,NPIXA2,DIM,IKIND)
     
     

          

      IXWIND = 32! 16               !window size of the order chunks

C     Get profile pos,FWHM distributions and fits x-order profile
C     determining output image and weights
      CALL CAPROF(WGT(1,IOFF),MADRID(PT2V),MADRID(PT2),MADRID(PT3)
     $     ,MADRID(PT4),MADRID(PT4E),MADRID(PT4M),MADRID(PT5)
     $     ,MADRID(PT6),MADRID(PT7),OUT,VOUT,NB1,NB2,NPIXA1,IXWIND,IXMIN
     $     ,IXMAX,DIM,DIMP,5,RON,CONAD,THRESH,BINSIZE,WIDTH,INDX,OBJSET
     +     ,SKYBKG,VARSKY)
     
     
     
C     Module STFCLO (f77): close a frame 

      CALL STFCLO(FID1,ISTAT)
      CALL STFCLO(FID2,ISTAT) 
      CALL STFCLO(FID2V,ISTAT) 
      CALL STFCLO(FID3,ISTAT)
      CALL STFCLO(FID4,ISTAT)
      CALL STFCLO(FID4E,ISTAT)
      CALL STFCLO(FID4M,ISTAT)
      CALL STFCLO(FID5,ISTAT)
      CALL STFCLO(FID6,ISTAT)
      CALL STFCLO(FID7,ISTAT)
      
      RETURN

      END
C =============================================================================
