C $Id: g_prof.for,v 1.2 2002/08/28 09:30:41 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)g_prof.for	1.0 (ESO-DMD) 28/08/02 12:26:19
C+
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  subroutine  G_PROF  version 1.0 011023 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C Optimally Extract echelle orders using Gaussian approximation for the cross-order light profile.
C
C CALLED BY: caprof (caprof.for)
C
C.SYNTAX
C
C.HISTORY:
C  2002.08.28-AM Created
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.2 $
C $Log: g_prof.for,v $
C Revision 1.2  2002/08/28 09:30:41  amodigli
C Scoproprated by necopt.for
C
C
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     28/08/02       Created                           all
C==================================================================================                    
C==================================================================================
C=====SUBROUTINE TO EXTRACT ORDERS=================================================
C==================================================================================
      
      SUBROUTINE G_PROF(PROF,VBVAL,FBVAL,MASK,PBUF,VBUF,COLUM,SIGMA
     $     ,OUT,VOUT,NB1,NB2,NPIXA1,IXMIN,IXMAX,DIM,DIMP
     +     ,PDIM,CONAD,THRESH,INDX,OBJSET,SKYBKG,VARSKY) 
     
      IMPLICIT NONE
      INTEGER N_MAX_ORD
      INTEGER NGPAR
      INTEGER NPOINT
      PARAMETER(N_MAX_ORD=50,NGPAR=5,NPOINT=101)

C
C     Interface variables
C
      INTEGER DIM                        !Y dimension of mask
      INTEGER DIMP                       !Y dimension (No. of points) of prof
      INTEGER PDIM                       !X dimension of mask
      INTEGER N_CURR_ORD                  !Current order No.
      INTEGER NPIXA1                     !X size of mask 
      INTEGER NB1                        !X size of out ima
      INTEGER NB2                        !Y size of out ima
      INTEGER INDX                       !current x-order pos
      INTEGER IXMIN                      !pix point correxponding to XMIN
      INTEGER IXMAX                      !pix point correxponding to XMAX
      INTEGER*2 MASK(NPIXA1,DIM)         !mask ima
     
      REAL VBVAL(NPIXA1,DIM)             !variance of ima
      REAL FBVAL(NPIXA1,DIM)             !raw ima
      REAL PROF(NPIXA1,DIMP)             !profile
      REAL PBUF(PDIM,NPIXA1)             !buffer for coefficients of profile
      REAL VBUF(PDIM,NPIXA1)             !buffer for coefficients of raw values
      REAL COLUM(DIM)                    !output weights
      REAL SIGMA(DIM)                    !errors associated to points?
      REAL CONAD                         !Analog To Digital conversion factor
      REAL THRESH                        !Threshold factor
      REAL OUT(NB1,NB2)                  !output ima
      REAL VOUT(NB1,NB2)                 !output variance ima
      REAL OBJSET                        !object offset
      REAL MID_S_N_CENT(N_MAX_ORD)       !tmp S/N value
      REAL OBJ_POS_CENT(N_MAX_ORD)       !tmp obj position
      REAL FWHM(N_MAX_ORD)               !tmp array to hold average FWHM
      REAL SKYBKG(NB1,NB2)               !Sky bkg image   
      REAL VARSKY(NB1,NB2)               !var Sky bkg image        
C
C     Local variables
C
      INTEGER I                          !loop variable
      INTEGER J                          !loop variable
      INTEGER NVAL                       !counter for un masked point of 
                                         !Gaussian profile
      INTEGER INDEX                      !tmp variable to hold temporary X pos
      INTEGER JMAX                       !Aux var
      INTEGER ISTAT                      !Aux variable for MIDAS env 
C     Local variables
      INTEGER CRCNT                      !cosmic ray event counter
      INTEGER CRMSKS                     !tmp variable to store a value
      INTEGER STATUS                     !return status from MIDAS call 
C     Integer variables set by common            
      INTEGER MTID                       !Mask Table ID
      INTEGER MIDX                       !Mask ID of X col
      INTEGER MIDY                       !Mask ID of Y col
      INTEGER MIDORD                     !Mask ID ORDER col
      INTEGER MIDFLX                     !Mask ID FLX col
      
      REAL PVAL                          !x-profile value
      REAL CHI2MIN                       !minimum of chi-square
      REAL CHI2DEL                       !delta of chi-square
      REAL A(NGPAR)                      !coefficient of profile
      REAL B(NGPAR)                      !weight holder in chi2 of prof. coeff.
      REAL X                             !X position
      REAL DY(NGPAR)                     !deriv of Gaus prof. respect to coeff.
      REAL THRESH2                       !Square of threshold
      REAL DMAX                          !Holder for max diff of Y pos     
      REAL CHI2G                         !Gaussian chi-quare value
      REAL DCHI1                         !function to get chi-square difference
      REAL X1                           !X1=(1-ALPHA*SQRT(1.0/DELTA))*PBUF(1,I)
      REAL ZBRENT                        !function to hold zero of polynomial
      REAL AMPERR                        !error amplitude
      REAL AVERSIG                       !Average of sigma
      REAL ERF                           !Error function
      REAL S_N_THRESH                    !thresh contribute proportional to S/N
      REAL S_N_WEIGHT                    !weight for linear term proportional 
                                         !to S/N in thresh
      REAL BASIC_THRESH2                 !basic threshold squared
      REAL S_N_WEIGHT2                   !weight for quadratic term 
                                         !proportional to S/N in thresh

      REAL ALPHA                         !Amplitude variation in percent
      REAL DELTA                         !CHI-CHIMIN
      CHARACTER*96 LINE                  !Buffer to display informations
      
      EXTERNAL FGAUSS_C                  !Function to get Y of gauusian+linear
                                         !prof. + deriv.
      EXTERNAL CHI2G                     !Function to get Gaussian chi-square
      EXTERNAL DCHI1                     !function to get chi-square difference
      EXTERNAL ZBRENT                    !Function to get zero points of polyn
      EXTERNAL ERF                       !ERF function
      EXTERNAL GETQCI                    !subroutine to get and display some QC info
      

      INCLUDE 'MID_INCLUDE:ST_DEF.INC'   !to get the definitions 
                                         !on the ST interfaces
      INCLUDE '../incl/common_qc_log.inc'
      INCLUDE '../incl/common_crmask.inc'
      INCLUDE '../incl/common_vmr.inc'      

      INCLUDE 'MID_INCLUDE:ST_DAT.INC'   !to get data values for 
                                         !definitions above
     
 
 
C
C     Initialization of local variables
C

C     Integers
      I=0    
      J=0
      k=0
      l=0
      NVAL=0 
     
      
      INDEX=0 
      JMAX=0
      ISTAT=0
      CRCNT = 0
C     Reals
      PVAL=0.0   
      CHI2MIN=0.0 
      CHI2DEL=0.0 

      DO I=1,NGPAR
         A(I)=0.0    
         B(I)=0.0
         DY(I)=0.0  
      ENDDO
 
      X=0.0
      THRESH2=0.0
      DMAX   =0.0     
C      CHI2G  =0.0    !external function
C      DCHI1  =0.0    !external function
      X1     =0.0
C      ZBRENT =0.0    !external function
      AMPERR =0.0
      AVERSIG=0.0
C      ERF    =0.0    !external function
                    
      S_N_THRESH=0.0
      S_N_WEIGHT=0.0 
      BASIC_THRESH2=0.0
      S_N_WEIGHT2=0.0 

      ALPHA=0
      DELTA=0

C     Doubles

C     Chars
      LINE(1:) = ' ' 

      CRCNT = CRMSKS
c    
c     Fit the parameters of all the gaussians fitted to the averaged
c     order profils. As a result you get the parameters of a gaussian
c     at each wavelength, stored in PBUF[]. The gaussians itself are
c     saved in FBVAL[].
c
     
      THRESH2 = THRESH**2
      BASIC_THRESH2=THRESH2


C QC check:
C Calculates medium S/N in central part (50 pixs) of the order 
C Calculates the medium object position in central part (50 pixs) of the order
C This data is stored as descriptor in the output frame to monitor
C optimal extraction quality    
      CALL GETQCI(IXMIN,IXMAX,PBUF,PDIM,NPIXA1,FBVAL,DIM,CONAD)                                
                              
      S_N_WEIGHT =  0.1         !Linear term
      S_N_WEIGHT2 = 0.0500      !quadratic term

      S_N_THRESH=S_N_WEIGHT*MID_S_N_CENT(N_CURR_ORD)+
     +           S_N_WEIGHT2*MID_S_N_CENT(N_CURR_ORD)*
     +                       MID_S_N_CENT(N_CURR_ORD)
            
      THRESH2 = BASIC_THRESH2+S_N_THRESH**2

    
c     The parameters of the gaussians at each wavelength are determined
c     Now calculate the gaussian profils and save them in FBVAL[].  
   
      DO I =  IXMIN,IXMAX

 101     PBUF(1,I) = 1.0        !amplitude of gaussian
         PBUF(4,I) = 0.0        !background (additional constant)
         PBUF(5,I) = 0.0

C if the order is very weak you may get a strange fit
C and have the position (=PBUF(2,I)) < 1 or > DIM
C the following fitting procedure may give crazy results
C in this case it is advisable to set artificially the position
C in the middle of the slit

         IF(PBUF(2,I).LT.1.OR.PBUF(2,I).GT. FLOAT(DIM)) THEN
            PBUF(2,I) = DIM/2.+OBJSET
            PBUF(3,I) = 2.
         ENDIF

C if the sigma (=PBUF(3,I)) is < 1 one may get again a strange fit
C below. It happens only in orders without signal.
C in this case it is advisable to set artificially to 1

         IF(PBUF(3,I).LT.1.)  PBUF(3,I)=1.
c     The amplitude and the background of the gaussian
c     may be determined analytically

         DO J = 1,5
            A(J) = 0.0
         ENDDO

         AVERSIG = 0.0
c        for each chunk .....         
         DO J = 1,DIM
            IF (VBVAL(I,J).LT.1.E-9) MASK(I,J) = 0
            IF (MASK(I,J).GT.0) THEN
               X = J
               CALL FGAUSS_C(X,PBUF(1,I),COLUM(J),DY)
               SIGMA(J) = VBVAL(I,J)
               PROF(I,J) = COLUM(J) !save the profile == weights!

C FBVAL is the value of the observed signal
C Column is the output of FGAUSS_C so the fit profile
C Found the optimal gaussian with fixed FWHM and POSITION
C calculates only the optimal amplitude + BKG(constant)
 
               
               AVERSIG = AVERSIG+SIGMA(J)    !sums
               B(1) = COLUM(J)/SIGMA(J)      !weighted mean
               B(2) = 1.0/SIGMA(J)           !weights
               A(1) = A(1) + B(1) * COLUM(J)
               A(2) = A(2) + B(1)
               A(3) = A(3) + B(1) * FBVAL(I,J)
               A(4) = A(4) + B(2)
               A(5) = A(5) + B(2) * FBVAL(I,J)
            ELSE
               PROF(I,J) = 0.0
            ENDIF
         ENDDO
         PBUF(4,I) = A(1)*A(4) - A(2)*A(2)
         IF (PBUF(4,I) .NE. 0) THEN
           PBUF(4,I) = (A(1)*A(5) - A(2)*A(3)) / PBUF(4,I)
         ELSE
           PBUF(4,I) = 1E33
           WRITE(LINE,*) '(4) BAD prof at X=',I,A(1),A(2),A(3),A(4),A(5)
           CALL STTPUT(LINE,STATUS)
         ENDIF
         IF (A(1) .NE. 0) THEN
           PBUF(1,I) = (A(3) - A(2)*PBUF(4,I)) / A(1)
         ELSE
           PBUF(1,I) = 1E33
           WRITE(LINE,*) '(1) BAD prof at X=',I,A(1),A(2),A(3),PBUF(4,I)
           CALL STTPUT(LINE,STATUS)
         ENDIF
         
c     Now the amplitude and the background are known
c     Determine the final gaussian profil

c     Do the kappa sigma clipping if enabled (THRESH>0) to remove
c     cosmic ray hits.
         
         IF (THRESH.GT.0) THEN
            JMAX = -1
            NVAL = 0
            DMAX = 0.0


              
            DO J = 1,DIM
            
c 13/4/2000 am:                  
c to monitor points which may be chopped of as out of threshold
c put before check on mask(I,J) to have all the points anyway                
C It would be nice to put values in a table to be used to do QC
c                  X = J
c                  CALL FGAUSS_C(X,PBUF(1,I),PVAL,DY)
c                  NVAL = NVAL + 1
c                  DY(1) = FBVAL(I,J) - PVAL
c                  DY(2) = DY(1)*DY(1)/SIGMA(J)
c                  IF ((I.GE.(IXMIN+500)).AND.(I.LT.(IXMIN+600))) THEN 
c                      WRITE(LINE,*), X,FBVAL(I,J),PVAL,SIGMA(J),DY(1),DY(2)
c                      CALL STTPUT(LINE,STATUS)
c                  ENDIF               
c end of check            
            
        
        
            
               IF (MASK(I,J).GT.0) THEN
                  X = J
                  CALL FGAUSS_C(X,PBUF(1,I),PVAL,DY)
                  NVAL = NVAL + 1
                  DY(1) = FBVAL(I,J) - PVAL
                  DY(2) = DY(1)*DY(1)/SIGMA(J)
                  
                  
                  IF (DY(2).GT.THRESH2) THEN
                     IF (DY(2) .GT. DMAX) THEN
                        JMAX = J
                        DMAX = DY(2)
                     ENDIF
                  ENDIF
               ENDIF


            ENDDO

C It is necessary to have at least 3 pixels left to fit the gaussian + BKGD
c here does a check to eliminate the point over THRESH and goes on only if
c it has at least 3 points for the gaussian fit      
           
          IF (JMAX.GT.0.AND.NVAL.GT.3) THEN
               CRCNT = CRCNT + 1
               CALL TBRWRI(MTID,CRCNT,1,MIDORD,N_CURR_ORD,ISTAT)
               CALL TBRWRI(MTID,CRCNT,1,MIDX,I,ISTAT)
               CALL TBRWRI(MTID,CRCNT,1,MIDY,JMAX,ISTAT)
               CALL TBRWRR(MTID,CRCNT,1,MIDFLX,FBVAL(I,JMAX),ISTAT)
               MASK(I,JMAX) = 0
               GOTO 101
            ENDIF
         ENDIF
         CRMSKS=CRCNT
         
c here we have the final profile including the amplitude
C we need to estimate the error on the amplitude
c first compute the minimum chi2 for which we need the 'original' profil
c with amplitude = 1.0
c was         ALPHA = 0.1
c 13/3/2000 am: was  ALPHA = 0.2
              ALPHA = 0.2
         
  150    CONTINUE
        
         CHI2MIN = CHI2G(PBUF(1,I),COLUM,FBVAL,SIGMA,MASK,PBUF(4,I)
     $        ,NPIXA1,DIM,I)



c NEW: to get the error on the amplitude we now do a parabola fit on
c the chi2 function. That provides us to use the root finding procedure
c which was instable due to a bad start interval.

C am: Here is the problem, now fixed !

         CHI2DEL = CHI2G(PBUF(1,I)*(1-ALPHA),COLUM,FBVAL,SIGMA,MASK
     $        ,PBUF(4,I),NPIXA1,DIM,I)

c 
c was From Pascal-sSebastian
c         DELTA = CHI2DEL - CHI2MIN + 1.E-6
c Stefano suggest:
         DELTA = CHI2DEL - CHI2MIN 

C AM: 13/03/2000 
C     Added check on DELTA .LT. e8 and corresponding
C     redefinition of DELTA=1e8 and AMPERR=1e-5
C     We noticed that with a Quasar @ R860 2x2
c     with IXWIND=64 DELTA=NAN and then GaussJ pauses
C     asking to type go to continue. In the end the program
c     was crashing when optimal extraction was done 
c     (Invalid cut values) 
C                
         IF((DELTA .GT. 0) .AND. (DELTA .LT. 1e8)) THEN
            X1 = (1 - ALPHA * SQRT( 1.0 / DELTA )) * PBUF(1,I)
            AMPERR = ALPHA * SQRT( 1.0 / DELTA ) * PBUF(1,I)
         ELSE IF ((DELTA .LE. 0) .AND. (ALPHA .LT. 0.9)) THEN
            ALPHA =ALPHA+0.1
            GOTO 150
         ELSE IF  (DELTA .GT. 1e8) THEN
            DELTA = 1e8
            X1=0
            AMPERR= 1e-5
         ELSE                     
            X1= 0
            AMPERR = 1e2            
         ENDIF

c Here calculates the variance
c OUT is the amplitude
c VOUT is its variance
C 2.5066283=sqrt(2*PI)
C VOUT= [D(OUT)/D(AMP)*ERR_AMP]^2
CAK 2002-02-23
CAK it looks like the integrated signal is computed here, so
CAK we have to use sqrt(pi)=1.77245385091 and not sqrt(2pi)=2.5066283
CAK (as in the FGAUS function we mis a 1/2 factor in the argument of the exponential)
CAK
CAK
CWAS
CWAS         OUT(I,INDX)  = 2.5066283*PBUF(3,I)*PBUF(1,I)
CWAS         VOUT(I,INDX) = (2.5066283*PBUF(3,I)*AMPERR)**2
            OUT(I,INDX)  = 1.77245385091*PBUF(3,I)*PBUF(1,I)
            VOUT(I,INDX) = (1.77245385091*PBUF(3,I)*AMPERR)**2
         SKYBKG(I,N_CURR_ORD) = PBUF(4,I)*DIM
C we assume that PBUF(4,I) is affected from same AMPERR error as PBUF(1,I)
         VARSKY(I,N_CURR_ORD) = (AMPERR*DIM)**2
        
         
         IF ((VOUT(I,INDX) .GT. 0) .AND. (VOUT(I,INDX) .LT. 1.e8)) THEN 
            CONTINUE
         ELSE            
             VOUT(I, INDX) = 1.e8
         ENDIF
      ENDDO  
        
      RETURN
      END
C*******************************************************************
