C $Id: my_fun.for,v 1.1 2001/11/01 07:18:37 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)my_fun.for	1.0 (ESO-DFS) 23/10/01 12:26:19
C+
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  subroutines my_fun.for  version 1.0 011023 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C Collects some useful functions used in optimal extraction.
C.SYNTAX
C 
C.HISTORY:
C  2001.10.23-AM
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.1 $
C $Log: my_fun.for,v $
C Revision 1.1  2001/11/01 07:18:37  amodigli
C First version of necopt with splitted data
C
C
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     23/10/01       Created                           all  
C==============================================================================     
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     FGAUSS_C
C------------------------------------------------------------------------------
C  Role: gives the gaussian + linera term at point X with given 
C        coefficients stored in the array A. Derivatives are stored in the
C        array DYDA 
C------------------------------------------------------------------------------

      SUBROUTINE FGAUSS_C(X,A,Y,DYDA)
      IMPLICIT NONE
      REAL X                     !X position
      REAL Y                     !Y value
      REAL A(5)                  !Gaussial+linear coefficients
      REAL DYDA(5)               !derivs of function respect to coefficients

C
C     Local variables
C
      REAL A3I                   !reciprocal of A(3)
      REAL ARG                   !argument of gaussian
      REAL EX                    !exponential contribute
      REAL FAC                   !variable holder to compact expression

C
C     Initialization and 1st Local variables setting
C            

      Y=0.
      A3I=1.0/A(3)
      ARG=(X-A(2))*A3I            !was  ARG=(X-A(2))/A(3)
      EX=EXP(-ARG**2)
      FAC=A(1)*EX*2.*ARG

      Y=A(1)*EX + A(4)*(1.+(X-A(2))*A(5))
      DYDA(1)=EX
      DYDA(2)=FAC*A3I-A(4)*A(5)   !was DYDA(2)=FAC/A(3)-A(4)*A(5)
      DYDA(3)=FAC*ARG*A3I         !was DYDA(3)=FAC*ARG/A(3)
      DYDA(4)=1.+(X-A(2))*A(5)
      DYDA(5)=A(4)*(X-A(2))
      END


C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     POLYN
C------------------------------------------------------------------------------
C  Role: gives the polinomial of order 3 (degree 2) at point X with given 
C        unitary coefficients  
C------------------------------------------------------------------------------

      SUBROUTINE POLYN(X,Y,NDIM)
      IMPLICIT NONE
      INTEGER NDIM
      REAL X                !X position
      REAL Y(3)             !polynomial derivatives respect to coefficients

      Y(1) = 1.0
      Y(2) = X
      Y(3) = X*X

      RETURN
      END


C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     POLYN3
C------------------------------------------------------------------------------
C  Role: gives the polinomial of order 3 (degree 2) at point X with given 
C        coefficients stored in the array A 
C------------------------------------------------------------------------------
      FUNCTION POLYN3(X,A)
      IMPLICIT NONE
      REAL X              !X position
      REAL A(3)           !polynomial coefficients
      REAL POLYN3         !result

      POLYN3 = A(1)+A(2)*X+A(3)*X*X
      RETURN
      END



C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     POLYN5
C------------------------------------------------------------------------------
C  Role: gives the polinomial of order 5 (degree 4) at point X with given 
C        coefficients stored in the array A 
C------------------------------------------------------------------------------
      FUNCTION POLYN5(X,A)
      IMPLICIT NONE
      REAL X                !X position
      REAL A(5)             !polynomial coefficients
      REAL POLYN5           !result

      POLYN5 = A(1)+
     &         A(2)*X+
     &         A(3)*X*X+
     &         A(4)*X*X*X+
     &         A(5)*X*X*X*X

     
      RETURN
      END


C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     MYAVRG
C------------------------------------------------------------------------------
C  Role: evaluates an average of points
C------------------------------------------------------------------------------
      SUBROUTINE MYAVRG(ARRAY,MASK,IX,IMIN,IMAX,ISLICE,RES)
      IMPLICIT NONE
      INTEGER IMIN        !min value of loop variable
      INTEGER IMAX        !max value of loop variable
      INTEGER ISLICE      !current X position along the order
      INTEGER IX          !X size
      INTEGER*2 MASK(5,IX)  !array to mask profile points
      
      REAL ARRAY(5,IX)    !array to hold raw points

C
C     Local variables
C      
      INTEGER NP          !counter
      INTEGER I           !loop variable
      REAL RES            !result

  
C
C    Initialization of Local variables
C

      RES=0.
      NP=0
      I=0
      
      DO I = IMIN,IMAX
         IF (MASK(2,I).GT.0) THEN    !NB We use only 2nd component of mask
            RES = RES+ARRAY(ISLICE,I)      
            NP=NP+1
         ENDIF      
      ENDDO
      RES=RES/NP
      
      RETURN  
      END
      
      
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     MYSIGM
C------------------------------------------------------------------------------
C  Role: evaluates the standard deviation of point data respect to an average
C------------------------------------------------------------------------------
      SUBROUTINE MYSIGM(ARRAY,AVREF,MASK,IX,IMIN,IMAX,ISLICE,RES)
      IMPLICIT NONE
      INTEGER IMIN              !min value of loop variable
      INTEGER IMAX              !max value of loop variable
      INTEGER ISLICE            !current X position along the order
      INTEGER IX                !X size
      INTEGER*2 MASK(5,IX)        !array to mask profile points
      REAL ARRAY(5,IX)          !array to hold raw points
      REAL AVREF                !average ref value

C
C     Local variables
C      
      INTEGER NP                !counter
      INTEGER I                 !loop variable
      REAL RES                  !result 

C
C     Initialization of local variables
C


      RES=0.
      NP=0
      I=0
      
      DO I = IMIN,IMAX
         IF (MASK(2,I).GT.0) THEN    !NB We use only 2nd component of mask
            RES = RES+(ARRAY(ISLICE,I)-AVREF)*(ARRAY(ISLICE,I)-AVREF)      
            NP=NP+1
         ENDIF      
      ENDDO
      RES=SQRT(RES/(NP-1))
      
      RETURN 
      END      
C ============================================================================      
