C $Id: necexor.for,v 1.6 2002/02/11 09:54:21 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR: D.PONZ
C
C.IDENTIFICATION
C  program  ECHEXOR  version 1.2 840114
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C
C  EXTRACT ECHELLE ORDERS IN CASPEC IMAGES. ORDER POSITION ARE DEFINED
C  BY COEFFICIENTS OF A REGRESSION. SLIT WIDTH, ANGLE AND OFFSET
C  ARE DEFINED BY PARAMETERS
C            WIDTH,ANGLE,OFFSET
C
C.ALGORITHM
C
C   ORDERS ARE EXTRACTED BY PASSING A NUMERICAL SLIT OF GIVEN WIDTH 
C  AND ANGLE IN POSITIONS DEFINED BY THE REGRESSION COEFFS. THE SLIT
C  WILL SAMPLE AT INCREMENTS OF ONE STEP IN X.
C
C  THERE ARE THREE METHODS :
C   LINEAR   - PIXEL VALUES ARE ADDED, LINEAR INTERPOLATION
C   AVERAGE  - PIXEL VALUES ARE AVERAGED, LINEAR INTERPOLATION
C   WEIGHTED - PIXEL VALUES ARE WEIGHTD PROPORTIONALLY TO THE
C              PROFI PERPENDICULAR TO THE DISPERSION DIRECTION
C.INPUT/OUTPUT
C
C   EXTRACT/ORDER output = input width,angle,offset 
C				 table coeffs [ord1,ord2]
C  the connection file contains
C  IN_A   input FRM
C  IN_B   input table with coeffs
C  OUT_A  output FRM
C  INPUTR  width,angle
C  INPUTC  coeffs
C  INPUTI  ord1,ord2
C  ACTION               T - table option
C                       B - batch order by order
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.6 $
C $Log: necexor.for,v $
C Revision 1.6  2002/02/11 09:54:21  amodigli
C Initialized some variables to reduce purify errors
C
C Revision 1.5  2002/01/14 09:10:21  amodigli
C dfs code rev step1: defined variables
C
C Revision 1.4  2001/07/23 09:12:44  amodigli
C Eliminated some warinngs indicated by ftncheck
C
C Revision 1.3  2001/01/04 12:31:31  amodigli
C Added Id, Name, Revision, Log conf. control info
C
C------------------------------------------------------------------------------
      PROGRAM ECHEXO
      IMPLICIT NONE
      INTEGER NDIM                  !Input frm dimension 
      PARAMETER (NDIM=2)
      INTEGER MADRID                !(MIDAS env) Memory Addres ID
      INTEGER NAC                   !(MIDAS env) No of words allocated per raw
      INTEGER NAR                   !(MIDAS env) No of rows allocated
      INTEGER IMNO                  !(MIDAS env) File Id of frm
      
      INTEGER I                     !Aux variable to loop over orders
      INTEGER IAV                   !(MIDAS env) Actual No of mapped data
      INTEGER INDX                  !Aux variable to hold size of string
      INTEGER IORD                  !Aux variable to hold tmp relative order No
      INTEGER IORD1                 !Minimum of order No
      
      INTEGER IORD2                 !Minimum of order No
      INTEGER ISTAT                 !(MIDAS env) Aux var for return status of op.
      INTEGER NA                    !NA     = (IPAR(3)+1)* (IPAR(4)+1)
      INTEGER NAXISA                !(MIDAS env) Dimension of input frm
      INTEGER NAXISB                !(MIDAS env) Dimension of output frm (=2)
      
      INTEGER NCOL                  !(MIDAS env) No of defined column in tab
      INTEGER NPAR                  !NPAR   INTG*4  NO. OF EXTRACTION PARAMETERS
      INTEGER NROW                  !(MIDAS env) No of defined rows in tab
      INTEGER NSORT                 !(MIDAS env) No of sorted column in tab
      
      INTEGER*8 PNTRA               !(MIDAS env) Pointer to mapped data (in frm)
      INTEGER*8 PNTRB               !(MIDAS env) Pointer to mapped data (out frm)
      INTEGER*8 PNTRC               !(MIDAS env) Pointer to mapped data 
      
      INTEGER STATUS                !(MIDAS env) Aux var for return status of op.
      INTEGER TID                   !(MIDAS env) Table indentifier (input tab)
      INTEGER INO                   !(MIDAS env) File id of frm (out frm)
      INTEGER TMPID                 !(MIDAS env) Column id
      
      INTEGER NPIXA(3)              !(MIDAS env) No of pixels per axis (in frm)
      INTEGER NPIXB(3)              !(MIDAS env) No of pixels per axis (out frm)
      INTEGER IPAR(10)              !(MIDAS env) Holder for descriptor: (ord min/max)
      INTEGER KUN                   !(MIDAS env) Units of character keyword
      INTEGER KNUL                  !(MIDAS env) No of null values of character keyword
      
      INTEGER NWDTH                 !Size of frame
C
      DOUBLE PRECISION STEPA(3)     !(MIDAS env) Step size of input frm
      DOUBLE PRECISION STEPB(3)     !(MIDAS env) Step size of output frm
      DOUBLE PRECISION STARTA(3)    !(MIDAS env) start values of input frm
      DOUBLE PRECISION STARTB(3)    !(MIDAS env) start values of output frm
      
      REAL  RPAR(10)                !(MIDAS env) Holder for input real descriptor (INPUTR)
      REAL  CUTS(6)                 !Ima cuts (Xmin,Xmax,Ymin,Ymax)
      
      REAL XMIN                     !Xmin out ima
      REAL XMAX                     !Xmax out ima
      REAL XMIN1                    !Xmin out ima  ?used?
      REAL XMAX1                    !Xmax out ima  ?used?
      
      REAL PARAMS(12)               !EXTRACTION PARAMETERS
C
      DOUBLE PRECISION DPAR(100)    !(MIDAS env) Hold descriptor coef of order def polyn
C
      CHARACTER*72   IDENA          !(MIDAS env) ASCII Identification of input frm
      CHARACTER*72   IDENB          !(MIDAS env) ASCII Identification of output frm
      
      CHARACTER*64   CUNITA         !(MIDAS env) Units of each axis+data of input frm
      CHARACTER*64   CUNITB         !(MIDAS env) Units of each axis+data of output frm
      CHARACTER*64   DESNAM         !(MIDAS env) Name of output frame descriptor (COEFFC)
      
      CHARACTER*1    METH           !Extraction method
      CHARACTER*60   FRMIN          !Char holder for "IN_A" descriptor for input frm name
      CHARACTER*60   FRMOUT         !Char holder for "IN_B" descriptor for input tab name
      CHARACTER*60   INA1           !Char holder for "IN_A" descriptor for input frm name
      CHARACTER*60   INA2           !Char holder for "OUT_A" descriptor for output frm name

      CHARACTER*60   TABLE          !Char holder for "IN_B" descriptor for input tab name
      CHARACTER*60   NAME           !Char holder for "INPUTC" descriptor for output frm name
      CHARACTER*60   TYPE           !Char holder for "COEFFC" descriptor for output frm name
C
      INCLUDE 'MID_INCLUDE:ST_DEF.INC'
      COMMON /VMR/MADRID(1)
      INCLUDE 'MID_INCLUDE:ST_DAT.INC'
C
      DATA CUNITB/
     +     'FLUX            PIXEL                           ORDER'/
     
     
C     initialize some variables     
C     Integer variables
     
      NAC=0                   !(MIDAS env) No of words allocated per raw
      NAR=0                   !(MIDAS env) No of rows allocated
      IMNO=0                  !(MIDAS env) File Id of frm
      
      I=0                     !Aux variable to loop over orders
      IAV=0                   !(MIDAS env) Actual No of mapped data
      INDX=0                  !Aux variable to hold size of string
      IORD=0                  !Aux variable to hold tmp relative order No
      IORD1=0                 !Minimum of order No
      
      IORD2=0                 !Minimum of order No
      ISTAT=0                 !(MIDAS env) Aux var for return status of op.
      NA=0                    !NA     = (IPAR(3)+1)* (IPAR(4)+1)
      NAXISA=0                !(MIDAS env) Dimension of input frm
      NAXISB=0                !(MIDAS env) Dimension of output frm (=2)
      
      NCOL=0                  !(MIDAS env) No of defined column in tab
      NPAR=0                  !NPAR   INTG*4  NO. OF EXTRACTION PARAMETERS
      NROW=0                  !(MIDAS env) No of defined rows in tab
      NSORT=0                 !(MIDAS env) No of sorted column in tab

C     Reals

      DO I=1,10
         RPAR(I)=0               !(MIDAS env) Holder for input real descriptor (INPUTR)
      ENDDO 
      DO I=1,6             
         CUTS(I)=0               !Ima cuts (Xmin,Xmax,Ymin,Ymax)
      ENDDO
      XMIN=0                     !Xmin out ima
      XMAX=0                     !Xmax out ima
      XMIN1=0                    !Xmin out ima  ?used?
      XMAX1=0                    !Xmax out ima  ?used?
      
      DO I=1,12
         PARAMS(I)=0             !EXTRACTION PARAMETERS
      ENDDO
      
C    Double precision
      
      DO I=1,3            
         STEPA(I)=0              !(MIDAS env) Step size of input frm
         STEPB(I)=0              !(MIDAS env) Step size of output frm
         STARTA(I)=0             !(MIDAS env) start values of input frm
         STARTB(I)=0             !(MIDAS env) start values of output frm
      ENDDO
      DO I=1,100
         DPAR(I)=0               !(MIDAS env) Hold descriptor coef of order def polyn
      ENDDO
      
C     CHARS
      IDENA(1:) = ' '            !(MIDAS env) ASCII Identification of input frm
      IDENB(1:) = ' '            !(MIDAS env) ASCII Identification of output frm
      
      CUNITA(1:) = ' '           !(MIDAS env) Units of each axis+data of input frm
      DESNAM(1:) = ' '           !(MIDAS env) Name of output frame descriptor (COEFFC)
      
      METH(1:)   = ' '           !Extraction method
      FRMIN(1:)  = ' '           !Char holder for "IN_A" descriptor for input frm name
      FRMOUT(1:) = ' '           !Char holder for "IN_B" descriptor for input tab name
      INA1(1:)   = ' '           !Char holder for "IN_A" descriptor for input frm name
      INA2(1:)   = ' '           !Char holder for "OUT_A" descriptor for output frm name

      TABLE(1:)  = ' '           !Char holder for "IN_B" descriptor for input tab name
      NAME(1:)   = ' '           !Char holder for "INPUTC" descriptor for output frm name
      TYPE(1:)   = ' '           !Char holder for "COEFFC" descriptor for output frm name
                 
C
C  initialize system
C
      CALL STSPRO('ECHEXO')
C     Reads input descriptors      
      CALL STKRDC('IN_A',1,1,60,I,INA1,KUN,KNUL,STATUS)
      CALL STKRDC('IN_B',1,1,60,I,TABLE,KUN,KNUL,STATUS)
      CALL STKRDC('IN_C',1,1,1,I,METH,KUN,KNUL,STATUS)
      CALL STKRDC('OUT_A',1,1,60,I,INA2,KUN,KNUL,STATUS)
      CALL STKRDC('INPUTC',1,1,60,I,NAME,KUN,KNUL,STATUS)
      CALL STKRDR('INPUTR',1,3,I,RPAR,KUN,KNUL,STATUS)
      CALL STKRDI('INPUTI',1,2,I,IPAR,KUN,KNUL,STATUS)
      IORD1  = IPAR(1)
      IORD2  = IPAR(2)
      FRMIN = INA1
      FRMOUT = INA2
      
C     check the input method is upper case, set a switch      
      IF (METH(1:1) .EQ. 'l') METH = 'L'
      IF (METH(1:1) .EQ. 'a') METH = 'A'
      IF (METH(1:1) .EQ. 'm') METH = 'M'
C
C  input parameters in table
C
      CALL TBTOPN(TABLE,F_U_MODE,TID,STATUS)
      CALL TBIGET(TID,NCOL,NROW,NSORT,NAC,NAR,STATUS)
C
C  map input FRM
C
      CALL STIGET(FRMIN,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,
     .            NDIM,NAXISA,NPIXA,STARTA,STEPA,IDENA,
     +            CUNITA,PNTRA,IMNO,STATUS)
      PARAMS(1) = RPAR(1)/ABS(STEPA(2))
      PARAMS(2) = RPAR(2)
      PARAMS(3) = RPAR(3)/ABS(STEPA(2))

      IF (METH(1:1) .EQ. 'M') THEN
         NWDTH = PARAMS(1) + 2
         CALL STFCRE('column',D_R4_FORMAT,F_X_MODE,1,NWDTH,TMPID,STATUS)
         CALL STFMAP(TMPID,F_X_MODE,1,NWDTH,IAV,PNTRC,STATUS)
      ENDIF
C
C  read coeffs
C
      NAME   = NAME(1:59)//' '
      INDX   = INDEX(NAME,' ') - 1
      DESNAM = NAME(1:INDX)//'C'
      CALL STDRDC(TID,DESNAM,1,17,4,IAV,TYPE,KUN,KNUL,ISTAT)
      TYPE   = 'MULT'
      IF (TYPE.EQ.'MULT') THEN
         DESNAM = NAME(1:INDX)//'I'
         CALL STDRDI(TID,DESNAM,4,4,IAV,IPAR,KUN,KNUL,ISTAT)
         NA     = (IPAR(3)+1)* (IPAR(4)+1)
         DESNAM = NAME(1:INDX)//'R'
         CALL STDRDR(TID,DESNAM,1,4,IAV,PARAMS(4),KUN,KNUL,ISTAT)
         DESNAM = NAME(1:INDX)//'D'
         CALL STDRDD(TID,DESNAM,1,NA,IAV,DPAR,KUN,KNUL,ISTAT)
         CALL TBTCLO(TID,STATUS)
C
C ... map output FRM
C
         IF (IORD1.EQ.IORD2 .AND. IORD1.EQ.0) THEN
            IORD1  = PARAMS(6)
            IORD2  = PARAMS(7)
         END IF
         STARTB(1) = STARTA(1)
         STARTB(2) = IORD1
         STEPB(1)  = STEPA(1)
         STEPB(2)  = 1.D0
         IDENB     = IDENA
         NAXISB    = 2
         NPIXB(1)  = NPIXA(1)
         NPIXB(2)  = IORD2 - IORD1 + 1
         CALL STIPUT(FRMOUT,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     +        NAXISB,NPIXB,STARTB,
     +        STEPB,IDENB,CUNITB,PNTRB,INO,STATUS) 
C     
C  extract order
C
         NPAR      = 10
         PARAMS(8) = IPAR(3)
         PARAMS(9) = IPAR(4)
         XMIN      = 1.E30
         XMAX      = -XMIN
         DO 10 I = IORD1,IORD2
            
            PARAMS(10) = I
            IORD       = I - IORD1 + 1
            
            IF (METH(1:1) .EQ. 'M') THEN
               CALL EXTR2M(MADRID(PNTRA),NPIXA(1),NPIXA(2),
     +              MADRID(PNTRB),NPIXB(1),NPIXB(2),MADRID(PNTRC),NWDTH,
     +              IORD,NPAR,PARAMS,NA,DPAR,XMIN1,XMAX1,METH,
     +              STARTA,STEPA)
            ELSE
               CALL EXTR1M(MADRID(PNTRA),NPIXA(1),NPIXA(2),
     +              MADRID(PNTRB),NPIXB(1),NPIXB(2),
     +              IORD,NPAR,PARAMS,NA,DPAR,XMIN1,XMAX1,METH,
     +              STARTA,STEPA)
            ENDIF

            XMIN   = AMIN1(XMIN,XMIN1)
            XMAX   = AMAX1(XMAX,XMAX1)
 10      CONTINUE
      ELSE
         CALL STTPUT(' Error in dispersion coefficients ',ISTAT)
         GO TO 20
      END IF
C
C  descriptor handling
C
      CUTS(1) = XMIN
      CUTS(2) = XMAX
      CUTS(3) = XMIN
      CUTS(4) = XMAX
      CUTS(5) = 0.
      CUTS(6) = 0.
      CALL DSCUPT(IMNO,INO,' ',STATUS)
      CALL STDWRR(INO,'LHCUTS',CUTS,1,6,KUN,STATUS)
C
C  free data
C
C      CALL STFCLO(INO,STATUS)
 20   CALL STSEPI
      END

      SUBROUTINE EXTR1M(IN,NPIXA1,NPIXA2,OUT,NB1,NB2,INDX,NPAR,PAR,ND,
     +     DPAR,XMIN,XMAX,METH,STARTA,STEPA)
C
C       EXTRACTION ROUTINE - BILINEAR INTERPOLATION
C
C   ARGUMENTS
C IN     REAL*4  INPUT IMAGE
C NPIXA1 INTG*4  NO. OF SAMPLES
C NPIXA2 INTG*4  NO. OF LINES
C OUT    REAL*4  EXTRACTED ORDERS
C NB1    INTG*4  NO. OF SAMPLES
C NB2    INTG*4  NO. OF ORDERS
C INDX   INTG*4  SEQUENTIAL NO. OF THE ORDER
C NPAR   INTG*4  NO. OF EXTRACTION PARAMETERS
C PAR    REAL*4  EXTRACTION PARAMETERS
C ND     INTG*4  NO. OF COEFFS.
C DPAR   REAL*8  COEFFS. TO DEFINE ORDER POSITION
C XMIN   REAL*4  MINIMUM VALUE
C XMAX   REAL*4  MAXIMUM VALUE
C
      IMPLICIT NONE
      INTEGER  NB1               !INTG*4  NO. OF SAMPLES
      INTEGER  NB2               !INTG*4 NO. OF ORDERS
      INTEGER  NPAR              !INTG*4  NO. OF EXTRACTION PARAMETERS
      INTEGER  NPIXA1            !INTG*4  NO. OF SAMPLES (X in frm size)
      INTEGER  NPIXA2            !INTG*4  NO. OF LINES   (Y in frm size)
      INTEGER  ND                !INTG*4  NO. OF COEFFS.
      
      REAL OUT(NB1,NB2)          !REAL*4  EXTRACTED ORDERS (output frm)
      REAL PAR(NPAR)             !REAL*4  EXTRACTION PARAMETERS
      
      
      REAL IN(NPIXA1,NPIXA2)     !REAL*4  INPUT IMAGE
      REAL XMIN                  !REAL*4  MINIMUM VALUE
      REAL XMAX                  !REAL*4  MAXIMUM VALUE
      
      DOUBLE PRECISION DPAR(ND)  !REAL*8  COEFFS. TO DEFINE ORDER POSITION
      DOUBLE PRECISION STARTA(2) !(MIDAS env) start values of input frm
      DOUBLE PRECISION STEPA(2)  !(MIDAS env) Step size of input frm
      
      DOUBLE PRECISION XVAL      !(MIDAS env) Holder for world coordinate X point
      DOUBLE PRECISION YVAL      !(MIDAS env) Holder for world coordinate Y point
      DOUBLE PRECISION AX        !AX=1.D0 + SKWOF ??

      DOUBLE PRECISION WIDTH     !WIDTH=PAR(1)
      DOUBLE PRECISION SKEW      !SKEW=PAR(2)
      DOUBLE PRECISION HWIDTH    !half of WIDTH
      DOUBLE PRECISION OFFSET    !OFFSET=PAR(3)
      DOUBLE PRECISION DC        !???
      
      
      DOUBLE PRECISION YV        !YV=PAR(10)
      DOUBLE PRECISION WW(50)    !??
      
      DOUBLE PRECISION SKWDX     !SKWDX  = -DSIN(DD)     !   DX PER PIXEL
      DOUBLE PRECISION SKWOF     !SKWOF  = -SKWDX*HWIDTH !   2D INTERPOLATION
      DOUBLE PRECISION SUM       !Holder for output image intensity
      
      DOUBLE PRECISION DEGREE    !conversion factor from DEG to RAD
      DOUBLE PRECISION DD        !DD=SKEW*DEGREE
      DOUBLE PRECISION DSIN      !sinus of an argument as double precision function
      
      INTEGER          IWIDTH    !Integer part of WIDTH
      INTEGER          NX        !NX= SKWOF
      INTEGER          N1        !N1=NX
      INTEGER          N2        !N2= NPIXA1 - NX
      INTEGER          K         !K=PAR(8)
      INTEGER          L         !L=PAR(9)
      INTEGER          IX        !Loop variable
      INTEGER          INDX      !INTG*4  SEQUENTIAL NO. OF THE ORDER
      
      INTEGER          IX1       !Loop variable
      INTEGER          IP        !Tmp variable
      INTEGER          L1        !Loop variable
      INTEGER          K1        !Loop variable
      INTEGER          IW        !Loop variable
      
C     To Correct the central position for the offset as the OFFSET does not take into account HWIDTH 
      INTEGER          LOSLIT    !LOSLIT = INT(YLOW) 
      INTEGER          HISLIT    !HISLIT = INT(YHIGH) 
      
      DOUBLE PRECISION YLOW      !YLOW   = YVAL - HWIDTH
      DOUBLE PRECISION YHIGH     !YHIGH  = YVAL + HWIDTH
      
      CHARACTER*1      METH      !Extraction method
      DOUBLE PRECISION IX_R8R8   !(MIDAS env) conversion pixel to world coordinate
      DOUBLE PRECISION XI_R8R8   !(MIDAS env) conversion world to pixel coordinate
C
C     DEGREE = RAD / DEGREE
C
      DATA DEGREE/0.017453292519943295769237D+0/
C

C
C ... define extraction constants
C
      WIDTH  = PAR(1)
      IWIDTH = WIDTH
      HWIDTH = 0.5D0*WIDTH
      SKEW   = PAR(2)
      SKWDX  = 0.0D0
      SKWOF  = 0.0D0
      XMIN   = 0.0
      XMAX   = XMIN
C      OFFSET = PAR(3) - HWIDTH + 0.5
      OFFSET = PAR(3)
      IF (SKEW.NE.0.0) THEN       
         DD      = SKEW*DEGREE
         SKWDX  = -DSIN(DD)     !   DX PER PIXEL
         SKWOF  = -SKWDX*HWIDTH !   2D INTERPOLATION
      END IF
      AX     = 1.D0 + SKWOF
      NX     = SKWOF
      N1     = NX
      N2     = NPIXA1 - NX
C
C ... get ordinate values
C
      K      = PAR(8)
      L      = PAR(9)
      YV     = PAR(10)
C
C ... loop over the samples
C
      DO 10 IX = 1,N1
         OUT(IX,INDX) = 0.
 10   CONTINUE

      DO 100 IX = N2 + 1,NPIXA1
         OUT(IX,INDX) = 0.
 100  CONTINUE

      DO 50 IX1 = N1 + 1,N2

         IP     = 0
         DC     = 1.D0
         YVAL   = 0.D0
         XVAL   = IX_R8R8(AX,STARTA(1),STEPA(1))

C Computes position of the center of the slit from the coefficients of
C the bivariate polynomial stored in table order.

         DO 30 L1 = 0,L
            IP     = IP + 1
            WW(IP) = DC
            YVAL   = YVAL + WW(IP)*DPAR(IP)
            DO 20 K1 = 1,K
               IP     = IP + 1
               WW(IP) = WW(IP-1)*XVAL
               YVAL   = YVAL + WW(IP)*DPAR(IP)
 20         CONTINUE
            DC     = DC*YV
 30      CONTINUE
C
C Corrects the central position for the offset
C OFFSET does not take into account HWIDTH 
C
         YVAL   = XI_R8R8(YVAL,STARTA(2),STEPA(2)) + OFFSET
C     
         YLOW   = YVAL - HWIDTH
         YHIGH  = YVAL + HWIDTH
         LOSLIT = INT(YLOW)
         HISLIT = INT(YHIGH)

         IF (LOSLIT.LT.1)      LOSLIT = 1
         IF (HISLIT.GT.NPIXA2) HISLIT = NPIXA2

         SUM    = 0.D0
         IX     = INT(AX)
C
         IF (LOSLIT.EQ.HISLIT) THEN
            SUM = (YHIGH-YLOW)*IN(IX,LOSLIT)
C
         ELSE
C
            DO 40 IW = LOSLIT,HISLIT
               
               IF (IW.EQ.LOSLIT) THEN
                  SUM = SUM + IN(IX,IW)*(LOSLIT+1.-YLOW)
                  
               ELSE IF (IW.EQ.HISLIT) THEN
                  SUM = SUM + IN(IX,IW)*(YHIGH-HISLIT)

               ELSE
                  SUM = SUM + IN(IX,IW)

               ENDIF


 40         CONTINUE
C
         ENDIF
C
         IF (METH(1:1) .EQ. 'L') THEN ! Linear extraction
            OUT(IX1,INDX) = SUM
         ELSE                   ! Average extraction
            OUT(IX1,INDX) = SUM/WIDTH
         ENDIF
         XMIN   = AMIN1(XMIN,OUT(IX1,INDX))
         XMAX   = AMAX1(XMAX,OUT(IX1,INDX))
         AX     = AX + 1.D0
 50   CONTINUE

      RETURN

      END



      SUBROUTINE EXTR2M(IN,NPIXA1,NPIXA2,OUT,NB1,NB2,COL,NWDTH,INDX,NPAR
     $     ,PAR,ND,DPAR,XMIN,XMAX,METH,STARTA,STEPA)
C
C       EXTRACTION ROUTINE - MEDIAN
C
C   ARGUMENTS
C IN     REAL*4  INPUT IMAGE
C NPIXA1 INTG*4  NO. OF SAMPLES
C NPIXA2 INTG*4  NO. OF LINES
C OUT    REAL*4  EXTRACTED ORDERS
C NB1    INTG*4  NO. OF SAMPLES
C NB2    INTG*4  NO. OF ORDERS
C INDX   INTG*4  SEQUENTIAL NO. OF THE ORDER
C NPAR   INTG*4  NO. OF EXTRACTION PARAMETERS
C PAR    REAL*4  EXTRACTION PARAMETERS
C ND     INTG*4  NO. OF COEFFS.
C DPAR   REAL*8  COEFFS. TO DEFINE ORDER POSITION
C XMIN   REAL*4  MINIMUM VALUE
C XMAX   REAL*4  MAXIMUMVALUE
C
      IMPLICIT NONE
      INTEGER  NB1               !INTG*4  NO. OF SAMPLES
      INTEGER  NB2               !INTG*4 NO. OF ORDERS
      INTEGER  NPAR              !INTG*4  NO. OF EXTRACTION PARAMETERS
      INTEGER  NPIXA1            !INTG*4  NO. OF SAMPLES (X in frm size)
      INTEGER  NPIXA2            !INTG*4  NO. OF LINES   (Y in frm size)
      INTEGER  NWDTH
      INTEGER  ND                !INTG*4  NO. OF COEFFS
      
      
      REAL OUT(NB1,NB2)          !REAL*4  EXTRACTED ORDERS (output frm)
      REAL PAR(NPAR)             !REAL*4  EXTRACTION PARAMETERS
      
      
      REAL IN(NPIXA1,NPIXA2)     !REAL*4  INPUT IMAGE
      REAL COL(NWDTH)
      
      REAL XMIN                  !REAL*4  MINIMUM VALUE
      REAL XMAX                  !REAL*4  MAXIMUM VALUE
      
      
      DOUBLE PRECISION DPAR(ND)  !REAL*8  COEFFS. TO DEFINE ORDER POSITION
      DOUBLE PRECISION STARTA(2) !(MIDAS env) start values of input frm
      DOUBLE PRECISION STEPA(2)  !(MIDAS env) Step size of input frm
      
      DOUBLE PRECISION XVAL      !(MIDAS env) Holder for world coordinate X point
      DOUBLE PRECISION YVAL      !(MIDAS env) Holder for world coordinate Y point
      DOUBLE PRECISION AX        !AX=1.D0 + SKWOF ??
      
      DOUBLE PRECISION WIDTH     !WIDTH=PAR(1)
      DOUBLE PRECISION SKEW      !SKEW=PAR(2)
      DOUBLE PRECISION HWIDTH    !half of WIDTH
      DOUBLE PRECISION OFFSET    !OFFSET=PAR(3)
      DOUBLE PRECISION DC        !???
      
      DOUBLE PRECISION YV        !YV=PAR(10)
      DOUBLE PRECISION WW(50)    !??
      
      DOUBLE PRECISION SKWDX     !SKWDX  = -DSIN(DD)     !   DX PER PIXEL
      DOUBLE PRECISION SKWOF     !SKWOF  = -SKWDX*HWIDTH !   2D INTERPOLATION
      
      DOUBLE PRECISION DEGREE    !conversion factor from DEG to RAD
      DOUBLE PRECISION DD        !DD=SKEW*DEGREE
      DOUBLE PRECISION DSIN      !sinus of an argument as double precision function

      INTEGER          IWIDTH    !Integer part of WIDTH
      INTEGER          NX        !NX= SKWOF
      INTEGER          N1        !N1=NX
      INTEGER          N2        !N2= NPIXA1 - NX
      INTEGER          K         !K=PAR(8)
      INTEGER          L         !L=PAR(9)
      INTEGER          IX        !Loop variable
      INTEGER          INDX      !INTG*4  SEQUENTIAL NO. OF THE ORDER
      
      INTEGER          IX1       !Loop variable
      INTEGER          IP        !Tmp variable
      INTEGER          L1        !Loop variable
      INTEGER          K1        !Loop variable
      INTEGER          IW        !Loop variable
      
C     To Correct the central position for the offset as the OFFSET does not take into account HWIDTH 
      INTEGER          LOSLIT    !LOSLIT = INT(YLOW) 
      INTEGER          HISLIT    !YHIGH  = YVAL + HWIDTH
      
      DOUBLE PRECISION YLOW      !YLOW   = YVAL - HWIDTH
      DOUBLE PRECISION YHIGH     !YHIGH  = YVAL + HWIDTH
      
      CHARACTER*1      METH      !Extraction method
      
      DOUBLE PRECISION IX_R8R8   !(MIDAS env) conversion pixel to world coordinate
      DOUBLE PRECISION XI_R8R8   !(MIDAS env) conversion world to pixel coordinate
C
C     DEGREE = RAD / DEGREE
C
      DATA DEGREE/0.017453292519943295769237D+0/
C     

C
C ... define extraction constants
C

      WIDTH  = PAR(1)
      IWIDTH = WIDTH
      HWIDTH = 0.5D0*WIDTH
      SKEW   = PAR(2)
      SKWDX  = 0.0D0
      SKWOF  = 0.0D0
      XMIN   = 0.0
      XMAX   = XMIN
C      OFFSET = PAR(3) - HWIDTH + 0.5
      OFFSET = PAR(3)
      IF (SKEW.NE.0.0) THEN       
          DD      = SKEW*DEGREE
          SKWDX  = -DSIN(DD)      !   DX PER PIXEL
          SKWOF  = -SKWDX*HWIDTH  !   2D INTERPOLATION
      END IF
      AX     = 1.D0 + SKWOF
      NX     = SKWOF
      N1     = NX
      N2     = NPIXA1 - NX
C
C ... get ordinate values
C
      K      = PAR(8)
      L      = PAR(9)
      YV     = PAR(10)
C
C ... loop over the samples
C
      DO 10 IX = 1,N1
         OUT(IX,INDX) = 0.
 10   CONTINUE

      DO 100 IX = N2 + 1,NPIXA1
         OUT(IX,INDX) = 0.
 100  CONTINUE

      DO 50 IX1 = N1 + 1,N2

         IP     = 0
         DC     = 1.D0
         YVAL   = 0.D0
         XVAL   = IX_R8R8(AX,STARTA(1),STEPA(1))

C Computes position of the center of the slit from the coefficients of
C the bivariate polynomial stored in table order.

         DO 30 L1 = 0,L
            IP     = IP + 1
            WW(IP) = DC
            YVAL   = YVAL + WW(IP)*DPAR(IP)
            DO 20 K1 = 1,K
               IP     = IP + 1
               WW(IP) = WW(IP-1)*XVAL
               YVAL   = YVAL + WW(IP)*DPAR(IP)
 20         CONTINUE
            DC     = DC*YV
 30      CONTINUE
C
C Corrects the central position for the offset
C OFFSET does not take into account HWIDTH 
C
         YVAL   = XI_R8R8(YVAL,STARTA(2),STEPA(2)) + OFFSET
C     
         YLOW   = YVAL - HWIDTH
         YHIGH  = YVAL + HWIDTH
         LOSLIT = INT(YLOW)
         HISLIT = INT(YHIGH)

         IF (LOSLIT.LT.1)      LOSLIT = 1
         IF (HISLIT.GT.NPIXA2) HISLIT = NPIXA2

         IX     = INT(AX)
C
         IF (LOSLIT.EQ.HISLIT) THEN
            OUT(IX1,INDX) = IN(IX,LOSLIT)
         ELSE
            K1 = 0
            DO 40 IW = LOSLIT,HISLIT
               K1 = K1 + 1
               IF (IW.EQ.LOSLIT) THEN
                  COL(K1) = IN(IX,IW)!*(LOSLIT+1.-YLOW)
               ELSE IF (IW.EQ.HISLIT) THEN
                  COL(K1) = IN(IX,IW)!*MIN(1.0,YHIGH-HISLIT)
               ELSE
                  COL(K1) = IN(IX,IW)
               ENDIF
 40         CONTINUE            !determine MEDIAN:
            L1 = K1 / 2
            IF (K1.GT.0) THEN
               CALL GET_MEDIAN(COL,NWDTH,K1,L1,OUT(IX1,INDX),IW)
               IF (IW.LT.0) print *,'ERROR at ',IX1,INDX,' occured.'
            ELSE
               OUT(IX1,INDX) = -1.0
            ENDIF
         ENDIF
C
         XMIN   = AMIN1(XMIN,OUT(IX1,INDX))
         XMAX   = AMAX1(XMAX,OUT(IX1,INDX))
         AX     = AX + 1.D0
 50   CONTINUE

      RETURN

      END


      SUBROUTINE GET_MEDIAN(RA,N,NUSE,LL,VALU,STATUS)
C     Returns the median of a set of values
      IMPLICIT NONE
C 
      INTEGER   N
      INTEGER   LL
      INTEGER   STATUS
      
      INTEGER   NUSE
C 
      REAL      RA(N)
      REAL      VALU
C 
      VALU = 0.0
      STATUS = 0
C 
      IF (NUSE.LE.3) THEN
         IF (NUSE.LT.1) THEN
            STATUS = -1
            RETURN
         ELSE IF (NUSE.LE.2) THEN
            STATUS = 2
            VALU = RA(1)
c            VALU = (RA(1) + RA(2))/2.0               !for ndim = 1,2, median is 1st elem.
            RETURN
         ELSE 
            STATUS = 3
            VALU = RA(2)               !for ndim = 3, median is 2nd elem.
            RETURN
         ENDIF
      ENDIF
C 
      CALL NSORT(RA,NUSE,LL,VALU)
      RETURN
C 
      END
