C $Id: necmerge.for,v 1.7 2002/02/11 09:55:25 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.7 $
C $Log: necmerge.for,v $
C Revision 1.7  2002/02/11 09:55:25  amodigli
C initialize some variables to reduce purify warnings
C
C Revision 1.6  2002/01/14 15:05:48  amodigli
C DFS code rev step 1: defined var meaning
C
C Revision 1.5  2001/07/23 09:12:45  amodigli
C Eliminated some warinngs indicated by ftncheck
C
C Revision 1.4  2001/06/29 12:33:11  amodigli
C AMIN alwais REAL, STARTA,STEPA,WI
C always double
C also to not have warings on alpha
C
C Revision 1.3  2001/01/04 12:31:30  amodigli
C Added Id, Name, Revision, Log conf. control info
C
C------------------------------------------------------------------------------
      PROGRAM ECHMRG
C+
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  22:26 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR: D.PONZ
C
C
C.IDENTIFICATION
C
C  program  ECHMERG 
C
C.MODIFICATIONS
C
C  910128   P. Ballester   Define and set variables for mmake -i
C  991005   S.Wolf         ECHMR2O added: does an optimal merging.
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, ORDER MERGING
C
C.PURPOSE
C
C  OBTAIN A 1D FRM CALIBRATED IN WAVELENGTHS FROM A 2D FRM
C  SAMPLED IN THE SPACE WAVELENGTH-ORDER.
C
C.ALGORITHM
C
C  OVERLAPPING REGIONS OF CONSECUTIVE ORDERS ARE PROCESSED IN TWO WAYS:
C  - CONCATENATION OF CONSECUTIVE ORDERS, USING THE MIDDLE POINT AS
C    CONCATENATING POSITION (METHOD='CONCATENATE', DEFAULT)
C  - AVERAGE OF OVERLAPPING REGION (METHOD='AVERAGE')
C  - WEIGHTED AVERAGE OF OVERLAPPING REGION (METHOD='OPTIMAL')
C    spmerged = (w1*sp1 + w2*sp2)/(w1+w2)
C  - NO MERGING. INDIVIDUAL ORDERS ARE WRITTEN IN DIFFERENT FILES
C    (METHOD = 'NOMERGE')
C  - WEIGHTED AVERAGE. WEIGHTS PROPORTIONAL TO SINC**2 (METHOD='SINC')
C  THE METHOD 'NOMERGE' WILL PRODUCE AS MANY FILES AS DEFINED ORDERS
C   WITH NAMES xxxyyyy, xxxx IS THE OUTPUT FILE NAME, yyyy IS THE 
C   ORDER NUMBER
C
C.INPUT/OUTPUT
C
C  MERGE/ECHELLE INPUT OUTPUT  [CONC]
C  MERGE/ECHELLE INPUT OUTPUT  ORD1,ORD2 NOCONCAT
C  MERGE/ECHELLE INPUT OUTPUT  DELTA     AVERAGE
C  MERGE/ECHELLE INPUT OUTPUT  DELTA     OPTIMAL WEIGHTS
C  OR
C  MERGE/ECHELLE INPUT OUTPUT  TABLE SINC
C
C-
      IMPLICIT  NONE
      INTEGER   MADRID          !(MIDAS env) Memory Addres ID
      
      INTEGER   I               !Loop variable over order No.
      INTEGER   II1             !holder for out frm name size dimension
      INTEGER   II2             !II2    = 10000 + I !what is this???
      INTEGER   IORD1           !Minimum of order No
      INTEGER   IORD2           !Maximum of order No
      
      INTEGER*8 IPNTRA          !(MIDAS env) Pointer to mapped data (in frm)
      INTEGER*8 IPNTRB          !(MIDAS env) Pointer to mapped data (out frm)
      INTEGER*8 IPNTRV          !(MIDAS env) Pointer to mapped data (out var frm)
      INTEGER*8 IPNTRW          !(MIDAS env) Pointer to mapped data (in weight frm)
      
      INTEGER   ISTAT           !(MIDAS env) Aux var for return status of op.
      INTEGER   NAXISA          !(MIDAS env) Dimension of input frm
      INTEGER   NAXISB          !(MIDAS env) Dimension of output frm (=2)
      INTEGER   NAXISW          !(MIDAS env) Dimension of output weight frm (=2)
C     INTEGER   STATUS
      DOUBLE PRECISION    DEL   !WRANGE(1)
      DOUBLE PRECISION    WEND  !wavelength end 
      DOUBLE PRECISION    WINIT !wavelength start 
      DOUBLE precision    WSTEP !wavelength step
C
      INTEGER NPIXA(2)          !(MIDAS env) X size of input frm
      INTEGER NPIXB(2)          !(MIDAS env) X size of output frm
      INTEGER NPIXW(2)          !(MIDAS env) X size of input weight frm
      INTEGER NPTOT(100)        !Total number of point along X on (each) order
      INTEGER ICOL(3)           !Column of ripple table??
      INTEGER KUN               !(MIDAS env) Units of character keyword
      INTEGER KNUL              !(MIDAS env) No of null values of character keyword
      
      INTEGER INIMA             !(MIDAS env) File id of in frm
      INTEGER OUTIMA            !(MIDAS env) File id of out frm
      INTEGER VARIMA            !(MIDAS env) File id of out var frm
      INTEGER WGTIMA            !(MIDAS env) File id of in weight frm
      INTEGER TID               !(MIDAS env) File id of ripple table
      
      DOUBLE PRECISION  WSTART(100)  !Descriptor collecting start wav at each order
      DOUBLE PRECISION  STARTA(2)    !(MIDAS env) start values of input frm
      DOUBLE PRECISION  STEPA(2)     !(MIDAS env) Step size of input frm
      DOUBLE PRECISION  STARTB(2)    !(MIDAS env) start values of output frm
      DOUBLE PRECISION  STEPB(2)     !(MIDAS env) Step size of output frm
      
      DOUBLE PRECISION  STARTW(2)    !(MIDAS env) start values of input weight frm
      DOUBLE PRECISION  STEPW(2)     !(MIDAS env) Step size of input weight frm
      
      REAL CUT(4)                    !Ima cuts (Xmin,Xmax,Ymin,Ymax)
      REAL XMIN                      !Xmin out ima
      REAL XMAX                      !Xmax out ima
      REAL WRANGE(2)                 !Wavelength range. To be more precise ??
      REAL VAL(3)                    !(MIDAS env) actual values of ripple columns
      REAL K(100)                    !holder for values from ripple table. K(I)   = VAL(2)
      REAL A(100)                    !holder for values from ripple table. A(I)   = VAL(3)
      
      REAL RO(100)                   !??what is really this?
      
      CHARACTER*60 INFRM             !Char holder for "P1" char key for input frm name
      CHARACTER*60 OUTFRM            !Char holder for "P2" char key for input tab name
      CHARACTER*60 VARFRM            !Char holder for "P6" char key for input tab name
      CHARACTER*60 WGTFRM            !Char holder for "P5" char key for input tab name
      
      CHARACTER OUTFIL*12            !Char holder for actual output frm name
      CHARACTER TABLE*12             !Char holder for "P3" char key for input tab name

      CHARACTER METHOD*1             !Merging method
      CHARACTER WS*5                 !aux char variable to display current order No.
      
      CHARACTER IDENTA*72            !(MIDAS env) ASCII Identification of input frm
      CHARACTER IDENTB*72            !(MIDAS env) ASCII Identification of output frm
      CHARACTER CUNITA*64            !(MIDAS env) Units of each axis+data of input frm
      CHARACTER CUNITB*64            !(MIDAS env) Units of each axis+data of output frm
      
      CHARACTER IDENTW*72            !(MIDAS env) ASCII Identification of input weight frm
      CHARACTER CUNITW*64            !(MIDAS env) Units of each axis+data of input weight frm
      
      CHARACTER*16 LABEL(3)          !label for table column name
      LOGICAL NULL(3)                !(MIDAS env) No of null values of character keyword
C
      INCLUDE 'MID_INCLUDE:ST_DEF.INC'
      COMMON /VMR/MADRID(1)
      INCLUDE 'MID_INCLUDE:ST_DAT.INC'
C
      DATA CUNITB/'FLUX            WAVELENGTH'/
      DATA LABEL(1)/'ORDER'/,LABEL(2)/'KFIT'/,LABEL(3)/'AFIT'/
C
C ... INITIALIZE SYSTEM
C

C     initialize local variables

C     integers
      I=0               !Loop variable over order No.
      II1=0             !holder for out frm name size dimension
      II2=0             !II2    = 10000 + I !what is this???
      IORD1=0           !Minimum of order No
      IORD2=0           !Maximum of order No
      
      ISTAT=0           !(MIDAS env) Aux var for return status of op.
      NAXISA=0          !(MIDAS env) Dimension of input frm
      NAXISB=0          !(MIDAS env) Dimension of output frm (=2)
      NAXISW=0          !(MIDAS env) Dimension of output weight frm (=2)
            
C     reals

      CUT(1)=0
      CUT(2)=0
      CUT(3)=0
      CUT(4)=0  !Ima cuts (Xmin,Xmax,Ymin,Ymax)
      XMIN=0                      !Xmin out ima
      XMAX=0                      !Xmax out ima
      WRANGE(1)=0
      WRANGE(2)=0       !Wavelength range. To be more precise ??
      VAL(1)=0
      VAL(2)=0
      VAL(3)=0      !(MIDAS env) actual values of ripple columns
      DO I=1,100
         K(I)=0                    !holder for values from ripple table. K(I)   = VAL(2)
         A(I)=0                    !holder for values from ripple table. A(I)   = VAL(3)
         RO(I)=0                   !??what is really this?
      ENDDO
      
 
C     doubles
      DEL=0             !WRANGE(1)
      WEND=0            !wavelength end 
      WINIT=0           !wavelength start 
      WSTEP=0           !wavelength step
      STARTW(2)=0       !(MIDAS env) start values of input weight frm
      STEPW(2)=0        !(MIDAS env) Step size of input weight frm

C     Chars
      INFRM(1:) = ' '             !Char holder for "P1" char key for input frm name
      OUTFRM(1:) = ' '            !Char holder for "P2" char key for input tab name
      VARFRM(1:) = ' '            !Char holder for "P6" char key for input tab name
      WGTFRM(1:) = ' '            !Char holder for "P5" char key for input tab name
      
      OUTFIL(1:) = ' '            !Char holder for actual output frm name
      TABLE(1:) = ' '             !Char holder for "P3" char key for input tab name

      METHOD(1:) = ' '             !Merging method
      WS(1:) = ' '                 !aux char variable to display current order No.
      
      IDENTA(1:) = ' '            !(MIDAS env) ASCII Identification of input frm
      IDENTB(1:) = ' '            !(MIDAS env) ASCII Identification of output frm
      CUNITA(1:) = ' '            !(MIDAS env) Units of each axis+data of input frm
     
      IDENTW(1:) = ' '            !(MIDAS env) ASCII Identification of input weight frm
      CUNITW(1:) = ' '            !(MIDAS env) Units of each axis+data of input weight frm
      
            

      CALL STSPRO('ECHMRG')
      INFRM = ' '
      OUTFRM = ' '
      VARFRM = ' '
      WGTFRM = ' '
      CALL STKRDC('P1',1,1,60,I,INFRM,KUN,KNUL,ISTAT)
      CALL STKRDC('P2',1,1,60,I,OUTFRM,KUN,KNUL,ISTAT)
      CALL STKRDC('P6',1,1,60,I,VARFRM,KUN,KNUL,ISTAT)
      CALL STKRDC('P5',1,1,60,I,WGTFRM,KUN,KNUL,ISTAT)
      CALL CLNFRA(INFRM,INFRM,0)
      CALL CLNFRA(OUTFRM,OUTFRM,0)
      CALL CLNFRA(VARFRM,VARFRM,0)
      CALL CLNFRA(WGTFRM,WGTFRM,0)
      CALL STKRDC('P4',1,1,1,I,METHOD,KUN,KNUL,ISTAT)
      CALL FORUPC(METHOD,METHOD)
      IF (METHOD.EQ.'S') 
     .    CALL STKRDC('P3',1,1,8,I,TABLE,KUN,KNUL,ISTAT)
      CALL STKRDR('INPUTR',1,2,I,WRANGE,KUN,KNUL,ISTAT)
      IF (METHOD.EQ.'A' .OR. METHOD.EQ.'O') THEN
          WINIT  = 0.D0
          WEND   = WINIT
          DEL    = WRANGE(1)
      ELSE
          WINIT  = WRANGE(1)
          WEND   = WRANGE(2)
      END IF
C
C ... MAP INPUT FRM
C
      CALL STIGET(INFRM,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,
     .            3,NAXISA,NPIXA,STARTA,STEPA,IDENTA,CUNITA,
     .            IPNTRA,INIMA,ISTAT)
      CALL STDRDD(INIMA,'WSTART',1,NPIXA(2),I,
     .            WSTART,KUN,KNUL,ISTAT)
      CALL STDRDI(INIMA,'NPTOT',1,NPIXA(2),I,NPTOT,
     .            KUN,KNUL,ISTAT)
C ... CHECK IF NPTOT(NPIXA(2)) IS 0 
c     the following may happen if the order hits the detector's border
5     IF (NPTOT(NPIXA(2)).EQ.0) THEN  
          NPIXA(2) = NPIXA(2) - 1
          GOTO 5
      ELSE
         CONTINUE          
      END IF
      WSTEP  = STEPA(1)
C
C ... MAP WEIGHT FRM
C
      IF (METHOD.EQ.'O') THEN
         CALL STIGET(WGTFRM,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,
     .        3,NAXISW,NPIXW,STARTW,STEPW,IDENTW,CUNITW,
     .        IPNTRW,WGTIMA,ISTAT)
         IF (NAXISW.NE.NAXISA.OR.
     .        STARTW(1).NE.STARTA(1).OR.STARTW(2).NE.STARTA(2).OR.
     .        STEPW(1).NE.STEPA(1).OR.STEPW(1).NE.STEPA(1)) THEN
            GOTO 9998
         END IF
      END IF
      IF (METHOD.EQ.'N') THEN
C
C ... NO MERGE - PRODUCE ONE FILE PER ORDER
C
          IF (WINIT.EQ.WEND .AND. WINIT.LT.0.5) THEN
              IORD1  = 1
              IORD2  = NPIXA(2)
          ELSE
              IORD1  = MAX(NINT(WINIT),1)
              IORD2  = MIN(NINT(WEND),NPIXA(2))
          END IF
          OUTFRM = OUTFRM(1:59)//' '
          II1    = INDEX(OUTFRM,' ') - 1
          II1    = MIN(II1,4)
          OUTFIL = OUTFRM(1:II1)
          DO 10 I = IORD1,IORD2
              II2    = 10000 + I
              WRITE (WS,9000) II2
              OUTFIL(II1+1:II1+4) = WS(2:5)
              NAXISB    = 1
              NPIXB(1)  = NPTOT(I)
              NPIXB(2)  = 1
              STARTB(1) = WSTART(I)
              STARTB(2) = 1.D0
              STEPB(1)  = WSTEP
              STEPB(2)  = 0.D0
              WRITE (IDENTB,9010) I
              IDENTB(11:72) = IDENTA(1:62)
              CALL STIPUT(OUTFIL,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     .                    NAXISB,NPIXB,STARTB,STEPB,
     .                     IDENTB,CUNITB,IPNTRB,OUTIMA,ISTAT)
              CALL COPY(MADRID(IPNTRA),NPIXA(1),NPIXA(2),MADRID(IPNTRB),
     .                  NPIXB(1),I,XMIN,XMAX)
              CUT(1) = XMIN
              CUT(2) = XMAX
              CUT(3) = XMIN
              CUT(4) = XMAX
              CALL DSCUPT(INIMA,OUTIMA,' ',ISTAT)
              CALL STDWRR(OUTIMA,'LHCUTS',CUT,1,4,KUN,ISTAT)
C              CALL SXDFND(OUTFIL,'WSTART',TYPE,NOELEM,IBYTEL,STATUS)
C              DESNAM = 'WSTART'
C              IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFIL,DESNAM,STATUS)
C              CALL SXDFND(OUTFIL,'NPTOT',TYPE,NOELEM,IBYTEL,STATUS)
C              DESNAM = 'NPTOT'
C              IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFIL,DESNAM,STATUS)
C              CALL SXDFND(OUTFIL,'NORDER',TYPE,NOELEM,IBYTEL,STATUS)
C              DESNAM = 'NORDER'
C              IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFIL,DESNAM,STATUS)
C
              CALL STFCLO(OUTIMA,ISTAT)
              CALL STTPUT('File '//OUTFIL//' created ...',ISTAT)
   10     CONTINUE

      ELSE
C
C ... MAP OUTPUT FRAME
C
          IF (WINIT.EQ.WEND) THEN
              WINIT  = WSTART(1)
              WEND   = WSTART(NPIXA(2)) + (NPTOT(NPIXA(2))-1)*WSTEP
c             the following may happen if the order hits the detector's border              
              if (WEND .LT. 0.) THEN
                 NPIXA(2)=NPIXA(2)-1
                 WINIT  = WSTART(1)
                 WEND   = WSTART(NPIXA(2)) + (NPTOT(NPIXA(2))-1)*WSTEP
              endif
          END IF
          NAXISB    = 1
          NPIXB(1)  = (WEND-WINIT)/WSTEP + 1
          NPIXB(2)  = 1
          STARTB(1) = WINIT
          STARTB(2) = 1.D0
          STEPB(1)  = WSTEP
          STEPB(2)  = 0.D0
          CALL STIPUT(OUTFRM,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     .                    NAXISB,NPIXB,STARTB,STEPB,
     .                    IDENTA,CUNITB,IPNTRB,OUTIMA,ISTAT)
C
C ... METHODS OF OVERLAPPING
C
          IF (METHOD.EQ.'C') CALL ECHMR1(MADRID(IPNTRA),
     +                            NPIXA(1),NPIXA(2),STARTA,
     +                            STEPA,WSTART,NPTOT,MADRID(IPNTRB),
     +                            NPIXB,STARTB,XMIN,XMAX)
          IF (METHOD.EQ.'A') CALL ECHMR2(MADRID(IPNTRA),
     +                            NPIXA(1),NPIXA(2),STARTA,
     +                            STEPA,WSTART,NPTOT,MADRID(IPNTRB),
     +                            NPIXB,STARTB,XMIN,XMAX,DEL)
          IF (METHOD.EQ.'O') THEN
             CALL STIPUT(VARFRM,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     .            NAXISB,NPIXB,STARTB,STEPB,
     .            IDENTA,CUNITB,IPNTRV,VARIMA,ISTAT)
             CALL ECHMR2O(MADRID(IPNTRA),MADRID(IPNTRW),
     +            NPIXA(1),NPIXA(2),STARTA,
     +            STEPA,WSTART,NPTOT,MADRID(IPNTRB),MADRID(IPNTRV),
     +            NPIXB,STARTB,XMIN,XMAX,DEL)
             CALL DSCUPT(INIMA,VARIMA,' ',ISTAT)
                  
          ENDIF
          IF (METHOD.EQ.'S') THEN
C
C ... READ RIPPLE TABLE
C
              CALL TBTOPN(TABLE,F_IO_MODE,TID,ISTAT)
              CALL TBLSER(TID,LABEL(1),ICOL(1),ISTAT)
              CALL TBLSER(TID,LABEL(2),ICOL(2),ISTAT)
              CALL TBLSER(TID,LABEL(3),ICOL(3),ISTAT)
              DO 20 I = 1,NPIXA(2)
                  CALL TBRRDR(TID,I,3,ICOL,VAL,NULL,ISTAT)
                  RO(I)  = VAL(1)
                  K(I)   = VAL(2)
                  A(I)   = VAL(3)
   20         CONTINUE
              CALL ECHMR3(MADRID(IPNTRA),
     +                      NPIXA(1),NPIXA(2),STARTA,STEPA,WSTART,
     +                      NPTOT,MADRID(IPNTRB),NPIXB,STARTB,XMIN,XMAX,
     +                      RO,K,A)
          END IF
C
C ... WRITE PROCESS DESCRIPTORS
C
          CUT(1) = XMIN
          CUT(2) = XMAX
          CUT(3) = XMIN
          CUT(4) = XMAX
          CALL DSCUPT(INIMA,OUTIMA,' ',ISTAT)
          CALL STDWRR(OUTIMA,'LHCUTS',CUT,1,4,KUN,ISTAT)
C          CALL STTPUT(' Warning: modified code ECHMERG',STATUS)
C          CALL SXDFND(OUTFRM,'WSTART',TYPE,NOELEM,IBYTEL,STATUS)
C          IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFRM,'WSTART',STATUS)
C          CALL SXDFND(OUTFRM,'NPTOT',TYPE,NOELEM,IBYTEL,STATUS)
C          IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFRM,'NPTOT',STATUS)
C          CALL SXDFND(OUTFRM,'NORDER',TYPE,NOELEM,IBYTEL,STATUS)
C          IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFRM,'NORDER',STATUS)
      END IF
      GOTO 9999
 9998 CALL STTPUT
     $     ('Error: Bad weight map! '/
     $     /'(START, STEP and NAXIS same as in INPUT frame !?)',ISTAT)
 9999 CALL STSEPI
 9000 FORMAT (I5)
 9010 FORMAT ('ORDER:',I3)
      END

      SUBROUTINE ECHMR1(X,NPIXA1,NPIXA2,STARTA,STEPA,WI,NP,Y,NY,YSTR,
     +                  XMIN,XMAX)
C
C MERGE THE ORDERS, USING SIMPLE CONCATENATION IN THE MIDDLE POINT
      IMPLICIT NONE
C
      INTEGER  NY                  !(MIDAS env) X size of output frm
      INTEGER  J2                  !J2=NINT((WEND-WI(I))/YSTEP) + 1         ! actual no of wav sampl points
      INTEGER  I                   !Aux loop variable to iterate over orders
      INTEGER  J1                  !J1=NINT((WSTART-WI(I))/YSTEP) + 1       ! actual no of wav sampl points
      INTEGER  JOFF                !JOFF=NINT((WI(I)-YSTR)/YSTEP)           ! actual off in wav sampl points units
      INTEGER  J                   !Aux variable to iterate over orders
      INTEGER  JJ                  !aux variable: JJ=J+JOFF
      
      REAL     XMIN                !Xmin out ima
      REAL     XMAX                !Xmax out ima
      REAL     WEN1                !current Pixel end wavelenght
      REAL     WST2                !current Pixel start wavelenght
      REAL     WSTART              !Order start wavelenght
      
      INTEGER  NPIXA1              !(MIDAS env) Dimension of input frm
      INTEGER  NPIXA2              !(MIDAS env) Dimension of output frm
      INTEGER  NP(NPIXA2)          !Total number of point along X on (each) order
      REAL     X(NPIXA1,NPIXA2)    !Input ima frm
      
      DOUBLE PRECISION STARTA(2)   !(MIDAS env) start values of input frm
      DOUBLE PRECISION STEPA(2)    !(MIDAS env) Step size of input frm
      DOUBLE PRECISION WI(NPIXA2)  !Starting wavelength at each order
      DOUBLE PRECISION YSTR        !Y start value for out frm
C 
      REAL     Y(NY)               !Y array for out frm
      REAL     AMAX1               !real function to return the max of 2 real values
      REAL     AMIN1               !real function to return the min of 2 real values
C 
      DOUBLE PRECISION WEND        !End wav at current pix
      DOUBLE PRECISION YSTEP       !Y step of out ima frm
      DOUBLE PRECISION YEND        !Y end at current pix
      DOUBLE PRECISION WSTR        !End wav at cur pix?
C
      XMIN   = 0.
      XMAX   = 0.
      DO 10 I = 1,NY
          Y(I)   = 0.
   10 CONTINUE
      YSTEP  = STEPA(1)
      YEND   = YSTR + (NY-1)*YSTEP
C
C ... ITERATION ON ORDERS
C
      WEND   = 0.D0
      DO 30 I = 1,NPIXA2
          WSTR   = DMAX1(WI(I),WEND+YSTEP)
          IF (I.EQ.NPIXA2) THEN
              WEND   = WI(I) + (NP(I)-1)*YSTEP
          ELSE
              WEN1   = WI(I) + (NP(I)-1)*YSTEP
              WST2   = WI(I+1)
              IF (WST2.LT.WEN1) THEN
                  WEND   = 0.5D0* (WEN1+WST2)
              ELSE
                  WEND   = WEN1
              END IF
          END IF

          IF (WSTR.GE.YEND) RETURN
          IF (WEND.GT.YSTR) THEN
C
C ... ITERATION ON WAVELENGTHS
C
              WSTART = DMAX1(YSTR,WSTR)
              WEND   = DMIN1(WEND,YEND)
              J1     = NINT((WSTART-WI(I))/YSTEP) + 1
              J2     = NINT((WEND-WI(I))/YSTEP) + 1
              JOFF   = NINT((WI(I)-YSTR)/YSTEP)
              DO 20 J = J1,J2
                  JJ     = J + JOFF
                  IF (JJ.GT.0) THEN
                      Y(JJ)  = X(J,I)
                      XMAX   = AMAX1(XMAX,Y(JJ))
                      XMIN   = AMIN1(XMIN,Y(JJ))
                  END IF

   20         CONTINUE
          END IF

   30 CONTINUE
      RETURN

      END

      SUBROUTINE ECHMR2(X,NPIXA1,NPIXA2,STARTA,STEPA,WI,NP,Y,NY,YSTR,
     +                  XMIN,XMAX,DEL)
C
C MERGE THE ORDERS, AVERAGE ON THE OVERLAPPING REGION
C
      IMPLICIT NONE
C
      INTEGER NPIXA1                !(MIDAS env) X Dimension of input frm
      INTEGER NPIXA2                !(MIDAS env) X Dimension of output frm
      INTEGER NY                    !(MIDAS env) X size of output frm
      
      INTEGER NP(NPIXA2)            !Total number of point along X on (each) order
      REAL X(NPIXA1,NPIXA2)         !Input ima frm
      DOUBLE PRECISION STARTA(2)    !(MIDAS env) start values of input frm
      DOUBLE PRECISION STEPA(2)     !(MIDAS env) Step size of input frm
      DOUBLE PRECISION YSTR         !Y start value for out frm
      DOUBLE PRECISION DEL          !WRANGE(1)
      DOUBLE PRECISION WI(NPIXA2)   !Starting wavelength at each order
      
      REAL Y(NY)                    !Y array for out frm
      REAL XMIN                     !Xmin out ima
      REAL XMAX                     !Xmax out ima
C
      INTEGER IORD1                 !Minimum of order No
      INTEGER IORD2                 !Maximum of order No
      INTEGER IPIX                  !Aux var to loop over orders
      INTEGER IPIX1                 !actual pix no relative to order start pix (curr ord)
      INTEGER IPIX2                 !actual pix no relative to order start pix (curr ord+1)
      
      DOUBLE PRECISION YSTEP        !Y step of out ima frm
      DOUBLE PRECISION W0           !WW0=I(IORD2) + DEL                       !order start wav value
      DOUBLE PRECISION W1           !W1=WI(IORD1) + (NP(IORD1)-1)*YSTEP - DEL !curr pix wav value
      DOUBLE PRECISION WL           !WL     = YSTR + (IPIX-1)*YSTEP           !lower lim of??
      DOUBLE PRECISION P1           !offset of current point within pix
      DOUBLE PRECISION P2           !??
      REAL AMAX1                    !real function to return the max of 2 real values
      REAL AMIN1                    !real function to return the min of 2 real values
C
      XMIN   = 0.
      XMAX   = 0.
      YSTEP  = STEPA(1)
      IORD1  = 1
      IORD2  = 2
      W0     = WI(IORD2) + DEL
      W1     = WI(IORD1) + (NP(IORD1)-1)*YSTEP - DEL
C
C ... ITERATION ON ORDERS
C
      DO 20 IPIX = 1,NY
          Y(IPIX) = 0.0
          WL     = YSTR + (IPIX-1)*YSTEP
          IF (WL.LT.W0) THEN
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              Y(IPIX) = X(IPIX1,IORD1)
              XMAX   = AMAX1(Y(IPIX),XMAX)
              XMIN   = AMIN1(Y(IPIX),XMIN)
          ELSE IF (WL.LT.W1) THEN
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              IPIX2  = NINT((WL-WI(IORD2))/YSTEP) + 1
              P2     = (WL-W0)/ (W1-W0)
              P1     = 1.D0 - P2
              IF (X(IPIX1,IORD1).LE.0.0) THEN
                  P2     = 1.D0
                  P1     = 0.D0
              END IF
              IF (X(IPIX2,IORD2).LE.0.0) THEN
                  P2     = 0.D0
                  P1     = 1.D0
              END IF
              Y(IPIX) = X(IPIX1,IORD1)*P1 + X(IPIX2,IORD2)*P2
              XMAX   = AMAX1(Y(IPIX),XMAX)
              XMIN   = AMIN1(Y(IPIX),XMIN)
          ELSE
              IORD1  = IORD1 + 1
              IF (IORD1.GT.NPIXA2) RETURN
              IORD2  = IORD2 + 1
              IF (IORD2.GT.NPIXA2) THEN
                  W0     = 1.D20
              ELSE
                  W0     = WI(IORD2) + DEL
              END IF
              W1     = WI(IORD1) + (NP(IORD1)-1)*YSTEP - DEL
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              Y(IPIX) = X(IPIX1,IORD1)
              XMAX   = AMAX1(Y(IPIX),XMAX)
              XMIN   = AMIN1(Y(IPIX),XMIN)
          END IF
   20 CONTINUE
      RETURN
      END

      SUBROUTINE ECHMR2O(X,WGT,NPIXA1,NPIXA2,STARTA,STEPA,WI,NP,Y,VAR,NY
     $     ,YSTR,XMIN,XMAX,DEL)
C
C MERGE THE ORDERS, AVERAGE ON THE OVERLAPPING REGION USING WEIGHTS
C
      IMPLICIT NONE
C
      INTEGER NPIXA1               !(MIDAS env) X Dimension of input frm
      INTEGER NPIXA2               !(MIDAS env) X Dimension of output frm
      INTEGER NY                   !(MIDAS env) X size of output frm
      
      INTEGER NP(NPIXA2)           !Total number of point along X on (each) order
      
      REAL X(NPIXA1,NPIXA2)        !Input ima frm
      REAL WGT(NPIXA1,NPIXA2)      !Input ima frm: weight
      
      DOUBLE PRECISION STARTA(2)   !(MIDAS env) start values of input frm
      DOUBLE PRECISION STEPA(2)    !(MIDAS env) Step size of input frm
      DOUBLE PRECISION YSTR        !Y start value for out frm
      DOUBLE PRECISION DEL         !WRANGE(1)
      DOUBLE PRECISION WI(NPIXA2)  !Starting wavelength at each order
      
      REAL Y(NY)                   !Y array for out frm
      REAL VAR(NY)                 !var array for out var frm
      REAL XMIN                    !Xmin out ima
      REAL XMAX                    !Xmax out ima
C
      INTEGER IORD1                !Minimum of order No
      INTEGER IORD2                !Maximum of order No
      INTEGER IPIX                 !Aux var to loop over orders
      INTEGER IPIX1                !actual pix no relative to order start pix (curr ord)
      INTEGER IPIX2                !actual pix no relative to order start pix (curr ord+1)
      INTEGER IBAD                 !counter for no of bad pix values
      
      
      DOUBLE PRECISION YSTEP       !Y step of out ima frm
      DOUBLE PRECISION WG1         !WG1 = WGT(IPIX1,IORD1)  !weight at (current order) pix point
      DOUBLE PRECISION WG2         !WG2 = WGT(IPIX2,IORD2)  !weight at (current order+1) pix point
      DOUBLE PRECISION W0          !WW0=I(IORD2) + DEL                       !order start wav value
      DOUBLE PRECISION W1          !W1=WI(IORD1) + (NP(IORD1)-1)*YSTEP - DEL !curr pix wav value
      DOUBLE PRECISION WL          !WL     = YSTR + (IPIX-1)*YSTEP           !lower lim of??
      
      REAL  AMAX1                  !real function to return the max of 2 real values
      REAL  AMIN1                  !real function to return the min of 2 real values
      
      CHARACTER*80 MES             !Aux char variable to display a message
C
      IBAD   = 0
      XMIN   = 0.
      XMAX   = 0.
      YSTEP  = STEPA(1)
      IORD1  = 1
      IORD2  = 2
      W0     = WI(IORD2) + DEL
      W1     = WI(IORD1) + (NP(IORD1)-1)*YSTEP - DEL
C
C ... ITERATION ON ORDERS
C
      DO 20 IPIX = 1,NY
          Y(IPIX) = 0.0
          VAR(IPIX) = 0.0
          WL     = YSTR + (IPIX-1)*YSTEP
          IF (WL.LT.W0) THEN
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              Y(IPIX) = X(IPIX1,IORD1)
              VAR(IPIX) = WGT(IPIX1,IORD1)
              IF (VAR(IPIX).NE.0.0) VAR(IPIX) = 1.0/VAR(IPIX)
              XMAX   = AMAX1(Y(IPIX),XMAX)
              XMIN   = AMIN1(Y(IPIX),XMIN)
          ELSE IF (WL.LT.W1) THEN
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              IPIX2  = NINT((WL-WI(IORD2))/YSTEP) + 1
              WG1 = WGT(IPIX1,IORD1)
              WG2 = WGT(IPIX2,IORD2)

              IF (WG1.LT.1e-10 .AND. WG2.LT.1e-10) THEN
                 Y(IPIX) = 0.0
                 VAR(IPIX) = 0.0
                 IBAD = IBAD + 1
              ELSE
                 VAR(IPIX) = 1.0D0/(WG1 + WG2)
                 Y(IPIX) = X(IPIX1,IORD1)*WG1 + X(IPIX2,IORD2)*WG2
                 Y(IPIX) = Y(IPIX) * VAR(IPIX)
                 XMAX   = AMAX1(Y(IPIX),XMAX)
                 XMIN   = AMIN1(Y(IPIX),XMIN)
              ENDIF
          ELSE
              IORD1  = IORD1 + 1
              IF (IORD1.GT.NPIXA2) GOTO 99
              IORD2  = IORD2 + 1
              IF (IORD2.GT.NPIXA2) THEN
                  W0     = 1.D20
              ELSE
                  W0     = WI(IORD2) + DEL
              END IF
              W1     = WI(IORD1) + (NP(IORD1)-1)*YSTEP - DEL
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              Y(IPIX) = X(IPIX1,IORD1)
              VAR(IPIX) = WGT(IPIX1,IORD1)
              IF (VAR(IPIX).NE.0.0) VAR(IPIX) = 1.0/VAR(IPIX)
              XMAX   = AMAX1(Y(IPIX),XMAX)
              XMIN   = AMIN1(Y(IPIX),XMIN)
          END IF
 20   CONTINUE
 99   IF (IBAD.GT.0) THEN
         WRITE(MES,*) IBAD,' undefined pixels ... set to 0.0!'
         CALL STTPUT(MES,IBAD)
      END IF
      RETURN
         
      END

      SUBROUTINE COPY(A,NA1,NA2,B,NB,I,XMIN,XMAX)
C
C COPY THE ORDER NUMBER I
C
      IMPLICIT NONE
      INTEGER NA1
      INTEGER NA2
      INTEGER NB
      INTEGER I
      INTEGER J     !Aux loop variable
      REAL A(NA1,NA2)
      REAL B(NB)
      REAL XMIN     !Xmin out ima
      REAL XMAX     !Xmax out ima
C
      XMIN = 0.
      XMAX = 0.
C
      DO 10 J = 1,NB
          B(J)   = A(J,I)
          XMIN   = AMIN1(B(J),XMIN)
          XMAX   = AMAX1(B(J),XMAX)
   10 CONTINUE
      RETURN
      END

      SUBROUTINE ECHMR3(X,NPIXA1,NPIXA2,STARTA,STEPA,WI,NP,Y,NY,YSTR,
     +                    XMIN,XMAX,RORDER,K,A)
C
C      IMPLICIT NONE
C MERGE THE ORDERS, AVERAGE ON THE OVERLAPPING REGION WITH
C WEIGHTS PROPORTIONAL TO THE SINC**2
C
      IMPLICIT NONE

      INTEGER NY                  !(MIDAS env) X size of output frm
      INTEGER IORD1               !Minimum of order No
      INTEGER IPIX                !Aux var to loop over orders
      INTEGER NF                  !Aux var to loop over out flux values (size of order overlap region)
      INTEGER IORD                !Aux var to loop over orders
      INTEGER IPIX1               !actual pix no relative to order start pix (curr ord)
      INTEGER I                   !Aux loop var
      INTEGER IFL                 !Aux loop var to make average of flux over overlap region
      INTEGER IORD2               !Maximum of order No
      
      DOUBLE PRECISION    YSTR    !Y start value for out frm
      
      REAL XMIN                   !Xmin out ima
      REAL XMAX                   !Xmax out ima
      REAL YSTEP                  !Y step of out ima frm
      REAL PI                     !used??
      REAL DW                     !used??
      REAL WL                     !WL     = YSTR + (IPIX-1)*YSTEP           !lower lim of??
      
      INTEGER NPIXA1              !(MIDAS env) X Dimension of input frm
      INTEGER NPIXA2              !(MIDAS env) X Dimension of output frm
      INTEGER NP(NPIXA2)          !Total number of point along X on (each) order
      REAL    X(NPIXA1,NPIXA2)    !Input ima frm
      
      DOUBLE PRECISION STARTA(2)  !(MIDAS env) start values of input frm
      DOUBLE PRECISION STEPA(2)   !(MIDAS env) step values of input frm
      
      REAL    Y(NY)               !Y array for out frm
      DOUBLE PRECISION WI(NPIXA2) !Starting wavelength at each order
      REAL RORDER(NPIXA2)         !??what is really this?
      REAL    K(NPIXA2)           !holder for values from ripple table. K(I)   = VAL(2)
      REAL    A(NPIXA2)           !holder for values from ripple table. A(I)   = VAL(3)
      DOUBLE PRECISION DK         !DK     = K(IORD)
      DOUBLE PRECISION DA         !DA     = A(IORD)
      DOUBLE PRECISION PA         !PA     = PI*DA
      DOUBLE PRECISION DM         !DM     = RORDER(IORD)
      DOUBLE PRECISION DC         !DC     = DM/DK
      DOUBLE PRECISION DX         !DX     = (PA*DM*DC)* (DW-1.D0/DC)
      DOUBLE PRECISION WEIGHT(3)  !weight at current X point in merging overlapping region
      DOUBLE PRECISION FLUX(3)    !flux at current X point in merging overlapping region
      DOUBLE PRECISION SW         !sum of weights
      DOUBLE PRECISION SF         !sum of fluxes
C
      XMIN   = 0.
      XMAX   = 0.
      PI     = 0.     !  To be updated (ULTRIX installation)
      DW     = 0.     !  To be updated (ULTRIX installation)
      YSTEP  = STEPA(1)
      IORD1  = 1
      IORD2  = MIN(IORD1+2,NPIXA2)
C
C ... ITERATION ON OUTPUT WAVELENGTHS
C
      DO 30 IPIX = 1,NY
          Y(IPIX) = 0.
          FLUX(1) = 0.D0
          FLUX(2) = 0.D0
          FLUX(3) = 0.D0
          WEIGHT(1) = 0.D0
          WEIGHT(2) = 0.D0
          WEIGHT(3) = 0.D0
          NF     = 0
          WL     = YSTR + (IPIX-1)*YSTEP
C
C ... ITERATION ON ORDERS
C
          DO 10 IORD = IORD1,IORD2
              IPIX1  = (WL-WI(IORD))/YSTEP + 1
              IF (IPIX1.GT.5 .AND. IPIX1.LT. (NP(IORD)-5)) THEN
C
C ... PIXEL IN THE ORDER RANGE
C
                  NF     = NF + 1
                  FLUX(NF) = X(IPIX1,IORD)
                  DK     = K(IORD)
                  DA     = A(IORD)
                  PA     = PI*DA
                  DM     = RORDER(IORD)
                  DC     = DM/DK
                  DX     = (PA*DM*DC)* (DW-1.D0/DC)
                  IF (DABS(DX).LT.1.D-10) THEN
                      WEIGHT(NF) = 1.D0

                  ELSE
                      WEIGHT(NF) = 1.D0/ (DSIN(DX)/DX)**4
                  END IF

              END IF

   10     CONTINUE
C
C ... AVERAGE OVER FLUXES
C
          IF (NF.GE.1) THEN
              SW     = 0.D0
              SF     = 0.D0
              DO 20 IFL = 1,NF
                  SW     = SW + WEIGHT(IFL)
                  SF     = SF + WEIGHT(IFL)*FLUX(IFL)
   20         CONTINUE
              Y(IPIX) = SF/SW
              XMIN   = MIN(XMIN,Y(IPIX))
              XMAX   = MAX(XMAX,Y(IPIX))
          END IF
C
C ... CHECK ON ORDER LIMITS
C
          IPIX1  = (WL-WI(IORD1))/YSTEP + 1
          IF (IPIX1.GE.NP(IORD1)-5) THEN
              IORD1  = IORD1 + 1
              IORD2  = MIN(IORD1+2,NPIXA2)
          END IF

          IF (IORD1.GT.NPIXA2) RETURN
   30 CONTINUE
      DO 40 I = IPIX + 1,NY
          Y(I)   = 0.
   40 CONTINUE
      RETURN

      END
