C $Id: necopt.for,v 1.76 2002/08/28 09:23:57 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)echopt.for	2.4 (ESO-IPG) 12/6/91 12:26:19
C+
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  program  ECHOPT  version 1.0 910120 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C 
C Extract orders from an echelle spectrum, the order position is defined by 
C the coefficients of a regression. The slit width and the orders to be 
C extracted are defined by the keyword INPUTI
C The flux is calculated by assigning to each pixel a weighting factor 
C proportional to the amount of signal in it.
C.ALGORITHM
C The algorithm is based on a paper of Koji Mukai ( 1990, Optimal Extraction of
C Cross-Dispersed Spectra, Pub. of Astr.Soc. of the Pacific,102:183-189)
C AND a paper of Keith Horne:
C (1986, Pub. of Astr.Soc. of the Pacific, 98:609-617)
C
C.SYNTAX
C   EXTRACT/ORDER input output  width,ord1,ord2 ron,g,sigma table coeffs
C
C  the connection file contains
C  IN_A      input FRM (has to be de-biased BEFORE)
C  OUTPUTC   variance image of input data (same dimension as input frame)
C  OUT_A     output FRM
C  OUT_B     weight map FRM
C  IN_B      table with positions of the orders
C  INPUTC    coeffs
C  INPUTI    slit,ord1,ord2, weight_determination_flag
C  INPUTR    ron,g,sigma (ron: readout-noise, g: inverse gain,
C                         sigma: threshold for cosmic ray rejection)
C
C.HISTORY:
C  1999.04.08-SW save weight map, make use of variances,
C                iterative cosmic ray rejection (negative sigma switches off)
C                calculates the variance of the extracted spectrum
C  1999.09.08-SW the variance of the extracted spectrum is calculated in
C                  any case of INPUTI(4).
C                bug fix: the variance has to be devided by the squared
C                  number of extracted lines (represented usually by the
C                  keyword SLIT) in order to be consistent with the
C                  extracted orders which are averaged data! The number of
C                  lines is specified by INPUTI(1).
C  1999.10.28-SW Do a gaussian fit to the profils (CAPROF(),
C                  G_PROF()).
C                Save the cosmic ray detections in a table if EXTSIGMA>0
C  1999.11.09-SW Prevent division by zero in CALCULx() and CALC_x().
C  1999.11.11-SW Profil normalization removed in VSAMPL() so that the
C                variances may be easily derived for the cosmic ray
C                rejection.
C  1999.12.07-SW Gaussian profil model implemented together with
C                S. Cristiani (scristia@eso.org)
C  1999.12.08-SW Virtual resampling of the variance frame. This enables
C                a better access to the variances in CAPROF() and G_PROF()
C                and they have not to be re-calculated everytime.
C                (necopt.prg should take account for the flatfielding
C                when the variance frame is determined!!)
C  1999.12.16-SC Made the gauss fit more robust, get initial position of the
C                the object reading MIDAS keyword OBJSET.
C  1999.12.19-SW removed the pause statement (not allowed within pipeline)
C  1999.12.20-SW do a parabola fit to the CHI2 to get error on amplitude
C                of the gaussian (= optimal extracted spectrum).
C  2000.02.14-AM Put a check on Imax=Imin condition (which verifies when
C                the center of one order has a distance from the detector's
C                y border less than half order's size) to skip to process
C                such an order
C  2000.03.03-AM-SC Fixed crash problem in optimal extraction; the variance
C                calculation was wrong. Now we calculate DELTA with a possibly
C                variable step, and calculate OUT and VOUT only if 
C                (DELTA !=0) .AND. (DELTA .LE. 0.9). It result a better
C                variance value and anyway we check it is .GT. 0 and .LE. e8
C                
C  2000.03.13-AM Cleaned
C  2000.03.17-AM Suppressed unnecessary Output
C  2000.04.17-AM patch to improove optimal extraction for high S/N (THRESH depends from S/N)
C  2000.05/18-AM S/N now calculated order per order and within an order column by column
C  2000.06.06-AM Added several comments to clarify
C  2000.06.27-AM Addec QC1 parameters MID_S_N_CENT, OBJ_POS_CENT
C  2000.06.30-AM Release to DFO
C  2000.08.04-AM Uniformed QC parameter naming conventions
C  2000.11.10-AM POLY_FITX to improove optimal extraction
C  2000.11.30-AM Added k-s clipping
C  2000.12.04-AM Added order_trace table to check optimaql extraction positions and POLFIT
C             (copy of LFIT from Numerical Recipes). LSFIT was wrong!!!
C  2000.12.15-AM Found stable set of numbers to have good extraction quality.
C  2000.12.18-AM Release to DFO
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.76 $
C $Log: necopt.for,v $
C Revision 1.76  2002/08/28 09:23:57  amodigli
C Scorporated caprof
C
C Revision 1.75  2002/08/28 07:39:05  amodigli
C Taken out g_prof
C
C Revision 1.74  2002/08/16 15:18:57  amodigli
C speed-up
C
C Revision 1.73  2002/06/05 14:06:46  amodigli
C Increased the weights for k-s thresholding
C of a factor 10 each to better extract some
C mid S/N test data.
C
C Revision 1.72  2002/06/03 15:53:53  amodigli
C Corrected Linear term from 0.05 to 0.01
C
C Revision 1.71  2002/05/30 06:09:13  amodigli
C Corrected some errors in OUT,VOUT,FWHM,S/N
C originally due to not to have properly taken
C a normlaized FGAUS function.
C
C Revision 1.70  2002/04/21 08:46:36  amodigli
C Typo on variance file name
C
C Revision 1.69  2002/02/11 10:58:10  amodigli
C Added LHCUTS
C
C Revision 1.68  2002/02/11 09:23:36  amodigli
C Added var image for sky image
C
C Revision 1.67  2002/02/07 15:54:00  amodigli
C Added D1COL in call to QCTSR otherwise the
C program get screwed up. This was a portability
C problem to Linux. The thing was that the calling
C routine was also initializing the variable
C
C Revision 1.66  2002/02/03 07:50:28  amodigli
C Eliminated unnecessary argument I_FILE in
C call to DMPQC1
C
C Revision 1.65  2002/01/31 14:16:38  amodigli
C The sky level should be integrated on the
C slit size=DIM
C
C Revision 1.64  2002/01/24 08:00:34  amodigli
C Eliminated check for error status as is not
C proper to do in MIDAS.
C STDWRI needs to dump 'string' not "string"!
C (this was a portability problem to Linux).
C
C Revision 1.63  2001/11/09 18:01:46  amodigli
C Created extra modules as part of DFS
C code rev step 2
C
C Revision 1.62  2001/11/08 15:39:41  amodigli
C Fixed some ftncheck warnings
C
C Revision 1.61  2001/10/23 15:54:28  amodigli
C First code splitting as for DFS code rev step2
C
C Revision 1.60  2001/10/23 09:37:58  amodigli
C Defined QCTSRC to compact the code
C
C Revision 1.59  2001/10/06 11:51:57  amodigli
C Created QCOTR1 to compact CAPROF subroutine
C
C Revision 1.58  2001/10/06 10:14:57  amodigli
C Defined  QCORTR to dump QC info on order-trace
C and compact the CAPROF subroutine
C
C Revision 1.57  2001/10/03 10:07:00  amodigli
C Defined QCORTR (QC ORDER TRACE) subroutine
C to compact and clarify code
C
C Revision 1.56  2001/10/03 08:59:59  amodigli
C Defined GETQCI (GET QC INFO) subroutine
C to better structure code
C
C Revision 1.55  2001/10/01 09:06:50  amodigli
C Filled CR table. Activated MYAVRG and MYSIGMA
C
C Revision 1.54  2001/09/22 14:45:11  amodigli
C Reduced max passed var to EXTOBJ to 25
C
C Revision 1.53  2001/09/22 13:30:13  amodigli
C isolated init of 2 debug tables
C
C Revision 1.52  2001/09/21 07:42:05  amodigli
C Reduced of 4 the number of variable passed
C to the main subroutines
C
C Revision 1.51  2001/09/21 07:07:30  amodigli
C started to use a common block to pass variables
C
C Revision 1.50  2001/09/20 16:59:37  amodigli
C Moved dump of qc1 to subroutine DMPQC1
C
C Revision 1.49  2001/09/20 15:28:39  amodigli
C Mooved write of ord-S/N-FWHM-obj pos in subroutine
C
C Revision 1.48  2001/08/08 13:16:44  amodigli
C Added again ESO prefix to QC key
C
C Revision 1.47  2001/07/15 14:14:48  amodigli
C Cleaned usinf GNU ftncheck
C
C Revision 1.46  2001/07/10 08:00:36  amodigli
C error-factor 2 in AMPERR was wrong   G_PROF-variance calculation
C
C Revision 1.45  2001/07/03 11:55:06  amodigli
C D1NAME,D2NAME char buffer now of dimension
C which is a multiple of 8 (portability to
C alpha). Added a few comments
C
C Revision 1.44  2001/06/29 12:36:45  amodigli
C Added BKG (sky) val,err
C Changed QC key name.
C An integer pointer was not *8
C
C Revision 1.43  2001/06/15 08:44:33  amodigli
C Changed QC naming conventions to be DICB
C compliant
C
C Revision 1.42  2001/06/07 08:52:03  amodigli
C Corrected some typo and some not proper
C initializations to fix portability problems
C to PC Linux
C
C Revision 1.41  2001/05/29 06:22:46  amodigli
C Corrected typo in IKIND info
C
C Revision 1.40  2001/05/28 16:24:26  amodigli
C Cleaned some more comments.
C
C Revision 1.39  2001/05/24 08:44:43  amodigli
C BINX and BINY must not be initialized to 0
C more than once!
C
C Revision 1.38  2001/05/23 14:58:56  amodigli
C Check error status in call to MIDAS env
C
C Revision 1.37  2001/05/22 17:48:32  amodigli
C Initialized variables to 0 or null string
C
C Revision 1.36  2001/05/22 17:46:38  amodigli
C Added some pseudocode
C
C Revision 1.35  2001/05/19 16:34:11  amodigli
C Further cleaning for DFS code rev step1
C
C Revision 1.34  2001/05/14 13:45:58  amodigli
C Initialized to blank string the buffers
C to fix a portability error to pc-linux of SB
C
C Revision 1.33  2001/05/10 13:10:52  amodigli
C Added comments on variables
C
C Revision 1.32  2001/03/16 09:32:58  amodigli
C Increased char size of LINE from 72 to 99
C to support long file names for input raw FITS
C data.
C
C Revision 1.31  2001/03/12 14:34:56  amodigli
C From MAX(arg1,0) to MAX(arg1,0.0001)
C to fix a portability error problem
C to DEC XP1000
C
C Revision 1.30  2001/02/05 07:41:33  amodigli
C Fixed some portability errors to ALPHA Linux
C due to:
C a) FWHM(N_CURR_ORD) non initialized to 0
C b) In case one order is 1 point in CALC_I
C a check more was necessary.
C
C Revision 1.29  2001/01/26 12:37:44  amodigli
C Added a dummy filling of displayed info
C (fwhm,S/N,obj pos) in case the order has
C only one point, for robustness.
C
C Revision 1.28  2001/01/24 08:29:39  amodigli
C IXWIND from 16 to 32 to speed-up and have
C less outliers
C Added switch to average fit in case even the
C linear is very strange
C threshold CHI of fit from 1.0 to 0.5
C thresholds on slope and curvature relaxed
C below, more strict above chi threshold.
C
C Revision 1.27  2001/01/18 14:18:37  amodigli
C As delivered to DFO.
C
C Revision 1.26  2001/01/17 15:58:06  amodigli
C Good Configuration with chunk=32
C and clean points t 2sigma in pos and fwhm
C and their errors
C
C Revision 1.25  2001/01/16 09:28:11  amodigli
C Displayed FWHM now right as average at order center
C Added removal of points outside 2 sigma from av pos
C Set checks on FWHM slope and curvature
C Fit uses errors equal to 1 (not weighted fit)
C
C Revision 1.24  2001/01/12 10:23:29  amodigli
C First version with also fwhm a parabola
C function of x (before was a constant)
C
C Revision 1.23  2001/01/11 15:07:12  amodigli
C Used TBRWRD and TBRWRR in place of TBEPUT
C to not to have problem with Linux and g77
C
C Revision 1.22  2001/01/09 08:01:34  amodigli
C Several small bug adjustment to allow
C compilation with Linux and g77
C debug table renamed order_trace
C debug2 table renamed xorder_trace
C
C Revision 1.21  2001/01/04 12:31:32  amodigli
C Added Id, Name, Revision, Log conf. control info
C
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     15/07/01       Cleaned usinf GNU ftncheck        all
C  AM     29/06/01       QC-added BKGVAL,BKGERR col        main,CAPROF
C                        to monitor Sky level
C  AM     15/06/01       QC-changed QC naming to be DICB   main
C                        compliant
C  AM     07/06/01       Corrected some typo and some not   main
C                        proper initializations to fix 
C                        portability problems to PC Linux
C  AM     28/05/01       Cleaned some more comments.        all
C  AM     23/05/01       Check error status in call to      all
C                        MIDAS env
C  AM     22/05/01       Initialized variables to 0 or      all
C                        null string
C  AM     22/05/01       added some pseudocode              all
C  AM     19/05/01       Further cleaning for DFS           all
C                        code rev step1
C  AM     14/05/01       Initialized to blank string the    main
C                        buffers to fix a portability error 
C                        to pc-linux of SB
C
C  AM     10/05/01       Added comments on variables        all
C  AM     08/05/01       clean-comment                      main
C  AM     16/08/02       Speed-up                           all

C------------------------------------------------------------------------------
C  Role: Initialize the Midas environment. Do In/Out. for each order do OBJ
C        or FF extraction. Close the environment.
C------------------------------------------------------------------------------
C  Pseudocode:
C    Initialize the system: 
C       define the program name,reads MIDAS descriptors
C       open input table and gets parameters
C       map variance frame
C       map input frame
C       get other data from input frame descriptors
C       crea ut variance
C       maps weight frame
C       init 'crm_' frame
C       maps output tables
C    display some info
C    DO I=ord_min,ord_max
C       if OBJ ima then
C              call EXTOBJ
C              display : Ord No., S/N, FWHM, Obj Pos, info
C       else then
C              call EXTFF
C       endif
C    ENDDO
C    DO I=ord_min,ord_max
C       store S/N, FWHM, Obj Pos info in out frm descriptors
C    ENDDO
C------------------------------------------------------------------------------
C Module dependances:                     
C                                            |->gauss_fitx (gauss_fit.c)
C                                            |->qcotr1.for->MIDAS
C                                            |->g_prof.for---------->|->MIDAS
C                                            |->myavrg(my_fun.for)   |->FGAUSS_C(gauss_fit.c)
C                                            |->mysigm(my_fun.for)   |->MIDAS
C                             |->caprof.for->|->polfit(gauss_fit.c)  |->getqci.for->MIDAS
C                             |              |->qcotro.for->MIDAS
C                             |              |->MIDAS
C                             |->cent.for
C              |->extobj.for->|->vsampl.for
C              |              |->MIDAS
C              |
C              |->extff.for-->|->calc_i.for
C              |              |->calc_p.for
C              |              |->cent.for
C              |              |->MIDAS
C              |              
C  echopt.for->|->qcordtra.for->MIDAS
C              |->dmpqc1.for  ->MIDAS
C              |->mnmx(necopt.for)
C              |->qctsr.for   ->MIDAS
C              |->MIDAS
C------------------------------------------------------------------------------


      PROGRAM ECHOPT

      IMPLICIT NONE

      INTEGER NDIM
      INTEGER N_MAX_ORD               !maximum expected order number 
      INTEGER NIRPAR                  !No of elements of int array IPAR
      INTEGER NPARAM                  !No of elements of array PARAMS
      INTEGER NDPAR                   !No of elements of array DPAR
      INTEGER NSIZIM                  !No of elements for ima descriptors
      INTEGER NSCAN                   !No of elements of SCAN array
      INTEGER LENFIL                  !Max len ofc File Name

      INTEGER ND1COL
      INTEGER ND2COL
      INTEGER NQC0                       !No of QC par dumped at order 0
      PARAMETER (NQC0=12)             !No of QC parameter dumped order 0
      PARAMETER (LENFIL=80)
      PARAMETER (NDIM=2)
      PARAMETER (ND1COL=18,ND2COL=4)
      PARAMETER (N_MAX_ORD=50)        !maximum expected order number 
                                      !To be changed accordingly in many places
                                      !(valid for UVES)
      PARAMETER (NIRPAR=10)           !No of elements of int array IPAR
      PARAMETER (NPARAM=12)           !No of elements of array PARAMS
      PARAMETER (NDPAR=40)            !No of elements of array DPAR
      PARAMETER (NSIZIM=3)            !No of elements for ima descriptors 
      PARAMETER (NSCAN=2)             !No of elements of SCAN array
     
 
      INTEGER IDQC0(NQC0)                  !aux variable to hold tab col ids

      INTEGER N_CURR_ORD              !Variable to store current order number
      INTEGER MADRID                  !Memory Addres Identification
      INTEGER NAC                     !No. of words alloc. per raw 
                                      !(MIDAS env. int.)

      INTEGER NAR                     !No. of raw allocated (MIDAS env int.)
      INTEGER IMNO                    !File id of frame (MIDAS env int.)
      INTEGER SCAN(NSCAN)             !Scan image descriptor
      INTEGER BINSIZE                 !bin size of virtual resampling
      
      INTEGER I                       !Auxilliary variable
      INTEGER IAV                     !Actual No. of vals read (MIDAS env int.)
      INTEGER INDX                    !Aux. variable for string manipulation
      INTEGER IORD                    !Auxilliary variable as order counter
      INTEGER IORD1                   !Minimum detected order value 
      INTEGER IORD2                   !Maximum detected order value
      INTEGER IORDER                  !Aux. variable as order counter=IORD
       
      INTEGER ICOL1                   !Column No.(MIDAS env int.)
      INTEGER ICOL2                   !Column No.(MIDAS env int.)
      
      INTEGER ISTAT                   !Status of operation (MIDAS env int.)
      INTEGER NA                      !(IPAR(3)+1)* (IPAR(4)+1)
      INTEGER NAXISA                  !No. of axis of Image (MIDAS env int.)
      INTEGER NAXISB                  !No. of axis of Image (MIDAS env int.)
      
      INTEGER NCOL                    !No. of def. cols of tab (MIDAS env int.)
      INTEGER NPAR                    !No. of parameters
      INTEGER NROW                    !No. of def. rows of tab (MIDAS env int.)
      INTEGER NSORT                   !Sorted columns of table (MIDAS env int.)
      
      INTEGER STATUS                  !Status of operation (MIDAS env int.)
      INTEGER TID                     !Table Id.(MIDAS env int.)
   
      INTEGER INO                     !File Id of frame (MIDAS env int.) 
                                      !(out ima)
      INTEGER FIDNO                   !File Id of frame (MIDAS env int.) 
                                      !(out ima)

      INTEGER INSKY                     !File Id of frame (MIDAS env int.) 
                                      !(out sky ima)
      INTEGER IVSKY                     !File Id of frame (MIDAS env int.) 
                                      !(out var sky ima)                                      
 
      INTEGER INVO                    !File Id of frame (MIDAS env int.) 
                                      !(out variance)
      
      INTEGER*8 PNTRA                 !Pointer to mapped data (MIDAS env int.)
      INTEGER*8 PNTRB                 !Pointer to mapped data (MIDAS env int.)
      INTEGER*8 PNTRC                 !Pointer to mapped data (MIDAS env int.)
      INTEGER*8 PNTRV                 !Pointer to mapped data (MIDAS env int.)
      INTEGER*8 PNTVO                 !Pointer to mapped data (MIDAS env int.)
      INTEGER*8 PNTRSK                 !Pointer to mapped data (MIDAS env int.)
      INTEGER*8 PNTRVS                 !Pointer to mapped data (MIDAS env int.)  
      
      INTEGER NPIXA(NSIZIM)           !No. of pix in ima (MIDAS env int.)
      INTEGER NPIXB(NSIZIM)           !No. of pix out ima (MIDAS env int.)
      INTEGER NPIXC(NSIZIM)           !No. of pix in ima (MIDAS env int.)
      
      INTEGER IPAR(NIRPAR)            !array for int. params (MIDAS env int.)
      INTEGER KUN                     !unit (MIDAS env int.)
      INTEGER KNUL                    !nul (MIDAS env int.)
      
      INTEGER IKIND                   !switch for even/odd pix No. slit width
      INTEGER IWIDTH                  !slit width in pix
      INTEGER DIMP                    !dimention of weight profile
      
      INTEGER NWY                     !NWY=DIMP*(IORD2 - IORD1 + 1)

      INTEGER WID                     !File id of frame (MIDAS env int.) 
                                      !for weight ima

      INTEGER VID                     !File id of frame (MIDAS env int.) 
 
      INTEGER STOCH                   !tmp variable to store a value
      INTEGER STOCH2                  !tmp variable to store a value
      INTEGER DTID1                   !Table id (MIDAS env int.)
      INTEGER DTID2                   !Table id (MIDAS env int.)
      INTEGER BINX                    !No. of pix per bin along X
      INTEGER BINY                    !No. of pix per bin along Y
      INTEGER DUN                     !Units (MIDAS env int.)
     
      INTEGER DETWGT                  !Switch to determine weights 
                                      !(1:determine 0: use weight map)
      INTEGER MTID                    !Table Id.(MIDAS env int.)                                      !for variance ima
      INTEGER CRMSKS                  !tmp variable to store a value                                      
      INTEGER MIDFLX                  !CR Mask col id for :FLUX 
      INTEGER MIDORD                  !CR Mask col id for :ORDER 
      INTEGER MIDX                    !CR Mask col id for :X
      INTEGER MIDY                    !CR Mask col id for :Y
       
      INTEGER I_FILE                  !Hold length of VARNAME file 
      INTEGER NAXISC                  !Number of axis of weight map


      INTEGER IDPCH  !
      INTEGER IDPF0  !
      INTEGER IDPF1  !
      INTEGER IDPER0 !
      INTEGER IDS0   !
      INTEGER IDS1   !
      INTEGER IDICH  !
      INTEGER IDMSK0 !
      INTEGER IDXCH0 !
      INTEGER IDORD  !
      INTEGER IDFWHM !
      INTEGER IDFWER !
      INTEGER IDDEG  !
      INTEGER IDCHI0 !
      INTEGER IDKS   !
      INTEGER IDBKGV !
      INTEGER IDBKGE !
      INTEGER IDMSK1 !

      INTEGER D1COL(ND1COL)
      INTEGER D2COL(ND2COL)


      REAL MID_S_N_CENT(N_MAX_ORD)    !Average S/N     at order center
      REAL OBJ_POS_CENT(N_MAX_ORD)    !Average OBJ pos at order center
      REAL FWHM(N_MAX_ORD)            !Average FWHM    at order center
  
      REAL  RPAR(NIRPAR)              !array for real params descriptor 
                                      !(MIDAS env int.)
      REAL PARAMS(NPARAM)             !real extraction parameters
      REAL OFFSET                     !offset
      REAL OBJSET                     !object offset
      REAL XMIN                       !max of ima
      REAL XMAX                       !max of ima
      REAL CUT(6)                     !LHCUTS

      DOUBLE PRECISION STEPA(NSIZIM)  !Step of input image
      DOUBLE PRECISION STEPB(NSIZIM)  !Step of output image
      DOUBLE PRECISION STARTA(NSIZIM) !Start of input image
      DOUBLE PRECISION STARTB(NSIZIM) !Start of output image
      DOUBLE PRECISION DPAR(NDPAR)    !array containing order polynomial coeff.

      CHARACTER*72 IDENA              !ASCII identification for frame
      CHARACTER*72 IDENB              !ASCII identification for frame
      CHARACTER*96 LINE               !Buffer to display informations
      
      CHARACTER*64 CUNITA             !Unit of each axis + data 
                                      !(MIDAS env int.)

      CHARACTER*64 CUNITB             !Unit of each axis + data 
                                      !(MIDAS env int.)

      CHARACTER*64 DESNAM             !char descriptor
      CHARACTER*64 D1NAME(ND1COL)     !Descriptor name
      CHARACTER*64 D2NAME(ND2COL)     !Descriptor name
      
C     NB LENFIL=80

      CHARACTER*80 FRMIN              !Input frame
      CHARACTER*84 FRMOUT             !Output frame
      CHARACTER*88 FRMVOUT            !output variance frame
      CHARACTER*92 FRMSKY             !output sky frame
      CHARACTER*92 FRVSKY            !output variance sky frame      
      CHARACTER*80 FRMWGT             !weight image frame
      CHARACTER*80 FRMVAR             !input variance frame
      
      CHARACTER*80 INA1           !buffer for data values (MIDAS env int.)
      CHARACTER*80 INA2           !buffer for data values (MIDAS env int.)
      CHARACTER*80 INA3           !buffer for data values (MIDAS env int.)
      CHARACTER*80 INA4           !buffer for data values (MIDAS env int.)
      CHARACTER*80 TABLE          !buffer for data values (MIDAS env int.)
      
     
      CHARACTER*8  NAME               !buffer for data values (MIDAS env int.)
      CHARACTER*8  TYPE               !buffer for data values (MIDAS env int.)
      CHARACTER    DBG_SW             !switch to dump extra QC info
    
      
      EXTERNAL EXTFF                  !Subroutine to extract FF
      EXTERNAL EXTOBJ                 !Subroutine to extract the OBJ
      EXTERNAL DMPQC1                 !Subroutine to dump QC1 Info

      EXTERNAL QCTSR
      EXTERNAL CAPROF
     
C      COMMON /DBGORT/DTID1,DTID2         !to reduce par in subroutine call
      COMMON /DBGPOS/STOCH,STOCH2        !to reduce par in subroutine call                                 

      COMMON /FIT/BINX,BINY              !Necessary to set tresholds
                                         !over polynomial fit slope and 
                                         !curvature BINX,BINY dependent

      COMMON /DBG/IORDER                 !Used for debugging purposes?

      COMMON /QC0NM/D1NAME,D2NAME,D1COL,D2COL,IDQC0
      INCLUDE 'MID_INCLUDE:ST_DEF.INC'   !to get the definitions 
                                         !on the ST interfaces
      INCLUDE '../incl/common_qc_log.inc' 
      INCLUDE '../incl/common_crmask.inc'  
      INCLUDE '../incl/common_dbgort.inc'  
      INCLUDE '../incl/common_vmr.inc'  
      INCLUDE '../incl/common_qc.inc'
                                  

      INCLUDE 'MID_INCLUDE:ST_DAT.INC'   !to get data values for 
                                         !definitions above
 
                                      


      DATA CUNITB/
     +     'FLUX            PIXEL                           ORDER'/

C     Initializes all int variables to 0 to be sure...

 
 
 
      N_CURR_ORD  =0
      I_FILE      =0
      MIDORD   = 0
      MIDX     = 0
      MIDY     = 0
      MIDFLX   = 0
C      MADRID      =0

      DO I=1,NSCAN
         SCAN(I)=0
      ENDDO

      DO I=1,NSIZIM
         NPIXA(I)=0
         NPIXB(I)=0
         NPIXC(I)=0
         SCAN(I)=0
      ENDDO

      DO I=1,NIRPAR 
         IPAR(I)=0
         RPAR(I)=0.0
      ENDDO

      FIDNO       =0
      NAC         =0 

      NAR         =0
      IMNO        =0

      BINSIZE     =0
      
      I           =0
      IAV         =0
      INDX        =0
      IORD        =0
      IORD1       =0 
      IORD2       =0
      IORDER      =0
       
      ICOL1       =0
      ICOL2       =0
      
      ISTAT       =0
      NA          =0
      NAXISA      =0
      NAXISB      =0
      
      NCOL        =0
      NPAR        =0
      NROW        =0
      NSORT       =0
      
      STATUS      =0
      TID         =0
      MTID        =0
      INO         =0
                           
      INVO        =0
      INSKY=0
      IVSKY=0
                  
C      To not have portability problems to PC-Linux,
C      better not initialize pointers PNTR A,B,C,V,O                                 
 
      
      KUN        =0
      KNUL       =0 
      
      IKIND      =0
      IWIDTH     =0
      DIMP       =0
      
      NWY        =0

      WID        =0 
                           

      VID        =0 
      CRMSKS     =0                                 
      STOCH      =0
      STOCH2     =0
      DTID1      =0
      DTID2      =0
      BINX       =0
      BINY       =0
      DUN        =0
      DETWGT     =0
      NAXISC     =0

      IDPCH =0 !
      IDPF0 =0 !
      IDPF1 =0 !
      IDPER0=0 !
      IDS0  =0 !
      IDS1  =0 !
      IDICH =0 !
      IDMSK0=0 !
      IDXCH0=0 !
      IDORD =0 !
      IDFWHM=0 !
      IDFWER=0 !
      IDDEG =0 !
      IDCHI0=0 !
      IDKS  =0 !
      IDBKGV=0 !
      IDBKGE=0 !
      IDMSK1=0 !
      DO I=1,ND1COL           
         D1COL(I)=0
      ENDDO
 
      DO I=1,ND2COL           
         D2COL(I)=0
      ENDDO



C     Initializes all real variables to 0 to be sure...
  
      DO I=1,N_MAX_ORD
         MID_S_N_CENT(I) =0.0    
         OBJ_POS_CENT(I) =0.0  
         FWHM(I)         =0.0
      ENDDO

      DO I=1,NPARAM
         PARAMS(I)=0.0
      ENDDO 
      OFFSET           =0.0    
      OBJSET           =0.0    
      XMIN = 0
      XMAX = 0
     
      DO I=1,NQC0
         IDQC0(I)=0
      ENDDO
      DO I=1,6
         CUT(I)=0
      ENDDO

C     Initializes all double precision variables to 0 to be sure...

       DO I=1,NDPAR
         DPAR(I)=0.0D0
       ENDDO

       DO I=1,NSIZIM
          STEPA(I)=0.0D0
          STEPB(I)=0.0D0
          STARTA(I)=0.0D0
          STARTB(I)=0.0D0 
       ENDDO


C     Initializes all char variables to null string to be sure...

      IDENA(1:) = ' '
      IDENB(1:) = ' '
      LINE(1:) = ' '
      CUNITA(1:) = ' '
C      CUNITB(1:) = ' '    !CUNITB is initialized by a DATA statement
      DESNAM(1:) = ' '
      FRMIN(1:) = ' '
      FRMOUT(1:) = ' '
      FRMVOUT(1:)= ' '
      FRMSKY(1:)= ' '
      FRVSKY(1:)= ' '
      FRMWGT(1:) = ' '
      FRMVAR(1:) = ' '
      INA1(1:)   = ' '
      INA2(1:)   = ' '
      INA3(1:)   = ' '
      TABLE(1:)  = ' '
     
      NAME(1:) = ' '
      TYPE(1:) = ' '
      DBG_SW(1:) = ' '
c      D1NAME(1:) = ' '
c      D2NAME(1:) = ' '




C
C  initialize system
C

C STSPRO = ST (F77) system: get into MIDAS environment
C          read key DB, init file control table and open logfile

C STKRDC = STK (F77) module: ReaD Character key
C STKRDI = STK (F77) module: ReaD Integer key
C STKRDR = STK (F77) module: ReaD Real key

C IN_A is the input frame
C IN_B is the table with the positions of the orders
C OUT_A is the output frame
C OUT_B is the weight map frame
C OUTPUTC is the variance image of input data
C INPUTC are the coefficients
C INPUTR are ron,gain,sigma
C INPUTI is slit,ord1,ord2


      CALL STSPRO('ECHOPT')              !set up MIDAS environment
                                         !+enamble automatic error abort
                                         
      CALL STKRDC('IN_A',1,1,LENFIL,I,INA1,KUN,KNUL,STATUS)  !get in frm name
    
      CALL STKRDC('IN_B',1,1,LENFIL,I,TABLE,KUN,KNUL,STATUS) !get in tab name
    
      CALL STKRDC('OUT_A',1,1,LENFIL,I,INA2,KUN,KNUL,STATUS) !set out frm name
    
      CALL STKRDC('OUT_B',1,1,LENFIL,I,INA3,KUN,KNUL,STATUS) !set out weight map 
                                                         !frm
    
      CALL STKRDC('OUTPUTC',1,1,LENFIL,I,INA4,KUN,KNUL,STATUS) !set out var im frm
    
      CALL STKRDC('INPUTC',1,1,8,I,NAME,KUN,KNUL,STATUS) !get in coeff
    
      CALL STKRDR('INPUTR',1,3,I,RPAR,KUN,KNUL,STATUS)   !get in ron,gain,sigma
    
      CALL STKRDI('INPUTI',1,4,I,IPAR,KUN,KNUL,STATUS)   !get in slit,ord1,ord2
    
      CALL STKRDI('SCAN',1,2,I,SCAN,KUN,KNUL,STATUS)     !get in scan
    
      CALL STKRDR('OFFSET',1,1,I,OFFSET,KUN,KNUL,STATUS) !get in offset
    
      CALL STKRDR('OBJSET',1,1,I,OBJSET,KUN,KNUL,STATUS) !get in objset
      CALL STKRDI('BINX',1,1,I,BINX,KUN,KNUL,STATUS)     !get in binx (used to
                                                         !set curvature limits)
      
      CALL STKRDI('BINY',1,1,I,BINY,KUN,KNUL,STATUS)     !get in biny (used to
                                                         !set curvature limits)
      CALL STKRDC('UVES_DEBUG',1,1,1,I,DBG_SW,KUN,KNUL,STATUS) !DEBUG SWITCH


C      CALL STKRDI('BINSIZE',1,1,I,BINSIZE,N1,N1,N1)

      BINSIZE =  10              !binsize for virtual resampling VSAMPL().
                                 !now 10 but when used is re-set to 1 !

      IORD1  = IPAR(2)           !starting detected order number
      IORD2  = IPAR(3)           !ending detected order number
      DETWGT = IPAR(4)           !1: determine 0: use weight map
      
C CLNFRA stand this for Call Link Frame?
      
      CALL CLNFRA(INA1,FRMIN,0)  !input frame (pixel-pixel-space)
      CALL CLNFRA(INA2,FRMOUT,0) !output frame (extracted orders)
      CALL CLNFRA(INA3,FRMWGT,0) !weight map (pixel-full_order-space)
      CALL CLNFRA(INA4,FRMVAR,0) !variance frame (pixel-pixel-space)
C 180102 Klaus suggest to eliminate CLNTAB:
C      CALL CLNTAB(TABLE,TABLE,0) !order table (order positions)

C
C  input parameters in table
C

      CALL TBTOPN(TABLE,F_U_MODE,TID,STATUS)          !(F77) open table

      CALL TBIGET(TID,NCOL,NROW,NSORT,NAC,NAR,STATUS) !(F77) read size params
      

C
C  map variance frame:
C  map the complete data frame for reading or updating and 
C  read the standard ard descriptors
C
      CALL STIGET(FRMVAR,D_R4_FORMAT,F_IO_MODE,F_IMA_TYPE,
     .     NDIM,NAXISA,NPIXA,STARTA,STEPA,IDENA,
     +     CUNITA,PNTRV,VID,STATUS)
     

     
C  map input FRM
C  map the complete data frame for reading or updating and 
C  read the standard ard descriptors
C
C This was to check how looks like the input frame

      CALL STIGET(FRMIN,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,
     .            NDIM,NAXISA,NPIXA,STARTA,STEPA,IDENA,
     +            CUNITA,PNTRA,IMNO,STATUS)


C   FRMIN:       input frame
C   D_R4_FORMAT: data type
C   F_I_MODE:    opening mode, F_I_MODE or F_IO_MODE
C   F_I_TYPE:    file type
C   NDIM:        max no. of dimensions
C   NAXISA:          no. of dimensions
C   NPIXA:       no. of pixels per axis
C   STARTA:      start values
C   STEPA:       step sizes
C   IDENA:       ASCII id for frame
C   CUNITA:      unit of each axis + data
C   PNTRA:       pointer to mapped data
C   IMNO:        file id of frame
C   STATUS:      status (0= ok)


      PARAMS(1) = IPAR(1)       !slitwidth (in spatial direction)
      PARAMS(2) = RPAR(3)       !threshold factor for CR-rejection
      PARAMS(3) = RPAR(2)       !invers gain factor [e-/ADU]
      PARAMS(4) = RPAR(1)       !readout-noise [e-]
      
      
C
C  read coeffs
C
      INDX   = INDEX(NAME//' ',' ') - 1
      DESNAM = NAME(1:INDX)//'C'
      
C     STD (f77) module: ReaD char descriptor
      CALL STDRDC(TID,DESNAM,1,17,4,IAV,TYPE,KUN,KNUL,ISTAT)  

      TYPE   = 'MULT'
      IF (TYPE.EQ.'MULT') THEN
          DESNAM = NAME(1:INDX)//'I'
          
C     STD (f77) module: ReaD integer descriptor          
          CALL STDRDI(TID,DESNAM,4,4,IAV,IPAR,KUN,KNUL,ISTAT)
          NA     = (IPAR(3)+1)* (IPAR(4)+1)
          DESNAM = NAME(1:INDX)//'R'
          
C     STD (f77) module: ReaD real descriptor           
          CALL STDRDR(TID,DESNAM,1,4,IAV,PARAMS(5),KUN,KNUL,ISTAT)
          DESNAM = NAME(1:INDX)//'D'
          
C     STD (f77) module: ReaD double descriptor           
          CALL STDRDD(TID,DESNAM,1,NA,IAV,DPAR,KUN,KNUL,ISTAT)

C
C ... map output FRM
C

          IF (IORD1.EQ.IORD2 .AND. IORD1.EQ.0) THEN
              IORD1  = INT(PARAMS(7))
              IORD2  = INT(PARAMS(8))
          END IF

         
          STARTB(1) = STARTA(1)
          STARTB(2) = IORD1                 !ORD1 is the starting order ID No
          STEPB(1)  = STEPA(1)
          STEPB(2)  = 1.D0
          IDENB     = IDENA
          NAXISB    = 2
          NPIXB(1)  = NPIXA(1)
          NPIXB(2)  = IORD2 - IORD1 + 1     !No of extracted orders
          FRMVOUT = 'var_'//FRMOUT
          I = MAX(INDEX(FRMVOUT,' ')-1,1)

          CALL STIPUT(FRMOUT,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     +         NAXISB,NPIXB,STARTB,STEPB,IDENB,CUNITB,PNTRB,INO,STATUS)

          WRITE(LINE,*) 'OUTPUT:',FRMOUT(1:I),' extracted spectrum'

          CALL STTPUT(LINE,STATUS)
          
                    
C
C ... variance of extracted orders
C

          IDENB = 'variance'
C         creates a new image frame, map it and store standard descriptors    
          CALL STIPUT(FRMVOUT,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     +         NAXISB,NPIXB,STARTB,STEPB,IDENB,CUNITB,PNTVO,INVO,STATUS)

          WRITE(LINE,*) '       ',FRMVOUT(1:I)
     +         ,' variance of extr. spectrum'
     
C         module STST (f77): display text and store it in MIDAS log file
C         IN=LINE:           text string
C         OUT=STATUS:        return status (0=ok)      
          CALL STTPUT(LINE,STATUS)

          FRMSKY = 'opt_sky_'//FRMOUT
          I = MAX(INDEX(FRMSKY,' ')-1,1)
          CALL CLNFRA(FRMSKY,FRMSKY,0) !variance frame (pixel-pixel-space)
            
          CALL STIPUT(FRMSKY,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     $         NAXISB,NPIXB,STARTB,STEPB,'extracted_sky_bkg',
     +         CUNITB,PNTRSK,INSKY,STATUS)
 
          WRITE(LINE,*) '       ',FRMSKY(1:I)
     +         ,' sky bkg of extr. spectrum'
     
C         module STST (f77): display text and store it in MIDAS log file
C         IN=LINE:           text string
C         OUT=STATUS:        return status (0=ok)      
          CALL STTPUT(LINE,STATUS)

          
          FRVSKY = 'var_opt_sky_'//FRMOUT
          I = MAX(INDEX(FRVSKY,' ')-1,1)
          CALL CLNFRA(FRVSKY,FRVSKY,0) !variance frame (pixel-pixel-space)
            
          CALL STIPUT(FRVSKY,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     $         NAXISB,NPIXB,STARTB,STEPB,'var_extracted_sky_bkg',
     +         CUNITB,PNTRVS,IVSKY,STATUS)
 
          WRITE(LINE,*) '       ',FRVSKY(1:I)
     +         ,' var of sky bkg '
    
C         module STST (f77): display text and store it in MIDAS log file
C         IN=LINE:           text string
C         OUT=STATUS:        return status (0=ok)      
          CALL STTPUT(LINE,STATUS)
                    
C
C  ... map weight FRM
C

                                                   !slit width in pix
          IWIDTH = INT(PARAMS(1))                  !number of pix=slit width

          IF (MOD(IWIDTH,2).EQ.0)THEN          
             IKIND=1            !even slit width
             DIMP = IWIDTH+1    !dimension of the weight profil
          ELSE
             IKIND=0            !odd slit width
             DIMP = IWIDTH+2    !dimension of the weight profil
          ENDIF

          NWY  = DIMP*(IORD2 - IORD1 + 1)

          STARTB(1) = STARTA(1)
          STARTB(2) = 1.D0
          STEPB(1)  = STEPA(1)
          STEPB(2)  = 1.D0
          IDENB     = 'weight map'
          NAXISC    = 2
          NPIXC(1)  = NPIXA(1)
          NPIXC(2)  = NWY

 

          IF (DETWGT .EQ. 1) THEN
C          
C         creates a new image frame, map it and store standard descriptors    
C
             CALL STIPUT(FRMWGT,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     +            NAXISC,NPIXC,STARTB,STEPB,
     +            IDENB,CUNITB,PNTRC,WID,STATUS)

      
     
C
C cosmic ray rejection ONLY if weights should be determined
C
             IF (PARAMS(2).LT.0) THEN
                WRITE(LINE,*) '       NO cosmic ray rejection'
                CALL STTPUT(LINE,STATUS) !display text and store it 
                                         !in MIDAS log file
                MTID = -1
             ELSE

C  create CR-mask table

                FRMOUT = 'crm_'//FRMIN
                I = MAX(INDEX(FRMOUT,' ')-4,1)
                FRMOUT = FRMOUT(1:I)//'tbl'
                I = I + 3
                
C               initialize and open table file                
                CALL TBTINI(FRMOUT,F_TRANS,F_O_MODE,4,10000,MTID
     $               ,ISTAT)

C               initialize a column in a table     
                CALL TBCINI(MTID,D_I4_FORMAT,1,'I','num',':ORDER',ICOL1
     $               ,ISTAT)
                
C               initialize a column in a table     
                CALL TBCINI(MTID,D_I4_FORMAT,1,'I','PIXEL',':X',ICOL1
     $               ,ISTAT)


C               initialize a column in a table     
                CALL TBCINI(MTID,D_I4_FORMAT,1,'I','PIXEL',':Y',ICOL2
     $               ,ISTAT)


C               initialize a column in a table     
                CALL TBCINI(MTID,D_R4_FORMAT,1,'G10.4','ADU',':FLUX'
     $               ,ICOL2,ISTAT)

                CALL TBCSER(MTID,':ORDER',MIDORD,ISTAT)
              
                CALL TBCSER(MTID,':X',MIDX,ISTAT)
                CALL TBCSER(MTID,':Y',MIDY,ISTAT)
                CALL TBCSER(MTID,':FLUX',MIDFLX,ISTAT)
    
                WRITE(LINE,*) '       Cosmic ray rejection (>',PARAMS(2)
     .               ,'*sigma)'
     
                CALL STTPUT(LINE,STATUS)  !display text and store it
                                          !in MIDAS log file   
                                                       
                WRITE(LINE,*) '         save detections on: '
     $               ,FRMOUT(1:I)
     
                CALL STTPUT(LINE,STATUS)  !display text and store it
                                          !in MIDAS log file                  
             ENDIF
          ELSE
          
C            map complete data frame for reading or updating 
C            and read the standard ard descriptors          
             CALL STIGET(FRMWGT,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,
     .            NDIM,NAXISC,NPIXC,STARTB,STEPB,IDENB,
     +            CUNITB,PNTRC,WID,STATUS)

          ENDIF

          I = MAX(1,INDEX(FRMWGT,' ')-1)
          WRITE(LINE,*) '       ',FRMWGT(1:I),' weight map'
          
          CALL STTPUT(LINE,STATUS)  !display text and store it 
                                         !in MIDAS log file
                                                   
          I = MAX(1,INDEX(TABLE,' ')-1)
          WRITE(LINE,*) '       ',TABLE(1:I),' order table'
          
          CALL STTPUT(LINE,STATUS)  !display text and store it
                                    !in MIDAS log file     
                                         
          WRITE(LINE,*) '       OFFSET to order positions:',OFFSET
          
          CALL STTPUT(LINE,STATUS)  !display text and store it
                                    !in MIDAS log file 
                                              
          WRITE(LINE,*) '       OFFSET to object positions:',OBJSET
          
          CALL STTPUT(LINE,STATUS)  !display text and store it
                                    !in MIDAS log file          

C  !!!!!!! BEWARE here set binsize to 1 before (at the beginning) was set to 10
          BINSIZE = 1

          WRITE(LINE,*) '       BINSIZE of virtually resampled data:'
     $         ,BINSIZE
          
          CALL STTPUT(LINE,STATUS)  !display text and store it
                                    !in MIDAS log file               
        
   
C
C  extract order
C
          NPAR       = 11
          PARAMS(9)  = IPAR(3)
          PARAMS(10) = IPAR(4)
          IF (SCAN(1).EQ.0) THEN
             SCAN(1) = 1
             SCAN(2) = NPIXA(2)
          ENDIF


          IF (DETWGT .EQ. 1) THEN
C
C crea a table to monitor some results
C      
              CALL ORDTRA(NPIXB,DIMP)
              
          ENDIF
 

      CRMSKS=0
 
      STOCH=0
      STOCH2=0
      CALL QCTSR(D1NAME,ND1COL,D2NAME,ND2COL,D1COL,D2COL,IDQC0)

          DO 10 I = IORD1,IORD2

 
              N_CURR_ORD=I
C
C here write out infos only for OBJ not for FF extraction
C
            
              PARAMS(11) = I
              IORD       = I - IORD1 + 1
              IORDER = IORD
              IF (DETWGT .EQ. 1) THEN                   
                  CALL STDWRI(DTID2,'ESO.QC.OPTEX.NORD',
     +                        NPIXB(2),1,1,DUN,STATUS)
             
                  
C The optimal extraction is performed only on the Object (not on FF)
C do an optimal extraction, save weights and variances on disk.


                 CALL EXTOBJ(MADRID(PNTRA),MADRID(PNTRV),NPIXA(1)
     $                ,NPIXA(2),MADRID(PNTRB),MADRID(PNTVO),NPIXB(1)
     $                ,NPIXB(2),MADRID(PNTRC),NPIXC(1),NPIXC(2),IORD
     $                ,NPAR,PARAMS,NA,DPAR,SCAN,OFFSET,BINSIZE,STARTA
     $                ,STEPA,IKIND,IWIDTH,DIMP,OBJSET,MADRID(PNTRSK)
     +                ,MADRID(PNTRVS))
                

              ELSE

C              
C the immediate extraction is performed either for OBJ and Flat Field
C do an immediate extraction using the weights of a previous optimal extract.
C
                 CALL EXTFF(MADRID(PNTRA),MADRID(PNTRV),NPIXA(1)
     $                ,NPIXA(2),MADRID(PNTRB),MADRID(PNTVO),NPIXB(1)
     $                ,NPIXB(2),MADRID(PNTRC),NPIXC(1),NPIXC(2),IORD
     $                ,NPAR,PARAMS,NA,DPAR,SCAN,OFFSET,STARTA,STEPA
     $                ,IKIND,IWIDTH,DIMP)
              ENDIF

10        CONTINUE

      ELSE
      
C         display text and store it in MIDAS log file       
          CALL STTPUT('Error in Dispersion coefficients',ISTAT)
          GOTO 20
      ENDIF
      
          
      IF (DETWGT .EQ. 1) THEN     
        
          FRMOUT = 'xb_'//FRMIN
          DUN=0
          I_FILE = MAX(INDEX(FRMOUT,' ')-1,1)
          CALL STFOPN(FRMOUT(1:I_FILE),D_R4_FORMAT,0,F_IMA_TYPE,FIDNO,
     +                ISTAT)
 
          
          DO 50 N_CURR_ORD=IORD1,IORD2
             CALL DMPQC1(FIDNO,MID_S_N_CENT(N_CURR_ORD),
     .                   OBJ_POS_CENT(N_CURR_ORD),
     +                   FWHM(N_CURR_ORD),N_CURR_ORD)  
   50     CONTINUE      
   
   
C
C ... WRITE PROCESS DESCRIPTORS
C
          NNNN = NPIXB(1)*NPIXB(2)
          CALL MNMX(MADRID(PNTRB),NNNN,XMIN,XMAX)
          CUT(1) = XMIN
          CUT(2) = XMAX
          CUT(3) = XMIN
          CUT(4) = XMAX
          
          CALL STDWRR(FIDNO,'LHCUTS',CUT,1,4,KUN,ISTAT)
                                               
          CALL STFCLO(FIDNO,ISTAT)                              
      ENDIF
      
      IF (MTID.GT.0) CALL TBTCLO(MTID,ISTAT) !close CR-mask table
     
 
   20 CALL STSEPI !Module STST (f77) close down the MIDAS env.
         
      END
      
      SUBROUTINE MNMX(X,N,XMIN,XMAX)
C
C FIND MIN MAX VALS
C
      IMPLICIT NONE
      INTEGER  N      !size of X array
      REAL     X(N)   !array of values
      REAL     XMIN   !reference min
      REAL     XMAX   !reference max
C
      INTEGER I       !loop variable
C
      XMIN   = X(1)
      XMAX   = XMIN
      DO 10 I = 2,N
          XMIN   = AMIN1(XMIN,X(I))
          XMAX   = AMAX1(XMAX,X(I))
   10 CONTINUE
      RETURN
      END

