C $Id: necrebi2d.for,v 1.6 2001/07/23 09:12:39 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetts Ave, Cambridge, 
C MA 02139, USA.
C
C Correspondence concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.6 $
C $Log: necrebi2d.for,v $
C Revision 1.6  2001/07/23 09:12:39  amodigli
C Eliminated some warinngs indicated by ftncheck
C
C Revision 1.5  2001/01/04 12:31:35  amodigli
C Added Id, Name, Revision, Log conf. control info
C
C------------------------------------------------------------------------------
      PROGRAM ECHREB
C
C+
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  10:37 - 11 JAN 1988
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR: D.PONZ
C
C
C.IDENTIFICATION
C
C  program  ECHREBI1
C
C.MODIFICATIONS
C
C  910128   P.Ballester   Define and set variables
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, REBIN
C
C.PURPOSE
C
C  OBTAIN A 1D FRM CALIBRATED IN WAVELENGTHS FROM A 2D FRM
C  OF TYPE ECHELLE WITH THE DISPERSION COEFFS DEFINED AS DESCRIPTORS
C
C.ALGORITHM
C
C  EACH LINE IN THE INPUT FRM IS REBINNED IN LINEAR STEP IN WAVELENGTH
C  ACCORDING TO THE DISPERSION COEFFS.
C
C.INPUT/OUTPUT
C
C  REBIN/ECHELLE INPUT OUTPUT STEP [METHOD] [TABLE]
C  OR
C  REBIN/ECHELLE INPUT OUTPUT REFER_IMAGE [METHOD] [TABLE]
C
C METHOD CAN BE EITHER "NONLINEAR" - NON LINEAR REBINNING USING
C                                     THE COEFFS IN THE TABLE
C                      "LINEAR"    - LINEAR REBINNING
C
C.MODIFICATIONS
C [2.2]  DEFINE THE SAMPLING DOMAIN VIA A REFER IMAGE
C 
C 000713
C 
C-
      IMPLICIT NONE
C
      INTEGER NMAX_ORDER                    !Max number of forseen orders
      INTEGER HS_MAX                        !Max size of a slit in pixels
      INTEGER NMAX_Y_OUT                    !Maximum dimension of out ima
      PARAMETER(NMAX_ORDER=100,HS_MAX=100)  !HS probably should be read 
                                            !from input frame
      PARAMETER(NMAX_Y_OUT=NMAX_ORDER*HS_MAX)  !Maximum dimension of out ima                                          
      INTEGER      MADRID                   !Using this array the addresss 
                                            !of in/out data 
                                            !are passed to subsequent routines
                                                 
      INTEGER      ID                       !Ima ID number
      INTEGER      IR                       !file id of output frame
      
      INTEGER      I                        !do-loop variable index
      INTEGER      J                        !do-loop variable index
      INTEGER      K                        !do-loop variable index
 
      INTEGER      I1                       !Auxilliary index
      INTEGER      IMNO                     !file id of frame
      
      INTEGER*8    IPNTRA                   !Pointer to input mapped data 
                                            !for MIDAS env interface
      INTEGER*8    IPNTRB                   !Pointer to output mapped data 
                                            !for MIDAS env interface

      INTEGER      ISTAT                    !status variable 
                                            !for MIDAS env interface
      INTEGER      NAXISA                   !No of dimension for input frame
      INTEGER      NAXISB                   !No of dimension for output frame
      
      INTEGER      PIXBN                    !PIXBN=STEPA(1)/WSTEP
                                            !Step size/sampling step:
                                            !NPIXB(1)*WSTEP=NPIXA*STEPA(1)
                                            !wav range is conserved
                                                 
      INTEGER      KUN                  !Units variable for MIDAS env interface
      INTEGER      KNUL                     !Number of null values
      INTEGER      NNNN                     !NNNN = NPIXB(1)*NPIXB(2)
      
      INTEGER      NPIXA(2)                 !Buffer for output data
                                            !No of pixel per axis for in  frame
      INTEGER      NPIXB(2)                 !Buffer for output data
                                            !No of pixel per axis for out frame
                                                 
      INTEGER      IORD(2)                  !Buffer for output data (Order)
      INTEGER      IORD2(2)                 !Starting(1) and Ending(2) 
                                            !relative order number
                                                 
      INTEGER      TID                      !Table ID
      
      INTEGER      IORDER(NMAX_Y_OUT)       !Buffer for data containing Norder
      INTEGER      NCOE(2)                  !Buffer for data containing coeffs
      INTEGER      IA1(NMAX_Y_OUT)          !Buffer for data containing coeffi
      INTEGER      NPTOT(NMAX_Y_OUT)        !Buffer for data containing nptot
      INTEGER      HS                       !Actual size of the extraction slit
      INTEGER      NORDERS                  !Number of effective orders to be rebinned
      
      DOUBLE PRECISION WSTART(NMAX_Y_OUT)   !Buffer for data containing wstart
      DOUBLE PRECISION STEPA(2)             !Stepsizes for input frame
      DOUBLE PRECISION STARTA(2)            !Start values for input frame
      
      DOUBLE PRECISION STEPB(2)             !Stepsizes for output frame
      DOUBLE PRECISION STARTB(2)            !Start values for output frame
      
      DOUBLE PRECISION WSTEP                !Sampling step
      DOUBLE PRECISION A1(7,NMAX_Y_OUT)     !Buffer for data containing coeffd
      DOUBLE PRECISION WAUX(NMAX_ORDER)     !Auxilliary array to temporally hold WSTART values after rebinning
      
      REAL         CUT(4)                   !Image(?) cuts
      REAL         XMIN                     !Image(?) min
      REAL         XMAX                     !Image(?) max
      
      CHARACTER*60 OUTFRM                   !Output Frame
      CHARACTER*60 INFRM                    !Input Frame
      CHARACTER*60 REFER                    !Reference frame for sampling step
      CHARACTER*60 TABLE                    !Buffer for (input table) data values
      
      CHARACTER    METHOD*1                 !Rebinning method
      CHARACTER    LINE*80                  !Auxilliary char buffer to display output messages
      CHARACTER    DSCNAM*8                 !Descriptor name
      
      
      CHARACTER    IDENTA*72                !out: ASCII identification for frame
      CHARACTER    CUNITA*64                !out: unit of each axis + data
      CHARACTER    CUNITB*64                !in:  unit of each axis + data
      
      LOGICAL      IREF
      INCLUDE      'MID_INCLUDE:ST_DEF.INC'
      COMMON /VMR/MADRID(1)
      INCLUDE      'MID_INCLUDE:ST_DAT.INC'
      DATA CUNITB/'Flux            Relative wavel.'/
C
C ... INITIALIZE SYSTEM
C
      CALL STSPRO('ECHREB')
      CALL STKRDC('IN_A',1,1,60,I,INFRM,KUN,KNUL,ISTAT)
      CALL STKRDC('OUT_A',1,1,60,I,OUTFRM,KUN,KNUL,ISTAT)
      CALL CLNFRA(INFRM,INFRM,0)
      CALL CLNFRA(OUTFRM,OUTFRM,0)
      CALL STKRDC('P3',1,1,60,I,REFER,KUN,KNUL,ISTAT)
      CALL STKRDC('P4',1,1,1,I,METHOD,KUN,KNUL,ISTAT)
      CALL STKRDI('HS',1,1,I,HS,KUN,KNUL,ISTAT)
      I1     = INDEX('.+1234567890',REFER(1:1))
      IF (I1.NE.0) THEN
          CALL GENCNV(REFER,4,1,NNNN,XMIN,WSTEP,ISTAT)
          IF (ISTAT.LT.1) CALL STETER(11,'Wrong sampling step')
C 
          WRITE(LINE,9010) WSTEP
          CALL STTPUT(LINE,ISTAT)
          IREF   = .FALSE.
      ELSE
          IREF   = .TRUE.
          CALL CLNFRA(REFER,REFER,0)
      END IF
      IF (METHOD.EQ.'N' .OR. METHOD.EQ.'n') THEN
          CALL STKRDC('IN_B',1,1,60,I,TABLE,KUN,KNUL,ISTAT)
          CALL CLNTAB(TABLE,TABLE,0)
          CALL TBTOPN(TABLE,F_I_MODE,TID,ISTAT)
          CALL STDRDI(TID,'ORDER',1,2,I,IORD,KUN,KNUL,ISTAT)
          CALL STDRDI(TID,'COEFS',1,2,I,NCOE,KUN,KNUL,ISTAT)
          CALL STDRDD(TID,'COEFD',1,7*NCOE(2),I,A1,
     .                KUN,KNUL,ISTAT)
          CALL STDRDI(TID,'COEFI',1,NCOE(2),I,IA1,
     .                KUN,KNUL,ISTAT)
          CALL TBTCLO(TID,ISTAT)
      END IF
C
C ... MAP INPUT IMAGE
C
      CALL STIGET(INFRM,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,
     .            3,NAXISA,NPIXA,STARTA,STEPA,IDENTA,CUNITA,
     .            IPNTRA,IMNO,ISTAT)

      IF (NAXISA.EQ.1 .OR. NPIXA(2).EQ.1) THEN
          NPIXA(2) = 1
          NAXISA = 1
      END IF

      IF (NPIXA(2).GT.(NMAX_Y_OUT))  THEN
          CALL STETER(9,
     +   'Module ECHRE31: Too many orders (limit 100*100). 
     +    Buffer overflow')
      ENDIF

      IF (STEPA(2).NE.1.)  THEN
          CALL STETER(9,'Module ECHREBI: Wrong step(2) .ne. 1 ')
      ENDIF

      IORD2(1) = STARTA(2)                  ! Starting relative order number
      IORD2(2) = STARTA(2) + NPIXA(2) - 1   ! Final relative order number
C
C ... MAP OUTPUT FRM
C
      IF (METHOD.EQ.'N' .OR. METHOD.EQ.'n') THEN
C ....LINEAR REBINNING      
          NAXISB = 2
          IF (IREF) THEN
C ........SAMPLING STEP GIVEN FROM REFERENCE FRAME          
              CALL STFOPN(REFER,D_R4_FORMAT,0,F_IMA_TYPE,
     .                    IR,ISTAT)
              CALL STDRDI(IR,'NPIX',1,2,I,NPIXB,KUN,KNUL,ISTAT)
              CALL STDRDD(IR,'START',1,2,I,STARTB,KUN,KNUL,ISTAT)
              CALL STDRDD(IR,'STEP',1,2,I,STEPB,KUN,KNUL,ISTAT)
              WSTEP  = STEPB(1)
              CALL STDRDI(IR,'NORDER',1,NPIXB(2),I,IORDER,
     .                    KUN,KNUL,ISTAT)
              CALL STDRDI(IR,'NPTOT',1,NPIXB(2),I,NPTOT,
     .                    KUN,KNUL,ISTAT)
              CALL STDRDD(IR,'WSTART',1,NPIXB(2),I,WSTART,
     .                    KUN,KNUL,ISTAT)
     
              DO 10 I = 1,NPIXB(2)
                  WSTART(I) = WSTART(I)/1000.D0
   10         CONTINUE
          ELSE
C ........SAMPLING STEP GIVEN AS NUMERICAL PARAMETER       
 
C             GET STARTING POSITIONS FOR EACH ORDER AND NO. OF PIXELS PER ORDER
C             GET LIMITS FOR THE FIRST ORDER (LOWER WAVELENGTHS)  
              CALL WRANGE(NCOE(2),A1,IA1,WSTEP,IORD,NPIXA(1),WSTART,
     .                    NPIXB(1),NPTOT,IORDER,STARTA,STEPA,IORD2)
     
              NPIXB(2)  = NPIXA(2)
              STARTB(1) = 0.D0
              STARTB(2) = STARTA(2)
              STEPB(1)  = WSTEP
              STEPB(2)  = STEPA(2)
          END IF

C         Creates output frame
          CALL STIPUT(OUTFRM,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     .                NAXISB,NPIXB,STARTB,STEPB,
     .                IDENTA,CUNITB,IPNTRB,ID,ISTAT)
C
C ... NON LINEAR REBINNING
C
          CALL ECHRE4(MADRID(IPNTRA),NPIXA(1),NPIXA(2),STARTA,STEPA,
     .                NCOE(2),A1,IA1,IORD,MADRID(IPNTRB),NPIXB(1),
     .                NPIXB(2),WSTART,WSTEP,IORD2,HS)
C
C ... WRITE PROCESS DESCRIPTORS
C
          NNNN = NPIXB(1)*NPIXB(2)
          

C ...     FIND MIN MAX VALS          
          CALL MNMX(MADRID(IPNTRB),NNNN,XMIN,XMAX)
          CUT(1) = XMIN
          CUT(2) = XMAX
          CUT(3) = XMIN
          CUT(4) = XMAX
C          print *,'NPIXB(2)=',NPIXB(2)
          NORDERS=NPIXB(2)/HS
          DO I = 1,NORDERS
             WAUX(I)=WSTART(I)
          ENDDO
          DO I = 1,NORDERS
             DO J=1,HS
                K=(I-1)*HS+J
                WSTART(K) = 1000.D0*WAUX(I)
C                print *,'WSTART(',K,')=',WSTART(K)
             ENDDO 
          ENDDO
   
C ...     WRITE PROCESS DESCRIPTORS          
          CALL STDCOP(IMNO,ID,3,DSCNAM,ISTAT)
          CALL STDWRD(ID,'WSTART',WSTART,1,NPIXB(2),KUN,ISTAT)
          CALL STDWRI(ID,'NPTOT',NPTOT,1,NPIXB(2),KUN,ISTAT)
          CALL STDWRI(ID,'NORDER',IORDER,1,NPIXB(2),KUN,ISTAT)
          CALL STDWRR(ID,'LHCUTS',CUT,1,4,KUN,ISTAT)
      ELSE
C ... LINEAR REBINNING
          IF (NPIXA(2).GT.1) THEN
CCCCCCCCCCCCC              CALL STDFND(IMNO,'WSTART',I1,I2,I3,ISTAT)
C              IF (TYPE(1:1).EQ.' ') THEN
C                  CALL STTPUT(' Wrong input image ... ',ISTAT)
C                  GO TO 40
CCCCCCCCCCCCC              END IF
              CALL STDRDD(IMNO,'WSTART',1,NPIXA(2),I,WSTART,
     .                    KUN,KNUL,ISTAT)
              CALL STDRDI(IMNO,'NPTOT',1,NPIXA(2),I,NPTOT,
     .                    KUN,KNUL,ISTAT)
          END IF
          CALL STDRDR(IMNO,'LHCUTS',1,4,I,CUT,KUN,KNUL,ISTAT)
          PIXBN     = STEPA(1)/WSTEP
          NAXISB    = NAXISA
          NPIXB(1)  = NPIXA(1)*PIXBN
          NPIXB(2)  = NPIXA(2)
          STARTB(1) = STARTA(1)
          STARTB(2) = STARTA(2)
          STEPB(1)  = WSTEP
          STEPB(2)  = STEPA(2)
          IF (NPIXB(2).GT.1) THEN
              DO 30 I = 1,NPIXB(2)
                  NPTOT(I) = NPTOT(I)*PIXBN
   30         CONTINUE
          ELSE
              WSTART(1) = STARTB(1)
              NPTOT(1) = NPIXB(1)
          END IF
          
C         Creates output frame          
          CALL STIPUT(OUTFRM,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     .                NAXISB,NPIXB,STARTB,STEPB,IDENTA,
     .                CUNITB,IPNTRB,ID,ISTAT)
     
C         Linear Rebinning     
          CALL ECHR01(MADRID(IPNTRA),NPIXA(1),NPIXA(2),STARTA,STEPA,
     .                NPTOT,WSTART,
     .                MADRID(IPNTRB),NPIXB(1),NPIXB(2),WSTEP)

          CALL STDCOP(IMNO,ID,3,DSCNAM,ISTAT)
          IF (NPIXB(2).GT.1) THEN
            CALL STDWRD(ID,'WSTART',WSTART,1,NPIXB(2),KUN,ISTAT)
            CALL STDWRI(ID,'NPTOT',NPTOT,1,NPIXB(2),KUN,ISTAT)
          END IF
          CALL STDWRR(ID,'LHCUTS',CUT,1,4,KUN,ISTAT)
      END IF
      
C     close down the MIDAS environment      
      CALL STSEPI
C10000 FORMAT(G<I1>.0)
 9000 FORMAT (G6.0)
 9010 FORMAT(' Sampling step = ',F10.4)
      END

      SUBROUTINE ECHRE3(X,NPIXA1,NPIXA2,STARTA,STEPA,NO,A,IA,IORD,
     +                  Y,NPIXB1,NPIXB2,YSTR,YSTP,IORD2)
C
C REBIN THE ORDERS
C
      IMPLICIT NONE
C
      INTEGER NO                            !No of coeffi
      INTEGER NPIXB1                        !No of out frame X points
      INTEGER NPIXB2                        !No of out frame Y points
      
      INTEGER NPIXA1                        !No of in  frame X points
      INTEGER NPIXA2                        !No of in  frame Y points
      INTEGER IORD(2)                       !Buffer for output data (Order)
      INTEGER IA(NO)                        !Buffer for data containing coeffi
      INTEGER IORD2(2)                      !Starting(1) and Ending(2) 
                                            !relative order number
      
      REAL X(NPIXA1,NPIXA2)                 !In  frame
      REAL Y(NPIXB1,NPIXB2)                 !Out frame
      
      DOUBLE PRECISION YSTR(1)              !Wav start
      DOUBLE PRECISION YSTP                 !Wav step
      DOUBLE PRECISION STARTA(2)            !Start values for input frame
      DOUBLE PRECISION STEPA(2)             !Step sizes for input frame
      
      DOUBLE PRECISION A(7,NO)              !Buffer for data containing coeffd
      DOUBLE PRECISION B1(3)                !Auxilliary vector to hold data
C
      INTEGER I0                            !Auxilliary variable
      INTEGER I2                            !Auxilliary variable
      INTEGER I1                            !Auxilliary variable
      INTEGER K1                            !Auxilliary variable for a loop
      INTEGER II                            !Auxilliary variable for a loop

      INTEGER I3                            !index corresponding to coeffd
                                            !of the appropriate order
      
      DOUBLE PRECISION WLSTP                !Wav sub step
      DOUBLE PRECISION WLST2                !Wav start
C
C ... GET LIMITS FOR THE FIRST ORDER (LOWER WAVELENGTHS)
C

      IF (IORD(1).LT.IORD(2)) THEN
          I0     = NO
          I2     = -1
      ELSE
          I0     = 1  
          I2     = 1
      END IF
      I1     = I0
      WLSTP  = YSTP/1000.D0
C
C ... ITERATION ON ORDERS
C
C     From bigger to lower order (from lower to bigger wav step) it is done a scan

      DO 20 K1 = IORD2(2), IORD2(1), -1
          I3 = I1 + IORD2(1) - 1
          IF (IA(I3).LT.0) THEN
              B1(1)  = A(1,I3)
              B1(2)  = -A(2,I3)
              B1(3)  = A(3,I3)
          END IF
          WLST2  = YSTR(I1)
          IF (IA(I3).LT.0) THEN
C  DATA REBINNING: CONVERSION FROM SAMPLES INTO WAVELENGTHS
C  ALGORITHM: LINEAR INTERPOLATION. THE TRANSFORMATION WAVELENGTH INTO
C             SAMPLE NUMBER IS DEFINED BY S = A0+A1*T+A2*T**2                  
              CALL LREBIN(X(1,I1),NPIXA1,STARTA,STEPA,B1,3,Y(1,I1),
     +                    NPIXB1,WLST2,WLSTP)
C
C ... SIGN CHANGE
              DO 10 II = 1,NPIXB1
                  Y(II,I1) = -Y(II,I1)
   10         CONTINUE
          ELSE
C         DATA REBINNING: CONVERSION FROM SAMPLES INTO WAVELENGTHS
C         ALGORITHM: LINEAR INTERPOLATION. THE TRANSFORMATION SAMPLE NUMBER INTO
C                    WAVELENGTH IS DEFINED BY W = A0 + A1*S + A2*S**2 + ...            
              CALL LREBI1(X(1,I1),NPIXA1,STARTA,STEPA,A(1,I3),IA(I3),
     +                     Y(1,I1),NPIXB1,WLST2,WLSTP)
          END IF
C         with the following statement at the following iteration I3 changes          
          I1     = I1 + I2
          IF (I1.GT.NO) RETURN
   20 CONTINUE
      RETURN
      END

C ===============================================================================
C Changed
C ===============================================================================
      SUBROUTINE ECHRE4(X,NPIXA1,NPIXA2,STARTA,STEPA,NO,A,IA,IORD,
     +                  Y,NPIXB1,NPIXB2,YSTR,YSTP,IORD2,HS)
C
C REBIN THE ORDERS
C
      IMPLICIT NONE
C
      INTEGER NO                            !No of coeffi
      INTEGER NPIXB1                        !No of out frame X points
      INTEGER NPIXB2                        !No of out frame Y points
      
      INTEGER NPIXA1                        !No of in  frame X points
      INTEGER NPIXA2                        !No of in  frame Y points
      INTEGER IORD(2)                       !Buffer for output data (Order)
      INTEGER IA(NO)                        !Buffer for data containing coeffi
      INTEGER IORD2(2)                      !Starting(1) and Ending(2) 
                                            !relative order number
      
      REAL X(NPIXA1,NPIXA2)                 !In  frame
      REAL Y(NPIXB1,NPIXB2)                 !Out frame
      
      DOUBLE PRECISION YSTR(1)              !Wav start
      DOUBLE PRECISION YSTP                 !Wav step
      DOUBLE PRECISION STARTA(2)            !Start values for input frame
      DOUBLE PRECISION STEPA(2)             !Step sizes for input frame
      
      DOUBLE PRECISION A(7,NO)              !Buffer for data containing coeffd
      DOUBLE PRECISION B1(3)                !Auxilliary vector to hold data
C
      INTEGER I0                            !Auxilliary variable
      INTEGER I2                            !Auxilliary variable
      INTEGER I1                            !Auxilliary variable
      INTEGER K1                            !Auxilliary variable for a loop
      INTEGER II                            !Auxilliary variable for a loop
      INTEGER IS                            !Auxilliary variable place holder
      INTEGER I3                            !index corresponding to coeffd
                                            !of the appropriate order
      INTEGER J                             !Auxilliary variable to loop whithin the order    
      INTEGER HS                            !Window size                                        
      
      DOUBLE PRECISION WLSTP                !Wav sub step
      DOUBLE PRECISION WLST2                !Wav start
C
C ... GET LIMITS FOR THE FIRST ORDER (LOWER WAVELENGTHS)
C
    
      IF (IORD(1).LT.IORD(2)) THEN
          I0     = NO
          I2     = -1
      ELSE
          I0     = 1  
          I2     = 1
      END IF
      I1     = I0
      WLSTP  = YSTP/1000.D0
C
C ... ITERATION ON ORDERS
C
C     From bigger to lower order (from lower to bigger wav step) it is done a scan
C      print *,'IORD2(1)=',IORD2(1),'IORD2(2)=',IORD2(2),'NO=',NO
      DO 20 K1 = IORD2(2), IORD2(1), -1

          I3 = I1 + IORD2(1) - 1
          IF (IA(I3).LT.0) THEN
              B1(1)  = A(1,I3)
              B1(2)  = -A(2,I3)
              B1(3)  = A(3,I3)
          END IF
          
          WLST2  = YSTR(I1)
C          print *,'WLST2=',WLST2,'I1=',I1
C          print *,'HS=',HS
          DO J=1,HS    
              IS=(I1-1)*HS+J   
C              print *,'IS=',IS    
          IF (IA(I3).LT.0) THEN
C  DATA REBINNING: CONVERSION FROM SAMPLES INTO WAVELENGTHS
C  ALGORITHM: LINEAR INTERPOLATION. THE TRANSFORMATION WAVELENGTH INTO
C             SAMPLE NUMBER IS DEFINED BY S = A0+A1*T+A2*T**2    
   
              CALL LREBIN(X(1,IS),NPIXA1,STARTA,STEPA,B1,3,Y(1,IS),
     +                    NPIXB1,WLST2,WLSTP)
C
C ... SIGN CHANGE
              DO 10 II = 1,NPIXB1
                  Y(II,IS) = -Y(II,IS)
   10         CONTINUE
          ELSE
C         DATA REBINNING: CONVERSION FROM SAMPLES INTO WAVELENGTHS
C         ALGORITHM: LINEAR INTERPOLATION. THE TRANSFORMATION SAMPLE NUMBER INTO
C                    WAVELENGTH IS DEFINED BY W = A0 + A1*S + A2*S**2 + ...            
              CALL LREBI1(X(1,IS),NPIXA1,STARTA,STEPA,A(1,I3),IA(I3),
     +                     Y(1,IS),NPIXB1,WLST2,WLSTP)
          END IF
        ENDDO
C         with the following statement at the following iteration I3 changes          
          I1     = I1 + I2
          IF (I1.GT.NO) RETURN
   20 CONTINUE
      RETURN
      END
      
C ===============================================================================      











      DOUBLE PRECISION FUNCTION POLEV(IA,A,XX)
C
C
C
      IMPLICIT NONE
C
      INTEGER IA
      DOUBLE PRECISION A(IA),XX,YY

      YY     = A(1) + XX* (A(2)+XX* (A(3)+XX* (A(4)+XX* (A(5)+XX* (A(6)+
     +         XX*A(7))))))
      POLEV  = YY
      RETURN
      END

      SUBROUTINE MNMX(X,N,XMIN,XMAX)
C
C FIND MIN MAX VALS
C
      IMPLICIT NONE
      INTEGER  N
      REAL     X(N), XMIN, XMAX
C
      INTEGER I
C
      XMIN   = X(1)
      XMAX   = XMIN
      DO 10 I = 2,N
          XMIN   = AMIN1(XMIN,X(I))
          XMAX   = AMAX1(XMAX,X(I))
   10 CONTINUE
      RETURN
      END

      SUBROUTINE WRANGE(NCOE,A,IA,WSTEP,IORD,NPIXA,WSTART,NPIXB,NPTOT,
     +                  IORDER,STARTA,STEPA,IORD2)
C
C GET STARTING POSITIONS FOR EACH ORDER AND NO. OF PIXELS PER ORDER
C
C
C ... GET LIMITS FOR THE FIRST ORDER (LOWER WAVELENGTHS)
C
      IMPLICIT NONE
C
      INTEGER NCOE                          !Buffer for data containing coeffs
      INTEGER NPIXA                         !Buffer for output data
                                            !No of pixel per axis for in  frame
      INTEGER NPIXB                         !Buffer for output data
                                            !No of pixel per axis for out  frame
      
      DOUBLE PRECISION A(7,NCOE)            !Buffer for data containing coeffd
      DOUBLE PRECISION POLEV                !Ploinomial evaluation function
      DOUBLE PRECISION B(3)                 !Auxilliary array to hold data
      DOUBLE PRECISION STARTA(1)            !Starting X point
      DOUBLE PRECISION STEPA(1)             !Step for X axis
      
      DOUBLE PRECISION WSTART(1)            !Starting wav
      DOUBLE PRECISION WSTEP                !wav step
      DOUBLE PRECISION WLST1                !wav start (from subroutine echord)
      DOUBLE PRECISION WLEN1                !wav len (from subroutine echord)
      
      INTEGER IA(NCOE)                      !Buffer for data containing coeffi
      INTEGER IORD(2)                       !Buffer for output data (Order)
      INTEGER NPTOT(1)                      !Buffer for data containing nptot
      INTEGER IORDER(1)                     !Buffer for data containing Norder
      INTEGER IORD2(2)                      !Starting(1) and Ending(2) 
                                            !relative order number
C

      INTEGER I0                            !Auxilliary index
      INTEGER I1                            !Auxilliary index
      INTEGER I2                            !Auxilliary index
      INTEGER I                             !Auxilliary index for loop
      INTEGER ISTEP                         !Auxilliary index
      INTEGER K                             !Auxilliary index for loop
      INTEGER N1                            !Number of points
      INTEGER I3                            !index corresponding to coeffd
                                            !of the appropriate order

c      INTEGER*8 N1
      DOUBLE PRECISION WS1                  !wav sub-step
      DOUBLE PRECISION WW1                  !variable to hold result from polev: not necessary!
      DOUBLE PRECISION IX_IR8               !function to convert from integer to world coordinates
C
      IF (IORD(1).LT.IORD(2)) THEN
          I0     = NCOE 
          I2     = -1
      ELSE
          I0     = 1 
          I2     = 1
      END IF
      I1     = I0
      NPIXB  = 0
      WS1    = WSTEP/1000.D0
C
C ... get order number
C
      ISTEP  = 1
      IF (IORD(1).GT.IORD(2)) ISTEP  = -1 ! Default case in present versions
      IORDER(1) = IORD(1) - IORD2(1) + 1
      DO 10 I = 2, IORD2(2) - IORD2(1) + 1
          IORDER(I) = IORDER(I-1) + ISTEP
   10 CONTINUE
C
C ... iteration on orders to get starting wavelength and no. of pixels
C
C     From bigger to lower order (from lower to bigger wav step) it is done a scan

      DO 20 K = IORD2(2), IORD2(1), -1
          I3 = I1 + IORD2(1) - 1
          IF (IA(I3).LT.0) THEN
              B(1)   = A(1,I3)
              B(2)   = -A(2,I3)
              B(3)   = A(3,I3)
              CALL ECHORD(B,3,NPIXA,WLST1,WLEN1,STARTA,STEPA)
          ELSE
              WW1    = POLEV(IA(I3),A(1,I3),IX_IR8(1,STARTA,STEPA))
              WLST1  = WW1
              WW1    = POLEV(IA(I3),A(1,I3),IX_IR8(NPIXA,STARTA,STEPA))
              WLEN1  = WW1
          END IF
          WLST1  = WLST1*10.D0
          WLEN1  = WLEN1*10.D0
          N1     = (WLEN1-WLST1)/WS1
          IF (N1.GT.65534) THEN !added swolf@eso.org
             N1 = 0             !if WLEN1 and WLST1 are NaN N1 gets wrong!
          ENDIF                 !--> NPIXB gets wrong, too!
          NPTOT(I1) = N1
          NPIXB  = MAX(NPIXB,N1)
          IF (I1.EQ.I0) THEN
              WSTART(I1) = WS1*NINT(WLST1/WS1) !before: WLST1 (swolf@eso.org)
          ELSE
              WSTART(I1) = WSTART(I0) + WS1*NINT((WLST1-WSTART(I0))/WS1)
          END IF
C         with the following statement at the following iteration I3 changes            
          I1     = I1 + I2
          IF (I1.GT.NCOE) GO TO 30
   20 CONTINUE
   30 NPIXB  = NPIXB + 3
      RETURN
      END

      SUBROUTINE ECHR01(A,NPIXA1,NPIXA2,STARTA,STEPA,NPTOT,WSTART,
     +                  B,NPIXB1,NPIXB2,WSTEP)
C
C REBIN ECHELLE ORDERS: Linear Rebinning
C
      IMPLICIT NONE
C
      INTEGER NPIXA1                        !Buffer for output data
                                            !No of pixel per X axis for in  frame
      INTEGER NPIXA2                        !Buffer for output data
                                            !No of pixel per Y axis for in  frame
      INTEGER NPIXB1                        !Buffer for output data
                                            !No of pixel per X axis for out frame
      INTEGER NPIXB2                        !Buffer for output data
                                            !No of pixel per Y axis for out frame
      
      DOUBLE PRECISION  STEPA(2)            !Stepsizes for input frame
      DOUBLE PRECISION  STARTA(2)           !Start values for input frame
      
      REAL A(NPIXA1,NPIXA2)                 !Input frame
      REAL B(NPIXB1,NPIXB2)                 !Output frame
      
      DOUBLE PRECISION  WSTART(NPIXA2)      !Starting wav array for all the orders
      DOUBLE PRECISION  WSTEP               !wav step size
      
      INTEGER NPTOT(NPIXA2)                 !Number of points in each order
C
      INTEGER NORD                          !Auxilliary index
      INTEGER IORD                          !Auxilliary index
      INTEGER I                             !Auxilliary index
      INTEGER NPREB                         !Auxilliary index: No of points in each order
      
      DOUBLE PRECISION XSTART               !Auxilliary var for starting wav in each order
      DOUBLE PRECISION XP                   !Auxilliary variable for current wav in each order
      DOUBLE PRECISION ADEX                 !Auxilliary variable for current bin in each order
      DOUBLE PRECISION PIXBN                !PIXBN  = WSTEP/STEPA(1)
      
      REAL   VBIN
C
      PIXBN  = WSTEP/STEPA(1)
      NORD   = NPIXA2                            !Y ima dimension = No of orders (?)
      DO 30 IORD = 1,NORD
          XSTART = WSTART(IORD)                  !starting wav point at the current order
          NPREB  = NPTOT(IORD)                   !number of points at the current order
          DO 10 I = 1,NPREB                      !scan the order
              XP     = XSTART + (I-1)*WSTEP      !wav pos at current pixel in current order
              ADEX   = (XP-XSTART)/STEPA(1) + 1.D0 !Actual pixel
              B(I,IORD) = VBIN(A(1,IORD),PIXBN,ADEX)
   10     CONTINUE
          DO 20 I = NPREB + 1,NPIXB1
              B(I,IORD) = 0.
   20     CONTINUE
   30 CONTINUE
      RETURN
      END

      REAL FUNCTION VBIN(ARRAY,PIXPBN,ADEX)
C
C FUNCTION TO RETURN A VALUE FOR A REBINNED BIN
C
C INPUT ARGUMENTS
C ARRAY REAL*4 ARRAY WITH REAL ORIGINAL PIXELS
C PIXPBN REAL*4 PIXELS PER BIN, NUMBER OF ORIGINAL
C   PIXELS IN THE NEW BIN, WITH FRACTIONS
C ADEX REAL*4 CENTER INDEX IN THE ARRAY FOR THE NEW BIN,
C   WITH FRACTIONS
C
      IMPLICIT NONE
      INTEGER          IBOTDX      !   TOP POINTER, WITHOUT FRACTION
      INTEGER          ITOPDX      !   BOTTOM POINTER, WITHOUT FRACTION
      INTEGER          IENLP
      INTEGER          I           !   Auxilliary index
      INTEGER          ISTLP
      
      REAL             HBIN        !   BOTTOM POINTER, FRACTION
      REAL             BOTDX       !   TOP POINTER, FRACTION
      REAL             TOPDX       !   BOTTOM POINTER, WITHOUT FRACTION
      REAL             SUM         !   Auxilliary variable
      REAL             BOTFRA      !   Auxilliary variable
      REAL             TOPFRA      !   START LOOP INDEX
      
      REAL             ARRAY(1)    !   Array with value of input frame
      DOUBLE PRECISION PIXPBN      !   PIXBN  = WSTEP/STEPA(1)
      DOUBLE PRECISION ADEX        !   Auxilliary variable for current bin
C  !   HALF BIN
      HBIN   = PIXPBN*0.5D0  !   BOTTOM POINTER, FRACTION
      BOTDX  = ADEX - HBIN  !   TOP POINTER, FRACTION
      TOPDX  = ADEX + HBIN  !   BOTTOM POINTER, WITHOUT FRACTION
      IBOTDX = BOTDX  !   TOP POINTER, WITHOUT FRACTION
      ITOPDX = TOPDX
C
      IF (IBOTDX.EQ.ITOPDX) THEN
C
C   BIN SMALLER THAN ORIGINAL PIXELS
C   AND NEW BIN WITHIN AN ORIGINAL PIXEL
C
          SUM    = PIXPBN*ARRAY(IBOTDX+1)
      ELSE
C
C   BIN LARGER THAN ORIGINAL PIXELS, OR
C   SMALL BIN OVER TWO PIXELS
C
          BOTFRA = 1. - (BOTDX-FLOAT(IBOTDX))
          TOPFRA = TOPDX - FLOAT(ITOPDX)  !   START LOOP INDEX
          ISTLP  = IBOTDX + 2
          IENLP  = ITOPDX
          IBOTDX = IBOTDX + 1
          ITOPDX = ITOPDX + 1
C
          SUM    = 0.
          IF (IENLP.GE.ISTLP) THEN
              DO 10 I = ISTLP,IENLP
                  SUM    = SUM + ARRAY(I)
   10         CONTINUE
          END IF  !   LOWER FRACTION
          SUM    = SUM + BOTFRA*ARRAY(IBOTDX)  !   UPPER FRACTION
          SUM    = SUM + TOPFRA*ARRAY(ITOPDX)
      END IF
C
C AREA CORRECTION
C
      VBIN   = SUM/PIXPBN
      RETURN
      END

      SUBROUTINE ECHORD(AC,NC,NPIX,W1,W2,STARTA,STEPA)
C
C
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  22:34 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR: D.PONZ
C
C.KEYWORDS:
C  SPECTROSCOPY, CASPEC, ORDER DEFINITION
C
C.PURPOSE:
C  GET WAVELENGTH RANGE OF A GIVEN ORDER
C
C.ALGORITHM:
C
C.INPUT/OUTPUT
C  INPUT ARGUMENTS
C
C  AC(NC) REAL*8 COEFFICIENTS OF THE TRANSFORMATION
C  NC  REAL*4 NO. OF COEFFICIENTS
C  NPIX  INTG*4 NO OF SAMPLES
C
C  OUTPUT ARGUMENTS
C
C  W1  REAL*4  LOWER LIMIT
C  W2  REAL*4  UPPER LIMIT
C
C-------------------------------------------------------------
C
C
      IMPLICIT NONE
      INTEGER NPIX                          !Buffer for output data
                                            !No of pixel per X axis for in  frame
      INTEGER NC                            !Dimention of AC
      
      DOUBLE PRECISION AC(NC)               !Auxilliary array to hold NC data
      DOUBLE PRECISION STARTA(1)            !Starting value
      DOUBLE PRECISION STEPA(1)             !Step value
      
      DOUBLE PRECISION W1                   !wav start
      DOUBLE PRECISION W2                   !wav bin size
      DOUBLE PRECISION IX_IR8               !To convert from pix to world coordinates
C
      DOUBLE PRECISION A0                   !Auxilliary variable
      DOUBLE PRECISION A1                   !Auxilliary variable
      DOUBLE PRECISION A2                   !Auxilliary variable
      DOUBLE PRECISION F1                   !Auxilliary variable
      DOUBLE PRECISION FN                   !Auxilliary variable
C
C
C ... ASSIGN COEFFS
C
C      F1     = 1.D0
C      FN     = NPIX

      F1     = IX_IR8(1,STARTA(1),STEPA(1))
      FN     = IX_IR8(NPIX,STARTA(1),STEPA(1))

      A0     = AC(1)
      A1     = AC(2)
      IF (NC.GE.3) THEN
          A2     = AC(3)
          W1     = (-A1+DSQRT(A1*A1-4.D0*A2* (A0-F1)))/ (2.D0*A2)
          W2     = (-A1+DSQRT(A1*A1-4.D0*A2* (A0-FN)))/ (2.D0*A2)

      ELSE
          W1     = (F1-A0)/A1
          W2     = (FN-A0)/A1
      END IF
      RETURN
      END


      SUBROUTINE LREBI1(X,NX,XSTR,XSTP,AC,NC,Y,NY,YSTR,YSTP)
C
C
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  0:22 - 4 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR: D.PONZ
C
C.IDENTIFICATION
C  SUBROUTINE  LREBIN1 VERSION 1   22 MAR 1983
C  J.D.PONZ
C
C.KEYWORDS:
C  SPECTROSCOPY, CASPEC, REBINNING
C
C.PURPOSE:
C  DATA REBINNING
C  CONVERSION FROM SAMPLES INTO WAVELENGTHS
C
C.ALGORITHM:
C  LINEAR INTERPOLATION. THE TRANSFORMATION SAMPLE NUMBER INTO
C  WAVELENGTH IS DEFINED BY
C  W = A0 + A1*S + A2*S**2 + ...
C
C.INPUT/OUTPUT
C  INPUT ARGUMENTS
C
C  X(NX) REAL*4 INPUT ARRAY
C  NX  INTG*4 NO OF SAMPLES
C  XSTR  REAL*4 STARTING VALUE OF INPUT
C  XSTP  REAL*4 INPUT STEP
C  AC(NC) REAL*8 COEFFICIENTS OF THE TRANSFORMATION
C  NC  REAL*4 NO. OF COEFFICIENTS
C  NY  INTG*4 NO. OF OUTPUT VALUES
C  YSTR  REAL*4 STARTING VALUE OF THE INPUT
C  YSTP  REAL*4 STEP OF THE OUTPUT
C
C  OUTPUT ARGUMENTS
C
C  Y(NY) REAL*4 OUTPUT ARRAY
C
C-------------------------------------------------------------
C
C
      IMPLICIT NONE
      INTEGER  NX
      INTEGER  NY
      INTEGER  NC
      
      REAL X(NX)                    !In  frame
      REAL Y(NY)                    !Out frame
      
      DOUBLE PRECISION YSTR         !wav start  out frame
      DOUBLE PRECISION YSTP         !wav sub step out frame
      DOUBLE PRECISION XSTR         !wav start  in frame
      DOUBLE PRECISION XSTP         !wav sub step in frame
C
      DOUBLE PRECISION AC(NC)       !Buffer for data containing coeffd
      DOUBLE PRECISION A0           !Auxilliary variable
      DOUBLE PRECISION A1           !Auxilliary variable
      DOUBLE PRECISION A2           !Auxilliary variable
      DOUBLE PRECISION A3           !Auxilliary variable
      DOUBLE PRECISION A4           !Auxilliary variable
      DOUBLE PRECISION A5           !Auxilliary variable
      DOUBLE PRECISION A6           !Auxilliary variable
      DOUBLE PRECISION FN           !Auxilliary variable
      DOUBLE PRECISION FL           !Auxilliary variable
      DOUBLE PRECISION WPOLY        !Auxilliary variable to hold wav
      
      
      DOUBLE PRECISION FACTOR       !Auxilliary variable
      DOUBLE PRECISION DPOLY        !Auxilliary variable
      DOUBLE PRECISION CNORM        !Auxilliary variable
      DOUBLE PRECISION IX_R8R8      !Function to pass from integer to world coordinates
      DOUBLE PRECISION XN           !Auxilliary variable
      DOUBLE PRECISION XNP1         !Auxilliary variable
      
      DOUBLE PRECISION PN           !Auxilliary variable
      DOUBLE PRECISION PNP1         !Auxilliary variable
      DOUBLE PRECISION TM           !Auxilliary variable
      DOUBLE PRECISION TMP1         !Auxilliary variable
      DOUBLE PRECISION C            !Auxilliary variable
      DOUBLE PRECISION COV          !Auxilliary variable
      DOUBLE PRECISION VAL          !Auxilliary variable
      DOUBLE PRECISION COVX         !Auxilliary variable
      
      DOUBLE PRECISION B2           !Auxilliary variable
      DOUBLE PRECISION B3           !Auxilliary variable
      DOUBLE PRECISION B4           !Auxilliary variable
      DOUBLE PRECISION B5           !Auxilliary variable
      DOUBLE PRECISION B6           !Auxilliary variable
      DOUBLE PRECISION DELT         !Auxilliary variable
      DOUBLE PRECISION STARTL       !Auxilliary variable
      
      INTEGER          M1           !Auxilliary variable
      INTEGER          N            !Auxilliary variable
      INTEGER          K            !Auxilliary variable
      INTEGER          MPT          !Auxilliary variable
C
      WPOLY(FL) = A0 + FL* (A1+FL* (A2+FL* (A3+FL* (A4+FL* (A5+FL*
     +            A6)))))
      DPOLY(FL) = A1 + FL* (B2+FL* (B3+FL* (B4+FL* (B5+FL*B6))))
C
C ... ASSIGN COEFFS
C
      A0     = AC(1)
      A1     = AC(2)
      A2     = AC(3)
      A3     = AC(4)
      A4     = AC(5)
      A5     = AC(6)
      A6     = AC(7)
      B2     = 2.D0*A2
      B3     = 3.D0*A3
      B4     = 4.D0*A4
      B5     = 5.D0*A5
      B6     = 6.D0*A6
      CNORM  = 10.D0/YSTP

C
C ... CHECK FOR OUTPUT START
C
      STARTL = YSTR - 0.5D0*YSTP
      FN     = 0.5D0
      XN     = IX_R8R8(FN,XSTR,XSTP)
      PN     = 10.D0*WPOLY(XN)

      DELT   = PN - STARTL
      IF (DELT.GT.0.) THEN
          M1     = DELT/YSTP + 1.D0
          TMP1   = PN + YSTP
          TM     = PN
      ELSE
          M1     = 1
          TMP1   = STARTL + YSTP
          TM     = STARTL
      END IF
C
C ... CHECK FOR INPUT START
C
      DO 10 N = 1,NX
          FN     = FN + 1.D0
          XNP1   = IX_R8R8(FN,XSTR,XSTP)
          PNP1   = 10.D0*WPOLY(XNP1)

          IF (PNP1.GT.TM) GO TO 20
          PN     = PNP1
   10 CONTINUE
      N      = NX + 1
      GO TO 30
   20 C      = X(N)
   30 VAL    = 0.D0

      COVX   = XNP1 - XN
      COV    = (PNP1 - PN)/COVX
C
C ... REBINNING LOOP
C
      MPT    = 1
      K      = 0
   35 CONTINUE
          IF ( .NOT. (MPT.LE.NY)) GO TO 80
          K      = K + 1
          IF ((K.LT.M1) .OR. (N.GT.NX)) GO TO 60
C 
   40     IF (PNP1.EQ.PN) GO TO 90
          VAL    = VAL + C* (DMIN1(TMP1,PNP1)-DMAX1(TM,PN))/COV
          IF (TMP1.LT.PNP1) GO TO 50
          N      = N + 1
          IF (N.GT.NX) GO TO 60

          PN     = PNP1
          XN     = XNP1

          FN     = FN + 1.D0
          XNP1   = IX_R8R8(FN,XSTR,XSTP)
          PNP1   = 10.D0*WPOLY(XNP1)

          C      = X(N)
          COV    = (PNP1 - PN)/(XNP1-XN)

          GO TO 40
   50     TM     = TMP1
          TMP1   = TMP1 + YSTP
   60     FACTOR = CNORM*DPOLY(IX_R8R8(FN,XSTR,XSTP))
C          FACTOR = FACTOR*10.D0*DABS(DPOLY(IX_R8R8(FN,XSTR,XSTP)))/YSTP
          Y(MPT) = VAL*FACTOR
          MPT    = MPT + 1
          VAL    = 0.D0
          GOTO 35
   80 CONTINUE
   90 CONTINUE
      DO 100 K = MPT + 1,NY
          Y(K)   = 0.
  100 CONTINUE
      RETURN
      END

      SUBROUTINE LREBIN(X,NX,XSTR,XSTP,AC,NC,Y,NY,YSTR,YSTP)
C
C
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  19:08 - 6 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR: D.PONZ
C
C.KEYWORDS:
C  SPECTROSCOPY, CASPEC, REBINNING
C
C.PURPOSE:
C  DATA REBINNING
C  CONVERSION FROM SAMPLES INTO WAVELENGTHS
C
C.ALGORITHM:
C  LINEAR INTERPOLATION. THE TRANSFORMATION WAVELENGTH INTO
C  SAMPLE NUMBER IS DEFINED BY
C  S = A0+A1*T+A2*T**2
C
C.INPUT/OUTPUT
C  INPUT ARGUMENTS
C
C  X(NX) REAL*4 INPUT ARRAY
C  NX  INTG*4 NO OF SAMPLES
C  XSTR  REAL*4 STARTING VALUE OF INPUT
C  XSTP  REAL*4 INPUT STEP
C  AC(NC) REAL*8 COEFFICIENTS OF THE TRANSFORMATION
C  NC  REAL*4 NO. OF COEFFICIENTS
C  NY  INTG*4 NO. OF OUTPUT VALUES
C  YSTR  REAL*4 STARTING VALUE OF THE INPUT
C  YSTP  REAL*4 STEP OF THE OUTPUT
C
C  OUTPUT ARGUMENTS
C
C  Y(NY) REAL*4 OUTPUT ARRAY
C
C-------------------------------------------------------------
C
C
      IMPLICIT NONE
      INTEGER NX
      INTEGER NY
      INTEGER NC
      INTEGER N
      INTEGER M1
      
      REAL X(NX)                          !In   frame
      REAL Y(NY)                          !Out  frame
      
      DOUBLE PRECISION AC(NC)             !Buffer for data containing coeffd
      DOUBLE PRECISION A0                 !Auxilliary variable
      DOUBLE PRECISION A1                 !Auxilliary variable
      DOUBLE PRECISION A2                 !Auxilliary variable
      DOUBLE PRECISION A3                 !Auxilliary variable
      DOUBLE PRECISION A4                 !Auxilliary variable
      DOUBLE PRECISION A5                 !Auxilliary variable
      DOUBLE PRECISION FN                 !Auxilliary variable
      
      DOUBLE PRECISION FACTOR             !Auxilliary variable
      DOUBLE PRECISION CNORM              !Auxilliary variable
      DOUBLE PRECISION START              !Auxilliary variable
      DOUBLE PRECISION PN                 !Auxilliary variable
      DOUBLE PRECISION DELT               !Auxilliary variable
      DOUBLE PRECISION STARTL             !Auxilliary variable
      
      DOUBLE PRECISION TM                 !Auxilliary variable
      DOUBLE PRECISION TMP1               !Auxilliary variable
      
      DOUBLE PRECISION IX_R8R8
      
      DOUBLE PRECISION XSTR                
      DOUBLE PRECISION XSTP
      DOUBLE PRECISION YSTR
      DOUBLE PRECISION YSTP
      
C
      INTEGER K                          !Auxilliary variable
      INTEGER MPT                        !Auxilliary variable
      
      DOUBLE PRECISION PNP1              !Auxilliary variable
      DOUBLE PRECISION C                 !Auxilliary variable
      DOUBLE PRECISION VAL               !Auxilliary variable
      DOUBLE PRECISION COV               !Auxilliary variable
      DOUBLE PRECISION DSQRT             !Auxilliary variable
      DOUBLE PRECISION DMIN1             !Auxilliary variable
      DOUBLE PRECISION DMAX1             !Auxilliary variable
      
      DOUBLE PRECISION XN                !Auxilliary variable
      DOUBLE PRECISION XNP1              !Auxilliary variable
      DOUBLE PRECISION COVX              !Auxilliary variable
C
C WPOLY(FL) = FL*(A1+FL*(A2+FL*(A3+FL*(A4+FL*A5))))
C
C ... ASSIGN COEFFS
C
      A0     = AC(1)
      A1     = -AC(2)
      A2     = AC(3)
      A3     = A1*A1
      A4     = 4.D0*A2
      A5     = 2.D0*A2
      CNORM  = 10.D0/ (A5*YSTP)
C A0 = AC(1)
C A1 = AC(2)
C A2 = AC(3)
C
C ... CHECK FOR OUTPUT START
C
      START  = YSTR - 0.5D0*YSTP
C STARTL= START - A0
      STARTL = START
      M1     = 1
      FN     = 0.5D0
      XN     = IX_R8R8(FN,XSTR,XSTP)

C PN    = WPOLY(IX_R8R8(FN,XSTR,XSTP))
      PN     = 10.D0* (A1+DSQRT(A3-A4* (A0-XN)))/A5
      DELT   = PN - STARTL
      IF (DELT.GT.0.) THEN
          M1     = DELT/YSTP + 1.D0
          TMP1   = STARTL + M1*YSTP
          TM     = TMP1 - YSTP
      ELSE
          TMP1   = STARTL + YSTP
          TM     = STARTL
      END IF
C
C ... CHECK FOR INPUT START
C
      DO 10 N = 1,NX
          FN     = FN + 1.D0
          XNP1   = IX_R8R8(FN,XSTR,XSTP)

C   PNP1 = WPOLY(IX_R8R8(FN,XSTR,XSTP))
          PNP1   = 10.D0* (A1+DSQRT(A3-A4* (A0-XNP1)))/A5
          IF (PNP1.GT.TM) GO TO 20
          PN     = PNP1
          XN     = XNP1
   10 CONTINUE
      N      = NX + 1
      GO TO 30
   20 C      = X(N)
   30 VAL    = 0.D0
      COVX   = XNP1 - XN
      COV    = (PNP1 - PN)/COVX
C
C ... REBINNING LOOP
C
      MPT    = 1
      K      = 0
C   35 CONTINUE
          IF ( .NOT. (MPT.LE.NY)) GO TO 80
          K      = K + 1
          IF ((K.LT.M1) .OR. (N.GT.NX)) GO TO 60
   40     VAL    = VAL + C* (DMIN1(TMP1,PNP1)-DMAX1(TM,PN))/COV
          IF (TMP1.LT.PNP1) GO TO 50
          N      = N + 1
          IF (N.GT.NX) GO TO 60
          PN     = PNP1
          XN     = XNP1
          FN     = FN + 1.D0
          XNP1   = IX_R8R8(FN,XSTR,XSTP)
C         PNP1 = WPOLY(IX_R8R8(FN,XSTR,XSTP))
          PNP1   = 10.D0* (A1+DSQRT(A3-A4* (A0-XNP1)))/A5

          C      = X(N)

          COVX   = XNP1 - XN
          COV    = (PNP1 - PN)/COVX

          GO TO 40
   50     TM     = TMP1
          TMP1   = TMP1 + YSTP
   60     FACTOR = CNORM*(DSQRT(A3-A4* (A0-XNP1))-
     +             DSQRT(A3-A4* (A0-XNP1-1.D0)))
          Y(MPT) = VAL*FACTOR
          MPT    = MPT + 1
          VAL    = 0.D0
   80 CONTINUE
      DO 90 K = MPT + 1,NY
          Y(K)   = 0.
   90 CONTINUE
      RETURN
      END

