C $Id: nr_fun.for,v 1.2 2001/11/01 09:30:19 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)nr_fun.for	2.4 (ESO-DFS) 23/10/01 12:26:19
C derived from necopt.for
C
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  subroutines  nr_fun.for  version 1.0 011023 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C See Role entry in each listed function
C
C.SYNTAX
C
C.HISTORY:
C  2001.10.23-AM Created
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.2 $
C $Log: nr_fun.for,v $
C Revision 1.2  2001/11/01 09:30:19  amodigli
C Eliminated several warnings from ftnchek
C
C Revision 1.1  2001/11/01 07:18:37  amodigli
C First version of necopt with splitted data
C
C
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     23/10/01       Created                           all

C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     zbrent
C------------------------------------------------------------------------------
C  Role: finds roots of equations. ZBRENT uses a combination of methods to give
C        assured and relatively efficient convergence.
C------------------------------------------------------------------------------

c
c Some numerical recipe routines:
c     
      FUNCTION zbrent(func,x1,x2,tol,
     $     PR,FBVAL,VAR,MASK,GPAR,NPIXA1,DIM,CHI2MIN,INDX)

      IMPLICIT NONE
      INTEGER ITMAX

      REAL zbrent,tol,x1,x2,func,EPS

      EXTERNAL func

      PARAMETER (ITMAX=100,EPS=3.e-8)

      INTEGER iter

      REAL a,b,c,d,e,fa,fb,fc,p,q,r,s,tol1,xm

      INTEGER NPIXA1, DIM, INDX
      INTEGER*2 MASK(NPIXA1,DIM)

      REAL PR(DIM),FBVAL(NPIXA1,DIM),VAR(DIM),GPAR(10)
     $     ,CHI2MIN

      a=x1
      b=x2
      fa=func(a, PR,FBVAL,VAR,MASK,GPAR,NPIXA1,DIM,CHI2MIN,INDX)
      fb=func(b, PR,FBVAL,VAR,MASK,GPAR,NPIXA1,DIM,CHI2MIN,INDX)

      if((fa.gt.0..and.fb.gt.0.).or.(fa.lt.0..and.fb.lt.0.))pause
     *'root must be bracketed for zbrent'
      c=b
      fc=fb

      do 11 iter=1,ITMAX

        if((fb.gt.0..and.fc.gt.0.).or.(fb.lt.0..and.fc.lt.0.))then
          c=a
          fc=fa
          d=b-a
          e=d
        endif

        if(abs(fc).lt.abs(fb)) then
          a=b
          b=c
          c=a
          fa=fb
          fb=fc
          fc=fa
        endif

        tol1=2.*EPS*abs(b)+0.5*tol
        xm=.5*(c-b)

        if(abs(xm).le.tol1 .or. fb.eq.0.)then
          zbrent=b
          return
        endif

        if(abs(e).ge.tol1 .and. abs(fa).gt.abs(fb)) then
          s=fb/fa

          if(a.eq.c) then
            p=2.*xm*s
            q=1.-s
          else
            q=fa/fc
            r=fb/fc
            p=s*(2.*xm*q*(q-r)-(b-a)*(r-1.))
            q=(q-1.)*(r-1.)*(s-1.)
          endif

          if(p.gt.0.) q=-q

          p=abs(p)

          if(2.*p .lt. min(3.*xm*q-abs(tol1*q),abs(e*q))) then
            e=d
            d=p/q
          else
            d=xm
            e=d
          endif

        else
          d=xm
          e=d
        endif

        a=b
        fa=fb

        if(abs(d) .gt. tol1) then
          b=b+d
        else
          b=b+sign(tol1,xm)
        endif

        fb=func(b, PR,FBVAL,VAR,MASK,GPAR,NPIXA1,DIM,CHI2MIN,INDX)
11    continue

      pause 'zbrent exceeding maximum iterations'
      zbrent=b


      return
      END

C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     erf
C------------------------------------------------------------------------------
C  Role:
C------------------------------------------------------------------------------


C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.

      FUNCTION erf(x)

      IMPLICIT NONE
      REAL erf,x

CU    USES gammp

      REAL gammp

      if(x.lt.0.)then
        erf=-gammp(.5,x**2)
      else
        erf=gammp(.5,x**2)
      endif

      return
      END

C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     gammp
C------------------------------------------------------------------------------
C  Role:
C------------------------------------------------------------------------------

C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.
      FUNCTION gammp(a,x)

      IMPLICIT NONE
      REAL a,gammp,x

CU    USES gcf,gser

      REAL gammcf,gamser,gln

      if(x.lt.0..or.a.le.0.) pause 'bad arguments in gammp'

      if(x.lt.a+1.)then
        call gser(gamser,a,x,gln)
        gammp=gamser
      else
        call gcf(gammcf,a,x,gln)
        gammp=1.-gammcf
      endif


      return
      END


C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     gser
C------------------------------------------------------------------------------
C  Role:
C------------------------------------------------------------------------------

C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.
      SUBROUTINE gser(gamser,a,x,gln)

      IMPLICIT NONE
      INTEGER ITMAX

      REAL a,gamser,gln,x,EPS

      PARAMETER (ITMAX=100,EPS=3.e-7)

CU    USES gammln

      INTEGER n

      REAL ap,del,sum,gammln


      gln=gammln(a)


      if(x.le.0.)then
        if(x.lt.0.)pause 'x < 0 in gser'
        gamser=0.
        return
      endif

      ap=a
      sum=1./a
      del=sum

      do 11 n=1,ITMAX
        ap=ap+1.
        del=del*x/ap
        sum=sum+del
        if(abs(del).lt.abs(sum)*EPS) goto 1
11    continue
      pause 'a too large, ITMAX too small in gser'

1     gamser=sum*exp(-x+a*log(x)-gln)

      return
      END

C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     gcf
C------------------------------------------------------------------------------
C  Role:
C------------------------------------------------------------------------------

C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.
      SUBROUTINE gcf(gammcf,a,x,gln)

      IMPLICIT NONE
      INTEGER ITMAX

      REAL a,gammcf,gln,x,EPS,FPMIN

      PARAMETER (ITMAX=100,EPS=3.e-7,FPMIN=1.e-30)

CU    USES gammln

      INTEGER i

      REAL an,b,c,d,del,h,gammln


      gln=gammln(a)
      b=x+1.-a
      c=1./FPMIN
      d=1./b
      h=d

      do 11 i=1,ITMAX
        an=-i*(i-a)
        b=b+2.
        d=an*d+b

        if(abs(d).lt.FPMIN)d=FPMIN
        c=b+an/c

        if(abs(c).lt.FPMIN)c=FPMIN

        d=1./d
        del=d*c
        h=h*del

        if(abs(del-1.).lt.EPS)goto 1

11    continue

      pause 'a too large, ITMAX too small in gcf'
1     gammcf=exp(-x+a*log(x)-gln)*h

      return
      END

C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     gammln
C------------------------------------------------------------------------------
C  Role: Natural logarithm of the gamma function for positive arguments
C------------------------------------------------------------------------------
C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.
      FUNCTION gammln(xx)
      IMPLICIT NONE

      REAL gammln,xx

      INTEGER j

      DOUBLE PRECISION ser,stp,tmp,x,y,cof(6)

      SAVE cof,stp

      DATA cof,stp/76.18009172947146d0,-86.50532032941677d0,
     *24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
     *-.5395239384953d-5,2.5066282746310005d0/


      x=DBLE(xx)
      y=x
      tmp=x+5.5d0
      tmp=(x+0.5d0)*log(tmp)-tmp
      ser=1.000000000190015d0

      do 11 j=1,6
        y=y+1.d0
        ser=ser+cof(j)/y
11    continue

      gammln=SNGL(tmp+log(stp*ser/x))


      return
      END

C==================================================================================
