C $Id: qcortra.for,v 1.2 2002/01/24 14:57:30 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)echopt.for	2.4 (ESO-IPG) 12/6/91 12:26:19
C+
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  program  ECHOPT  version 1.0 910120 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C 
C Extract orders from an echelle spectrum, the order position is defined by 
C the coefficients of a regression. The slit width and the orders to be 
C extracted are defined by the keyword INPUTI
C The flux is calculated by assigning to each pixel a weighting factor 
C proportional to the amount of signal in it.
C.ALGORITHM
C The algorithm is based on a paper of Koji Mukai ( 1990, Optimal Extraction of
C Cross-Dispersed Spectra, Pub. of Astr.Soc. of the Pacific,102:183-189)
C AND a paper of Keith Horne:
C (1986, Pub. of Astr.Soc. of the Pacific, 98:609-617)
C
C.SYNTAX
C   EXTRACT/ORDER input output  width,ord1,ord2 ron,g,sigma table coeffs
C
C  the connection file contains
C  IN_A      input FRM (has to be de-biased BEFORE)
C  OUTPUTC   variance image of input data (same dimension as input frame)
C  OUT_A     output FRM
C  OUT_B     weight map FRM
C  IN_B      table with positions of the orders
C  INPUTC    coeffs
C  INPUTI    slit,ord1,ord2, weight_determination_flag
C  INPUTR    ron,g,sigma (ron: readout-noise, g: inverse gain,
C                         sigma: threshold for cosmic ray rejection)
C
C.HISTORY:
C  2001.11.09-AM Created from necopt.for 
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.2 $
C $Log: qcortra.for,v $
C Revision 1.2  2002/01/24 14:57:30  amodigli
C Eliminated check for error status as is not
C proper to do in MIDAS.
C STDWRI needs to dump 'string' not "string"!
C (this was a portability problem to Linux).
C
C Revision 1.1  2001/11/09 18:01:42  amodigli
C Created extra modules as part of DFS
C code rev step 2
C
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     09/11/01      Created from necopt.for           all
C------------------------------------------------------------------------------
C  Role: Initialize the order tracing table

C==================================================================================
C=====SUBROUTINE TO CREATE TABLE TO MONITOR ORDER TRACING==========================
C==================================================================================      
      SUBROUTINE ORDTRA(NPIXB,DIMP)
      
      IMPLICIT NONE
      INTEGER NSIZIM                    !No of elements for ima descriptors
      PARAMETER (NSIZIM=3)              !No of elements for ima descriptors 
C     Interface variables

      INTEGER NPIXB(NSIZIM)             !No. of pix in ima (MIDAS env int.) 
      INTEGER DIMP                      !dimention of weight profile

C     common variables
      INTEGER DTID1                     !Table id (MIDAS env int.)
      INTEGER DTID2                     !Table id (MIDAS env int.)                  
C     Local variables      
      INTEGER ICOL1                     !Column No.(MIDAS env int.)
      INTEGER IS                        !Status of operation (MIDAS env int.)
      INTEGER DUN                       !Units (MIDAS env int.)      
      

        
      INCLUDE 'MID_INCLUDE:ST_DEF.INC'   !to get the definitions 
                                         !on the ST interfaces
      INCLUDE '../incl/common_dbgort.inc'  
      INCLUDE '../incl/common_vmr.inc'                                          

      INCLUDE 'MID_INCLUDE:ST_DAT.INC'   !to get data values for 
                                         !definitions above
      
           
      CALL TBTINI('order_trace',F_TRANS,F_O_MODE,16,10000,DTID1,IS)  

      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F5.0','none',':ORDER',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F3.0','bin',':I_CH',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F6.0','bin',':X_CH_0',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'G8.2','bin',':P_ERR0',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'G10.4','bin',':S_CH_0',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'G10.4','bin',':S_CH_1',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','bin',':P_FWHM',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'G10.4','bin',':P_FWHM_ERR',
     +            ICOL1,IS)

      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','bin',':P_CH',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','bin',':P_F0',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','bin',':P_F1',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F2.0','none',':M0',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F2.0','none',':M1',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F3.0','none',':DEG',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'G10.4','none',':CHI0',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F3.0','none',':KS',ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','counts',':BKGVAL',
     +            ICOL1,IS)
      CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','counts',':BKGERR',
     +            ICOL1,IS)

      CALL STDWRI(DTID1,'ESO.QC.OPTEX.XSIZE',NPIXB(1),1,1,DUN,IS)
      CALL STDWRI(DTID1,'ESO.QC.OPTEX.YSIZE',DIMP,1,1,DUN,IS)
      CALL STDWRI(DTID1,'ESO.QC.OPTEX.NORD',NPIXB(2),1,1,DUN,IS)

      CALL TBTINI('xorder_trace',F_TRANS,F_O_MODE,4,200000,DTID2,IS)
      CALL TBCINI(DTID2,D_R8_FORMAT,1,'F5.0','none',':ORDER',ICOL1,IS)  
      CALL TBCINI(DTID2,D_R8_FORMAT,1,'F5.0','none',':I_CH',ICOL1,IS)
      CALL TBCINI(DTID2,D_R8_FORMAT,1,'F5.0','bin',':Y_CH',ICOL1,IS)
      CALL TBCINI(DTID2,D_R8_FORMAT,1,'F5.0','ADU',':FLUX',ICOL1,IS)

      CALL STDWRI(DTID1,'ESO.QC.OPTEX.XSIZE',NPIXB(1),1,1,DUN,IS)
      CALL STDWRI(DTID1,'ESO.QC.OPTEX.YSIZE',DIMP,1,1,DUN,IS)
      CALL STDWRI(DTID2,'ESO.QC.OPTEX.NORD',NPIXB(2),1,1,DUN,IS)
              
      RETURN
      END
