C $Id: vsampl.for,v 1.2 2001/11/01 09:30:20 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)vsample.for	1.0 (ESO-DFS) 23/10/01 12:26:19
C+ derived from necopt.for
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  program  ECHOPT  version 1.0 9011023 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C See Role below
C.SYNTAX
C
C.HISTORY:
C  2001.10.23-AM  Created
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.2 $
C $Log: vsampl.for,v $
C Revision 1.2  2001/11/01 09:30:20  amodigli
C Eliminated several warnings from ftnchek
C
C Revision 1.1  2001/11/01 07:18:38  amodigli
C First version of necopt with splitted data
C
C
C------------------------------------------------------------------------------

C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     VSAMPL
C  AM     29/05/01       Corrected typo in IKIND info      EXTFF
C------------------------------------------------------------------------------
C  Role: Does the sampling in the virtual space
C------------------------------------------------------------------------------
      

      SUBROUTINE VSAMPL(IN,PROF,YVAL,FBVAL,MASK,DIMP,
     1     IWIDTH,BINSIZE,NPIXA1,IXMIN,IXMAX,NPIXA2,DIM,IKIND)
C
C     Do the sampling in the virtual space.
C
      IMPLICIT NONE

      INTEGER IWIDTH                 !size of slit in pixel
      INTEGER DIM                    !Y size of ima
      INTEGER BINSIZE                !Bin size
      INTEGER DIMP                   !profile dimension
      INTEGER NPIXA1                 !X dim of ima
      INTEGER NPIXA2                 !Y dim of ima
      INTEGER IKIND                  !switch for even/odd pix No. slit width
      INTEGER IXMIN                  !index corresponding to XMIN
      INTEGER IXMAX                  !index corresponding to XMAX
      INTEGER*2 MASK(NPIXA1,DIM)     !Mask ima

      DOUBLE PRECISION YVAL(NPIXA1)  !current Y order position(X)

      REAL FBVAL(NPIXA1,DIM)         !Output Intensity at X,Y position
      REAL PROF(NPIXA1,DIMP)         !Profile at X,Y position
      REAL IN(NPIXA1,NPIXA2)         !Input raw frame

C
C     Local variables
C

      INTEGER I                      !loop variable
      INTEGER J                      !loop variable
      INTEGER IX                     !tmp val holding pix position along order
      INTEGER IND                    !tmp variable to hold values
      INTEGER IY                     !Actual order value
      INTEGER NPOINT                 !Point counter
      INTEGER K                      !loop variable
      
      REAL DX                        !Fractional pixel X discrepancy 
      REAL DY                        !Fractional pixel Y discrepancy
      REAL YDEC                      !Y position at pixel center
      REAL SUM                       !tmp sum of x-order intensity at pos X

      DOUBLE PRECISION YDVAL         !Y pos in double precision
                                     !YDVAL=YVAL(IND)-IWIDTH/2-1 + 
                                     !              (J-1.)/BINSIZE
      DOUBLE PRECISION XDVAL         !X pos in double precision
      

C
C     Initialization of local variables
C

C     Integers

      I =0
      J =0
      IX =0
      IND=0
      IY=0 
      NPOINT=0
      K=0     

C     Reals
      
      DX=0
      DY=0
      YDEC=0
      SUM=0

C     Doubles

      YDVAL=0.0D0
      XDVAL=0.0D0




      
      DO J=1,DIM
         DO I=IXMIN,IXMAX
            FBVAL(I,J) = 0
            MASK(I,J) = 0
         ENDDO
      ENDDO

      DO J=1,DIMP
         DO I=IXMIN,IXMAX
            PROF(I,J) = 0
         ENDDO
      ENDDO
      
       
      
      IF (IKIND.EQ.0) THEN
      
         DO I = IXMIN,IXMAX
            SUM = 0
c change?

            K = INT(YVAL(I)-DBLE(IWIDTH/2.0)-1) 
            
c     am: Found the error: 
c         if K<0 (as when for order 1 at the botton of the detector=> 
c         YVAL(I)<IWIDTH+1) the array IN(I,K) explodes
c         so we try to skip this case with the following IF  
c         do the same as before but starting from k=0 (bottom of detector)   
            IF (K .LT. 1) THEN
               DO  J = 1,DIMP
                  PROF(I,J) = IN(I,J)
               ENDDO
               GOTO 10 
            ENDIF  
                         
            PROF(I,1) = IN(I,K)         
            
            DO  J = K+1,K+DIMP-2
               PROF(I,J-K+1) = IN(I,J)
               SUM = SUM+IN(I,J)
            ENDDO
            
            IF ( NINT(YVAL(I)).EQ.INT(YVAL(I))) THEN
               SUM = SUM + IN(I,K)
            ELSE
               SUM = SUM + IN(I,K+DIMP-1)
            ENDIF  
                      
            PROF(I,DIMP) = IN(I,K+DIMP-1)
            
c            IF (SUM.EQ.0)  SUM=1
c            DO J = K,K+DIMP-1
c               PROF(I,J-K+1) = PROF(I,J-K+1)/SUM
c            ENDDO

c      am: added the following CONTINUE to escape from loop
c          in case K<0

   10    CONTINUE
         ENDDO
      ELSE
      
         DO I=IXMIN,IXMAX
         
            SUM = 0
c change?
            K = INT(YVAL(I)-DBLE(IWIDTH/2.0))
            DO J = K,K+DIMP-1
               PROF(I,J-K+1) = IN(I,J)
               SUM = SUM+IN(I,J)
            ENDDO

c            IF (SUM.EQ.0) SUM = 1  
c            DO J = K,K+DIMP-1
c               PROF(I,J-K+1) = PROF(I,J-K+1)/SUM
c            ENDDO

         ENDDO
      ENDIF
      
      IF (IKIND.EQ.0) THEN



         DO J=1,DIM
            NPOINT =0
            DO I= IXMIN,IXMAX
               XDVAL = I
               IND = I
c change?
               K = INT(YVAL(I)-DBLE(IWIDTH/2)-1)
               



            IF (K .LT. 1) THEN
              GOTO 20 
            ENDIF  
            
                                  
               YDVAL = YVAL(IND)-DBLE(IWIDTH/2.0)-1 + 
     !                      DBLE((J-1.)/BINSIZE)
               IX = INT(XDVAL)
               YDEC = INT(YDVAL)+0.5
               DX = SNGL(XDVAL-IX)
               DY = SNGL(YDVAL-DBLE(YDEC))
               IY  = INT(YDVAL)
               IF (DY.GT.-0.5.AND.DY.LE.0) THEN
                  IF ((IY-K).GT.0) THEN
                     FBVAL(I,J) =  (1+DY)*PROF(IX,IY-K+1) -
     1                    DY*PROF(IX,IY-K)
                     MASK(I,J) = 1
                     NPOINT = NPOINT+1
                  ENDIF
               ELSE IF (DY.LT.0.5.AND.DY.GE.0) THEN
                  IF ((IY-K+1).LT.DIMP) THEN
                     FBVAL(I,J) =  (1-DY)*PROF(IX,IY-K+1) +
     1                    DY*PROF(IX,IY-K+2)
                     MASK(I,J) = 1
                     NPOINT = NPOINT+1
                  ENDIF
               ENDIF

c           am: added to allow to skip if K .LT. 1

20          CONTINUE   

            ENDDO
         ENDDO
      ELSE




         DO J=1,DIM
            NPOINT =0
            DO I= IXMIN,IXMAX
               XDVAL = I
               IND = I
c change?
               K = INT(YVAL(I)-DBLE(IWIDTH/2.0))
               YDVAL = YVAL(IND)-DBLE(IWIDTH/2)-1 + DBLE((J-1.)/BINSIZE)
               IX = INT(XDVAL)
               YDEC = INT(YDVAL)+0.5
               DX = SNGL(XDVAL-IX)
               DY = SNGL(YDVAL-DBLE(YDEC))
               IY  = INT(YDVAL)
               IF (DY.GT.-0.5.AND.DY.LE.0.AND.
     1              (IY-K+1).LE.DIMP.AND.(IY-K).GE.0) THEN
                  IF ((IY-K).NE.0.AND.(IY-K).NE.DIMP) THEN
                     FBVAL(I,J) =  (1+DY)*PROF(IX,IY-K+1) -
     1                    DY*PROF(IX,IY-K)
                  ELSE IF((IY-K).EQ.0) THEN
                     FBVAL(I,J) = PROF(IX,IY-K+1) 
                  ELSE
                     FBVAL(I,J) = PROF(IX,IY-K)
                  ENDIF
                  MASK(I,J) = 1
                  NPOINT = NPOINT+1
               ELSE IF (DY.LT.0.5.AND.DY.GE.0.AND.
     1                 (IY-K+1).LE.DIMP.AND.(IY-K).GE.0) THEN
                  IF ((IY-K+1).NE.DIMP.AND.(IY-K+1).NE.0) THEN
                     FBVAL(I,J) =  (1-DY)*PROF(IX,IY-K+1) +
     1                    DY*PROF(IX,IY-K+2)
                  ELSE IF ((IY-K+1).EQ.DIMP) THEN
                     FBVAL(I,J) = PROF(IX,IY-K+1)
                  ELSE
                     FBVAL(I,J) = PROF(IX,IY-K+2)
                  ENDIF
                  MASK(I,J) = 1
                  NPOINT = NPOINT+1
               ENDIF
            ENDDO
         ENDDO
      ENDIF
      
      RETURN
      END
C =================================================================================
