/* ===========================================================================
 * $Id: gauss_fit.c,v 1.6 2002/08/16 15:29:02 amodigli Exp $
 * $Name:  $
 * $Revision: 1.6 $
 * $Log: gauss_fit.c,v $
 * Revision 1.6  2002/08/16 15:29:02  amodigli
 * Speed-up
 *
 * Revision 1.5  2001/02/12 10:10:00  amodigli
 * Eliminated function chknan as it was using
 * c library function isnanf not any more
 * supported from HP11. Anyway it was NOT used!
 *
 * Revision 1.4  2001/01/04 13:23:04  amodigli
 * Added Id, Name, Revision, Log conf. control info
 *
 * ===========================================================================
 */
/* GAUSS FIT
   uses: nonlfit.c
   need: mrqmin.c, gaussj.c covsrt.c
*/
#include <math.h>



void fpoly_c(x,a,y,dyda)
float x,a[5],*y,dyda[5];
{

     *y=0.0;
     *y = a[1]+
          a[2]*x+
          a[3]*x*x+
          a[4]*x*x*x+
          a[5]*x*x*x*x;

          
     dyda[1]=1;
     dyda[2]=x;
     dyda[3]=x*x;
     dyda[4]=x*x*x;
     dyda[5]=x*x*x*x;

     
}

void fgauss_c(x,a,y,dyda)
float x,a[5],*y,dyda[5];
{
  double fac=0,ex=0,arg=0,arg2=0;
  double a1_p_ex=0, fac_d_a3=0, one_d_a3=0;

  *y=0.0;
  one_d_a3 = 1.0/a[3];
  arg=(x-a[2])*one_d_a3;
  arg2=arg*arg;
  /* to Fix problems to ALPHA Linux */
  if(arg2 < 200.)
  {
     ex=exp(-arg2);
  }
  else
  {
     ex=exp(-200.0);
  }
  
  a1_p_ex = a[1]*ex;
  fac=a1_p_ex*2.0*arg;
  fac_d_a3 = fac*one_d_a3;
  *y = a1_p_ex + a[4]*(1.0+(x-a[2])*a[5]);  
  dyda[1]=ex;
  dyda[2]=fac_d_a3-a[4]*a[5];
  dyda[3]=fac_d_a3*arg;
  dyda[4]=1.0 + (x-a[2])*a[5];
  dyda[5]=a[4]*(x-a[2]);
}

/* necessary for FORTRAN call in MIDAS */
/* CALL GAUSS_FIT(...) */
/* esoext.exe will interpret this call as GAUSST !! */
int gausst_(y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *y, *sig, *param, *errp, *chisq;
int   *ndata, *npar, *mfit, *listp, *istat;
{
  gauss_fit(y-1,sig-1,*ndata,param-1,*npar,listp-1,*mfit,errp-1,chisq,istat);
}

int gausst(y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *y, *sig, *param, *errp, *chisq;
int   *ndata, *npar, *mfit, *listp, *istat;
{
  gauss_fit(y-1,sig-1,*ndata,param-1,*npar,listp-1,*mfit,errp-1,chisq,istat);
}


int gaussx(x,y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *x, *y, *sig, *param, *errp, *chisq;
int   *ndata, *npar, *mfit, *listp, *istat;
{
  gauss_fitx(x-1,y-1,sig-1,*ndata,param-1,*npar,listp-1,*mfit,errp-1,chisq,istat);
}


int gaussx_(x,y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *x, *y, *sig, *param, *errp, *chisq;
int   *ndata, *npar, *mfit, *listp, *istat;
{
  gauss_fitx(x-1,y-1,sig-1,*ndata,param-1,*npar,listp-1,*mfit,errp-1,chisq,istat);
}





int gauss_fitx(x,y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *x, *y, *sig, *param, *errp, *chisq;
int   ndata, npar, mfit, *listp, *istat;
{
  void  fgauss_c();

  *istat = nonlfit(x,y,sig,ndata,param,npar,listp,mfit,fgauss_c,errp,chisq);
}

int gauss_fit(y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *y, *sig, *param, *errp, *chisq;
int   ndata, npar, mfit, *listp, *istat;
{
  void  fgauss_c();

  *istat = non_lfit(y,sig,ndata,param,npar,listp,mfit,fgauss_c,errp,chisq);
}




/* necessary for FORTRAN call in MIDAS */
/* CALL GAUSS_FIT(...) */
/* esoext.exe will interpret this call as GAUSST !! */
int polyt_(y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *y, *sig, *param, *errp, *chisq;
int   *ndata, *npar, *mfit, *listp, *istat;
{
  poly_fit(y-1,sig-1,*ndata,param-1,*npar,listp-1,*mfit,errp-1,chisq,istat);
}

int polyt(y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *y, *sig, *param, *errp, *chisq;
int   *ndata, *npar, *mfit, *listp, *istat;
{
  poly_fit(y-1,sig-1,*ndata,param-1,*npar,listp-1,*mfit,errp-1,chisq,istat);
}

int polyfx(x,y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *x, *y, *sig, *param, *errp, *chisq;
int   *ndata, *npar, *mfit, *listp, *istat;
{
  poly_fitx(x-1,y-1,sig-1,*ndata,param-1,*npar,listp-1,*mfit,errp-1,chisq,istat);
}


int polyx(x,y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *x, *y, *sig, *param, *errp, *chisq;
int   *ndata, *npar, *mfit, *listp, *istat;
{
  poly_fitx(x-1,y-1,sig-1,*ndata,param-1,*npar,listp-1,*mfit,errp-1,chisq,istat);
}


int polyx_(x,y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *x, *y, *sig, *param, *errp, *chisq;
int   *ndata, *npar, *mfit, *listp, *istat;
{
  poly_fitx(x-1,y-1,sig-1,*ndata,param-1,*npar,listp-1,*mfit,errp-1,chisq,istat);
}


int polyfx_(x,y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *x, *y, *sig, *param, *errp, *chisq;
int   *ndata, *npar, *mfit, *listp, *istat;
{
  poly_fitx(x-1,y-1,sig-1,*ndata,param-1,*npar,listp-1,*mfit,errp-1,chisq,istat);
}

int poly_fitx(x,y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *x, *y, *sig, *param, *errp, *chisq;
int   ndata, npar, mfit, *listp, *istat;
{
  void  fpoly_c();

  *istat = nonlfit(x,y,sig,ndata,param,npar,listp,mfit,fpoly_c,errp,chisq);
}



int poly_fit(y,sig,ndata,param,npar,listp,mfit,errp,chisq,istat)
float *y, *sig, *param, *errp, *chisq;
int   ndata, npar, mfit, *listp, *istat;
{
  void  fpoly_c();

  *istat = non_lfit(y,sig,ndata,param,npar,listp,mfit,fpoly_c,errp,chisq);
}
