/* ===========================================================================
 * $Id: mrqmin.c,v 1.4 2002/08/16 15:29:55 amodigli Exp $
 * $Name:  $
 * $Revision: 1.4 $
 * $Log: mrqmin.c,v $
 * Revision 1.4  2002/08/16 15:29:55  amodigli
 * Add header description
 *
 * Revision 1.3  2001/01/04 13:23:05  amodigli
 * Added Id, Name, Revision, Log conf. control info
 *
 * ===========================================================================
 */
int mrqmin(x,y,sig,ndata,a,ma,lista,mfit,covar,alpha,chisq,funcs,alamda)
float x[],y[],sig[],a[],**covar,**alpha,*chisq,*alamda;
int ndata,ma,lista[],mfit;
void (*funcs)();
/* Lavenberg-Marquardt method, attempting to reduce the value of chi2 of a fit between a set of
   NDATA points X(I),Y(I) with individual standard deviations SIG(I), and a non linear function 
   dependent on MA coefficients A. The array LISTA numbers the parameters A such that the first
   MFIT elements correspond to values actually being adjusted; the remaining MA-FIT parameters 
   are held fixed at their input value. The program so turns current best-fit values for the MA 
   fit parameters A and CHI2, CHISQ. The arrays COVAR(NCA,NCA), ALPHA(NCA,NCA) with physical 
   dimension NCA (>= MFIT) are used as working space during most iterations. Supply a subroutine 
   FUNCS(X,A,YFIT,DYDA,NA) that evaluates the fitting function YFIT, and its derivatives DYDA 
   with respect to the fitting parameters A at X. On the first call provide an initial guess 
   for the parameters A, and set ALAMBDA<0 for initialization (which then sets ALAMBDA=0.001).
   If a step succeeds CHISQ becomes smaller and ALAMDA decreases by a factor 10. If a step
   fails ALAMBDA grows by a factor 10. You must call this routine repeatedly untill convergence 
   is achieved. Then make a final call with ALAMDA=0, so that COVAR(I,J) returns the covariance
   matrix, and ALPHA(I,J) the curvature matrix.
*/
{
	int k,kk,j,ihit,err;
	static float *da,*atry,**oneda,*beta,ochisq;
	float *vector(),**matrix();
	void mrqcof(),covsrt(),nrerror(),free_matrix(),free_vector();
	int gaussj();

	err = 0;
	if (*alamda < 0.0) {
		oneda=matrix(1,mfit,1,1);
		atry=vector(1,ma);
		da=vector(1,ma);
		beta=vector(1,ma);
		kk=mfit+1;
		for (j=1;j<=ma;j++) {
			ihit=0;
			for (k=1;k<=mfit;k++)
				if (lista[k] == j) ihit++;
			if (ihit == 0)
				lista[kk++]=j;
			else if (ihit > 1) nrerror("Bad LISTA permutation in MRQMIN-1");
		}
		if (kk != ma+1) nrerror("Bad LISTA permutation in MRQMIN-2");
		*alamda=0.001;
		mrqcof(x,y,sig,ndata,a,ma,lista,mfit,alpha,beta,chisq,funcs);
		ochisq=(*chisq);
	}
	for (j=1;j<=mfit;j++) {
		for (k=1;k<=mfit;k++) covar[j][k]=alpha[j][k];
		
		/* To fix portability problem to ALPHA Linux */
		if ( *alamda >= 1.0e30 ) *alamda = 1.0e30;
		
		covar[j][j]=alpha[j][j]*(1.0+(*alamda));
		oneda[j][1]=beta[j];
	}
	err = gaussj(covar,mfit,oneda,1);
	for (j=1;j<=mfit;j++)
		da[j]=oneda[j][1];
	if (*alamda == 0.0) {
		covsrt(covar,ma,lista,mfit);
		free_vector(beta,1,ma);
		free_vector(da,1,ma);
		free_vector(atry,1,ma);
		free_matrix(oneda,1,mfit,1,1);
		return(err);
	}
	for (j=1;j<=ma;j++) atry[j]=a[j];
	for (j=1;j<=mfit;j++)
		atry[lista[j]] = a[lista[j]]+da[j];
	mrqcof(x,y,sig,ndata,atry,ma,lista,mfit,covar,da,chisq,funcs);
	if (*chisq < ochisq) {
		*alamda *= 0.1;
		ochisq=(*chisq);
		for (j=1;j<=mfit;j++) {
			for (k=1;k<=mfit;k++) alpha[j][k]=covar[j][k];
			beta[j]=da[j];
			a[lista[j]]=atry[lista[j]];
		}
	} else {
		*alamda *= 10.0;
		*chisq=ochisq;
	}
	return(err);
}



/* mrq_min() is the same as mrqmin() except for that x is assumed to be
   an array starting at 1 and having a step size of 1: 1,2,3,4,...,ndata.
   As x is only used in mrqcof or mrq_cof() respectively, the changes had
   only to be done in mrq_cof()!
 */
int mrq_min(y,sig,ndata,a,ma,lista,mfit,covar,alpha,chisq,funcs,alamda)
float y[],sig[],a[],**covar,**alpha,*chisq,*alamda;
int ndata,ma,lista[],mfit;
void (*funcs)();
{
	int k,kk,j,ihit,err;
	static float *da,*atry,**oneda,*beta,ochisq;
	float *vector(),**matrix();
	void mrqcof(),covsrt(),nrerror(),free_matrix(),free_vector();
	int gaussj();

	if (*alamda < 0.0) {
		oneda=matrix(1,mfit,1,1);
		atry=vector(1,ma);
		da=vector(1,ma);
		beta=vector(1,ma);
		kk=mfit+1;
		for (j=1;j<=ma;j++) {
			ihit=0;
			for (k=1;k<=mfit;k++)
				if (lista[k] == j) ihit++;
			if (ihit == 0)
				lista[kk++]=j;
			else if (ihit > 1) nrerror("Bad LISTA permutation in MRQMIN-1");
		}
		if (kk != ma+1) nrerror("Bad LISTA permutation in MRQMIN-2");
		*alamda=0.001;
		mrq_cof(y,sig,ndata,a,ma,lista,mfit,alpha,beta,chisq,funcs);
		ochisq=(*chisq);
	}
	for (j=1;j<=mfit;j++) {
		for (k=1;k<=mfit;k++) covar[j][k]=alpha[j][k];
		covar[j][j]=alpha[j][j]*(1.0+(*alamda));
		oneda[j][1]=beta[j];
	}
	err = gaussj(covar,mfit,oneda,1);
	for (j=1;j<=mfit;j++)
		da[j]=oneda[j][1];
	if (*alamda == 0.0) {
		covsrt(covar,ma,lista,mfit);
		free_vector(beta,1,ma);
		free_vector(da,1,ma);
		free_vector(atry,1,ma);
		free_matrix(oneda,1,mfit,1,1);
		return(err);
	}
	for (j=1;j<=ma;j++) atry[j]=a[j];
	for (j=1;j<=mfit;j++)
		atry[lista[j]] = a[lista[j]]+da[j];
	mrq_cof(y,sig,ndata,atry,ma,lista,mfit,covar,da,chisq,funcs);
	if (*chisq < ochisq) {
		*alamda *= 0.1;
		ochisq=(*chisq);
		for (j=1;j<=mfit;j++) {
			for (k=1;k<=mfit;k++) alpha[j][k]=covar[j][k];
			beta[j]=da[j];
			a[lista[j]]=atry[lista[j]];
		}
	} else {
		*alamda *= 10.0;
		*chisq=ochisq;
	}
	return(err);
}
