/* ===========================================================================
 * $Id: nonlfit.c,v 1.3 2002/08/16 15:30:46 amodigli Exp $
 * $Name:  $
 * $Revision: 1.3 $
 * $Log: nonlfit.c,v $
 * Revision 1.3  2002/08/16 15:30:46  amodigli
 * Fix a memory leak
 *
 * Revision 1.2  2001/01/04 13:23:06  amodigli
 * Added Id, Name, Revision, Log conf. control info
 *
 * ===========================================================================
 */
#include <string.h>
#include <stdio.h>
#include <tblsys.h>
#include <tblerr.h>
#include <tbldef.h>
#include <macrogen.h>
#include <atype.h>
#include <midas_def.h>
#include <math.h>
/* #include <sunmath.h> */
#include "nrutil.h"

#define TRUE 1
#define FALSE 0
#define MLEN  1024
#define IMAX  300

int   Mlev;
char  Mes[MLEN];


int nonlfit(x,y,sig,ndata,param,npar,listp,mfit,funcs,errp,chi2)
float *x, *y, *sig, *param, *errp, *chi2;
int   *listp;
int   ndata, npar, mfit;
void (*funcs)();
{
  int    i, err, tmp_err;
  float  **covar;
  float  **alpha;
  float  ochisq, chisq, alamda;


  covar = matrix(1,npar,1,npar);
  alpha = matrix(1,npar,1,npar);


  alamda = -1.0;   /* initialization */
  err = mrqmin(x,y,sig,ndata,param,npar,listp,mfit,
	       covar,alpha,&chisq,funcs,&alamda);
  if (err < 0){
    /* to fix an leack error */
    tmp_err = err;
    alamda = 0.0;
    err = mrqmin(x,y,sig,ndata,param,npar,listp,mfit,
	       covar,alpha,&chisq,funcs,&alamda);
    err = tmp_err;
    /* end fix */

    free_matrix(covar,1,npar,1,npar);
    free_matrix(alpha,1,npar,1,npar);
    return(err);
  }

  i = 0;
  do
    {
      ochisq = chisq;

      err=mrqmin(x,y,sig,ndata,param,npar,listp,mfit,
		 covar,alpha,&chisq,funcs,&alamda);
      if (err < 0){
        /* to fix an leack error */
        tmp_err = err;
        alamda = 0.0;
        err = mrqmin(x,y,sig,ndata,param,npar,listp,mfit,
	            covar,alpha,&chisq,funcs,&alamda);
        err = tmp_err;
        /* end fix */

	free_matrix(covar,1,npar,1,npar);
	free_matrix(alpha,1,npar,1,npar);
	return(err);
      }
      i++;
    }while ((alamda > 1.E-6) && (alamda < 1.E38) && (i<IMAX));

  /* Calculate covariance and curvature matrix. */

  alamda=0.0;
  err=mrqmin(x,y,sig,ndata,param,npar,listp,mfit,
	     covar,alpha,&chisq,funcs,&alamda);

  for (i=1;i<=npar;i++) errp[i] = covar[i][i];
  *chi2 = chisq;

  free_matrix(covar,1,npar,1,npar);
  free_matrix(alpha,1,npar,1,npar);
  return(err);
}



/* the following macro has to be done for FORTRAN codes
   as the linker tries to find nonlft instead of non_lfit!
   This is due to the esoext.exe pre-compiler!!
 */
int nonlft(y,sig,ndata,param,npar,listp,mfit,funcs,chi2)
float *y, *sig, *param, *chi2;
int   *listp;
int   *ndata, *npar, *mfit;
void (*funcs)();
{
  non_lfit(y,sig,*ndata,param,*npar,listp,*mfit,funcs,chi2);
}

int non_lfit(y,sig,ndata,param,npar,listp,mfit,funcs,errp,chi2)
float *y, *sig, *param, *errp, *chi2;
int   *listp;
int   ndata, npar, mfit;
void (*funcs)();
{
  int    i, err;
  float  **covar;
  float  **alpha;
  float  ochisq, chisq, alamda;

  covar = matrix(1,npar,1,npar);
  alpha = matrix(1,npar,1,npar);

  alamda = -1.0;   /* initialization */
  err = mrq_min(y,sig,ndata,param,npar,listp,mfit,
	       covar,alpha,&chisq,funcs,&alamda);
  if (err < 0) {
    free_matrix(covar,1,npar,1,npar);
    free_matrix(alpha,1,npar,1,npar);
    return(err);
  }

  i = 0;
  do
    {
      ochisq = chisq;

      err=mrq_min(y,sig,ndata,param,npar,listp,mfit,
		 covar,alpha,&chisq,funcs,&alamda);
      if (err < 0){
	free_matrix(covar,1,npar,1,npar);
	free_matrix(alpha,1,npar,1,npar);
	return(err);
      }
      i++;
    }while ((alamda > 1.E-6) && (alamda < 1.E38) && (i<IMAX));

  /* Calculate covariance and curvature matrix. */
  alamda=0.0;
  err=mrq_min(y,sig,ndata,param,npar,listp,mfit,
	     covar,alpha,&chisq,funcs,&alamda);

  for (i=1;i<=npar;i++) errp[i] = covar[i][i];
  /*
  for (i=1;i<=npar;i++) printf("%f ",errp[i]);
  printf("chisq: %f\n",chisq/ndata);
  */
  *chi2 = chisq;

  free_matrix(covar,1,npar,1,npar);
  free_matrix(alpha,1,npar,1,npar);
  return(err);
}


int calculfit(x, y, fit, res, funcs, param, num, npar)
float *x, *y, *fit, *res, *param;
int   num, npar;
void (*funcs)();
{
  int i;
  float *dy;

  dy = vector(1,npar);
  for (i=1;i<=num;i++){
    (*funcs)(x[i], param, &fit[i], dy);
    /*    printf("%d %f %f %f\n",i,x[i],y[i],fit[i]); */
    res[i] = y[i] - fit[i];
  }
  free_dvector(dy,1,npar);
}



int calcul_fit(y, fit, res, funcs, param, num, npar)
float *y, *fit, *res, *param;
int   num, npar;
void (*funcs)();
{
  int i;
  float x, *dy;

  dy = vector(1,npar);
  for (i=1;i<=num;i++){
    x = i;
    (*funcs)(x, param, &fit[i], dy);
    /*    printf("%d %f %f %f\n",i,x[i],y[i],fit[i]); */
    res[i] = y[i] - fit[i];
  }
  free_dvector(dy,1,npar);
}


int show_param(param,listp,num,mfit)
float *param;
int   *listp, num, mfit;
{
  int i;
  
  sprintf(Mes,"Initial parameter set:\n\t\t\tVALUE\t\tUSED-FLAG");
  for (i=1;i<=num;i++){
    sprintf(&Mes[strlen(Mes)],"\nparameter_%d:\t%f\t%d",i,param[i],listp[i]);
  }
    
  sprintf(&Mes[strlen(Mes)],"\n(USED-FLAG: 0 indicates a fixed parameter)");
  /*  messout(Mlev,4,"nonlfit",Mes); */
  SCTPUT(Mes);
}
