! $Id: uves_align.prg,v 1.3 2001/01/04 10:39:43 amodigli Exp $
!+++ uves_align +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_align.prg
!
!.KEYWORDS      Get reference spectrum of a list of standard stars
!
!.USAGE         @p (IDENT) input reflist accur
!
!.INPUT         input   (P1): name of input frame
!                             standard star exposure
!               reflist (P2): reference table of standard stars
!                             3D table with standard star spectra
!               accur   (P3): pointing accuracy of the telescope [arcsec]
!
!.OUTPUT        
!
!.RETURN        Q1:  0: successful return
!                    otherwise: a successful return failed
!		OUT_A: name of the identified standard star spectrum
!
!.DEFAULT       P3:     18.0
!
!.PURPOSE       Get reference spectrum of a list of standard stars
!
!.ALGORITHM     Compare the coordinates of the standard star measurement with
!               the ones of the reference std.star catalog and store the
!		spectrum in a table. The name of the identified standard star
!		is returned via OUT_A. The name of the table is the name of
!		the standard star ("{out_a}.tbl").
!
!.ENVIRON       UVES
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1999/11/05  Changed copy of fors_img_cal_align.prg
!
!.COMMENT       
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.3 $
! $Log: uves_align.prg,v $
! Revision 1.3  2001/01/04 10:39:43  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name of std-*-observation or RA,DEC [deg]:"
define/param p2 ? c "Enter name of std.star reference catalog:"
define/param p3 60.0 n "Enter pointing accuracy (default=60.0 arcsec):"
!
! constants
!
define/local pid/c/1/20 align ? +lower_levels
define/local DR/d/1/1 {p3}                              !accuracy in arcsec
!
! local variable declaration
!
define/local i1/i/1/1 1
define/local j1/i/1/1 1
define/local k1/i/1/1 1
define/local l1/i/1/1 1
define/local catal/i/1/1 1
define/local num/i/1/1 0
define/local nlit/i/1/1 0
define/local RAr/d/1/1 0.0
define/local DEr/d/1/1 0.0
define/local sDEr/d/1/1 0.0
define/local cDEr/d/1/1 0.0
define/local RAm/d/1/1 0.0
define/local DEm/d/1/1 0.0
define/local sDEm/d/1/1 0.0
define/local cDEm/d/1/1 0.0
define/local obj/c/1/100 " " ALL
define/local rspe/c/1/100 " " ALL +lower_levels
define/local tlit/c/1/100 " " ALL +lower_levels

set/format F15.10
i1 = m$index(p1,",")
if i1 .gt. 2 then					!RA,DEC pair specified
  i1 = i1 - 1
  RAm = {p1(1:{i1})}
  i1 = i1 + 2
  DEm = {p1({i1}:)}
else
  if m$existd(p1,"O_POS") .eq. 1 then
    RAm = m$value({p1},O_POS(1))			!RA and DEC of the
    DEm = m$value({p1},O_POS(2))			!observation [deg].
  else
    goto ERR1
  endif
endif
cDEm = m$cos( DEm )
sDEm = m$sin( DEm )

tlit = p2
show/table {tlit} >Null
nlit  = outputi(2)					!number of std. stars
if nlit .le. 0 goto ERR3

comput/tabl {tlit} :{c_fxoname} = collapse(:{c_fxoname}) !to remove spaces
create/colu {tlit} :DIST "arcsec" F9.2 R*8 >Null	!distance of std star
compute/table {tlit} :DIST = 0.0





set/format F9.4 I1
mess/out 4 {pid} "Search for coincidences ..."
mess/out 4 {pid} "  assumed pointing accuracy: {DR} arcsec"

set/format F15.10
!
! We have to use compute/table although select/table could be a more direct way
! as select/table has accuracy problems as soon as the mathematical expression
! gets too long.
!
compute/table {tlit} -
  :DIST = {sDEm}*SIN(:{c_decl}) + {cDEm}*COS(:{c_decl})*COS({RAm}-:{c_rect})
compute/table {tlit} :DIST = ABS(ACOS(MIN(:DIST, 1.0))) * 3600.

select/table {tlit} :DIST.le.{DR} >Null
j1 = {Q1}						!Index of first detect.
num = outputi(1)					!Total number of detec.
if num .gt. 1 goto ERR4					!There are more than
if num .eq. 0 goto ERR5					!one coincidence.
if num .eq. 1 then					!One coincidence found
  rspe = "{{p2},:{c_fxoname},{j1}}"
  mess/out 4 {pid} "Reference spectrum found: `{rspe}`."
  get/refflux {tlit} {j1} {rspe}			!... SAVE coincidence.
endif
select/table {tlit} ALL					!Reset selection.

out_a = rspe
return 0



!Error handling:
ERR1:
  mess/out 1 {pid} "Bad input frame: no coordinates found!"
  return -1
ERR3:
  mess/out 1 {pid} "Empty standard star catalog!"
  return -3
ERR4:
  mess/out 1 {pid} "More than one coincidence found!"
  return -4
ERR5:
  mess/out 1 {pid} "No standard star could be identified !"
  return -5
