! $Id: uves_ccdtestb.prg,v 1.5 2002/01/29 17:34:16 amodigli Exp $
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.IDENTIFICATION: ccdtestb.prg
!.PURPOSE:        Procedure to determine the ccd bias characteristics
!.AUTHORS:        Rein H. Warmels, Thomas Augusteijn
!.USE:            @s ccdtestb
!.                where p1 = test to be done B1 to B5 or all,
!.                      p2 = input catalogue containing the ccd bias frames
!.                      p3 = output identifier [bias]
!.                      p4 = method for computing the combined bias
!.                      p5 = row range to be averaged
!.                      p6 = column range to be averaged
!.                      p7 = area
!.                      p8 = radius of smoothing box for median filtering,
!.                           decimation factor for image size reduction
!.USE:            @s ccdtestb p1 p2 p3 p4 p5 p6 p7 p8 
!.Algorithm:      The following tests are done:
!                 Test B1: Creation of the combine bias frame. The combining
!                          method is taken from P3.
!                          The result is load onto the display and a hardcopy
!                          is generated in file {p3}_aver.ps
!                 Test B2: From the combined bias frame row and columns are 
!                          averaged and plotted. The result plot are stored
!                          in the files {p3}_avcol.ps and {p2}_avrow.ps.
!                 Test B3: Find the hot pixels.
!                          The combined bias frame is median filtered and 
!                          subtracted from the original. A plot is generated 
!                          showing the position of the hot pixels and the 
!                          affected columns (file {p3}_hotpix.ps). A MIDAS
!                          table ({p3}_hotpix.tbl).
!                 Test B4: Inspection of single frame: the first and the last 
!                          one in the catalogue. For the first one a histogram
!                          of the pixel values is made, ignoring the hot pixels
!                          The plot file can be found in {p3}_hist.ps. For the
!                          last frame in the catalogue is first corrected for 
!                          hot pixels and then rebin 
!                 Test B5: Determination of the bias level and standard 
!                          deviation. For each input frame in the catalogue 
!                          determine the  mean bias and standard deviation 
!                          after hot pixel correction, box averaging and 
!                          median filtering. The overall mean bias level and 
!                          mean sigma is computed."
!.VERSION:        940620 RHW creation
!.VERSION:        950115 RHW Inclusion of the subtest 
!                 960429 PS  new defaults for windows.
!                 970510 PS more statistics of Bias images
!                 991219 plot control added for use in pipeline (NICE_CREA)
!--------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.5 $
! $Log: uves_ccdtestb.prg,v $
! Revision 1.5  2002/01/29 17:34:16  amodigli
! Fixed error running ccdtest on RED data with
! DISPLAY set to off
!
! Revision 1.4  2001/01/04 10:39:39  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!--------------------------------------------------------------------------
DEFINE/PARAMETER P1  ALL          C  "Enter test to be executed;"
DEFINE/PARAMETER P2  ?            C  "Enter input catalogue:"
DEFINE/PARAMETER P3  BIAS         C  "Enter combined output frame:"
DEFINE/PARAMETER P4  median       C  "Combining method:"
DEFINE/PARAMETER P5  <,>          C  "Enter row numbers for averaging:" 
DEFINE/PARAMETER P6  <,>          C  "Enter column numbers for averaging:" 
DEFINE/PARAMETER P7  [<,<:>,>]    C  "Enter area:"
DEFINE/PARAMETER P8  5,10         N  "Radius med. filtering, and dec. factor:"
!
WRITE/KEY    INPUTI/I/1/2     {P8}
WRITE/KEY    RADIUS/I/1/1     {INPUTI(1)}
write/key biasmean/r/1/1 0.0
write/key biassigm/r/1/1 0.0
write/key hot_pix/i/1/1 0
write/key hot_tabl/c/1/40  " " 
DEFINE/LOCAL NBOX/I/1/1       {INPUTI(2)}
DEFINE/LOCAL BSSTA/R/1/1      0.0
DEFINE/LOCAL BSEND/R/1/1      0.0
DEFINE/LOCAL BSSTE/R/1/1      0.0
DEFINE/LOCAL XSTA/R/1/1       0.0
DEFINE/LOCAL XEND/R/1/1       0.0
DEFINE/LOCAL YSTA/R/1/1       0.0
DEFINE/LOCAL YEND/R/1/1       0.0
DEFINE/LOCAL REBIN/C/1/3      " "
DEFINE/LOCAL XLAB/R/1/1       0.0
DEFINE/LOCAL YLAB/R/1/1       0.0
DEFINE/LOCAL SCALE/R/1/1       0.0
DEFINE/LOCAL XSCALE/I/1/1      0
DEFINE/LOCAL YSCALE/I/1/1      0
DEFINE/LOCAL dispflag/I/1/2   0,0
log/tof
WRITE/OUT "     PROCEDURE TO DETERMINE THE CCD BIAS CHARACTERISTICS"
write/out "     ==================================================="
write/out " "
write/out " "
write/out "  command line: "
write/out " @@ ccdtestb {p1} {p2} {p3}"
write/out "    {p4} {p5} {p6} {p7} {p8}"
write/out " "

! *** check if the catalogue exists
IF M$EXIST("{P2}") .EQ. 0 THEN
   WRITE/OUT "*** FATAL <CCDTESTB>: Catalogue {P2} not existing"
   RETURN/EXIT
ENDIF
!
!set/midas output=logonly
read/icat {p2}
set/midas output=no
if NICE_CREA .eq. "Y" then
  show/disp >Null
  dispflag(1) = outputi(1)
  dispflag(2) = outputi(12)
  if dispflag(1) .eq. -1 crea/display 0
  assign/disp 0
  clear/chan over
  if dispflag(2) .eq. -1 crea/gra 0
  assign/gra g,0
  set/graph default
endif
set/midas output=yes
!
define/local catal/i/1/1 0
define/local nfrm/i/1/1 0
LOOP1:
   store/frame in_a {p2} 1 CONT1
   nfrm = nfrm + 1
   if nfrm .eq. 1 then
       define/local frm1/c/1/60 {in_a}
   endif
   goto LOOP1
CONT1:
define/local frmn/c/1/60 {in_a}
!
BRANCH P1(1:2) B1,B2,B3,B4,B5 TESTB1,TESTB2,TESTB3,TESTB4,TESTB5
!
TESTB1:
write/out " "
WRITE/OUT "Test B1: Creating combined master bias frame"
write/out "----------------------------------------------------------------"
write/out "All frames in the input catalogue are combined using the"
write/out "<{P4}> method. The output frame will be {P3}.bdf.
write/out " "
!
! *** first set all the keywords for the combining
IF M$EXIST("{P3}.bdf") .EQ. 0 THEN
   define/local sav_met/c/1/20   "{bs_met}"       ! save the combining method
   write/key    bs_met/c/1/20    "{p4}"           ! store new method
   define/local sav_del/c/1/3    "{bs_del}"       ! save the combining method
   write/key    bs_del/c/1/3     "NO"             ! store new method
   define/local sav_typ/c/1/60   "{bs_typ}"       ! save the exposure type
   write/key    bs_typ/c/1/60    "?"              ! store new method
   delete/ima {p3} no
!   @s make_cal bs {p2} {p3}                      ! use standard call for
!   @s ccdcombine bs {p2} {p3}			  ! future compatibility
   combine/ccd  bs {p2} {p3}
   write/key    bs_met/c/1/20    "{sav_met}"      ! restore method
   write/key    bs_del/c/1/3     "{sav_del}"      ! restore deletion option
   write/key    bs_typ/c/1/60    "{sav_typ}"      ! restore exposure type
else 
  WRITE/OUT "Combined bias frame {P3}.bdf exists; no new one created"
ENDIF
write/key SC_BSFRM/c/1/60 {p3}
!
statist/imag {p3} ? ? ? FN              ! calculate statistics
bssta = OUTPUTR(3) - 5.0 * OUTPUTR(4)   ! eliminate hotpixel contrib.
bsend = OUTPUTR(3) + 5.0 * OUTPUTR(4)
statist/imag {p3} ? ? {bssta},{bsend} FN
bssta = OUTPUTR(3) - 1.5 * OUTPUTR(4)
bsend = OUTPUTR(3) + 1.5 * OUTPUTR(4)
cuts {p3} {bssta},{bsend}

scale = {{p3},NPIX(1)}. / ididev(2)
if scale .lt. 1.0 then
   xscale = 1.0 / scale
  else
   xscale = -scale - 1
endif

scale = {{p3},NPIX(2)}. / ididev(3)
if scale .lt. 1.0 then
   yscale = 1.0 / scale
  else
   yscale = -scale - 1
endif

!load/itt expo
if NICE_CREA .eq. "Y" then
  load/image {p3} scale={xscale},{yscale}
  set/midas output=no
  copy/display p5=noprint
  -copy screen{mid$sess(11:12)}.ps {p3}.ps
  set/midas output=yes
  write/out "Image of average bias frame is in file {p3}.ps"
  delete/ima screen{mid$sess(11:12)}.ima NO
endif
write/out " "
write/out "End of test B1"
write/out " "

IF P1(1:3) .NE. "ALL" RETURN
!
!
TESTB2:
write/out "Test B2: Inspection of combined bias frame:"
write/out "----------------------------------------------------------------"
!
write/out "Averaging rows {p5} of the stacked bias frame;"
write/out "Output frame will be {p3}_avrow.bdf;"
average/row {p3}_avrow = {p3} {p5}                    ! average rows
@p uves_ccdtestb,plot {p3}_avrow -140,-40,25,75 1
!
! *** Plot the bias level averaged over columns
write/out " "
write/out "Averaging columns {p6} of the stacked bias frame;"
write/out "Output frame will be {p3}_avcol.bdf"
average/col {p3}_avcol = {p3} {p6}                    ! average columns
@p uves_ccdtestb,plot {p3}_avcol -140,-40,25,15 2 {p3}_aver.ps
write/out " "
write/out "End of test B2"
write/out " "
!delete/imag {p3}_clock NO                             ! delete rotated image
IF P1(1:3) .NE. "ALL" RETURN
!
!

TESTB3:
write/out "Test B3: Finding the hot pixels"
write/out "------------------------------------------------------------------"
write/out "Generate a table with hot pixels in area {p7}"
write/out "by subtracting the median smoothed stacked bias from the "
write/out "unsmoothed one and doing statistics of the result frame."
write/out "A plot and a MIDAS table with the hot pixels area is produced."
write/out "If hot pixels are found, a new average row in range {p5}"
write/out "and column in range {p6} are produced to remove the effect"
write/out "of possible hot columns. "
write/out " "
!
IF M$EXIST("{P3}_hotpix.bdf") .EQ. 0  THEN
   write/out "     WARNING: Median filtering will take a while; don't panic."
   write/out " "
   set/midas output=no
   set/format i1
   filter/median {p3} &s {RADIUS},{RADIUS},0.0 ? {p7}
   set/midas output=yes
   compute/image {p3}_hotpix = {p3} - &s
else
   WRITE/OUT "Hot pixel frame {p3}_hotpix.bdf exists;"
   write/out "no new one created"
   write/out " "
endif
!
set/midas output=no
write/desc {p3}_hotpix IDENT/C/1/40 "{DETEC} - Hot pixels"
stat/ima {p3}_hotpix {p7} ? ? FN
!
bssta = OUTPUTR(3) - 5 * OUTPUTR(4)
bsend = OUTPUTR(3) + 5 * OUTPUTR(4)
stat/ima {p3}_hotpix {p7} ? {bssta},{bsend} FN
!
bssta = OUTPUTR(3) + 5 * OUTPUTR(4)
bsend = OUTPUTR(3) + 9 * OUTPUTR(4)
set/gra frame=square xaxis= yaxis= pmode=1
extr/ima &s = {p3}_hotpix{p7}
if NICE_CREA .eq. "Y" plot/cont &s ? ? {bssta} NEG 
bssta = -999999
find/pixel &s {bssta},{bsend} out A dummtab 50000
set/midas output=yes
IF OUTPUTI(1) .EQ. 0 THEN
   set/format f8.2
   WRITE/OUT "*** INFO: No pixels found outside range {bssta}, {bsend}"
   WRITE/OUT "          No hot pixel image and table produced"
   write/out " "
   set/format i1 e15.5
   IF P1 .EQ. "ALL" THEN 
      GOTO TESTB4
   ELSE 
      RETURN
   ENDIF
ENDIF

name/colu dummtab :x_pix :x
name/colu dummtab :y_pix :y

sort/tab dummtab :x,:y         
write/keyw in_a/c/1/60  dummtab
write/keyw out_a/c/1/60 {p3}_hotpix
set/format i1 f10.0
write/out " "
run STD_EXE:ccdhotpix
set/format
!
hot_tabl = "{p3}_hotpix"
set/midas output=no
select/tabl {p3}_hotpix :VALUE.GT.0.0
read/tabl {p3}_hotpix
if NICE_CREA .eq. "Y" then
   overpl/tabl {p3}_hotpix :x :y 5
endif
show/tab {p3}_hotpix
hot_pix = outputi(2)
if NICE_CREA .eq. "Y" then
   copy/gra postscript
   -copy postscript.ps {p3}_hotpix.ps
endif
set/midas output=yes
write/out " "
write/out "The contour plot with hot pixels inside the area "
write/out "{p7} is stored in the file "
write/out "{p3}_hotpix.ps. The MIDAS table containing "
write/out "the hot pixels/cols is in {p3}_hotpix.tbl"
write/out " "
write/out "There were {hot_pix} hot pixels/columns found "
write/out " "

write/out "Now we do again the row and column averages with a clean image:"
write/out " "
write/out "Averaging rows {p5} of the stacked bias frame;"
write/out "Output frame will be {p3}_avrow_clean.bdf;"
set/midas output=no
modify/colu {p3},{p3}_hotpix &a V                   ! modify the hot pixels
average/row {p3}_avrow_clean = &a {p5}                 ! average rows
@p uves_ccdtestb,plot {p3}_avrow_clean -140,-40,25,75 1

! *** Plot the bias level averaged over columns
set/midas output=yes
write/out " "
write/out "Averaging columns {p6} of the stacked bias frame"
write/out "Output frame will be {p3}_avcol_clean.bdf"
set/midas output=no
average/col {p3}_avcol_clean = &a {p6}                ! average columns
set/midas output=yes
@p uves_ccdtestb,plot {p3}_avcol_clean -140,-40,25,15 2 {p3}_aver_clean.ps
write/out "End of test B3"
write/out " "
delete/imag &a no
delete/imag &s NO
delete/tabl dummtab NO
IF P1(1:3) .NE. "ALL" RETURN
!
!
TESTB4:
set/format i1
write/out "Test B4: Inspection of single bias frames:"
write/out "------------------------------------------------------------------"
write/out "A histogram of the pixel intensities of frame {frm1}"
write/out "over the area {p7} is computed, ignoring the hot pixels"
write/out "and cosmics, and stored in the table {p3}_hotpix.tbl."
write/out "The frame {frmn} is cleaned from hot pixels and cosmics. "
write/out "Hereafter the clean frame is rebinned using a median filter "
write/out "over {RADIUS}x{RADIUS} pixels; output in {p3}_rebin.ps"
!write/out "The rows {p5} of the single bias frame will be averaged"
!write/out "and the FFT will be computed. Output frames are {P3}_real.bdf"
!write/out "and {P3}_imag.bdf;"
write/out " {frm1} {p7}"

! *** Plot a histogram distribution of a bias frame 
set/midas output=no
statist/image {frm1} {p7} ? ? FN
xsta = OUTPUTR(3) - 30.
xend = OUTPUTR(3) + 30.
statist/image {frm1} {p7} 1 {xsta},{xend} FN
set/midas output=yes
if NICE_CREA .eq. "Y" then
  set/gra xaxis={xsta},{xend} yaxis= frame=rect pmode=1
  plot/hist {frm1}
  copy/gra postscript 
  -copy postscript.ps {p3}_hist.ps
  write/out " "
  write/out "Plot of the histogram is in file {p3}_hist.ps"
  set/grap clear=on
endif
!
write/out " Now produce clean and rebined bias frame"
! set/midas output=no

if hot_pix .NE. 0 then
 modify/colu {frmn},{p3}_hotpix &a V                   ! modify the hot pixels
else
 write/out "Hot_pix eq 0"
 comp/ima &a = {frmn}
endif

extr/ima middumma = middumma{p7}
write/key rebin/c/1/3 "med"
in_a  = "middumma"
out_a = "{p3}_rebin"
run STD_EXE:ccdrebin
write/out " {out_a}  and {p3}_rebin"
write/out " restore CCD coordinates "
xsta = {&a,start(1)} + {nbox}/2
ysta = {&a,start(2)} + {nbox}/2

write/desc {p3}_rebin start/d/1/2 {xsta},{ysta}
write/desc {p3}_rebin step/d/1/2 {nbox},{nbox}

statis/ima {p3}_rebin
xsta = OUTPUTR(3) - 1.0*OUTPUTR(4)
xend = OUTPUTR(3) + 1.0*OUTPUTR(4)

write/out " Cut {p3}_rebin "
cuts/imag {p3}_rebin {xsta},{xend}
scale = {{p3}_rebin,NPIX(1)}. / ididev(2)
if scale .lt. 1.0 then
   xscale = 1.0 / scale
  else
   xscale = -scale - 1
 endif
scale = {{p3}_rebin,NPIX(2)}. / ididev(3)
if scale .lt. 1.0 then
   yscale = 1.0 / scale
  else
   yscale = -scale - 1
 endif
if NICE_CREA .eq. "Y" then
  write/out " load {out_a} or {p3}_rebin "
  load/imag {p3}_rebin scale={xscale},{yscale} 
  ! load/imag {out_a} scale={xscale},{yscale}
  copy/display p5=noprint
  set/midas output=yes
  -copy screen{mid$sess(11:12)}.ps {p3}_rebin.ps
  write/out "Cleaned and rebinned bias frame is in file {p3}_rebin.ps"
  delete/imag screen{mid$sess(11:12)}.ps no
  delete/imag screen{mid$sess(11:12)}.ima no
endif
delete/imag &a no
delete/imag &b no
write/out " "
write/out "End of test B4"
write/out " "
IF P1(1:3) .NE. "ALL" RETURN
!
!
TESTB5:
write/out "Test B5: Determination of the bias level and standard deviation"
write/out "------------------------------------------------------------------"
write/out "For each input frame in the catalogue determine the mean"
write/out "BIAS and standard deviation AFTER hot pixel correction,"
write/out "Box averaging and median filtering, and using area {P7}." 
write/out "Finally, the overall mean bias level and sigma are computed."
write/out
define/local mean/r/1/1 0.0
define/local sigm/r/1/1 0.0
define/local meandif/r/1/1 0.0
define/local sigmdif/r/1/1 0.0
define/local meanstd/r/1/1 0.0
define/local sigmstd/r/1/1 0.0
define/local frame/c/1/60 " "
write/key    catal/i/1/1 0
write/key    rebin/c/1/3 "mea"
define/local ifrm/i/1/1 0
create/tab   {p3}_mean 3 10 null
create/col   {p3}_mean :NAME  "   " A20  C*20
create/col   {p3}_mean :MEAN  "ADU" F9.4 R*4
create/col   {p3}_mean :SIGMA "ADU" F9.4 R*4


LOOP2:
   store/frame in_a {p2} 1 FINISH
   ifrm  = ifrm + 1
   frame = in_a
   set/midas output=no
IF OUTPUTI(1) .EQ. 0 THEN
   modify/colu {in_a},{p3}_hotpix middumma V          ! modify the hot pixels
ELSE
   comp/ima middumma = {frame}
ENDIF
   set/midas output=no
   extr/ima middumma = middumma{p7}
   in_a  = "middumma"
   out_a = "bs_mean"
   out_b = "bs_sigm"
   run STD_EXE:ccdrebin
   filter/median {OUT_A} &a {RADIUS},{RADIUS},0.0 
   filter/median {OUT_B} &b {RADIUS},{RADIUS},0.0 
   statist/ima &a ? ? ? NN
   mean  = outputr(3)
   statist/ima &b ? ? ? NN
   sigm = outputr(3)
   {p3}_mean,:NAME,@{ifrm}  = "{frame}"
   {p3}_mean,:MEAN,@{ifrm}  =  mean
   {p3}_mean,:SIGMA,@{ifrm} =  sigm
   set/midas output=yes
   set/format t6.2
   write/out "     Statistics of {frame}:  mean =  {mean};  noise ={sigm}"
   goto LOOP2
FINISH:
set/midas output=no
ifrm = ifrm + 1
sele/table {p3}_mean seq.lt.{ifrm}
{p3}_mean,:NAME,@{ifrm}  = "all_average"
stat/tab {p3}_mean :MEAN
biasmean = outputr(3)
{p3}_mean,:MEAN,@{ifrm}  =  biasmean
meandif = outputr(2) - outputr(1)
meanstd = outputr(4)
stat/tab {p3}_mean :SIGMA
biassigm = outputr(3)
{p3}_mean,:SIGMA,@{ifrm} =  biassigm
sigmdif = outputr(2) - outputr(1)
sigmstd = outputr(4)
ifrm = ifrm - 1
delete/imag {OUT_A} no
delete/imag {OUT_B} no
delete/imag &a no
delete/imag &s no

set/midas output=yes
set/format i1 t6.2
write/out " "
write/out -
       "     Average of  {ifrm} bias frames:  mean =  {biasmean};  noise ={biassigm}"
write/out " "
write/out -
       "     mean/noise diff.(max-min):  mean =  {meandif};  noise ={sigmdif}"  
write/out -
       "     mean/noise std. deviation:  mean =  {meanstd};  noise ={sigmstd}"  
write/out " "
write/out "The MIDAS table with results is in file {p3}_mean.tbl"
write/out " "
write/out "End of test B5"
write/out " "
set/format
!
! cleanup 
delete/ima bs_sigm.bdf NO
delete/ima bs_mean.bdf NO
delete/ima postscript.ps NO
delete/ima screen*.ps NO
delete/ima screen*.ima NO
delete/ima graph*.plt NO
delete/ima middumma.bdf NO
delete/tab dummtab.tbl NO
set/graph default
RETURN
!



!=== SUBROUTINES ==============================================================

ENTRY plot
define/param p1 ? c "Enter frame to plot:"
define/param p2 ? c "Enter diagram characteristic:"
define/param p3 ? c "Enter 1: first part, 2: second part of plot:"
define/param p4 + c "Enter rootname:"

define/local ysta/r/1/1 0.0
define/local yend/r/1/1 0.0

if NICE_CREA .eq. "Y" then
  statist/imag {p1} ? ? ? GN                      ! calculate statistics
  ysta = OUTPUTR(8) - 4                           ! get relevant y range
  yend = OUTPUTR(8) + 4
  set/gra yaxis={ysta},{yend} pmode=1             ! set the y axis
  plot/row {p1} @1 ?  {p2}              ! plot the row
  if {p3} .eq. 1 then
    set/grap clear=off
  else if {p3} .eq. 2 then
    set/gra
    copy/gra postscript
    -copy postscript.ps {p4}
    write/out " "
    write/out "Plot file is in {p4}
    write/out " "
  endif
  return 0
endif
return -1



