! $Id: uves_ccdtestc.prg,v 1.3 2001/01/04 10:39:39 amodigli Exp $
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.IDENTIFICATION: ccdtestc.prg
!.PURPOSE:        Procedure to determine the ccd contamination
!.AUTHORS:        A. Kaufer
!.USE:            @s ccdtestc
!.                where p1 = input catalogue containing the ccd flat frames
!.                      p2 = reference field
!. 			     layout: 1 2 3
!.                                   4 5 6
!.                                   7 8 9
!.                      p3 = window size for measurement
!.                      p4 = result table
!.                      p5 = plot table
!.USE:            @s ccdtestc p1 p2 p3 p4
!.Algorithm:      
!.VERSION:        991212 AKA   creation
!		  991219 SW    Plot switch parameter added for use in pipeline.
!----------------------------------------------------------------------
! $Name:  $
! $Revision: 1.3 $
! $Log: uves_ccdtestc.prg,v $
! Revision 1.3  2001/01/04 10:39:39  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!----------------------------------------------------------------------
DEFINE/PARAMETER P1  ?            C  "Enter input catalogue:"
DEFINE/PARAMETER P2  5            N  "Enter reference field:"
DEFINE/PARAMETER P3  256          N  "Enter window size:"
DEFINE/PARAMATER P4  ?            C  "Enter result table:"
DEFINE/PARAMATER P5  ?            C  "Enter result table:"
!
define/local reffield/i/1/1 {p2}
define/local box/i/1/1      {p3}
define/local restab/c/1/80  {p4}
define/local infile/c/1/80  " " all
!
define/local catal/i/1/1 0

define/local f/i/1/1 0
define/local nfrm/i/1/1 0
define/local win/i/1/4  0,0,0,0
define/local cen/i/1/2  0,0
define/local res/r/1/9  0 all
define/local npix/i/1/2 0,0
!
!
DEFINE/LOCAL BSSTA/R/1/1      0.0
DEFINE/LOCAL BSEND/R/1/1      0.0
DEFINE/LOCAL BSSTE/R/1/1      0.0
DEFINE/LOCAL SCALE/R/1/1       0.0
DEFINE/LOCAL XSCALE/I/1/1      0
DEFINE/LOCAL YSCALE/I/1/1      0
DEFINE/LOCAL dispflag/I/1/2   0,0
!
log/off
WRITE/OUT "     PROCEDURE TO DETERMINE CCD CONTAMINATION"
write/out "     ========================================"
write/out " "
write/out " "
write/out "  command line: "
write/out "  @@ ccdtestc {p1} {p2} {p3} {p4}" 
write/out " "

! *** check if the catalogue exists
IF M$EXIST("{P1}") .EQ. 0 THEN
   WRITE/OUT "*** FATAL <CCDTESTC>: Catalogue {P1} not existing"
   RETURN/EXIT
ENDIF
!
!set/midas output=logonly
!read/icat {p1}
set/midas output=no
if P5(1:1) .eq. "Y" then
  show/disp >Null
  dispflag(1) = outputi(1)
  dispflag(2) = outputi(12)
  if dispflag(1) .eq. -1 crea/display 0
  assign/disp 0
  clear/chan over
  if dispflag(2) .eq. -1 crea/gra 0 
  assign/gra g,0
endif
set/midas output=yes
!
crea/tab {restab} 20 100
crea/col {restab} :file C*30
do f = 1 9 
 crea/col {restab} :f{f} R*4 F7.1
 crea/col {restab} :n{f} R*4 F5.3 
enddo 
crea/col {restab} :ref R*4 F7.1
!
LOOP1:
   store/frame infile {p1} 1 CONT1
   nfrm = nfrm + 1
   ! bias substraction
   if M$EXIST("{SC_BSFRM}.bdf") .EQ. 1 then
    compute/imag &a = {infile} - {SC_BSFRM}
   else
    IF M$TSTNO("{BIASMEAN}") .EQ. 0 THEN
     write/OUT -
     "     *** INFO: KEYWORD BIASMEAN is not filled with a number"
     write/OUT "               No bias offset subtracted"
     write/out " "
      compute/imag  &a = {in_a}
     else
      compute/imag  &a = {in_a} - {BIASMEAN}
    endif
   endif
   ! 
   ! display input frame
   statist/imag &a ? ? ? FN            ! calculate statistics
   bssta = OUTPUTR(3) - 5.0 * OUTPUTR(4)   ! eliminate hotpixel contrib.
   bsend = OUTPUTR(3) + 5.0 * OUTPUTR(4)
   statist/imag &a ? ? {bssta},{bsend} FN
   bssta = OUTPUTR(3) - 1.5 * OUTPUTR(4)
   bsend = OUTPUTR(3) + 1.5 * OUTPUTR(4)
   cuts &a {bssta},{bsend}
!
   scale = {&a,NPIX(1)}. / ididev(2)
   if scale .lt. 1.0 then
    xscale = 1.0 / scale
   else
    xscale = -scale - 1
   endif
!
   scale = {&a,NPIX(2)}. / ididev(3)
   if scale .lt. 1.0 then
    yscale = 1.0 / scale
   else
    yscale = -scale - 1
   endif
!
   if P5(1:1) .eq. "Y" load/image &a scale={xscale},{yscale}
!
   npix(1) = {&a,NPIX(1)}
   npix(2) = {&a,NPIX(2)}
!
!  define corresponding positions 
   do f = 1 9
     if {f} .eq. 1 then
	cen(1) = box
        cen(2) = npix(2)-box
     elseif {f} .eq. 2 then
	cen(1) = npix(1)/2
        cen(2) = npix(2)-box	
     elseif {f} .eq. 3 then
	cen(1) = npix(1)-box
        cen(2) = npix(2)-box	
     elseif {f} .eq. 4 then
	cen(1) = box
        cen(2) = npix(2)/2	
     elseif {f} .eq. 5 then
	cen(1) = npix(1)/2
        cen(2) = npix(2)/2	
     elseif {f} .eq. 6 then
	cen(1) = npix(1)-box
        cen(2) = npix(2)/2	
     elseif {f} .eq. 7 then
	cen(1) = box
        cen(2) = box	
     elseif {f} .eq. 8 then
	cen(1) = npix(1)/2
        cen(2) = box	
     elseif {f} .eq. 9 then
	cen(1) = npix(1)-box
        cen(2) = box	
     else
	cen(1) = npix(1)/2
        cen(2) = npix(2)/2	
    endif 
!   define window coordinates
    win(1) = cen(1)-box/2
    win(2) = cen(2)-box/2
    win(3) = cen(1)+box/2
    win(4) = cen(2)+box/2
!   do the statistics on the box
    if P5(1:1) .eq. "Y" then
      assign/gra D,0 >Null
      plo/ax [@{win(1)},@{win(2)}:@{win(3)},@{win(4)}]
    endif
    stat/ima &a [@{win(1)},@{win(2)}:@{win(3)},@{win(4)}] ? ? FNNN
    {restab},:f{f},@{nfrm} = outputr(8)
    {restab},:file,@{nfrm} = "{infile}"
!
    if {f} .eq. {reffield} then
     {restab},:ref,@{nfrm} = {{restab},:f{f},@{nfrm}}
    endif
  enddo
  read/tab {restab} 
 goto LOOP1
CONT1:

do f = 1 9
 comp/tab {restab} :n{f} = :f{f}/:ref
 stat/tab {restab} :n{f} 
 res({f}) = outputr(3)
enddo 
copy/kd res/r/1/9 {restab}.tbl fields/r/1/9 

if P5(1:1) .eq. "Y" then
  assign/gra
  set/graph default
endif
set/format F4.2
write/out "  {res(1)} {res(2)} {res(3)}"
write/out "  {res(4)} {res(5)} {res(6)}"
write/out "  {res(7)} {res(8)} {res(9)}"

RETURN
