! $Id: uves_ccdtestt.prg,v 1.2 2001/01/04 10:39:38 amodigli Exp $
! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.IDENTIFICATION: ccdtestt.prg
!.PURPOSE:        Procedure to test the ccd linearity and transfer
!.AUTHOR:         Rein H. Warmels, Thomas Augusteijn
!.USE:            @s ccdtestt p1  p2 p3 p4 p5 p6
!.                where p1 = test to be executed
!.                      p2 = input catalogue containing the ccd frames
!.                      p3 = output identifier [TRANS]
!.                      p4 = image section to be used for the median and
!.                           variance determination
!.                      p5 = time interval to fit the shutter offset
!.                      p6 = selection criteria table 
!.                      p7 = plot switch
!.Algorithm:      The following tests are done:
!.                Test T1: Creation of the transfer and linearity table"
!.                Test T2: Determination of linearity curve and shutter error
!.                Test T3: Determination of the transfer curve
!.VERSION:        940524 RHW creation
!		  960429 PS  change of default windows areas for test
!		             change of algorithm to find shutter offset.
!			     improved formats and some error checking.
!                            Default selection: not saturated.
!		  991219 SW  Plot switch parameter added for use in pipeline.
!--------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_ccdtestt.prg,v $
! Revision 1.2  2001/01/04 10:39:38  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!--------------------------------------------------------------------
DEFINE/PARAMETER P1  ALL           C  "Enter test to be executed:"
DEFINE/PARAMETER P2  ?             C  "Enter input catalogue:"
DEFINE/PARAMETER P3  TRANS         C  "Enter combined output frame:"
DEFINE/PARAMETER P4  [<,<:>,>]     C  "Enter the window for analysis:"
DEFINE/PARAMETER P5  -2.0,2.0      C  "Time interval to fit the shutter offset"
DEFINE/PARAMETER P6  all           C  "Selection criterium for table:"
DEFINE/PARAMETER P7  Y             C  "Create plots Y/N:"
! 
write/key aducf/r/1/1 0.0
write/key errcf/r/1/1 0.0
write/key enoise/r/1/1 0.0
write/key erren/r/1/1 0.0
write/key eshutter/r/1/1 0.0
write/key counts/r/1/1 0.0
define/local xlab/r/1/1 0.0
define/local ylab/r/1/1 0.0
define/local aducf1/r/1/1 0.0
define/local aducf2/r/1/1 0.0
define/local errcf1/r/1/1 0.0
define/local errcf2/r/1/1 0.0
define/local enoise1/r/1/1 0.0
define/local enoise2/r/1/1 0.0
define/local counts1/r/1/1 0.0
define/local counts2/r/1/1 0.0
define/local shval/r/1/3 0.0,0.0,0.0
define/local sherr/r/1/1 0.0
define/local shut/r/1/1 0.0
define/local shoff1/r/1/1 0.0
define/local shoff2/r/1/1 0.0
define/local nstep/i/1/1 0
define/local iter/i/1/1 0
define/local n/i/1/1 0
define/local noff/i/1/1 0
write/key    linerr/r/1/6 0.0 0.0
!write/key    shval/r/1/3 {p5},0.01   ! done later
define/local crit/c/1/50 {p6}

log/tof
WRITE/OUT "     PROCEDURE TO TEST THE CCD LINEARITY AND TRANSFER"
write/out "     ================================================"
write/out " "
write/out " "

!
! *** check if the catalogue exists
IF M$EXIST("{P2}") .EQ. 0 THEN
   WRITE/OUT "*** FATAL <CCDTESTT>: Catalogue {P2} not existing"
   RETURN/EXIT
ENDIF
!
set/midas output=logonly
read/icat {p2}
set/midas output=no
if P7(1:1) .eq. "Y" then
  show/disp >Null
  if outputi(12) .eq. -1 crea/gra 0
  set/graph default
  set/graph pmode=1
endif
set/midas output=yes
!
define/local col1/c/1/20  :UT_tim1 
define/local col2/c/1/20  :UT_tim2 
define/local col3/c/1/20  :Exp_tim1 
define/local col4/c/1/20  :Exp_tim2
define/local col5/c/1/20  :Med_cnt1
define/local col6/c/1/20  :Med_cnt2
define/local col7/c/1/20  :Variance
define/local col8/c/1/20  :Frc_cnt1
define/local col9/c/1/20  :el_adu1 
define/local col10/c/1/20 :Frc_cnt2
define/local col11/c/1/20 :el_adu2 
define/local col12/c/1/20 :scratch
define/local el_adc/r/1/1 0.0
!
BRANCH P1(1:2) T1,T2,T3 TESTT1,TESTT2,TESTT3

TESTT1:
set/format f10.2
write/out " "
write/out "Test T1: Creation of the transfer and linearity table"
write/out "-------------------------------------------------------------------"
write/out "The table will contain 7 columns: column 1 for the UT of the"
write/out "of the exposure of the first image; column 2 for the exposure"
write/out "time of the first image; column 3 the exposure time of the "
write/out "second image; column 4 the median pixel intensity over the"
write/out "selected image section in frame a; column 5 the median pixel"
write/out "intensity over the selected frame area in frame b; column 6"
write/out "the variance of the difference of the frame a and b, and" 
write/out "column 7 is a 2 point aproximation of the Conv. Factor used "
write/out "to select out bad or saturated pairs of images."
write/out "The frames used have, if present, been corrected with the bias"
write/out "frame {sc_bsfrm} in the keyword SC_BSFRM,"
write/out "or else corrected for the bias offset {BIASMEAN} in keyword BIASMEAN."
write/out " "
set/format
!
IF M$EXIST("{P3}.tbl") .EQ. 0 THEN
   create/tab   {p3} 9 20 null
   create/tab   {p3} 9 20 null
   create/col   {p3} {col1}      "hours" F9.2 R*4
   create/col   {p3} {col2}      "hours" F9.2 R*4
   create/col   {p3} {col3}      "sec" F9.2 R*4
   create/col   {p3} {col4}      "sec" F9.2 R*4   
   create/col   {p3} {col5}      "ADU" F9.1 R*4
   create/col   {p3} {col6}      "ADU" F9.1 R*4
   create/col   {p3} {col7}      "ADU" F9.1 R*4
   create/col   {p3} {col8}      " % " F9.2 R*4
   create/col   {p3} {col9}      "e-/adu" F9.2 R*4
   create/col   {p3} {col10}     " % " F9.2 R*4
   create/col   {p3} {col11}     "e-/adu" F9.2 R*4
   create/col   {p3} {col12}     "   " F9.2 R*4
!
   define/local ut_time/c/1/9 "O_TIME(5)"
!   define/local ut_time/c/1/9 "TM_START"
   define/local catal/i/1/1 0
   define/local ifrm/i/1/1 0
   define/local frm1/c/1/60 " "
   define/local frm2/c/1/60 " "
   define/local var/r/1/1 0.0
   define/local tdef/d/1/1 0.0

!
   set/format ,e20.15
   LOOP1:
      store/frame in_a {p2} 1 CONT1
      frm1 = "{in_a}"
      store/frame in_b {p2} 1 CONT1
      frm2 = "{in_b}"
      tdef = {{frm1},{O_DESC}} - {{frm2},{O_DESC}}
      tdef = M$ABS(tdef)
      if tdef .GT. 0.02 then
         write/out "     *** WARNING: exposure times {frm1} amd {frm2} differ "
         write/out "                  more than .02 sec: image pair skipped"
         write/out " "

      else
        if M$EXIST("{SC_BSFRM}.bdf") .EQ. 1 then
          compute/imag &a = {in_a} - {SC_BSFRM}
          compute/imag &b = {in_b} - {SC_BSFRM}
        else
          IF M$TSTNO("{BIASMEAN}") .EQ. 0 THEN
            write/OUT -
             "     *** INFO: KEYWORD BIASMEAN is not filled with a number"
            write/OUT "               No bias offset subtracted"
            write/out " "
            compute/imag  &a = {in_a}
            compute/imag  &b = {in_b}
          else
            compute/imag  &a = {in_a} - {BIASMEAN}
            compute/imag  &b = {in_b} - {BIASMEAN}
          endif
        endif

         ifrm = ifrm + 1
         {p3},{col1},@{ifrm} = {{frm1},{ut_time}}
         {p3},{col2},@{ifrm} = {{frm2},{ut_time}}
         {p3},{col3},@{ifrm} = {{frm1},{O_DESC}}
         {p3},{col4},@{ifrm} = {{frm2},{O_DESC}}
         stat/ima &a {p4} ? ? RN
         {p3},{col5},@{ifrm} = outputr(3)
         stat/ima &b {p4} ? ? RN
         {p3},{col6},@{ifrm} = outputr(3)
         comp/ima &a = {frm1} - {frm2}
         stat/ima &a {p4} ? ? RN
         var = outputr(4) * outputr(4) / 2.0
         {p3},{col7},@{ifrm} = var
         if var .GT. 0 then
           {p3},{col9},@{ifrm} = M$VALUE({p3},{col5},@{ifrm}) / var
           {p3},{col11},@{ifrm} = M$VALUE({p3},{col6},@{ifrm}) / var
         else
           {p3},{col9},@{ifrm} = 0.0
           {p3},{col11},@{ifrm} = 0.0
         endif
      endif
   goto LOOP1
   CONT1:
!
   delete/imag &a NO
   delete/imag &b NO
   write/out "Content of unedited table {p3}:"
   write/out " "
   read/tab {p3} {col1},{col3},{col4},{col5},{col6},{col7},{col9}

else
   WRITE/OUT "  *** INFO: Linearity table {p3} exists; no new one created"
   write/out " "
ENDIF
!
!
! *** table ready do the printed and plotted
SELECT/TAB {P3} ALL
sort/tab {p3} {col3}
read/tab {p3} {col1},{col3},{col4},{col5},{col6},{col7},{col9}
write/out " "
!write/out "Table with exposure sequence is in file {p3}_table.txt"
!write/out " "
!
sort/tab {p3} {col1}
if P7(1:1) .eq. "Y" then
  plot/table {p3} {col1} {col3} -60,-40,25,75 18 2
  xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1))/10
  ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5))/10
  label/gra  "Exposure sequence" {xlab},{ylab} ? ? 1
endif
!
sort/tab {p3} {col3}
if P7(1:1) .eq. "Y" then
  set/graph clear=off yform=e6.2
  plot/tab {p3} {col3} {col5} -60,-40,25,15 7 2
  xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
  ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
  label/grap  "Linearity Curve (1)" {xlab},{ylab} ? ? 1
endif
!
if P7(1:1) .eq. "Y" then
  plot/tab {p3} {col4} {col6} -60,-40,115,15 7 2
  xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
  ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
  label/grap "Linearity Curve (2)" {xlab},{ylab} ? ? 1
!
  set/grap xform=e6.0
  plot/tab {p3} {col5} {col7} -60,-40,115,75 7 2
  xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
  ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
  label/grap "Transfer Curve" {xlab},{ylab} ? ? 1
  copy/grap postscript 
  -copy postscript.ps {p3}_lin1.ps
  write/out "Plot with exposure sequence vs exposure time, linearity curve (1),"
  write/out "linearity curve (2), and transfer curve are stored in file"
  write/out "{p3}_lin1.ps"
  set/grap clear=on
endif
!
write/out " "
write/out "End of test T1"
write/out " "
IF P1(1:3) .NE. "ALL" RETURN
!
!
TESTT2:
write/out "Test T2: Determination of linearity curve and the shutter error"
write/out "----------------------------------------------------------------"
write/out "Entries in the table {p3} not fullfilling the"
write/out "selection criteria <{p6}>"
write/out "will now be selected out."
write/out "If the selection is all, values of simple e-/adu calculation"
write/out "will be used to eliminate saturated images."
write/out "From the remaining entries in the table a linear fit is"
write/out "done to determine the linearity curve for frames 1 and 2."
write/out " "
wait 5
set/midas output=no

!
! *** linearity curve of frames 1

if "{p6}" .eq. "all" then
	sele/tab {p3} {col9}.LT.15.AND.{col9}.GT.0
	stat/tab {p3} {col9}
	el_adc = outputr(3)            ! get mean of el_adc
	enoise1 = outputr(4)               ! std dev.
	stat/tab {p3} {col11}
	el_adc = (el_adc + outputr(3)) / 2.0
	enoise1 = (enoise1 + outputr(4)) * 2.5   ! /2 * 5
	set/format f9.2
	crit = "ABS({col9}-{el_adc}).LT.{enoise1}"
endif
set/midas output=yes
write/out "selection criteria: {crit}"
write/out " "
set/midas output=no

select/tab {p3} {crit}
if P7(1:1) .eq. "Y" then
  set/grap stype=7 ltype=0 xform= yform=e6.2
  plot/tab {p3} {col3} {col5} -60,-40,25,75 
  set/grap clear=off
  xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
  ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
  label/grap "Linearity Curve (1)" {xlab},{ylab} ? ? 1
endif
regression/lin {p3} {col5} {col3}
save/regress {p3} regress
Compute/regress {p3} {col12} = regress
set/midas output=yes
set/format ,F10.2
write/out "     Linearity results data set 1:"
write/out "     Mean count rate: {outputd(2)} ADU/sec"
set/format
set/format F6.3
sherr = outputd(1) / outputd(2) 
write/out "     Exposure offset: {sherr} seconds (+ = slow; - = fast)" 
set/midas output=no
if P7(1:1) .eq. "Y" then
  set/grap ltyp=2 stype=0
  overplot/tab {p3} {col3} {col12}
endif
!
! *** linearity curve of frames 2
select/tab {p3} {crit}
if P7(1:1) .eq. "Y" then
  set/grap stype=7 ltype=0 xform= yform=e6.2
  plot/tab {p3} {col4} {col6} -60,-40,115,75
  xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
  ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
  label/grap "Linearity Curve (2)" {xlab},{ylab} ? ? 1
endif
regression/lin {p3} {col6} {col4}
save/regress {p3} regress
compute/regress {p3} {col12} = regress
set/midas output=yes
set/format ,F10.2
write/out " "
write/out "     Linearity results data set 2:"
write/out "     Mean count rate: {outputd(2)} ADU/sec"
set/format
set/format F6.3
sherr =  outputd(1) / outputd(2) 
write/out "     Exposure offset: {sherr} seconds (+ = slow; - = fast)" 
if P7(1:1) .eq. "Y" then
  set/grap stype=0 ltype=2
  overplot/tab {p3} {col4} {col12}
endif
!
! *** First set: fractional count rate versus median count
write/descr {p3}.tbl HISTORY_UPDA/I/1/1 0
write/out " "
write/out "     Determining shutter offset from fractional count rates (1)"
set/midas output=no
set/format

write/key    shval/r/1/2 {p5}      
write/key    shval/r/3/1 0.01
nstep = ( shval(2) - shval(1) )/ shval(3) 

ITER1:
shoff1 = 99999999.
do N = 1  nstep 
   shut =  shval(1)  + ( n - 1) * shval(3) 
   compute/tab {p3} {col8} = {col5}/({col3} + {shut})
   select/tab {p3} {crit}
   stat/tab {p3} {col8}                  !get stderr of col8
   shoff2 =  outputr(4)       ! get stderr to minimize

   if shoff2 .GT. shoff1 .and. iter .eq. 0 then  ! ready to finer step
      shval(1) = shval(1) + (n - 3) * shval(3)   ! step back 2 to find min.
      shval(3) = shval(3)/10                      ! 20 steps with fine step
      nstep = 20
      iter  = 1

      goto ITER1
   elseif shoff2 .GT. shoff1 .and. iter .eq. 1 then
      shut = shval(1) + (n - 2) * shval(3)       ! minimum found to 1/100s
      goto DONE1
   else
      shoff1 =  shoff2 
   endif
end do

! if comming out here: no minimum found. Set shutter offset to 0
shut = 0.0
set/format f6.2
set/midas output=yes
write/out -
"*** Method not converging. Shutter offsets from {shval(1)} to {shval(2)} sec."
write/out "    Shutter offset set to 0."
!
! plot the fractional count rate and the fit
DONE1:
write/descr {p3}.tbl HISTORY_UPDA/I/1/1 1
eshutter =  shut 
set/midas output=yes
set/format f6.3
write/out "     Shutter offset: {shut} seconds"
set/format

compute/tab {p3} {col8} = {col5}/({col3} + {shut})
set/midas output=no
select/tab {p3} {crit}
stat/tab {p3} {col8}
set/midas output=yes
set/format f6.2
counts1 = outputr(3)
write/out "     Average Count Rate: {counts1} ADU/sec"
set/format
compute/tab {p3} {col8} = ({col8}-{outputr(3)})/{outputr(3)}*100
set/midas output=no
select/tab {p3} {crit}
if P7(1:1) .eq. "Y" then
  set/grap xform=e6.0 yform=f6.2 clear=off
  plot/tab {p3} {col5} {col8} -60,-40,25,15 7 2
  xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
  ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
  label/grap "Diff. Count Rate in % (1)" {xlab},{ylab} ? ? 1
endif
!
stat/tab {p3} {col8}
linerr(1) = outputr(1)
linerr(2) = outputr(2)
set/format f6.2
set/midas output=yes
write/out " "
write/out -
"     Linearity error after shutter correction: +{linerr(2)}/{linerr(1)} %"
set/midas output=no
set/format
!
regression/lin {p3} {col8} {col3}
set/midas output=yes
save/regress {p3} regress
compute/regress {p3} {col12} = regress
if P7(1:1) .eq. "Y" then
  set/grap ltype=4 stype=0
  overplot/tab {p3} {col5} {col12} 
endif
!
!
! *** Second set: fractional count rate versus median count
write/descr {p3}.tbl HISTORY_UPDA/I/1/1 0
write/out " "
write/out "     Determining shutter offset from fractional count rates (2)"
!
set/midas output=no
write/key    shval/r/1/2 {p5}
write/key    shval/r/3/1 0.01
nstep = ( shval(2) - shval(1) ) / shval(3) 
!
ITER = 0
!
ITER2:
shoff1 = 999.
do N = 1 nstep
   shut =  shval(1) + ( n - 1) * shval(3) 
   compute/tab {p3} {col10} = {col6}/({col4}+{shut})
   select/tab {p3} {crit}
   stat/tab {p3} {col10}
   shoff2 = outputr(4)

   if shoff2 .GT. shoff1 .and. iter .eq. 0 then
      shval(1) =  shval(1) + (n - 3) * shval(3) 
      shval(3) = shval(3)/10
      nstep = 20
      iter  = 1

      goto ITER2
   elseif shoff2 .GT. shoff1 .and. iter .eq. 1 then
      shut = shval(1) + (n - 2) * shval(3)
      goto DONE2
   else
      shoff1 = shoff2
   endif
end do

! if comming out here: no minimum found. Set shutter offset to 0
shut = 0.0
set/format f6.2
set/midas output=yes
write/out -
"*** Method not converging. Shutter offsets from {shval(1)} to {shval(2)} sec."
write/out "    Shutter offset set to 0."

!
! plot the fractional count rate and the fit
DONE2:
write/descr {p3}.tbl HISTORY_UPDA/I/1/1 1
eshutter = (eshutter + shut)/2.0
set/midas output=yes
set/format f6.3
write/out "     Shutter offset: {shut} seconds"
set/midas output=no
set/format
compute/tab {p3} {col10} = {col6}/({col4}+{shut})
select/tab {p3} {crit}
stat/tab {p3} {col10}
set/midas output=yes
set/format f6.2
counts2 = outputr(3)
write/out "     Average Count Rate: {counts2} ADU/sec"
set/format
set/midas output=no
compute/tab {p3} {col10} = ({col10}-{outputr(3)})/{outputr(3)}*100
select/tab {p3} {crit}

if P7(1:1) .eq. "Y" then
  set/grap xform=e6.0 yform=f6.2 clear=off
  plot/tab {p3} {col6} {col10} -60,-40,115,15 7 2
  xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
  ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
  label/grap "Diff. Count Rate in % (2)" {xlab},{ylab} ? ? 1
endif
!
stat/tab {p3} {col10}
linerr(3) = outputr(1)
linerr(4) = outputr(2)
set/format f6.2
set/midas output=yes
write/out " "
write/out -
"     Linearity error after shutter correction: +{linerr(4)}/{linerr(3)} %"
set/format
linerr(5) = (linerr(1) + linerr(3))/2.0
linerr(6) = (linerr(2) + linerr(4))/2.0
!
set/midas output=no
regression/lin {p3} {col10} {col4}
save/regress {p3} regress
compute/regress {p3} {col12} = regress
if P7(1:1) .eq. "Y" then
  set/grap ltype=4 stype=0
  overplot/tab {p3} {col6} {col12}
  set/gra clear=on
!
!*** all done; get the postscript file
  copy/gra postscript 
  -copy postscript.ps {p3}_lin2.ps
  set/midas output=yes
  write/out " "
  write/out "Plot with fitted linearity curves in file {p3}_lin2.ps"
  set/grap clear=on
endif
write/out " "
write/out "End of test T2"
write/out " "
wait 5


IF P1(1:3) .NE. "ALL" RETURN
!
!
TESTT3:
write/out "Test T3: Determination of the transfer curve"
write/out "----------------------------------------------------------------"
write/out "From the selected entries the table a linear regression"
write/out "analysis is done to determine the readout noise. The readout"
write/out "noise is determined by the inverse of the slope between"
write/out "the median and the variance" multiplied by the sigma of the"
write/out "bias (determined by TESTB/CCD and stored in keyword BIASSIGM."
write/out " "
!
!set/midas output=logonly
set/midas output=no
select/tab {p3} {crit}
if P7(1:1) .eq. "Y" then
  set/grap xformat=e6.0 yformat=e6.2
  !plot/tab {p3} {col5} {col7}  -140,-50,30,85 7 0
  plot/tab {p3} {col5} {col7}  -140,-40,25,75 7 0
  xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
  ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
  label/gra "Transfer curve (1), edited" {xlab},{ylab} ? ? 1
  set/grap clear=off
endif
regression/lin {p3} {col7} {col5}
save/regress {p3} regress
compute/regress {p3} {col12} = regress
select/tab {p3} {crit}
ADUCF1  = 1 / OUTPUTD(2) 
errcf1  = outputr(4)/outputd(2)/outputd(2)
ENOISE1 = aducf1 * BIASSIGM
set/midas output=yes
set/format f10.3,f10.3
write/out -
 "     Transfer curve 1 : slope: {outputd(2)}, offset in y: {outputd(1)}"
write/out "     Conv. factor: {aducf1} e-/ADU; Elec. RON: {enoise1} e-"
set/format
if P7(1:1) .eq. "Y" overplot/tab {p3} {col5} {col12} ? 0 3
!
!set/midas output=logonly
set/midas output=no
select/tab {p3} {crit}

if P7(1:1) .eq. "Y" then
  !plot/tab {p3} {col6} {col7}  -140,-50,30,15 7 0
  plot/tab {p3} {col6} {col7}  -140,-40,25,15 7 0
  xlab =  plrgrap(1)  + ( plrgrap(2) - plrgrap(1) )/10
  ylab =  plrgrap(6)  - ( plrgrap(6) - plrgrap(5) )/10
  label/grap "Transfer curve (2), edited" {xlab},{ylab} ? ? 1
endif
regression/lin {p3} {col7} {col6}
save/regress {p3} regress
compute/regress {p3} {col12} = regress
select/tab {p3} {crit}
aducf2 = 1 / OUTPUTD(2) 
errcf2  = outputr(4)/outputd(2)/outputd(2)
enoise2 = aducf2 * BIASSIGM 
set/midas output=yes
set/format f10.3,f10.3
write/out -
 "     Transfer curve 2 : slope: {outputd(2)}, offset in y: {outputd(1)}"
write/out "     Conv. factor: {aducf2} e-/ADU; Elec. RON: {enoise2} e-"
set/format
ADUCF  = (aducf1 + aducf2) / 2
errcf  = (errcf1 + errcf2) / 2
ENOISE = aducf * BIASSIGM
erren  = errcf * BIASSIGM
set/format f10.3
write/out " "
write/out "     Mean conv. factor: {ADUCF} +/- {errcf} e-/ADU" 
WRITE/OUT "     Mean Elec. RON:   {ENOISE} +/- {erren} e-"
write/out "     Values are stored in the CCD keywords ADUCF and ENOISE"
counts = (counts1 + counts2)/2.0
write/out " "
write/out "     Average Count Rate after shutter correction: {counts} ADU/sec"

if P7(1:1) .eq. "Y" then
  overplot/tab {p3} {col6} {col12} ? 0 3
  copy/grap postscript 
  -copy postscript.ps {p3}_trans.ps
  write/out " "
  write/out "Plot with fitted transfer curves in file {p3}_trans.ps"
  set/grap clear=on
  !set/grap default
endif
write/out " "
write/out "End of test T3"
write/out " "
!
if P7(1:1) .eq. "Y" then
  delete/ima postscript*.ps NO
  delete/ima graph*.plt NO
endif
RETURN
