! $Id: uves_corr_hotpix.prg,v 1.6 2001/08/08 13:14:44 amodigli Exp $
!+++ UVES_CORR_HOTPIX +++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!
!.IDENT         uves_corr_hotpix.prg 
!
!.KEYWORDS      spectroscopy, echelle, cleaning
!
!.USAGE         @p (IDENT) in_ima in_tab method 
!
!.INPUT         in_ima   (P1): input frame to be cleaned
!               in_tab   (P2): input bad pix table 
!               method   (p3): hot pix correction method L(inear)/R(andom) 
!.OUTPUT        cleaned image (P1)
!
!.RETURN        Q1:  0: successful return
!                   -1: return failure
!
!.DEFAULT
!
!.PURPOSE       Clean images from hot pixels artifacts
!
!.ALGORITHM     Call uves_corr_hotpix.c to correct bad pix in p1 as specified 
!               by p2, using method p3
!               context: CORRECT/HPIX, FIND/HPIX, FIND/DPIX
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 99NOV
!
!.AUTHOR        Andrea Modigliani, ESO-DMO
!
!.VERSION       1.0 2001/03/01  Create
!
!.COMMENT	
!		
!.TODO		
!
!------------------------------------------------------------------------------
!$Name:  $
!$Revision: 1.6 $
!$Log: uves_corr_hotpix.prg,v $
!Revision 1.6  2001/08/08 13:14:44  amodigli
!Added again prefix ESO to QC keywords
!
!Revision 1.5  2001/06/28 12:20:35  amodigli
!Updated QC name from ESO.PRO.QC.BP.CORR
!to QC.BADPIX.CORR
!
!Revision 1.4  2001/05/07 13:49:46  amodigli
!Corr typo in default name for P2
!
!Revision 1.3  2001/03/12 06:48:26  amodigli
!Typos in header description
!
!Revision 1.2  2001/03/08 15:59:22  amodigli
!Added descriptor specifying that a
!correction of bad pixel occurred
!
!Revision 1.1  2001/03/05 07:01:39  amodigli
!created
!
!------------------------------------------------------------------------------
crossref in_ima in_tab method
define/param p1 ?            ima "enter name of input image:"
define/param p2 "hp.tbl" tbl "enter name of hot pixel table:"
define/param p3 "L"            c "enter correction method: [L]/A"

define/local pid/c/1/20 "uves_corr_hotpix" ? +lower

write/key in_a {p1}
write/key in_b {p2}
write/key method/c/1/80 {p3}
method = m$upper(method)

if method(1:1) .eq. "A" .OR. method(1:1) .eq. "L" then
   run PIPE_EXE:uves_corr_hotpix
   write/desc {p1} ESO.QC.BADPIX.CORR/c/1/30 "TRUE"
   write/dhelp {p1} ESO.QC.BADPIX.CORR  "Hot Pix Corr"
else
   mess/out 2 {pid} "The method {method} is not supported. No frame correction."
endif

return 0
!=================================================================



