! $Id: uves_crea_deadcol_tab.prg,v 1.3 2001/11/28 15:25:32 amodigli Exp $
!+++ UVES_DEADCOL_TAB +++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!
!.IDENT         uves_crea_deadcol_tab.prg 
!
!.KEYWORDS      spectroscopy, echelle, cleaning
!
!.USAGE         @p (IDENT) in_ima out_tab 
!
!.INPUT         ima      (P1): input frame to be cleaned
!               out table (P2): table dead col pix table
!.OUTPUT        dead col pixel table (dc.tbl)
!
!.RETURN        Q1:  0: successful return
!                   -1: return failure
!
!.DEFAULT
!
!.PURPOSE       Get a table to clean images from trap columns artifacts
!
!.ALGORITHM     According to the bin defines Xs,Ys,Xe,Ye of the dead col to be 
!               cleaned (in RED MIT chip), to be used as input of CORRECT/HPIX
!      context: FIND/DPIX, CORRECT/HPIX
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 99NOV
!
!.AUTHOR        Andrea Modigliani, ESO-DMO
!
!.VERSION       1.0 2001/02/23  Create
!
!.COMMENT	
!		
!.TODO		
!
!------------------------------------------------------------------------------
!$Name:  $
!$Revision: 1.3 $
!$Log: uves_crea_deadcol_tab.prg,v $
!Revision 1.3  2001/11/28 15:25:32  amodigli
!Corrected 2 typos and added a check on chip
!ID
!
!Revision 1.2  2001/03/12 06:46:01  amodigli
!typos in header description
!
!Revision 1.1  2001/03/05 17:14:54  amodigli
!Added uves_crea_deadcol_tab.prg
!
!Revision 1.1  2001/03/05 06:58:18  amodigli
!Simplified. Now binning treated inside proc.
!
!------------------------------------------------------------------------------
crossref in_ima out_tab
define/param p1 ?          ima "enter name of input image:"
define/param p2 "dc.tbl" tbl "enter name of out tab: "

define/local pid/c/1/20 "uves_crea_deadcol_tab" ? +lower
if {PATHID} .ne. 2 then
   mess/out 1 {pid} "A dead column is at the moment known only for REDU chip"
endif


define/local br1/i/1/4 0 all +lower_levels

define/local bin/c/1/10 " "
define/local binX/i/1/1 0
define/local binY/i/1/1 0
set/format i1
binX = {{p1},{h_xwinbin}}
binY = {{p1},{h_ywinbin}}

bin = "{binX}x{binY}"
set/format
w/o "Arm = 
!-----------------------------
! set the bad raw Xs,Ys,Xe,Ye
!-----------------------------
     if "{bin}" .eq. "1x1" then 
        write/key br1/i/1/4  1,491,3272,492

else if "{bin}" .eq. "1x2" then                 !means sizeX=2,sizeY=1
                                                !X and Y are swapped
        write/key br1/i/1/4  1,491,1636,492

else if "{bin}" .eq. "2x2" then
        write/key br1/i/1/4  1,245,1636,246

else if "{bin}" .eq. "2x3" then            !means sizeX=3,sizeY=2 
                                           !X and Y are swapped
        write/key br1/i/1/4  1,245,1091,246
endif
!----------------------------
! creates the bad pixel table
!----------------------------
@p uves_crea_deadcol_tab,CREA_BAD_PIX_TAB {p2} 

!=================================================================
!=================================================================
ENTRY CREA_BAD_PIX_TAB
!-------------------------------------------------
!creates a given table with 4 columns and 7 raws
!-------------------------------------------------
define/parameter p1 ?       c "Enter Table Name: "

crea/tab {p1}
CREATE/COLU {p1} :XSTARTPIX
CREATE/COLU {p1} :YSTARTPIX
CREATE/COLU {p1} :XENDPIX
CREATE/COLU {p1} :YENDPIX

@p uves_crea_deadcol_tab,ADD_ROW {p1} 1 {br1(1)} {br1(2)} {br1(3)} {br1(4)} 

return 0
!=================================================================
!=================================================================
ENTRY ADD_ROW
!--------------------------------------------------
! add a raw entry at a given index in a given table
!--------------------------------------------------
define/parameter p1 ? c "Enter Table Name: "
define/parameter p2 ? n "Enter bad raw index (1)"
define/parameter p3 ? n "Enter X start Pix: "
define/parameter p4 ? n "Enter Y start Pix: "
define/parameter p5 ? n "Enter X end Pix: "
definE/parameter p6 ? n "Enter Y end Pix: "

define/local x1/i/1/1 {p3}
define/local y1/i/1/1 {p4}
define/local x2/i/1/1 {p5}
define/local y2/i/1/1 {p6}


WRITE/TABLE {p1} :XSTARTPIX @{p2}   {x1} 
WRITE/TABLE {p1} :YSTARTPIX @{p2}   {y1}
WRITE/TABLE {p1} :XENDPIX   @{p2}   {x2}
WRITE/TABLE {p1} :YENDPIX   @{p2}   {y2}

return 0
!=================================================================



