! $Id: uves_creadisp.prg,v 1.4 2002/03/04 09:16:16 amodigli Exp $
!+++ UVES_CREADISP ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT	(C) 1998  European Southern Observatory
!
!.IDENT		uves_creadisp
!
!.KEYWORDS	Creates image and graphic displays needed for pipeline
!
!.USAGE		@p (IDENT) [max_x] [max_y] [use_x] [use_y]
!
!.INPUT		max_X   (P1): maximum X dimension of your terminal
!		max_Y   (P2): maximum Y dimension of your terminal
!		use_X 	(P3): X dimension to be used, OR
!				in case its less than 1 interpret as
!				percentage use parameter.
!		use_Y   (P4): Y dimension to be used
!
!.OUTPUT	terminal output
!
!.RETURN	Q1:  0: successful return
!		    -1: a successful return failed
!
!.DEFAULT	P1	1280
!		P2	1024
!		P3	+
!		P4	+
!
!.PURPOSE	Create 3 image displays on top of the screen. Below these
!		2 graphic windows will be created and 1 big graphic window
!		(iconified).
!		The global keywords used are already created by setting the
!		UVES-context.
!
!.ALGORITHM	Calls standard MIDAS commands. Checks if P3 is less than 1.0
!		at the beginning which is interpreted as a perecentage of
!		the terminal area to be used.
!
!.ENVIRON	UVES context
!   
!.LANGUAGE	MIDAS 98NOV
!
!.AUTHOR	Sebastian Wolf, ESO-DMO
!
!.VERSION	1.0 1998/08/24	Creation
! 020225	last modif (KB)
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.4 $
! $Log: uves_creadisp.prg,v $
! Revision 1.4  2002/03/04 09:16:16  amodigli
! Speed-up from Klaus
!
! Revision 1.3  2001/05/07 13:42:12  amodigli
! Typo in default size of windows
!
! Revision 1.2  2001/01/04 10:39:37  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
! 
define/param P1 1280 n "maximum allowed X-dimension [hardware]:"
define/param P2 1024 n "maximum allowed Y-dimension [hardware]:"
define/param P3 + n "used X-dimension:"
define/param P4 + n "used Y-dimension:"

define/local i1/i/1/1 0
define/local xsz/i/1/1 0
define/local ysz/i/1/1 0
define/local xdim/i/1/1 0
define/local ydim/i/1/1 0
define/local xoff/i/1/1 0
define/local yoff/i/1/1 0
define/local off/i/1/1 6
define/local dd/r/1/1 0.0


if m$existk("nice_crea") .eq. 0 then
  write/keyword nice_crea/c/1/1 N
else if nice_crea .eq. "Y" then
  assign/graph g,3
  modify/graph W			!due to MODIFY/GRAP Icon
  wait/sec 0.1				!I have to display the graph. window
  modify/graph I			!first and then iconize it!
  return 0
else
  nice_crea = "N"
endif

if pipeline(5) .eq. 0 return -1		!no display

if P3(1:1) .eq. "+" P3 = P1
if P4(1:1) .eq. "+" P4 = P2

if {P3} .le. 1.0 then
  xdim = m$abs({P3}) * {P1}
  ydim = m$abs({P3}) * {P2}
else 
  xdim = {P3}
  ydim = {P4}
  if {P3} .gt. {P1} .OR. {P3} .lt. 0 then
    mess/out 1 creadisp "Bad X-size: {P3} (<={P1} and should be positive!)"
    return -1
  endif
  if {P4} .gt. {P2} .OR. {P4} .lt. 0 then
    mess/out 1 creadisp "Bad Y-size: {P4} (<={P2} and should be positive!)"
    return -2
  endif
endif
xsz = xdim					!size of the used
ysz = ydim					!screen area
 
dd = (xdim - 3 * off) / 3.0
xdim = m$nint({dd})
dd = (ydim - 120) / 2.0
ydim = m$nint({dd})

xoff = -xdim - off
yoff = {P2} - ydim - 56
do i1 = 1 3
  xoff = xoff + xdim + off
  creat/disp {i1} {xdim},{ydim},{xoff},{yoff} >Null
  if i1 .eq. 1 wait 0.1
enddo

write/key NICE_DIDS/i/1/10 0 ALL
write/key NICE_DIDS/i/1/3 1,1,1
write/key UVES_DXSZ/i/1/3 {xdim},{xdim},{xdim}
write/key UVES_DYSZ/i/1/3 {ydim},{ydim},{ydim}


ydim = UVES_DXSZ(1) / 1.4
UVES_GXSZ(1) = UVES_DXSZ(1)
UVES_GYSZ(1) = ydim
UVES_GXSZ(2) = UVES_DXSZ(1)
UVES_GYSZ(2) = ydim
UVES_GYSZ(3) = yoff + ydim - 56 * 4
UVES_GXSZ(3) = UVES_DXSZ(1) * 3 + off * 2

yoff = yoff - ydim - 28

xoff = (UVES_DXSZ(1) + {off})
crea/gra 1 {UVES_DXSZ(1)},{UVES_GYSZ(1)},{xoff},{yoff} >Null
xoff = {xoff} + {UVES_DXSZ(1)} + {off}
crea/gra 2 {UVES_GXSZ(2)},{UVES_GYSZ(2)},{xoff},{yoff} >Null
xoff = 0.0
crea/gra 3 {UVES_GXSZ(3)},{UVES_GYSZ(3)},{xoff},{yoff} >Null
write/key NICE_GIDS/i/1/10 0 ALL
write/key NICE_GIDS/i/1/3 1,1,1

load/lut heat

NICE_CREA = "Y"

modify/graph W >Null
wait/secs 0.1
modify/graph I >Null

return 0
