! $Id: uves_efficiency.prg,v 1.22 2002/02/08 15:54:21 amodigli Exp $
!+++ UVES_EFFICIENCY ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_efficiency.prg
!
!.KEYWORDS      
!
!.USAGE         @p (IDENT) incat outcat refcat
!
!.INPUT         incat 	(P1): catalog of raw and reference frames
!				standard star frame (raw data)
!		incat 	(P1): catalog of raw and reference frames
!				DRS setup table [DRS_SETUP_x]
!				order table [ORDER_TABLE_x]
!				background table [BACKGR_TABLE_x]
!				flux standard table [FLUX_STD_TABLE]
!				atmospheric extinction table [EXTCOEFF_TABLE]
!.OUTPUT        refcat	(P2): catalog of products
!				efficiency table for each detector
!				[EFFICIENCY_TABLE_x] (x:REDL, REDU, BLUE)
!
!.RETURN        Q1: =0: successful return
!                   -1: a successful return failed
!
!.DEFAULT	
!
!.PURPOSE       Determine the instrument efficiency
!
!.ALGORITHM     Do a order extraction of the standard star without averaging
!		after having subtracted the background. Resample the data
!		and devide by the corresponding reference spectrum of the
!		flux standard catalog. In order to get the efficiency ...
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Andreas Kaufer, O. Stahl and Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/10/08  Created. (O.Stahl)
!		    1999/12/19	Adapted version for the pipeline (AK,SW)
!
!.COMMENT	FLUXTAB, ORDTAB, BAKTAB will be reset.
!		SAMPLE keyword will be read from DRS table
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.22 $
! $Log: uves_efficiency.prg,v $
! Revision 1.22  2002/02/08 15:54:21  amodigli
! Eliminated an unnecessary redefinition of wlen
!
! Revision 1.21  2001/10/30 12:48:47  amodigli
! Increased P3 (matching box) in align/std
! from 30 to 60 arcecs to improove robustness
!
! Revision 1.20  2001/09/08 15:30:44  amodigli
! PIPEFILE->h_pipefile
! h_namecros without QC prefix. We use UVES_ICS.
!
! Revision 1.19  2001/08/12 16:14:48  amodigli
! Passing p_version as string par
!
! Revision 1.18  2001/08/10 15:32:39  amodigli
! used h_inspath and h_insmode in place of their
! correnspondent FITS keywords
!
! Revision 1.17  2001/07/15 14:18:57  amodigli
! Eliminated unnecessary call to uves_getpversion.prg
!
! Revision 1.16  2001/07/10 09:45:37  amodigli
! Used QC.INS.GRATi.NAME to log CHIP name
!
! Revision 1.15  2001/07/04 07:50:04  amodigli
! New QC1 Naming conv
!
! Revision 1.14  2001/06/20 14:21:15  amodigli
! Using QC1 LOG and PAF files
!
! Revision 1.13  2001/06/16 06:56:54  amodigli
! Changing QC-LOG naming conventions
! Now portid is a global key defined in uvesinit.prg
!
! Revision 1.12  2001/06/05 11:28:27  amodigli
! Taken in count that with LINEAR method
! the pixel are added and so in presence
! of possible different extraction slits
! for sky1,obj,sky2 one have to rescale
! the intensity taking in count of the
! relative obj/skyN ratio.
!
! Revision 1.11  2001/05/29 06:32:03  amodigli
! taken in count of offset in eff calculation
!
! Revision 1.10  2001/05/07 08:47:14  amodigli
! Eliminated monitor of backgr image
!
! Revision 1.9  2001/05/07 08:46:22  amodigli
! Set radSmoX to 300 *sizeX/4096
!
! Revision 1.8  2001/05/04 13:56:35  amodigli
! Added bkg image as data product
! Changed bkg extraction parameters
!
! Revision 1.7  2001/01/24 08:40:06  amodigli
! Made changes to take in count of sky
! subtraction in STD star extraction
! as suggested from AKA in 12/11/00 e-mail
!
! Revision 1.6  2001/01/04 10:39:52  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
define/param P1 ? I "Enter object frame: "
define/param P2 ? c "Enter output catalog:"
define/param P3 ? c "Enter reference catalog:"
!
! Procedure identification
!
define/local pid/c/1/30	"Efficiency" ? +lower

define/local modext/c/1/25 " " ALL
define/local ifrm/c/1/100 " " ALL
define/local refc/c/1/100 " " ALL
define/local eff/c/1/100 " " ALL
define/local extinc/c/1/100 " " ALL
define/local stdstar/c/1/25 " " ALL
define/local ext/c/1/10 " "
define/local num/i/1/1 0
define/local catal/i/1/1  0
def/local do_class/c/1/25 " " all

if m$exist(p2) .eq. 0 .AND. p2(1:1) .ne. "+" -
  create/icat {p2} NULL {UVES_IDSTRNG} >Null


						!Check input type of P1
parse/path {p1}					!Get file extension.
ext = m$upper(q3)
if ext(1:4) .eq. ".CAT" then			!Check if we got a catalog.
  show/icat {p1} >Null
  num = outputi(1)				!Check number of entries.
  if num .lt. 1 goto ERR1			!No entries in the catalog.
  goto CATALOG
else if ext(1:4) .eq. ".BDF" then
  ifrm = p1
  goto SINGLE
else
  goto ERR2					!Bad P1 specification
endif



CATALOG:					!Loop through the input catalog
  store/frame ifrm {p1} 1 FINI

SINGLE:
  write/out " "
  mess/out 3 {pid} "Do the efficency calculation for\n`{ifrm}` ..."
  CONFIG/INSTR {ifrm}
  modext = q2


  search/icat {p3} {DRSRC_ID({PATHID})} NB
  if m$strlen(out_a) .eq. 0 then
    append/out "No appropriate DRS setup table found!"
    if FORCE_DRS(1:1) .eq. "Y" goto ERR4
    mess/out 2 {pid}
    mess/out 3 {pid} "Use global keyword settings (SHOW/ECHELLE)!"
  else
    savini/echelle {out_a} READ
  endif
  search/icat {p3} {ORDER_ID({PATHID})} NB	!get the order table
  if m$strlen(out_a) .eq. 0 then		!... does not exist
    mess/out 1 {pid} "No appropriate order table found!"
    goto ERR4
  else
    set/echelle ordtab={out_a}
  endif
  search/icat {p3} {BACK_ID({PATHID})} NB	!interorder background table
  if m$strlen(out_a) .eq. 0 then
    set/echelle BKGOPT=NO
    mess/out 1 {pid} "No appropriate background table found!"
    goto ERR4
  else
    set/echelle BAKTAB={out_a}			!BKGOPT never used in echelle!
  endif
  search/icat {p3} {LINTAB_ID({PATHID})}2 NB	!get the line table
  if m$strlen(out_a) .eq. 0 then		!... does not exist
    mess/out 1 {pid} "No appropriate line table found!"
    goto ERR4
  else
    set/echelle lintab={out_a}
  endif
  search/icat {p3} {STDFLX_ID} NB		!get the flux standard catalog
  if m$strlen(out_a) .eq. 0 then		!... does not exist
    mess/out 1 {pid} "No appropriate std star flux table found!"
    goto ERR4
  else
    refc = out_a
  endif
  search/icat {p3} {ATMEXT_ID} NB		!get the extinction table
  if m$strlen(out_a) .eq. 0 then		!... does not exist
    mess/out 1 {pid} "No extinction table found!"
    goto ERR4
  else
    extinc = out_a
  endif


  mess/out 3 {pid} "Get reference from catalog `{refc}` ..."
!
! AKA Error box increased to 30 arcsec
  ALIGN/STD {ifrm} {refc} 60			!Extracts the reference spect.
  if {q1} .lt. 0 goto ERR4			!from reference catalog, its
						!name is returned in OUT_A
  set/echelle FLUXTAB="{out_a}.tbl"
  mess/out 3 {pid} "Reference found: `{out_a}`."
  stdstar = "{out_a}"

  @p uves_efficiency,CALC_EFF {ifrm} {extinc} {stdstar}
  eff = q1

  classify/product {eff} {EFFI_ID}{MODEXT} {d_pre} {d_geo},{d_det} OK y n
  if p2(1:1) .ne. "+" add/icat {p2} {eff} >Null
  ! am: adding SCIENCE bkg (if exist, usually yes)
  if m$exist("bg_{ifrm}") .eq. 1 then
      do_class = "BKG_STD{modext}"
      classify/product bg_{ifrm} {do_class} {d_pre} {d_pho},{d_geo},{d_det} OK y n
      if p2(1:1) .ne. "+" add/icat {p2} bg_{ifrm} >Null
  endif


SKIP:
  if ext(1:4) .eq. ".BDF" goto FINI		!single frame -> finished.
  goto CATALOG
FINI:
if eff(1:1) .eq. " " goto ERR5

return 0

ERR1:
  mess/out 1 {pid} "Empty input catalog !"
  return -1
ERR2:
  mess/out 1 {pid} "You have to specify full name of input frame (.cat, .bdf)!"
  return -1
ERR4:
  append/out "--> skip input frame: `{ifrm}'"
  mess/out 1 {pid}
  goto SKIP
ERR5:
  mess/out 1 {pid} "Efficiency computation failed!"
  return -1




ENTRY CALC_EFF

define/param p1 ? I "Enter name of standard star frame (raw data):"
define/param p2 ? I "Enter name of extinction table:"
define/param p3 ? c "Enter name of standard star:"
!
! Local variables
!
def/local N/i/1/1 0
def/local nord/i/1/1 0
def/local nptot/i/1/1 0
def/local texp/r/1/1 0.0
def/local ccd_gain/r/1/1 0.0
def/local resppx/r/1/1 0.0
def/local Atel/r/1/1 0.0
def/local max/r/1/3 0,0,0
def/local smooth/r/1/1 1.0
def/local wcen/r/1/1 1.0
def/local wlen/r/1/1 0.0
def/local w/r/1/1  0.0
def/local e/r/1/1  0.0
def/local am/r/1/1 0.0

!24/01/01 am: from AKA suggestion
def/local fslit/r/1/1 0.0
def/local eslit/r/1/1 0.0

def/local eslit1/r/1/1 0.0
def/local eslit2/r/1/1 0.0
def/local eslit3/r/1/1 0.0

def/local eoff/r/1/1  0.0
def/local result/c/1/100 " " all
def/local prot/c/1/100 " " all
def/local modext/c/1/10 " " all
def/local fname/c/1/30 " " all
!
def/local h/r/1/1 6.64e-34	! Planck constant
def/local c/r/1/1 3.e8		! speed of light [m/s]
! UT2 NasB
def/local tdia/r/1/1 818 	! telescope primary diameter [cm]
def/local odia/r/1/1 140	! telescope central obstruction diam [cm]
!
def/local a/r/1/1 0.0
def/local b/r/1/1 0.0
!
CONFIG/INSTR {p1}
modext = q2
!
parse/path {p1}
prot = "effprot_{q2}.tbl"
result = "eff_{q2}"

get/pathid {p1}
wlen = m$value({p1},{h_cwlen({PATHID})})	!get central wavelength
set/format
if m$existd(p1,h_origname) .eq. 1 then
  fname = "{{p1},{h_origname}}"
else
  fname = "{{p1},{h_obsname}}_{{p1},{h_obstpno}}"
endif

define/local date/c/1/30 " " all
date   = "{{P1},ESO.TPL.START}"

!
! output to the logging system (AKA 2000-03-18) 
!
set/format F13.6


define/local dpr_type/c/1/80 " " all
define/local tpl_id/c/1/80 " " all
define/local arcfile/c/1/80 " " all
define/local pro_catg/c/1/80 " " all

qc1log/out 2 
set/format i1 f8.4,f8.4
qc1log/out 1 {p1} {h_qc_did}  {d_qc_did}    "Data Dictionary for QC"
qc1log/out 1 {p1} "PRO.REC1.PIPE.ID" "{p_version}" "Pipeline (unique) identifier" PRO
qc1log/out 1 {p1} "QC.TEST1.ID" "Efficiency-Test-Results"  "Name of QC Test"

if m$existd(p1,h_dprtype) .eq. 1 then
  dpr_type = "{{p1},{h_dprtype}}"
  qc1log/out 1 "{result}.tbl" {h_dprtype} "{dpr_type}" + DPR
endif
if m$existd(p1,h_tpid) .eq. 1 then
  tpl_id = "{{p1},{h_tpid}}"
  qc1log/out 1 {result}.tbl {h_tpid} "{tpl_id}" + TPL
endif
if m$existd(p1,h_arcfile) .eq. 1 then
  arcfile = "{{p1},{h_arcfile}}"
  qc1log/out 1 {result}.tbl {h_arcfile} {arcfile} + PRIMARY-FITS
endif
if m$existd(p1,h_procatg) .eq. 1 then
  pro_catg = "{{p1},{h_procatg}}"
  !NB here we have to use EFFI_ID to have the proper product category
  qc1log/out 1 {result}.tbl {h_procatg} {EFFI_ID}{MODEXT} + PRO
endif
qc1log/out 1 {result}.tbl {h_pipefile} {result}.tbl + PRO

if m$existd(p1,"{h_inspath}") .eq. 1 then 
    qc1log/out 1 {result}.tbl {h_inspath} {{p1},{h_inspath}}                 "Optical path used."         UVES_ICS
else
   mess/out 2 {pid} "Missing descriptor {h_inspath}"
endif
if m$existd(p1,"{h_insmode}") .eq. 1 then 
   qc1log/out 1 {result}.tbl {h_insmode} {{p1},{h_insmode}}                  "Instrument mode used."      UVES_ICS  
else
   mess/out 2 {pid} "Missing descriptor {h_insmode}"
endif
    
qc1log/out 1 {result}.tbl "{h_namecros({PATHID})(5:18)}"      {CROSS}  "Cross disperser ID"               UVES_ICS           
qc1log/out 1 {result}.tbl "{h_cwlen({PATHID})(5:)}"             {wlen}  "Grating central wavelength [nm]" UVES_ICS
qc1log/out 1 {result}.tbl "{h_fits_nameccd({PATHID})(5:)}"     {CCD}   "Grating unique ID"                CCDDCS
qc1log/out 1 {result}.tbl "OBS.TARG.NAME"                       {p3}    "OB target name"                  OBS
set/format 

!
if m$existd(p1,"BACKGROUND") .eq. 0 then	!BACKGROUND descrip. indicates
  mess/out 3 {pid} "Subtract background from `{p1}' ..."
  set/grap yaxis=-50,450
  
!am: old code  
!  subtract/back {p1} bg_{p1} &b >Null		!Subtract background f. science
! -rename middummb.bdf {p1}			        !and overwrite input frame.
!am: new code 06/04/01
   -copy {p1} pre_bg_sub_{p1}
   subtract/back {p1} tmp_bg_{p1} &b >Null
   define/local sizeX/i/1/1  0
   define/local sizeY/i/1/1  0
   sizeX = m$value({p1},NPIX(1))
   sizeY = m$value({p1},NPIX(2))
   define/local radSmoX/i/1/1 25
   define/local radSmoY/i/1/1 10
   radSmoX = 300 * {sizeX}/4096   
   wlen = m$value({p1},{h_cwlen({PATHID})})
   if wlen .eq. 860 then
     radSmoY = 500 * {sizeY}/2048  
   else
     radSmoY = 200 * {sizeY}/2048
   endif
   w/o "radSmoX={radSmoX} radSmoY={radSmoY}"
   filter/smooth tmp_bg_{p1}  bg_{p1} {radSmoX},{radSmoY} >Null
   compute/image {p1} = pre_bg_sub_{p1} - bg_{p1}
!am: end new code  
 
  if BKGVISU(1:1) .eq. "Y" then
    if m$exist("bg_{p1}") .eq. 1 then
      copy/grap postscript
      -rename postscript.ps bg_{p1}.ps
      assign/disp 3
      load/ima bg_{p1} sca=-8,-5 cuts=f,3sig
    endif
  endif
else						!the background subtraction!
  mess/out 2 {pid} "Background from `{p1}' already subtracted !"
endif


! calculates the obj offset
stat/table {ORDTAB} :ORDER >Null
offset/echelle {p1} {slit} {outputr(2)} ? SILENT >Null
mess/out 3 {pid} "Offset={OFFSET}, outputr(1)={outputr(1)}"
OFFSET = outputr(1)
define/local dwin/i/1/1 1

outputr(4) = m$value({p1},{h_pixscl})		!Pixelscale ["/pixel]
outputr(2) = m$value({p1},{h_sllen({PATHID})})
!24/01/01 am: from AKA suggestion
fslit = outputr(2)/outputr(4)/{CCDBIN(2)} - 2.0
mess/out 3 {pid} "Full slit length in pixel: {fslit}"
mess/out 3 {pid} "Extract orders (EXTMTD=LINEAR) of {p1} ..."
eslit = fslit/2.0
outputr(3) = fslit - eslit/2.0
SKYWIND(4) = fslit / 2.0
SKYWIND(1) = -SKYWIND(4)
SKYWIND(2) = OFFSET - DWIN - eslit/2
SKYWIND(3) = OFFSET + DWIN + eslit/2
outputr(1) = SKYWIND(2) - SKYWIND(1)
outputr(2) = SKYWIND(4) - SKYWIND(3)
mess/out 3 {pid} "Sky Wind: {skywind(1)} {skywind(2)} {skywind(3)} {skywind(4)}"
mess/out 3 {pid} "skyw(2)-skyw(1): {outputr(1)} skyw(4)-skyw(3): {outputr(2)}" 
nsky = 2

if outputr(1) .lt. 4 then			!Use only second sky window
   mess/out 3 {pid} "Use only second sky window!"
   skywind(1) = skywind(3)
   skywind(2) = skywind(4)
   nsky = 1
   !LINTAB_1 = LINTAB_2				!rename line table !!
endif
if outputr(2) .lt. 4 then			!Use only first sky window
   mess/out 3 {pid} "Use only first sky window!"
   skywind(3) = skywind(1)
   skywind(4) = skywind(2)
   nsky = 1
endif
eslit2 = fslit/2.0
eslit1 = skywind(2)-skywind(1)
eslit3 = skywind(4)-skywind(3)
  
! object
eslit = fslit/2.0
eoff  = 0.0
! write/out "object: {fslit} {eslit} {eoff}"
extract/order {p1} e_{p1} {eslit},0,{eoff} LINEAR {ORDTAB} COEFF
! lower sky
eslit = skywind(2) - skywind(1)		!Size of 1st sky window
eoff  = skywind(1) + slit/2.			!Offset to center of sky window
! write/out "sky lo: {fslit} {eslit} {eoff}"
extract/order {p1} sl_{p1} {eslit},0,{eoff} LINEAR {ORDTAB} COEFF
if nsky .eq. 2 then 
   ! upper sky
   eslit = skywind(4) - skywind(3)		!Do the same for the second
   eoff  =  skywind(3) + slit/2
   ! write/out "sky up: {fslit} {eslit} {eoff}"
   extract/order {p1} su_{p1} {eslit},0,{eoff} LINEAR {ORDTAB} COEFF
   ! do sky subtraction
   compute/ima x_{p1} = e_{p1} - ({eslit2}/{eslit1}*sl_{p1} + {eslit2}/{eslit3}*su_{p1})/2.0
else
   compute/ima x_{p1} = e_{p1} - ({eslit2}/{eslit1}*sl_{p1})
endif
!NB 
!as we use LINEAR method which add the pixels,
!Due to possible different extraction slit for sky1,obj,sky2
!we have in general to rescale each sky contribute to the obj
!extraction slit (and if 2 are the contribute we average)
copy/dd {p1} *,3 x_{p1}

set/format F7.5
if SAMPLE(1:3) .eq. "+++" then			!Get the wavelength binsize
  get/sample 					!at the centre of the first
  SAMPLE = "{outputd(1)}"			!order.
else
  mess/out 2 {pid} "SAMPLE = {SAMPLE} read from DRS-table !"
endif
smooth = 10*{sample}
mess/out 3 {pid} "Resample data, use smooth sample: {smooth}"
rebin/echelle x_{p1} wx_{p1} {smooth}	>Null
copy/dd {p1} *,3 wx_{p1}

convert/table   &o = {FLUXTAB} :{c_fxwave} :{c_fxflux}  wx_{p1} SPECTR
verify/echelle  &o AGAINST wx_{p1}
compute/image   resp_{p1} = wx_{p1}/&o >Null
copy/dd {p1} *,3 resp_{p1}

nord = {resp_{p1},NPIX(2)}
define/local topeff/r/1/{nord} 0 ALL
!
! create result table
crea/tab {prot} 2 {nord} 
crea/col {prot} :wave R*4 F7.1
crea/col {prot} :eff  R*4 F5.3 
!
mess/out 3 {pid} "Compute telescope + instrument DQE " 

merge/eche resp_{p1} eff 1,{nord} NOAPP  >Null
comp/airmass resp_{p1} >Null
!
am = m$value(resp_{p1},O_AIRM)
set/format F4.2
mess/out 3 {pid} "Extinction corr. for AM = {am}"
set/format
! 
start:
do N = 1 {nord}
! write/out "Order {N}: "
 copy/dd resp_{p1} *,3 eff{N}
 extinc/spec eff{N} &x ? {p2}   		>Null
 filter/median &x ext{N} 3			>Null
enddo
!
! compute telescope effective area in cm^2
Atel = 3.141/4.0*(tdia**2-odia**2)
! get exposure time
texp     = m$value(resp_{p1},O_TIME(7))
! get CCD conversion factor
ccd_gain = m$value(resp_{p1},{h_confac({uves_portid({PATHID})})})
!
!if {PATHID} .eq. 3 then
! resppx   = 192500
!else
! resppx   = 242000
!endif
!
set/format T10.3
mess/out 3 {pid} "Exposure time = {texp} seconds    "
mess/out 3 {pid} "CCD gain      = {ccd_gain} e-/ADU "
mess/out 3 {pid} "Tel eff Area  = {Atel} cm^2        "
!mess/out 3 {pid} "R/pixel       = {resppx}          "
!
! output to the logging system (AKA 2000-03-18) 
!
set/format F13.6

set/format i1 f8.4,f8.4
qc1log/out 1 {result}.tbl "{h_confac({uves_portid({PATHID})})}" {ccd_gain}  "Conversion from ADUs to electrons" CCDDCS
set/format i1 f8.0,f8.0
qc1log/out 1 {result}.tbl "DET{uves_portid({PATHID})}.WIN{uves_portid({PATHID})}.UIT1"     {texp}  "user defined subintegration time" CCDDCS
!we suppress the following QC key on one side as the telescope will not move elsewere from Paranal, 
!so the Telescope area is known on the other side there is no special dictionary keyword to define this value

!qc1log/out 1 {result}.tbl "QC.EFF.TEL.AREA"                    {Atel} 
set/format i1 f8.4,f8.4
qc1log/out 1 {result}.tbl "AIRMASS"            {am} "Averaged airmass" PRIMARY-FITS

!
set/format

do N = 1 {nord}
 copy/it ext{N}.bdf ext{N}.tbl :wave
 name/col ext{N}.tbl #2 :ctsperg
!save order number
 comp/tab ext{N}.tbl :order = {N} 
!ctsperg =  counts per 1e-16 erg/cm^2/A/sec] per pixel
 comp/tab  ext{N}.tbl :ctsperg = :ctsperg/1.e-16
!compute the conversion factor erg to photons [wave in A]
 comp/tab ext{N}.tbl :erg2phot = :wave*1.e-17/({h}*{c})
!
!Lin. Interpolate binsize as fkt. of wavelength within the order 
 plot/calibrat {N},{N} N
 a = (outputd(4)-outputd(6))/(outputd(3)-outputd(5))
 b = outputd(4)-a*outputd(3)
!
 comp/tab ext{N}.tbl :binsize = {a}*:wave+{b}
! comp/tab ext{N}.tbl :binsize = :wave/{resppx}
! compute expected photons per count
 comp/tab ext{N}.tbl :nphot = (1./:ctsperg)*:erg2phot*:binsize*{texp}*{Atel}
! Finally compute the efficiency
 comp/tab ext{N}.tbl :eff = {ccd_gain}/:nphot
!



 if N .eq. 1 then
  copy/tab ext{N}.tbl {result}.tbl
 else   
  merge/tab {result}.tbl ext{N}.tbl &tab
  copy/tab &tab {result}.tbl
 endif
 
!NB only now result.tbl is present so I can create the descriptor.. 
write/descr {result}.tbl AIRMASS/c/1/40
write/dhelp {result}.tbl AIRMASS "Averaged airmass"

!
!am 3/11/00 Added FITS keys from source raw file
copy/dd {p1} *,3 {result}.tbl

 wcen = m$value(wx_{p1},WSTART({N}))
 nptot = m$value(wx_{p1},NPTOT({N}))
 wcen = {nptot}*{smooth}/2.0 + wcen
 {prot},:wave,@{N} = wcen
!
 max(1) = wcen+nptot/10*smooth
 max(2) = wcen-nptot/10*smooth
 sele/tab ext{N}.tbl :wave.gt.{max(2)}.and.:wave.lt.{max(1)} >Null
 copy/tab ext{N}.tbl &median
 sort/tab &median :eff
 show/tab &median >Null
 outputi(2) = outputi(2)*0.9
 topeff({N}) = {&median,:eff,{outputi(2)}}
 {prot},:eff,@{N} = topeff({N})
 !
 ! output to the logging system (AKA 2000-03-18) 
 !
 set/format F13.6 i1
 w = m$value({prot},:wave,@{N})/10.
 e = m$value({prot},:eff,@{N})
 qc1log/out 1 {result}.tbl "QC.BLAZEFF{N}"      "{e}" "Blaze Efficiency"
 qc1log/out 1 {result}.tbl "QC.BLAZWLEN{N}"    "{w}"  "Blaze wavelength"
 set/format
enddo
qc1log/out 3

copy/kd topeff/r/1/{nord} {result}.tbl TOPEFF/r/1/{nord}
!
if NICE_CREA .eq. "Y" then
 assign/gra g,3
 modify/gra w
 stat/tab {result}.tbl :wave 	>Null
 max(1) = outputr(1)-10
 max(2) = outputr(2)+10
 set/grap xaxis={max(1)},{max(2)} yaxis=-0.01,0.2
 plot/axes ? ? ? "Wavelength [\AA ]" "DQE [Telescope+Instrument+Detector]" 
 over/tab {result}.tbl :wave :eff ? 0
 set/gra color=4
 over/tab {prot} :wave :eff ? 0
 set/gra color=1
 label/grap "UVES Detection Quantum Efficiency" {max(1)},0.205 ? ? 1
 copy/gra postscript
 -rename postscript.ps {result}.ps
! read/tab {result} :wave :eff
endif
!
return {result}.tbl
