! $Id: uves_getframes.prg,v 1.7 2002/10/25 09:54:34 amodigli Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       uves_getframes.prg
!
!.KEYWORDS    Pipeline input frame conversion
!
!.USAGE       get/frames  frm_set ref_set cat_accept [cat_reject]
!
!.INPUT       frm_set  	  (P1)   Name of ASCII file containing the raw frames
!             ref_set  	  (P2)   Name of ASCII file containing the reference 
!                                frames
!
!.OUTPUT      cat_accept  (P3)   Catalog name containing accepted frames
!             cat_reject  (P4)   Catalog name containing rejected frames
!
!.RETURNS     The recipe returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status.
!
!.PURPOSE     The procedure reads the ASCII file with the names of the input
!   	      data frames and copies the files to the current working 
!             directory. The files are converted from FITS format to MIDAS
!             internal formats. Finally, the file type of each frame is
!             determined and the name is written to the appropriate keywords.
!
!.ENVIRON     UVES context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!		   1999/03/10:	Copy of FORS context (SW)
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.7 $
! $Log: uves_getframes.prg,v $
! Revision 1.7  2002/10/25 09:54:34  amodigli
! Added support of some extra frames
!
! Revision 1.5  2002/06/10 16:13:55  amodigli
! Added support for IS mode (bug fix)
!
! Revision 1.4  2002/06/03 16:15:36  amodigli
! Many changes to support extra modes
! either for UVES and FLAMES
!
! Revision 1.3  2001/12/21 13:34:53  amodigli
! To fix problem of dpr tech always ECHELLE
!
! Revision 1.2  2001/01/04 10:39:48  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

crossref input reference accept reject

define/param p1 ?  c    "Enter raw frame set file:"
define/param p2 ?  c    "Enter reference frame set file:"
define/param p3 ?  c    "Enter accepted frames catalog name:"
define/param p4 +  c    "Enter rejected frames catalog name:"

define/maxpar 4


! Procedure identification
define/local procID/c/1/80      "GetFrames"


! Local constant
define/local cat_tmp/c/1/10     "tmpfrm.cat"


! Local variables
define/local pos/i/1/1          -1
define/local i/i/1/1             0
define/local status/i/1/1        0
define/local catflag/i/1/1       0
define/local catal/i/1/1         0
define/local n_fmax/i/1/1        0
define/local n_frames/i/1/1      0
define/local n_accept/i/1/1      0
define/local n_reject/i/1/1      0
define/local frm_set/c/1/80      {p1}
define/local ref_set/c/1/80      {p2}
define/local cat_accept/c/1/80   {p3}
define/local cat_reject/c/1/80   {p4}
define/local cat_entry/c/1/80    " "           all
define/local obsmode/c/1/80      " "           all
define/local orgname/c/1/256     " "           all
define/local fname/c/1/80   " " all


! Read input set of frames 
mess/out 3 {procID} "Processing SetOfFrames {frm_set} ...!"
@p set_of_frames IN {frm_set} {cat_tmp} CONV >Null
if outputi(1) .lt. 0 then
  mess/out 1 {procID} "Cannot read SetOfFrames {frm_set}!"
  status = 1
  goto clean
else
  n_frames = outputi(1)
endif
!w/o "n_frames={n_frames} {outputi(1)}"

! Check if number of input frames exceed pipeline limit
if n_frames .gt. p_maxfrm(1) then
  set/format i1
  mess/out 1 {procID} -
           "Number of input frames exceeds pipeline limit ({p_maxfrm(1)})!"
  set/format
  status = 1
  goto clean
endif


! Extract instrument mode from the input frame header
if m$existd("#1_{cat_tmp}",H_DPRTECH) .eq. 1 then
  copy/dk #1_{cat_tmp} {H_DPRTECH} obsmode
  obsmode = m$upper(obsmode)
endif
!am
!w/o {obsmode}
!w/o "input obsmode: {obsmode}"
!begin FORCE
!wr/o "+++ FORCE obsmode to be ECHELLE !"
!am
!obsmode = "ECHELLE"

define/local tmp_obsmode/c/1/25 " " all
do i = 1 n_frames 1

  @p uves_ext_fname.prg #{i}_{cat_tmp}
  fname = q1
  !w/o "fname={fname}"
  !w/o {H_DPRTECH}
  !read/des #{i}_{cat_tmp} {H_DPRTECH}
  if m$existd("#{i}_{cat_tmp}",H_DPRTECH) .eq. 1 then
     !w/o "uno"
     copy/dk #{i}_{cat_tmp} {H_DPRTECH} tmp_obsmode
          if m$index(tmp_obsmode,"ECHELLE") .gt. 0 then
        !mess/out 3 {procID} "Obs mode {tmp_obsmode} is a supported one"
        !read/des #{i}_{cat_tmp} {H_DPRTECH}
     else if m$index(tmp_obsmode,"ECHELLE,ABSORPTION-CELL") .gt. 0 then
        !mess/out 3 {procID} "Obs mode {tmp_obsmode} is a supported one"
        !read/des #{i}_{cat_tmp} {H_DPRTECH}
     else if m$index(tmp_obsmode,"ECHELLE,ABSORPTION-CELL,SLIC") .gt. 0 then

     else if m$index(tmp_obsmode,"ECHELLE,SLIC") .gt. 0 then
        !mess/out 3 {procID} "Obs mode {tmp_obsmode} is a supported one"
        !read/des #{i}_{cat_tmp} {H_DPRTECH}        
     else if  m$index(tmp_obsmode,"IMAGE") .gt. 0 then
        !mess/out 3 {procID} "Obs mode {tmp_obsmode} is a supported one"
        !read/des #{i}_{cat_tmp} {H_DPRTECH}
     else if   m$index(tmp_obsmode,"MOS") .gt. 0 then
        !mess/out 3 {procID} "Obs mode {tmp_obsmode} is a supported one"
        !read/des #{i}_{cat_tmp} {H_DPRTECH}
     else
        mess/out 2 {procID} "Obs mode {tmp_obsmode} not supported force it to be echelle"
        write/desc #{i}_{cat_tmp} {h_dprtech}/c/1/25 ECHELLE
     endif     
  else if m$index(fname,".bdf") .gt. 0 then
     write/desc #{i}_{cat_tmp} {h_dprtech}/c/1/25 ECHELLE
     copy/dk #{i}_{cat_tmp} {H_DPRTECH} obsmode
  else
     mess/out 3 {procID} "#{i}_{cat_tmp}" is a table with empty DPR.TECH"
     write/desc #{i}_{cat_tmp} {h_dprtech}/c/1/25 {obsmode}
  endif
enddo
!end


write/key UVES_NCHIPS/i/1/1 0
if m$existd("#1_{cat_tmp}",H_NCHIPS) .eq. 1 then
  copy/dk #1_{cat_tmp} {H_NCHIPS} UVES_NCHIPS
else
  status = 1
  goto clean
endif


! Associate observing technique with internal observing mode names
!am
!w/o "obsmode={obsmode}"
     if obsmode .eq. "ECHELLE" then
   i = 1
else if obsmode .eq. "MOS" then
   i = 2
else if obsmode .eq. "ECHELLE,ABSORPTION-CELL" then
   i = 3
else if obsmode .eq. "IMAGE" then
   i = 4
else if obsmode .eq. "ECHELLE,SLIC#1" then
   i = 5      
else if obsmode .eq. "ECHELLE,SLIC#2" then
   i = 6     
else if obsmode .eq. "ECHELLE,SLIC#3" then
   i = 7   
else if obsmode .eq. "ECHELLE,ABSORPTION-CELL,SLIC#1" then
   i = 8
else if obsmode .eq. "ECHELLE,ABSORPTION-CELL,SLIC#2" then
   i = 9
else if obsmode .eq. "ECHELLE,ABSORPTION-CELL,SLIC#3" then
   i = 10
else if obsmode .eq. "ECHELLE,ABSORBTION-CELL,SLIC#1" then
   i = 11
else if obsmode .eq. "ECHELLE,ABSORBTION-CELL,SLIC#2" then
   i = 12
else if obsmode .eq. "ECHELLE,ABSORBTION-CELL,SLIC#3" then
   i = 13
else if obsmode .eq. "ECHELLE,ABSORBTION-CELL" then
   i = 14
else
   i = 15
endif


mode_loop:

!w/o "nmodes: {p_nmodes} i={i} {p_supported({i})}"
!am:
!w/o "i={i} obsmode={obsmode} p_supported={p_supported({i})}"

if obsmode .ne. p_supported({i}) then
  i = i + 1
  if i .le. p_nmodes then
    goto mode_loop
  else
    mess/out 1 {procID} "Unsupported observing mode encountered!"
    mess/out 1 {procID} "Check header keyword {H_DPRTECH} of input frames!"
    status = 1
    goto clean
  endif
endif

UVES_INSMODE = m$upper(p_modes({i}))

!am:
!w/o "INS.MODE={UVES_INSMODE} n_frames={n_frames}"
if n_frames .gt. 1 then
  do i = 2 n_frames 1
    if m$existd("#{i}_{cat_tmp}",H_DPRTECH) .eq. 1 then
      !w/o "DPR.TECH={#{i}_{cat_tmp},{H_DPRTECH}} obsmode={obsmode}"
      !w/o "#{i}_{cat_tmp}"
      !w/o "DPR.TECH={#{i}_{cat_tmp},{H_DPRTECH}}"
      if "{#{i}_{cat_tmp},{H_DPRTECH}}" .eq. "IMAGE" then
         if "{#{i}_{cat_tmp},{H_DPRTYPE}}" .eq. "BIAS" then
            !do nothing the BIASES miss the TECH but are Ok
         endif
      else if "{#{i}_{cat_tmp},{H_DPRTECH}}" .ne. obsmode then
        UVES_INSMODE = "UNDEFINED"
        mess/out 1 {procID} -
                 "Ambiguous observing mode encountered in input frame set!"
        mess/out 1 {procID} "Check header keyword {H_DPRTECH} of input frames!"
        status = 1
        goto clean
      endif
    else if "{#{i}_{cat_tmp},{H_DPRTECH}}" .ne. obsmode then
      mess/out 1 {procID} "Header keyword {h_dprtech} not present in header!"
      status = 1
      goto clean
    endif
  enddo
endif


! Initialize quality control status of raw frames if not already set
!01-08-2002: AM. not necessary anymore: DFS1008
!do i = 1 n_frames 1
!  if m$existd("#{i}_{cat_tmp}",H_PROQCSTS) .eq. 0 then
!    write/descr #{i}_{cat_tmp} {H_PROQCSTS}/c/1/10 "{d_uchk}"
!  endif
!enddo


! Get readout port setup
config/ports #1_{cat_tmp}
if {q1} .ne. 0 then
  status = 1
  goto clean
endif


! Get filter setup
config/filter #1_{cat_tmp}


! Read reference set of frames if present
!w/o "n_frames={n_frames} {outputi(1)}"
if p2 .ne. "+" then
  mess/out 3 {procID} "Processing SetOfFrames {ref_set} ...!"
  @p set_of_frames IN {ref_set} {cat_tmp},app CONV >Null
  if outputi(1) .lt. 0 then
    mess/out 1 {procID} "Cannot read SetOfFrames {ref_set}!"
    status = 1
    goto clean
  else
    n_frames = n_frames + outputi(1)
  endif

!am
!w/o "n_frames={n_frames} {outputi(1)}"


! Check if number of reference frames exceed pipeline limit
  if outputi(1) .gt. p_maxfrm(2) then
    set/format i1
    mess/out 1 {procID} -
        "Number of reference frames exceeds pipeline limit ({p_maxfrm(2)})!"
    set/format
    status = 1
    goto clean
  endif
endif


! Merge input and reference catalogs into one. Frame identification is 
! done on the fly.
pos = m$indexb(cat_accept,".cat") - 1
if pos .lt. 0 then
  cat_accept = cat_accept // ".cat"
endif
create/icat {cat_accept} NULL {UVES_IDSTRNG} >Null

if cat_reject .ne. "+" then
  pos = m$indexb(cat_reject,".cat") - 1
  if pos .lt. 0 then
    cat_reject = cat_reject // ".cat"
  endif
  create/icat {cat_reject} NULL {UVES_IDSTRNG} >Null
  catflag = 1
endif

set/format i1
!w/o "n_frames={n_frames} {outputi(1)}"
n_fmax = n_frames - 1

!am
!read/icat {cat_tmp}
!w/o "n_fmax={n_fmax}"

do i = 0 n_fmax 1
  store/frame cat_entry {cat_tmp} 1
  if "{{cat_entry},{UVES_IDSTRNG}}" .eq. UVES_NOCLASS then
    if catflag .eq. 1 then
      add/icat {cat_reject} {cat_entry} >Null
      n_reject = n_reject + 1
    endif
    mess/out 1 {procID} "Frame type of {cat_entry} is unknown."
  else
    add/icat {cat_accept} {cat_entry} >Null
    n_accept = n_accept + 1
    if m$existd(cat_entry,H_ORIGNAME) .eq. 1 then
      orgname = "{{cat_entry},{H_ORIGNAME}}"
    else if m$existd(cat_entry,H_FILENAME) .eq. 1 then
      orgname = "{{cat_entry},{H_FILENAME}}"
    endif

    if m$strlen(orgname) .gt. 0 then
      pos = m$indexb(orgname,"/") + 1
      if pos .le. 0 then
        pos = 1
      endif
      mess/out 3 {procID} "Frame {orgname({pos}:)}"
    else
      mess/out 3 {procID} "Frame #{i}"
    endif
    mess/out 3 {procID} -
             "  converted to {cat_entry} ({{cat_entry},{UVES_IDSTRNG}})"
  endif
enddo

set/format

set/format j4 f7.2
mess/out 3 {procID} " "
mess/out 3 {procID} "*** Processing Log ***"
mess/out 3 {procID} " "
mess/out 3 {procID} "Name of input data set:     {frm_set}"
mess/out 3 {procID} "Name of reference data set: {ref_set}"
mess/out 3 {procID} "No. frames processed:       {n_frames}"
mess/out 3 {procID} "No. accepted frames:        {n_accept}"
mess/out 3 {procID} "No. rejected frames:        {n_reject}"
mess/out 3 {procID} " "
set/format

if n_accept .eq. 0 then
  mess/out 1 {procID} "No frames have been accepted!"
  status = 1
elseif n_accept .ne. n_frames then
  mess/out 2 {procID} "Some frames have been rejected!"
endif


! Cleanup temporary files
clean:
-delete {cat_tmp}
$echo {status}
$echo {q1}
return {status}
