! $Id: uves_getports.prg,v 1.4 2002/10/25 09:55:14 amodigli Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       uves_getports.prg
!
!.KEYWORDS    Determine readout port configuration
!
!.USAGE       config/ports  frame
!
!.INPUT       frame       (P1)   Input frame from which the detector port
!                                setup is extracted.
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is 
!             updated with the exit status.
!
!.PURPOSE     Get the number and the indices of all active detector ports.
!
!.ENVIRON     UVES context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/07/03:  Creation
!		   1999/03/10:	(SW) Copy from UVES
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.4 $
! $Log: uves_getports.prg,v $
! Revision 1.4  2002/10/25 09:55:14  amodigli
! Eliminated some commented lines
!
! Revision 1.2  2001/01/04 10:39:47  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
crossref  frame


define/param p1 ? c "Enter name of input frame:"
define/maxpar 1


! Procedure identification
define/local procID/c/1/80      "UvesGetPorts"


! Local variables
define/local i/i/1/1         0
define/local np/i/1/1        1
!AM define/local lockno/i/1/2    0     all
define/local frame/c/1/100   {p1}
define/local dscname/c/1/80  " "  all



! Unlock keywords if necessary
info/keyw uves_nout
!AMo lock key not necessary anymore
!AMo lockno(1) = mid$info(4)
!AMo if lockno(1) .ne. 0 then
!AMo   unlock/key uves_nout {lockno(1)}
!AMo endif

info/keyw uves_outi
!AMo lock key not necessary anymore
!AMo lockno(2) = mid$info(4)
!AMo if lockno(2) .ne. 0 then
!AMo  unlock/key uves_outi {lockno(2)}
!AMo endif


! Reset port status
do i = 1 UVES_MAXOUT 1
  uves_outi({i}) = 0
enddo


! Get number of outputs
if m$existd(frame,h_nout) .eq. 1 then
  uves_nout = {{frame},{h_nout}}
else
  mess/out 1 {procID} "Header keyword {h_nout} not present!"
  return 1
endif

set/format i1
do i = 1 UVES_MAXOUT 1
  dscname = kbase_outi // "{i}" // ksuff_outi
  if m$existd(frame,dscname) .eq. 1 then
    uves_outi({np}) = i
    np = np + 1
  endif
enddo
set/format

np = np - 1

if np .eq. 0 then
  mess/out 1 {procID} "No readout port information found in frame header!"
  return 1
else if uves_nout .ne. np then
  mess/out 2 {procID} -
     "Number of active ports not equal to header keyword {h_nout}"
endif


! Protect keywords
!AMo lock key not necessary anymore
!AMo if lockno(1) .ne. 0 then
!AMo  lock/key uves_nout {lockno(1)}
!AMo endif

!AMo if lockno(2) .ne. 0 then
!AMo  lock/key uves_outi {lockno(2)}
!AMo endif

return 0
