! $Id: uves_hcopy.prg,v 1.3 2002/02/18 08:33:57 amodigli Exp $
!+++ UVES_HCOPY +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_hcopy.prg
!
!.KEYWORDS      keywords
!
!.USAGE         @p (IDENT) print psname bdfname
!
!.INPUT         print	(P1): print option:
!				ON/OFF: enables/disables hardcopies.
!				LASER: print directly on default printer
!				NOPRINT: creates only the PS-file!
!				POSTSCRIPT: create a postscript file from the
!					input file (P3). The input file (P3)
!					has not to be displayed via LOAD/IMA!
!				Append a ,G to indicate a hardcopy of the
!				currently active graphics window (e.g: N,G).
!		psname  (P2): name of POSTSCRIPT file
!		bdfname	(P3): name of BDF input file, only used in connection
!				with P1=POSTSCRIPT
!		
!.OUTPUT        {P2}
!
!.RETURN        Q1:  0:  successful return
!                    <0: a successful return failed
!
!.DEFAULT	P1	N
!		P2	+	(take default: screenXX.ps)
!		P3	+
!		
!.PURPOSE       Create a hardcopy of the image display OR the graphics window
!
!.ALGORITHM     At first check the parameter P1 to decide wether we have to
!		handle graphics windows or image displays. Makes use of the
!		corresponding MIDAS commands (COPY/GRAP, COPY/DISPLAY or
!		COPY/PSCR. To improve the documentation part of the COPY/PSCR
!		the UVES context command ANNOT/PLOT is used to put relevant
!		UVES information on the plot.
!
!.ENVIRON       UVES context.
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1999/06/21  Created.
!		    1999/07/22	POSTSCRIPT option added.
!		    1999/07/23	Check the NICE_HCOPY keyword at first!
!		    1999/09/22	Enable/Disable hardcopies (ON/OFF) implemented.
!		    1999/09/23	Check of NICE_HCOPY shifted after ON/OFF check.
!		    1999/10/05	Put some information on terminal (MESS/OUT).
!
!.COMMENT	(1) The use of the postscript option is recommended as the
!		usual hardcopy command (COPY/DISP) is not secure -- there you
!		have to ensure that the display window is not covered!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.3 $
! $Log: uves_hcopy.prg,v $
! Revision 1.3  2002/02/18 08:33:57  amodigli
! Fixed small bugs indicated from Klaus as
! were used some MIDAS reserved keywords
!
! Revision 1.2  2001/01/04 10:39:46  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
crossref print psname bdfname

define/param p1 N c "Enter ON, OFF, POSTSCRIPT, LASER or NOPRINT: "
define/param p2 + c "Enter PS-name: "
define/param p3 + c "Enter BDF-name: "

define/local spathid/i/1/1 0
define/local fid/i/1/2 0 all
define/local clen/c/1/10 0
define/local swid/c/1/10 0
define/local itt/c/1/20 " " all
define/local juser/c/1/60 " " all


p1 = m$upper(p1)

if P1(1:2) .eq. "ON" then				!Enable hardcopy
  NICE_HCOPY = "YES"
  mess/out 3 hcopy "Hardcopy facility enabled."
  return 0
else if P1(1:3) .eq. "OFF" then				!Disable hardcopy
  NICE_HCOPY = "NO"
  mess/out 3 hcopy "Hardcopy facility disabled."
  return 0
endif


if NICE_HCOPY(1:1) .ne. "Y" goto WRN1

if m$index(p1,",") .gt. 0 then				!Hardcopy of graphic
  copy/graph postscript					!window
  if p1(1:1) .eq. "L" then				!Send directly to the
    -print postscript.ps				!default printer
    outputc = m$symbol("PRINTER")
    mess/out 3 hcopy "Hardcopy sent to printer: {outputc}."
  else if P2(1:1) .ne. "+" then				!Save as specified
    -rename postscript.ps {p2}
    mess/out 3 hcopy "POSTSCRIPT file `{p2}` created."
  else
    mess/out 3 hcopy "POSTSCRIPT file `postscript.ps` created."
  endif

  return 0
endif
							!Hardcopy of an image
							!display
if P1(1:1) .eq. "N" then				!Create the hardcopy
  copy/display LASER NOSTOP N ? NOPRINT
  if P2(1:1) .ne. "+" then
    -rename screen{MID$SESS(11:12)}.ps {p2}		!Save as specified
    mess/out 3 hcopy "POSTSCRIPT file `{p2}` created."
  endif
else if P1(1:1) .eq. "L" then				!Send directly to the
  copy/display LASER NOSTOP N ? PRINT			!default printer
  outputc = m$symbol("PRINTER")
  mess/out 3 hcopy "Hardcopy sent to printer: {outputc}."
else if P1(1:1) .eq. "P" then				!Create POSTSCRIPT copy
  if P2(1:1) .eq. "+" goto ERR1				!Error handling
  if P3(1:1) .eq. "+" goto ERR2				!Error handling
  copy/dk {p3} IDENT IDENT				!Clear IDENT descriptor
  write/desc {p3} IDENT " " ALL				!in order to disable
							!output by copy/pscr
  copy/pscr {p3} {p2} nf				!Copy bdf to ps

  copy/kd IDENT {p3} IDENT				!reset IDENT

  spathid = PATHID					!Save global key PATHID
  get/pathid {p3}					!Get the PATHID
  juser = m$symbol("USER")				!Get USER name
  itt = hcittlut(1:20)					!Get ITT used
  open/file annot write fid				!Write comments into
							!annotation file
							!Do not change spacing:
  write/file {fid(1)} "INSTRUMENT:  {{p3},{h_instrume}}"
  write/file {fid(1)} "CCD:                  {h_nameccd({PATHID})}"
  set/format F5.2
  if m$existd(p3,h_cwlen({PATHID})) .ne. 0 then
    clen = "{{p3},{h_cwlen({PATHID})}}"			!central wavelength
    write/file {fid(1)} "CENTER:           {clen} nm"
  endif
  if m$existd(p3,h_slwidth({PATHID})) .ne. 0 then
    swid = "{{p3},{h_slwidth({PATHID})}}"		!slit width
    write/file {fid(1)} "SLIT:                  {swid} arcsec"
  endif
  write/file {fid(1)} "FRAME:             {p3}"
  write/file {fid(1)} "ITT:                    {itt}"
  write/file {fid(1)} "CUTS:               {{p3},lhcuts(1)}, {{p3},lhcuts(2)}"
  write/file {fid(1)} "USER:                {juser}"
  set/format
  close/file {fid(1)}					!Close annotation file
  annot/plot {p2} annot h 0,3,0,9,1			!Write the annotations
  -delete annot						!into PS-file

  annot/plot {p2} "{IDENT}" h 0,24,0,20,1		!Title

  PATHID = spathid					!reset PATHID
  mess/out 3 hcopy "POSTSCRIPT file `{p2}` created."
endif

return 0

ERR1:
  mess/out 1 hcopy "Name of the PS file is missing!"
  return -1
ERR2:
  mess/out 1 hcopy "Name of the BDF file is missing!"
  return -2
WRN1:
  mess/out 2 hcopy "Hardcopies currently disabled! (use HARDC/PLOT ON)"
  return -3
