! $Id: uves_mplot_chun.prg,v 1.10 2002/03/14 09:35:36 amodigli Exp $
!+++ UVES_MPLOT_CHUN +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 2000  European Southern Observatory
!
!.IDENT         uves_mplot_chun.prg
!
!.KEYWORDS      uves optimal extraction QC check
!
!.USAGE         @p (IDENT) OrdTab yp_size switch
!
!.INPUT 
!               OrdTab        (p1): Order table name (order_trace.tbl)
!               yp_size       (p2): Half Size of plot
!               switch        (p3): [object]/fwhm switch between obj and fwhm plots
!          
!.OUTPUT        None
!
!.RETURN        1 on error, 0 else
!
!.DEFAULT	   
!               p1  order_trace.tbl
!               p2  3 (pix)
!               p3  obj
!
!.PURPOSE: 	    Optimal extraction QC check. 
!		        Plots chunk positions/fwhm before fit, after first and final fit to verify
!               optimal extraction accuracy. Creates a matrix of plots.
!
!.ALGORITHM     Does several multiple selection and correspondent plot/overplot
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 99NOV
!
!.AUTHOR        A.Modigliani, ESO/DMD
!
!.VERSION       2000/12/13  Created (AM).
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.10 $
! $Log: uves_mplot_chun.prg,v $
! Revision 1.10  2002/03/14 09:35:36  amodigli
! using ssize=3 instead of 1 (not supported by 02FEB)
!
! Revision 1.9  2001/08/08 13:14:46  amodigli
! Added again prefix ESO to QC keywords
!
! Revision 1.8  2001/06/28 12:50:42  amodigli
! Eliminated prefix ESO from QC.OBJ...keys
!
! Revision 1.7  2001/06/16 07:02:33  amodigli
! Adapted to new QC-LOG naming conventions
!
! Revision 1.6  2001/05/30 16:31:14  amodigli
! clean/doc
!
! Revision 1.5  2001/05/18 08:32:10  amodigli
! -Added automatic x-y rescaling
! -substituted call to entry DISPL_TIT with
!  call to uves_mplt_title
! -typo
! -small but in calling mess/out
!
! Revision 1.4  2001/03/23 12:52:13  amodigli
! renamed
! uves_plot_multi_debug -> uves_mplot_chun
!
! Revision 1.3  2001/03/12 06:45:09  amodigli
! Added check on P3 (switch parameter)
!
! Revision 1.2  2001/02/21 09:54:04  amodigli
! Added crossref
!
! Revision 1.1  2001/02/05 16:38:11  amodigli
! Redefined
! uves_plot_multi_debug.prg uves_mplot_chun.prg
! uves_plot_obj_pos_debug.prg uves_plot_chun.prg
!
! Revision 1.17  2001/02/05 16:28:37  amodigli
! Defined entry REPEAT_PLOT to simplify procedure
!
! Revision 1.16  2001/02/05 08:42:27  amodigli
! x_x_base must be real to have proper rescaling!
! check existence of "ESO.QC.OPS.VAL{ord}" and
! "ESO.QC.OPS.VAL{ord}" as opt extr
! sometimes skip an order.
! Improoved header info
!
! Revision 1.15  2001/01/25 07:42:30  amodigli
! Supported now automatic resizing of plots
!
! Revision 1.14  2001/01/22 15:51:30  amodigli
! Grouped displayed info.
! Using mess/out in place of w/o
!
! Revision 1.13  2001/01/18 13:59:13  amodigli
! Version improoved
!
! Revision 1.12  2001/01/11 14:38:06  amodigli
! Added parameter to switch between
! pos and fwhm plots
!
! Revision 1.11  2001/01/08 12:57:23  amodigli
! Renamed debug.tbl order_trace.tbl
! debug2.tbl xorder_trace.tbl
!
! Revision 1.10  2001/01/04 10:39:53  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
! Who    When        Why               Where
! AM     30/05/01    clean/doc         all routine
!------------------------------------------------------------------------------
! Short description:
!  do minimal check on input par
!  get graph size
!  determines how many plot to do
!  determines how many plot to do on X and Y directions
!  set rescaling factors according to graph size got and number of X,Y plots
!  plot the title
!  do the plots
!------------------------------------------------------------------------------
crossref intab yp_size switch
!
define/param p1 "order_trace.tbl" c "Enter table name [order_trace.tbl]"
define/param p2 3           n "Enter half y size of plot [3]"
define/param p3 "pos"       c "Enter info to display [pos]/fwhm"

define/local pid/c/1/10 "mplot_chun" ? +lower_levels

define/local XLL/r/1/1 30.0 ? +lower_levels
define/local YLL/r/1/1 15.0 ? +lower_levels
define/local XSZ/r/1/1 0.0 ? +lower_levels
define/local YSZ/r/1/1 0.0 ? +lower_levels

define/local intab/c/1/80 {p1} ? +lower_levels
define/local yp_size/r/1/1 {p2} ? +lower_levels
define/local info/c/1/80  {p3}  ? +lower_levels

info = m$upper(info)
     if "{info(1:3)}" .eq. "POS" then
else if "{info(1:3)}" .eq. "FWH" then
else
   mess/out 1 {pid} "P3={info} not allowed! Allowed only P3=pos or P3=fwhm"
   return -1
endif 


define/local ord/i/1/1  0 ? +lower_levels

set/format I1
show/disp >Null                       !check if graph.window already exists
if outputi(12) .lt. 0 then
    CONFIG/DISP P3=0.6
endif
ASSIGN/GRAP G,3
modify/grap W
clear/grap

!get grap window size
@p uves_plt_set_size

!Now appropriately defines scaling factors
define/local x_base/i/1/1 8   !Arbitrary value: 8 plots per row
define/local y_base/i/1/1 8   !Arbitrary value: 8 plots per col

!Determines how mani plots to do
define/local no/i/1/1 0
set/format i1
no = {{intab},ESO.QC.OPTEX.NORD}
set/format

!determines how many plot to do on X and on Y
x_base = m$nint(m$sqrt(no)-1)  !to get the "proper" No. of X plots
y_base = m$nint(no/(x_base-1)) !to get the "proper" No. of Y plots



!Based on the 7 plots per raw value and a default X-Y size we evaluate the
!Following coefficients 
define/local x_sc/r/1/1 -32 
define/local y_sc/r/1/1 -20
define/local x_res/r/1/1 40.
define/local y_res/r/1/1 24.

x_sc = x_sc*(XSZ/242)*(7/x_base)   !The ref value 242 is OK for a default X size
                                   !x_base, number of plots per raw can now be arbitrary
y_sc = y_sc*(YSZ/147)*(7/y_base)   !The ref value 147 is OK for a default Y size 
                                   !to fit 5 rows along Y 
x_res = x_res*(XSZ/242)*(7/x_base)
y_res = y_res*(YSZ/147)*(7/y_base)




!defines some useful variables
define/local fct/r/1/1 1
define/local nloop0/i/1/1 0
define/local nloop1/i/1/1 0
define/local jloop/i/1/1 0
define/local i/i/1/1 0
define/local j/i/1/1 0

set/format i1
   nloop0 = no/x_base
   nloop1 = no - nloop0*x_base
set/format 

clear/grap
set/grap tsize=0.5
set/grap colour=1
set/grap pmode=-1
plot/axes
set/grap pmode=2

!display minimal info
mess/out 3 {pid} "plot all points as black dots"
mess/out 3 {pid} "overplot as red points the ones used in the first fit"
mess/out 3 {pid} "overplot as green points the results of the first fit"
mess/out 3 {pid} "overplot as blue points the results of the last fit"
mess/out 3 {pid} "overplot as magenta points the points used in the last fit"

!now put a title in standard format
@p uves_mplot_title "{intab}" "QC on opt. extr. OBJ {info} vs. X for all orders"
!if no .le. 20 fct = 1.5

!Do the actual loop
jloop = x_base - 1
nloop0 = nloop0 - 1
do i = 0 nloop0 1
do j = 0 jloop 1
    @p uves_mplot_chun,REPEAT_PLOT {i} {j} {x_sc} {y_sc} {x_res} {y_res}
enddo
enddo

i = i+1
nloop1 = nloop1 - 1
do j = 0 nloop1 1
    @p uves_mplot_chun,REPEAT_PLOT {i} {j} {x_sc} {y_sc} {x_res} {y_res}
enddo

return 0
!========================================================================================================
!========================================================================================================
ENTRY REPEAT_PLOT
define/param p1 ? N "Enter X-plot index sequence coordinate:"
define/param p2 ? N "Enter Y-plot index sequence coordinate:"
define/param p3 ? N "Enter X scale :"
define/param p4 ? N "Enter Y scale :"
define/param p5 ? N "Enter X resolution :"
define/param p6 ? N "Enter Y resolution :"


define/local des/c/1/80 " "
define/local y_cen/r/1/1 0.0
define/local i/i/1/1 {p1}
define/local j/i/1/1 {p2}
define/local x_sc/r/1/1 {p3}
define/local y_sc/r/1/1 {p4}
define/local x_res/r/1/1 {p5}
define/local y_res/r/1/1 {p6}
define/local x_off/r/1/1 0
define/local y_off/r/1/1 0

define/local xLabOrd/r/1/1  0.0
define/local yLabOrd/r/1/1  0.0
define/local x_thick_big/r/1/1 1000
define/local y_thick_big/r/1/1 5
define/local x_thick_small/r/1/1 100
define/local y_thick_small/r/1/1 1
y_thick_big = yp_size/3.0
y_thick_small = yp_size/12.0

define/local x_lab0/c/1/20 "X"
define/local y_lab0/c/1/20 "Y"
define/local lab/i/1/1 0

define/local x_min/r/1/1  0
define/local x_max/r/1/1  4096
x_max = {{intab},ESO.QC.OPTEX.XSIZE}

define/local y_min/r/1/1  0
define/local y_max/r/1/1  4096
y_max = {{intab},ESO.QC.OPTEX.YSIZE}

    set/format i1
    ord = ord + 1

    if "{info(1:3)}" .eq. "POS" then
       if m$existd(intab,"ESO.QC.OBJ{ord}.POS") .eq. 1 then
          des = "ESO.QC.OBJ{ord}.POS"
       else  !sometime an order may be skipped from optimal extraction
          mess/out 2 {pid} "Value of descriptor ESO.QC.OBJ{ord}.POS not present, we skip!"
          return 0
       endif
    else
       if m$existd(intab,"ESO.QC.OBJ{ord}.FWHM") .eq. 1 then
          des = "ESO.QC.OBJ{ord}.FWHM"
       else  !sometime an order may be skipped from optimal extraction
          mess/out 2 {pid} "Value of descriptor ESO.QC.OBJ{ord}.FWHM not present, we skip!"
          return 0
       endif
    endif
    
    !get Y center
    set/format
    y_cen = {{intab},{des}}
    y_max = y_cen+yp_size
    y_min = y_cen-yp_size

    x_off = j*x_res+10 
    y_off = i*y_res+10
    xLabOrd = x_min+(x_max-x_min)*0.1
    yLabOrd = y_min+(y_max-y_min)*0.9

    !plot a dark frame in the proper position
    set/grap colour=1
    overplot/axes {x_min},{x_max},{x_thick_big},{x_thick_small} -
                  {y_min},{y_max},{y_thick_big},{y_thick_small} -
                  {x_sc},{y_sc},{x_off},{y_off} {x_lab0} {y_lab0} 
    
    !put a lable to indicate which order corresponds to
    set/format i1
    lab = ord
    label/graph {lab} {xLabOrd},{yLabOrd},wo 0 2
    set/format 

    !Overplot the actual data
    @p uves_mplot_chun,PLOT_DATA {intab} {ord} {info}
return 0
!========================================================================================================
!========================================================================================================

ENTRY PLOT_DATA
define/param p1 ? c "Enter table name:"
define/param p2 ? N "Enter order number:"
define/param p3 ? c "Enter info to display:"



set/grap stype=1
set/grap ssize=3
set/grap colour=1



select/table {p1} :ORDER.EQ.{p2} >Null
if "{p3(1:3)}" .eq. "POS" then
   overpl/tab {p1} :X_CH_0 :P_CH
else
   overpl/tab {p1} :X_CH_0 :P_FWHM
endif 

select/table {p1} :ORDER.EQ.{p2}.AND.:M0.EQ.1 >Null
!set/grap stype=2
set/grap colour=2


if "{p3(1:3)}" .eq. "POS" then
   overpl/tab {p1} :X_CH_0 :P_CH
else
   overpl/tab {p1} :X_CH_0 :P_FWHM
endif

!set/grap stype=3
set/grap colour=3

if "{p3(1:3)}" .eq. "POS" then
   overpl/tab {p1} :X_CH_0 :P_F0
else
   overpl/tab {p1} :X_CH_0 :S_CH_0
endif

!set/grap stype=4
set/grap colour=4

if "{p3(1:3)}" .eq. "POS" then
   overpl/tab {p1} :X_CH_0 :P_F1
else
   overpl/tab {p1} :X_CH_0 :S_CH_1
endif

select/table {p1} :ORDER.EQ.{p2}.AND.:M1.EQ.1 >Null
!set/grap stype=6
set/grap colour=6

if "{p3(1:3)}" .eq. "POS" then
   overpl/tab {p1} :X_CH_0 :P_CH
else
   overpl/tab {p1} :X_CH_0 :P_FWHM
endif
   
return 0
!========================================================================================================
