! $Id: uves_mplot_xchun.prg,v 1.1 2001/05/18 08:36:14 amodigli Exp $
!+++ UVES_MPLOT_XCHUN ++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 2000  European Southern Observatory
!
!.IDENT         uves_mplot_xchun.prg
!
!.KEYWORDS      uves optimal extraction QC check
!
!.USAGE         @p (IDENT) xOrdTab order 
!
!.INPUT 
!               xOrdTab       (p1): Order table name (xorder_trace.tbl)
!               order         (p2): order number [10]
!          
!.OUTPUT        None
!
!.RETURN        1 on error, 0 else
!
!.DEFAULT	   
!               p1  xorder_trace.tbl
!               p2  10 
!
!.PURPOSE: 	    Optimal extraction QC check. 
!		        Plots chunk x-order profile
!
!.ALGORITHM     Does several multiple selection and correspondent plot
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 01FEB
!
!.AUTHOR        A.Modigliani, ESO/DMD
!
!.VERSION       2001/05/16  Created (AM).
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.1 $
! $Log: uves_mplot_xchun.prg,v $
! Revision 1.1  2001/05/18 08:36:14  amodigli
! Created
!
!------------------------------------------------------------------------------
!
crossref intab order
!
define/param p1 "order_trace.tbl" c "Enter table name [xorder_trace.tbl]"
define/param p2 10           n "Enter order number [10]"

define/local pid/c/1/15 "mplot_xchun" ? +lower_levels

define/local XLL/r/1/1 30.0 ? +lower_levels
define/local YLL/r/1/1 15.0 ? +lower_levels
define/local XSZ/r/1/1 0.0 ? +lower_levels
define/local YSZ/r/1/1 0.0 ? +lower_levels

define/local intab/c/1/80 {p1} ? +lower_levels
define/local order/i/1/1  {p2} ? +lower_levels

define/local ord/i/1/1  0 ? +lower_levels
define/local chun/i/1/1  0 ? +lower_levels


set/format I1
show/disp >Null                       !check if graph.window already exists
if outputi(12) .lt. 0 then
    CONFIG/DISP P3=0.6
endif
ASSIGN/GRAP G,3
modify/grap W
clear/grap

@p uves_plt_set_size
define/local x_base/i/1/1 8   !Arbitrary value: 8 plots per row
define/local y_base/i/1/1 8   !Arbitrary value: 8 plots per col
define/local no/i/1/1 0

sel/tab {intab} :ORDER.eq.{order} >Null
no = outputi(1)

! at least 10 chunks should be present
sel/tab {intab} :ORDER.EQ.{order}.AND.:I_CH.EQ.10 >Null 
if outputi(1) .ne. 0 no = no / outputi(1)
x_base = m$nint(m$sqrt(no)-1)  !to get the "proper" No. of X plots
y_base = m$nint(no/(x_base-1)) !to get the "proper" No. of Y plots


!Based on the 7 plots per raw value and a default X-Y size we evaluate the
!Following coefficients 
define/local x_sc/r/1/1 -32 
define/local y_sc/r/1/1 -20
define/local x_res/r/1/1 40.
define/local y_res/r/1/1 24.

x_sc = x_sc*(XSZ/242)*(7/x_base)   !The ref value 242 is OK for a default X size
                                   !x_base, number of plots per raw can now be arbitrary
y_sc = y_sc*(YSZ/147)*(7/y_base)   !The ref value 147 is OK for a default Y size 
                                   !to fit 5 rows along Y 
x_res = x_res*(XSZ/242)*(7/x_base)
y_res = y_res*(YSZ/147)*(7/y_base)

define/local fct/r/1/1 1
define/local nloop0/i/1/1 0
define/local nloop1/i/1/1 0
define/local jloop/i/1/1 0
define/local i/i/1/1 0
define/local j/i/1/1 0


set/format i1
   nloop0 = no/x_base
   nloop1 = no - nloop0*x_base
set/format 

clear/grap
set/grap tsize=0.5
set/grap colour=1
set/grap pmode=-1
plot/axes
set/grap pmode=1

mess/out 3 {pid} "plot all points as black dots"
mess/out 3 {pid} "overplot as red points the ones used in the first fit"

set/format i1
@p uves_mplot_title "{intab}" "QC on opt. extr. x-order profiles order {order}"
set/format

jloop = x_base - 1
nloop0 = nloop0 - 1
do i = 0 nloop0 1
do j = 0 jloop 1
    @p uves_mplot_xchun,REPEAT_PLOT {i} {j} {x_sc} {y_sc} {x_res} {y_res}
enddo
enddo

if {q1} .eq. -1 return 0
i = i+1
nloop1 = nloop1 - 1
do j = 0 nloop1 1
    @p uves_mplot_xchun,REPEAT_PLOT {i} {j} {x_sc} {y_sc} {x_res} {y_res}
enddo
if {q1} .eq. -1 return 0

return 0
!========================================================================================================
!========================================================================================================
ENTRY REPEAT_PLOT
define/param p1 ? N "Enter X-plot index sequence coordinate:"
define/param p2 ? N "Enter Y-plot index sequence coordinate:"
define/param p3 ? N "Enter X scale :"
define/param p4 ? N "Enter Y scale :"
define/param p5 ? N "Enter X resolution :"
define/param p6 ? N "Enter Y resolution :"




define/local des/c/1/80 " "
define/local y_cen/r/1/1 0.0
define/local i/i/1/1 {p1}
define/local j/i/1/1 {p2}
define/local x_sc/r/1/1 {p3}
define/local y_sc/r/1/1 {p4}
define/local x_res/r/1/1 {p5}
define/local y_res/r/1/1 {p6}
define/local x_off/r/1/1 0
define/local y_off/r/1/1 0

define/local xLabOrd/r/1/1  0.0
define/local yLabOrd/r/1/1  0.0
define/local x_thick_big/i/1/1 10
define/local y_thick_big/i/1/1 100
define/local x_thick_small/i/1/1 5
define/local y_thick_small/i/1/1 50
y_thick_big = 100
y_thick_small = 10

define/local x_lab0/c/1/20 "X"
define/local y_lab0/c/1/20 "Y"
define/local lab/i/1/1 0

define/local x_min/r/1/1  0
define/local x_max/r/1/1  30

define/local y_min/r/1/1  0
define/local y_max/r/1/1  200

    set/format i1
    chun = chun + 1
    sel/tab {intab} :ORDER.EQ.{order}.AND.:I_CH.EQ.{chun} >Null
    if outputi(1) .eq. 0 return -1

    stat/tab {intab} :Y_CH  >Null
    x_min = 0
    x_max = outputr(2)

    stat/tab {intab} :FLUX >Null
    y_min = outputr(1)
    y_max = outputr(2)

    set/format

    x_off = j*x_res+10 
    y_off = i*y_res+10
    xLabOrd = x_min+(x_max-x_min)*0.1
    yLabOrd = y_min+(y_max-y_min)*0.9

    set/grap colour=1
   
    overplot/axes {x_min},{x_max},{x_thick_big},{x_thick_small} -
                  {y_min},{y_max},{y_thick_big},{y_thick_small} -
                  {x_sc},{y_sc},{x_off},{y_off} {x_lab0} {y_lab0} 
    !mess/out 3 {pid} "Plot order No. {ord}"
    set/format i1
    lab = chun
    label/graph {lab} {xLabOrd},{yLabOrd},wo 0 2
    set/format 

    set/grap colour=3
    @p uves_mplot_xchun,PLOT_DATA {intab} {order} {chun}
    set/grap colour=1
return 0
!========================================================================================================
!========================================================================================================

ENTRY PLOT_DATA
define/param p1 ? c "Enter table name:"
define/param p2 ? N "Enter order number:"
define/param p3 ? c "Enter info to display:"


set/grap ltype=1
set/grap stype=0
set/grap ssize=1
set/grap colour=1


overpl/table {p1} :Y_CH :FLUX
   
sel/tab {p1} all >Null

return 0
!========================================================================================================
