! $Id: uves_msrawxy.prg,v 1.12 2001/01/04 10:39:42 amodigli Exp $
!+++ UVES_MSRAWXY +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_msrawxy.prg
!
!.KEYWORDS      uves physical model, spectroscopy, echelle,
!
!.USAGE         @p (IDENT) wlc pline.tbl
!
!.INPUT         wlc     	(P1): wavelength calibration frame
!               pline.tbl   (P2): Projection table
!				            (P3): Measurement method (GAUSS/MOMENT)
!				            (P4): Visualisation option
!				            (P5): Measurement position offsets (X,Y) 
!					              from the predicted position.
!				            (P6): Scale of (X,Y) axis
!				            (P7): Dimension of measurement box (X,Y)
!		        fitreg 		(p8): Fits regression:
!				                  YES: Fits the residual differences.
!				                  NO:  No fitting.
!
!.OUTPUT        rline.tbl : predicted position (Y,Y)
!		        npline.tbl : projection map with corrected predicted position.
!		
!
!.RETURN        
!
!.DEFAULT	P1	{wlc}
!		P2	pline
!		P3	GAUSS
!		P4	YES
!		P5	0,0
!		P6	0,0
!		P7	40,40
!		p8	Y
!
!.Purpose: 	This procedure measures the raw XY positions and widths
!		of Th-Ar lines on an ThAr arc echelle exposure. 
!		It computes the differences with the predicted positions. 
!
!.ALGORITHM     Creates tables rline.tbl containing predicted positions
!		(XMOD,YMOD) and the differences, fits with REGRESS/POLY 
!		this difference depending on the positions (X,Y), computes
!		the projection map with COMPUTE/REGRESS and save them in the
!		table npline.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        P.Ballester, O.Boitquin, ESO/DMD
!
!.VERSION       1999/09/01  Created (OB).
!		1999/10/04-OB  Add usefull options P8 to fit the phys. model
!		1999/12/07-SW  Consequent use of P4!
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.12 $
! $Log: uves_msrawxy.prg,v $
! Revision 1.12  2001/01/04 10:39:42  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!
!
define/param P1 {WLC}     IMA   "Measurement Frame:" 
define/param P2 pline.tbl TAB   "Projection table:"
define/param P3 GAUSS   CHAR    "Measurement method (GAUSS/MOMENT):"
define/param P4 YES     CHAR    "Visualisation option:" 
define/param P5 0,0     NUMB    "Measurement position offsets (X,Y): " 
define/param P6 0,0     NUMB    "Scale of (X,Y) axis: " 
define/param P7 40,40   NUMB    "Dimension of measurement box (X,Y): " 
define/param P8 Y       CHAR    "Fits regression: Y or N ?"

! For messout
define/local pid/C/1/20 "uves_msrawxy"
mess/out 3 {pid} "Measure the raw XY positions of the lines"

verify/echelle {P1}

define/local MODTAB/C/1/30 {P2} 
define/local XYOFF/R/1/2   {P5}
define/local XYSCALE/R/1/2 {P6}
define/local MBOX/R/1/2    {P7} 

select/table {MODTAB} :IDENT.GT.0. >Null
copy/table {MODTAB} &l 

compute/table &l :XSTART = :XMOD*(1.+{XYSCALE(1)}) + {XYOFF(1)} - {MBOX(1)}/2.
compute/table &l :XEND   = :XMOD*(1.+{XYSCALE(1)}) + {XYOFF(1)} + {MBOX(1)}/2.
compute/table &l :YSTART = :YMOD*(1.+{XYSCALE(2)}) + {XYOFF(2)} - {MBOX(2)}/2.
compute/table &l :YEND   = :YMOD*(1.+{XYSCALE(2)}) + {XYOFF(2)} + {MBOX(2)}/2.

define/local limits/D/1/4 0,0
limits(1) = imsize(1) - mbox(1)/2
limits(2) = imsize(2) - mbox(2)/2
limits(3) = mbox(1)/2
limits(4) = mbox(2)/2

select/table &l :XSTART.GT.{limits(3)}..AND.:XSTART.LT.{limits(1)} >Null
select/table &l SELECT.AND.:YSTART.GT.{limits(4)}.AND.:YSTART.LT.{limits(2)} >Null
copy/table   &l &m
copy/table   &l rline

if p3(1:1) .eq. "G" then
  center/gauss {P1},middummm.tbl middummm.tbl >Null
else
  center/moment {P1},middummm.tbl middummm.tbl >Null
endif

copy/tt &m :XCEN    rline.tbl :XMES
copy/tt &m :YCEN    rline.tbl :YMES
copy/tt &m :STATUS  rline.tbl :STATUS

!AKA 2000-03-18:  save measured positions without status check for 
!                 stability test
copy/tab rline mline
!

select/table rline :STATUS.EQ.0. >Null
copy/table   rline  &p
copy/table   &p     rline 

if p4(1:1) .eq. "Y" then
 @ creifnot 2 heat
 clear/chan over
 load/table rline  :XMES :YMES  ? 0 5
 load/table rline  :XMOD :YMOD  ? 0 2
endif

compute/table rline :XDIF = :XMES - :XMOD
compute/table rline :YDIF = :YMES - :YMOD

! Fits the residual differences
if P8(1:1) .eq. "Y" then
  regress/poly  rline :XDIF :XMOD,:YMOD 2,2 {SESSOUTV}
  save/regress  {MODTAB} XDIF
  save/regress  rline XDIF
  regress/poly  rline :YDIF :XMOD,:YMOD 2,2 {SESSOUTV}
  save/regress  {MODTAB} YDIF
  save/regress  rline YDIF
  compute/regress rline :XDIFREG = XDIF {SESSOUTV}
  compute/regress rline :YDIFREG = YDIF {SESSOUTV}
  compute/table   rline :XMODREG = :XMOD + :XDIFREG
  compute/table   rline :YMODREG = :YMOD + :YDIFREG
  
  select/table {MODTAB} all >Null
  copy/table {MODTAB} npline
  copy/tt rline :XDIF npline :XDIF
  copy/tt rline :YDIF npline :YDIF
  compute/regress npline :XDIFREG = XDIF {SESSOUTV}
  compute/regress npline :YDIFREG = YDIF {SESSOUTV}
  compute/table   npline :XMOD = :XMOD + :XDIFREG
  compute/table   npline :YMOD = :YMOD + :YDIFREG

  !clear/chan over
  if P4(1:1) .eq. "Y" load/table npline :XMOD :YMOD ?  0 7
else
  !clear/chan over
  !load/table rline :XMOD :YMOD ?  0 4
endif  
