! $Id: uves_plotmod.prg,v 1.4 2001/01/04 10:39:36 amodigli Exp $
!+++ UVES_PLOTMOD +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_plotmod.prg
!
!.KEYWORDS      uves physical model, spectroscopy, echelle,
!
!.USAGE         @p (IDENT) rline.tbl wlc trans
!
!.INPUT         rline.tbl  (P1): name of measurement table
!               wlc     (P2): wavelength calibration frame
!               trans   (P3): Translation of the ccd (X,Y)
!			(P4): Angle offsets (echelle, CD, CCD rot)
!
!.OUTPUT        postscript.ps.
!
!.RETURN        
!
!.DEFAULT	P1	rline.tbl
!		P2	{wlc}
!		P3	0,0
!		P4	0.,0.,0.
!
!.PURPOSE       MIDAS procedure to display differences between measurements
!               of the raw XY position of Th-Ar lines on an arc echelle exposure
!               and the predicted positions as a function of pixel number 
!		in X and Y.
!
!		Creates 6 graphs and the postscript file with these 6 plots :
!		. plots column :XDIFF vs column :XMOD and :YMOD,
!		. plots column :YDIFF vs column :XMOD and :YMOD,
!		. plots column :YDIFF vs column :XDIFF,
!		. plots column :YMOD vs column :XMOD.
!
!
!.ALGORITHM     Creates a postscript file with plots. 
!
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        O.Boitquin, ESO/DMD
!
!.VERSION       1999/09/01  Created (OB).
!		1999/09/30-SW	Send more meaningful information!
!				Check return status of get/pathid 
!		1999/10/01-OB  Header with angle offsets (echelle, CD, CCD rot)
!		1999/10/07-SW  date,user plot label added (uves_plt_ident.prg).
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.4 $
! $Log: uves_plotmod.prg,v $
! Revision 1.4  2001/01/04 10:39:36  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------


define/param p1 rline.tbl c "Enter name of measurement table:"
define/param p2 {wlc} c "Enter name of wavelength calibration frame WLC:"
define/param p3   0,0 	NUMB  "Translation of the ccd (X,Y): "
define/param P4 0.,0.,0. NUMB 	"Angle offsets (echelle, CD, CCD rot):"

define/local K/I/1/1  0
!define/local colour/I/1/1  0
define/local Ncolour/I/1/1 4
define/local colsel/I/1/4 6,4,3,2
define/local typsel/I/1/4 5,3,2,8
define/local NX/I/1/1 0
define/local NY/I/1/1 0
define/local UPY/I/1/1 0
define/local LWY/I/1/1 0
define/local MINident/R/1/1  0.
define/local MAXident/R/1/1  0.
!define/local CENident/R/1/1  0.
define/local MNDX/I/1/1 -20
define/local MXDX/I/1/1 20
define/local MNDY/I/1/1 -20
define/local MXDY/I/1/1 20
!
! title
!
define/local hfname/c/1/100 " " ALL
define/local fname/c/1/100 " " ALL
define/local cname/c/1/100 " " ALL
define/local datobs/c/1/100 " " ALL
define/local swid/c/1/10 " " ALL
define/local temp/c/1/10 " " ALL
define/local wlen/c/1/10 " " ALL

define/local trans/D/1/2 {p3}
define/local angoff/D/1/3 {P4}

! For messout
define/local pid/C/1/20 "uves_plotmod"

append/out "Plot the differences of the physical model solution\n"
append/out "and the measurements `{WLC}`."
mess/out 3 {pid}

get/pathid {p2}
if {Q1} .ne. 0 return -1

set/format F5.1
wlen = "{{P2},{h_cwlen({PATHID})}}"	!get central wavelength

temp = "{{P2},{h_tempcam({PATHID})}}"
swid = "{{P2},{h_slwidth({PATHID})}}"
set/format
if m$existd(P2,h_origname) .eq. 1 then
  fname = "{{P2},{h_origname}}"
  hfname = "Filename :"
else
  fname = "{{P2},{h_obsname}}_{{P2},{h_obstpno}}"
  hfname = "OB name :"
endif
cname = h_nameccd({PATHID})

outputd(1) = -m$value({p2},O_TIME(4))
@p uves_jd2date {outputd(1)}
datobs = outputc
!
! get the dimension of the WLC-image
!
NX = m$value({p2},NPIX(1))
NY = m$value({p2},NPIX(2))
!
! Determine the statistic of the table
!
select/table {P1} all >Null
!
! Configure the plot
!
set/grap
assign/grap POSTSCRIPT.l
set/grap font=1 !yaxis=auto !pmode=0
!
! 1.plot
!
select/table {P1} all >Null
statis/table {P1} :XDIF >Null
MNDX = {outputr(1)}
MXDX = {outputr(2)}
statis/table {P1} :YDIF >Null
MNDY = {outputr(1)}
MXDY = {outputr(2)}

plot/axes  0,{NX} {MNDX},{MXDX} -65,-65,15,100 "\+ \+ X  (pixels)" "\+ \+ XDIF (in pixels)"

UPY = 0
!DO K = {ECHORD(4)} {ECHORD(5)} 1  	!order selection
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K}
! SELECT/TABLE {P1} :Y_1.EQ.{K}   	!order selection
  select/table {P1} :YMOD.GE.{LWY}.AND.:YMOD.LT.{UPY} >Null
! colour = {K}/{Ncolour}  		!order selection
  set/grap colour={colsel({K})}
  overpl/table {P1}   :XMOD :XDIF -65,-65,15,100 {typsel({K})}
enddo

!
! 2.plot
!
set/grap colour=1
select/table {P1} all >Null
overplot/axes  0,{NX} {MNDY},{MXDY} -65,-65,15,15 "\+ \+ X  (pixels)" "\+ \+ YDIF (in pixels)"

UPY = 0
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K} 
  select/table {P1} :YMOD.GE.{LWY}.AND.:YMOD.LT.{UPY} >Null
  set/grap colour={colsel({K})}
  overpl/table {P1}   :XMOD :YDIF -65,-65,15,15 {typsel({K})}
enddo
set/grap colour=1

if mid$sess(16:25) .eq. "99NOVpl1.1" then
  @p uves_plt_ident 0 15 csize=0.9	!write the bottom label (date,user)
endif 

!
! 3.plot
!
set/grap colour=1
select/table {P1} all >Null
overplot/axes  0,{NY} {MNDX},{MXDX} -65,-65,104,100 "\+ \+ Y  (pixels)" "\+ \+ XDIF (in pixels)"

UPY = 0
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K} 
  select/table {P1} :YMOD.GE.{LWY}.AND.:YMOD.LT.{UPY} >Null
  set/grap colour={colsel({K})}
  overpl/table {P1}   :YMOD :XDIF -65,-65,104,100 {typsel({K})}
enddo

!
! 4.plot
!
set/grap colour=1
select/table {P1} all >Null
overplot/axes 0,{NY} {MNDY},{MXDY} -65,-65,104,15 "\+ \+ Y  (pixels)" "\+ \+ YDIF (in pixels)"

UPY = 0
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K} 
  select/table {P1} :YMOD.GE.{LWY}.AND.:YMOD.LT.{UPY} >Null
  set/grap colour={colsel({K})}
  overpl/table {P1}   :YMOD :YDIF -65,-65,104,15 {typsel({K})}
enddo

!
! 5.plot
!
set/grap colour=1
select/table {P1} all >Null
overplot/axes {MNDX},{MXDX} {MNDY},{MXDY} -65,-65,194,100 "\+ \+ XDIF " "\+ \+ YDIF "

UPY = 0
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K} 
  select/table {P1} :YMOD.GE.{LWY}.AND.:YMOD.LT.{UPY} >Null
  set/grap colour={colsel({K})}
  overpl/table {P1}   :XDIF  :YDIF -65,-65,194,100 {typsel({K})}
enddo

!
! 6.plot
!
set/grap colour=1
select/table {P1} all >Null
overplot/axes 0,{NX} 0,{NY} -65,-65,194,15 "\+ \+ X  (pixels)" "\+ \+ Y  (pixels)"

UPY = 0
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K} 
  select/table {P1} :YMOD.GE.{LWY}.AND.:YMOD.LT.{UPY} >Null
  set/grap colour={colsel({K})}
  overpl/table {P1}   :XMOD :YMOD -65,-65,194,15 {typsel({K})}
enddo
select/table {P1} all >Null
!
! Plot the title
!
set/gra pmode=-1 colour=1
overpl/axes >Null


if mid$sess(16:25) .eq. "99NOVpl1.1" then
    set/grap tsize=1.3					!main title
    label/grap "Central wavelength: {wlen} nm, slit: {swid} arcsec, CCD: {cname}, {temp} \circ C" 90,167,mm

    set/grap tsize=1					!sub title
    outputi(1) = m$len(datobs)
    label/grap "{hfname} {fname}, Observ. date: {datobs(1:{outputi(1)})}" 90,162,mm

    set/grap tsize=1.0					!sub title: statistic
    SET/FORM f5.3
    label/grap "Shift in X = {trans(1)} , Shift in Y = {trans(2)}" 90,157,mm
    label/grap "Angle shift: echelle = {angoff(1)}, CrossDisp = {angoff(2)}, CCD rot = {angoff(3)}" 90,152,no
else
    set/grap tsize=1.3					!main title
    label/grap "Central wavelength: {wlen} nm, slit: {swid} arcsec, CCD: {cname}, {temp} \circ C" 0.5,0.99,no

    set/grap tsize=1					!sub title
    outputi(1) = m$len(datobs)
    label/grap "{hfname} {fname}, Observ. date: {datobs(1:{outputi(1)})}" 0.5,0.96,no

    set/grap tsize=1.0					!sub title: statistic
    SET/FORM f5.3
    label/grap "Shift in X = {trans(1)} , Shift in Y = {trans(2)}" 0.5,0.93,no
    label/grap "Angle shift: echelle = {angoff(1)}, CrossDisp = {angoff(2)}, CCD rot = {angoff(3)}" 0.5,0.90,no
    @p uves_plt_ident 0.02 0.02 csize=0.9	!write the bottom label (date,user)
endif
!
! Save the postscript file:
!
-rename postscript.ps pm_{wlen(1:3)}_{cname}.ps

