! $Id: uves_plt_resid.prg,v 1.6 2002/02/08 08:17:31 amodigli Exp $
!+++ UVES_PLT_RESID +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_plt_resid.prg
!
!.KEYWORDS      spectroscopy, echelle, wavelength calibration, residuals, plot
!
!.USAGE         @p (IDENT) lintab lincat
!
!.INPUT         lintab  (P1): name of line table (result of IDENT/ECHE)
!               lincat  (P2): name of line reference catalog
!
!.OUTPUT        graphic window
!
!.RETURN        -1 on error, else 0
!
!.DEFAULT	p1	{LINTAB}
!		p2	{LINCAT}
!
!.PURPOSE       Create a residual plot of the wavelength calibration.
!
!.ALGORITHM     Select all lines which were identified by IDENT/ECHELLE
!		(:SELECT.gt.1). Finally, plot the line table ({LINTAB})
!		using columns :ORDER and :RESIDUAL.
!		The title of the plot show informations like central
!		wavelength, slit size in arcsec, name of CCD, pixel size in
!		Angstroems, ...
!		The bottom label shows the parameters used for the line
!		identification and the user name together with the date of
!		creation of the plot.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1999/09/29  Created.
!		    1999/10/15	SHOW/DISP workaround implemented.
!
!.COMMENT	Put in a workaround to get the right size of the graphic
!		windows -- show/disp doesn't work properly in 98NOV!
!		This may be removed (see workaround)
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.6 $
! $Log: uves_plt_resid.prg,v $
! Revision 1.6  2002/02/08 08:17:31  amodigli
! Added check of nice_crea also on capital Y.
! Reduced XSZ1 from -150 to -135 to better fit
! in graphycal display window.
!
! Revision 1.5  2001/01/04 10:39:35  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------


define/param p1 {LINTAB} c "Enter name of line table [LINTAB]:"
define/param p2 {LINCAT} c "Enter name of line reference catalog [LINCAT]:"
!
! title
!
define/local xpos/r/1/1 0.0
define/local ypos/r/1/1 0.0
define/local hfname/c/1/100 " " ALL
define/local fname/c/1/100 " " ALL
define/local hlname/c/1/100 " " ALL
define/local lname/c/1/100 " " ALL
define/local cname/c/1/100 " " ALL
define/local datobs/c/1/100 " " ALL
define/local swid/c/1/10 " " ALL
define/local temp/c/1/10 " " ALL
define/local wlen/c/1/10 " " ALL
define/local samp/c/1/10 " " ALL
define/local mrms/c/1/10 " " ALL
define/local nbad/i/1/1 0
!
! Do the settings of the graphics device
!
!@ creifnot 1
set/graph default
set/grap pmode=0 font=1

show/disp >Null
!workaround:
if nice_crea .eq. "y" .or. nice_crea .eq. "Y" then
   outputi(13) = UVES_GXSZ( {outputi(12)} )
   outputi(14) = UVES_GYSZ( {outputi(12)} )
endif
!workaround_end.
define/local xrat/r/1/1 1.0
xrat = 1.0*outputi(13)/625.0
define/local yrat/r/1/1 1.0
yrat = 1.0*outputi(14)/425.0
define/local XSZ1/r/1/1 -135.0
XSZ1 = XSZ1 * xrat
define/local XOF1/r/1/1 25.0
XOF1 = XOF1 * xrat
define/local YSZ1/r/1/1 -80.0
YSZ1 = YSZ1 * yrat
define/local YOF1/r/1/1 25.0
YOF1 = YOF1 * yrat
!
! Plot the residuals of the wavelength calibration
!
set/format
set/graph color=1 xformat=AUTO

select/table {p1} (:SELECT.NE.0) >Null
plot/table {p1} :ORDER :RESIDUAL {XSZ1},{YSZ1},{XOF1},{YOF1}
!
!
! Prepare the title of the plot
!
get/pathid {p1}
if {q1} .ne. 0 return -1

set/format F5.1
wlen = "{{P1},{h_cwlen({PATHID})}}"	!get central wavelength
temp = "{{P1},{h_tempcam({PATHID})}}"	!get camera temperature
swid = "{{P1},{h_slwidth({PATHID})}}"	!get slit width
set/format F6.4
samp = "{{P1},PIXEL(1)}"		!get pixelsize (sampling) in Angstroems
if m$existd(p1,"AVRMS") .eq. 1 then
  set/format F9.5
  mrms = "{{P1},AVRMS(1)}"		!get mean RMS of wavel.cal.
  outputr = m$value({p1},AVRMS(2))
endif

set/format
cname = h_nameccd({PATHID})		!get name of the CCD
outputd(1) = -m$value({p1},O_TIME(4))
@p uves_jd2date {outputd(1)}
datobs = outputc			!get the date of observation

if m$existd(P1,h_origname) .eq. 1 then	!get original file name if exists
  fname = "{{P1},{h_origname}}"
  hfname = "Line Tbl. of:"
else if m$existd(P1,h_filename) .eq. 1 then
  fname = "{{P1},{h_filename}}"
  parse/path {fname}
  fname = "{q2}{q3}"
  hfname = "Line Tbl. of:"
else					!else: take the name of the OB
!  fname = "{{P1},{h_obsname}}_{{P1},{h_obstpno}}"
  fname = P1
  hfname = "Line Tbl.:"
endif

if m$existd(P1,h_procatg) .eq. 1 then	!get product categoty
  fname = "{fname} [{{P1},{h_procatg}}]"
endif

if P2(1:1) .ne. "+" then		!LINCAT passed to procedure?
  if m$existd(P2,h_origname) .eq. 1 then
    lname = "{{P2},{h_origname}}"	!get original file name if exists
  else if m$existd(P2,h_origname) .eq. 1 then
    lname = "{{P2},{h_filename}}"
    parse/path {lname}
    lname = "{q2}{q3}"
  else
    lname = "{LINCAT}"
  endif
  hlname = "Line Cat.:"
endif
!
!
! Plot the title
!
xpos = -XSZ1 / 2.0
ypos = -YSZ1 + 15 * yrat


if mid$sess(16:25) .eq. "99NOVpl1.1" then

    set/grap tsize=1.3					!main title
    label/grap "Central wavelength: {wlen} nm, slit: {swid}, CCD: {cname}, {temp} \circ C" {xpos},{ypos},mm
    !90,167,mm

    set/grap tsize=1					!sub title
    ypos = ypos - 7 * yrat
    label/grap "{hfname} {fname}, {hlname} {lname}" {xpos},{ypos},mm

    ypos = ypos - 5 * yrat
    outputi(1) = m$len(datobs)
    label/grap -
    "Observ. date: {datobs(1:{outputi(1)})}; Pixelsize: {samp} A; RMS: {mrms} A" {xpos},{ypos},mm


    inputc = m$symbol("USER")				!bottom label
    out_a = m$time()
    outputr(1) = XOF1 - 2
    outputr(2) = YOF1 * 0.9
    if nice_crea .eq. "y" .or. nice_crea .eq. "Y" then
       outputr(3) = 0.45 / xrat
       label/grap -
      "{out_a}; USER: {inputc}" -{outputr(1)},-{outputr(2)},mm 0 {outputr(3)} 1
    endif  
    if m$existd(P1,"WCPARAM") .eq. 1 then
      out_b = "Parameters: {{P1},WCPARAM}"
      outputr(1) = -XSZ1
      label/grap "{out_b}" {outputr(1)},-{outputr(2)},mm 0 {outputr(3)} 2
    endif

else

    set/grap tsize=1.3					!main title
    label/grap "Central wavelength: {wlen} nm, slit: {swid}, CCD: {cname}, {temp} \circ C" 0.5,0.98,no
    !90,167,mm

    set/grap tsize=1					!sub title
    ypos = ypos - 7 * yrat
    label/grap "{hfname} {fname}, {hlname} {lname}" 0.5,0.94,no

    ypos = ypos - 5 * yrat
    outputi(1) = m$len(datobs)
    label/grap -
    "Observ. date: {datobs(1:{outputi(1)})}; Pixelsize: {samp} A; RMS: {mrms} A" 0.5,0.90,no


    inputc = m$symbol("USER")				!bottom label
    out_a = m$time()
    if nice_crea .eq. "y" .or. nice_crea .eq. "Y" then
       outputr(3) = 0.45 / xrat
       label/grap -
      "{out_a}; USER: {inputc}" 0.02,0.02,no 0 {outputr(3)} 1
    endif  
    if m$existd(P1,"WCPARAM") .eq. 1 then
      out_b = "Parameters: {{P1},WCPARAM}"
      label/grap "{out_b}" 0.9,0.02,no 0 {outputr(3)} 2
    endif
endif    

return 0
