! $Id: uves_plt_uves.prg,v 1.4 2001/01/04 10:39:35 amodigli Exp $
!+++ UVES_PLT_UVES ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_plt_uves.prg
!
!.KEYWORDS      spectroscopy, echelle, display
!
!.USAGE         @p (IDENT) fname sprange title hcopy spsize
!
!.INPUT         fname   (P1): filename of merged echelle spectrum
!               x-range (P2): 1-dim data: wavelength range in Angstrom
!			      2-dim data: sub-spectra range
!               y-range (P3): y-axis range for each sub-spectrum
!               title   (P4): title of the plot
!               hcopy   (P5): hardcopy filename
!
!.OUTPUT        
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!		OUT_A:  date and user (lower left label)
!		OUT_B:  y-range (lower right label)
!
!.DEFAULT	P2	2,-1
!		P3	+
!		P4	+
!		P5	2009
!
!.PURPOSE       Creates a multiple spectrum plot of an echelle spectrum
!
!.ALGORITHM     Checks at first if the echelle spectrum has been already
!		merged -- if not the procedure exits.
!		The merged spectrum will be split into sub-spectra of
!		specified size (P2). The selected number of subspectra will
!		be plotted within one plot-frame with increasing offsets.
!		The zero-line and the x-ticks for each subspectrum will set,
!		correspondingly.
!		The subspectrum range may be specified in two ways: absolute
!		and relative:
!		absolute:	n1,n2	start at subspectrum {n1}, end at {n2}
!		relative:	n1,-n2	start at subspectrum {n1},
!					end at last spectrum - n2
!				-n1	show the spectra +/- n1 around central
!					sub-spectrum
!		wave.units:	w1,w2	start and end of wavelength range in A.
!				w1	start at wavelength w1 in A
!				n1	show order n1 in wavelength units.
!
!		The title of the plot is optional -- a standard one will be
!		used, otherwise.
!
!.ENVIRON       UVES, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1999/03/31  Created.
!		    1999/07/14	Spectral range in wavelength units [A] may be
!				entered.
!		    1999/09/09	Now create x-axis at y-range minimum and label
!		               	plot with yrange at lower right edge and store
!				it out_a, out_b.
!		    1999/10/05	Design improved, X-axis scale bug fixed!
!		    1999/10/12	Computation of {ny} was wrong for 1-dim case.
!				due to wrong XLOW, set to xmin!
!		    1999/11/03	For 1-dim spectra specify order number to
!				show only the desired order.
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.4 $
! $Log: uves_plt_uves.prg,v $
! Revision 1.4  2001/01/04 10:39:35  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

crossref fname sprange title hcopy spsize

define/param p1 ? c "Enter name of reduced echelle spectrum: "
define/param p2 + n "Enter X-range (ystart,yend,xwid): "
define/param p3 + n "Enter Y-range (ystart,yend): "
define/param p4 + c "Enter title:"
define/param p5 + c "Enter POSTCRIPT filename for hardcopy:"
!
! local constants
!
define/local pid/c/1/10 "plt_uves" ? +lower_levels
!
define/local CSZ/r/1/1 0.6 ? +lower_levels
define/local PCOL/i/1/1 4			!BLUE, RED
define/local XSIZE/r/1/1 210			!total size [mm]
define/local YSIZE/r/1/1 160
define/local PXMM/r/1/1 3.5			!pixel per mm
!
! global variables
!
!XLL = 20 YLL = 15
define/local XLL/r/1/1 30.0 ? +lower_levels
define/local YLL/r/1/1 15.0 ? +lower_levels
define/local XSZ/r/1/1 0.0 ? +lower_levels
define/local YSZ/r/1/1 0.0 ? +lower_levels
define/local XLOW/r/1/1 0.0 ? +lower_levels
define/local XUPP/r/1/1 0.0 ? +lower_levels
define/local XMIN/r/1/1 0.0 ? +lower_levels
define/local XMAX/r/1/1 0.0 ? +lower_levels
define/local YMIN/r/1/1 0.0 ? +lower_levels
define/local YMAX/r/1/1 0.0 ? +lower_levels
define/local nx/i/1/1 0 ? +lower_levels
define/local ny/i/1/1 0 ? +lower_levels
define/local nax/i/1/1 0 ? +lower_levels
define/local ista/i/1/1 0 ? +lower_levels
define/local iend/i/1/1 0 ? +lower_levels
define/local xwid/r/1/1 60.0 ? +lower_levels
define/local sp1/i/1/1 0 ? +lower_levels
define/local ystp/r/1/1 0.0 ? +lower_levels
define/local yoff/r/1/1 0.0 ? +lower_levels
define/local ymea/r/1/1 0.0 ? +lower_levels
define/local ifrm/c/1/100 {p1} ? +lower_levels
!
! local variables
!
define/local i1/i/1/1 0
define/local i2/i/1/1 0
define/local ysca/r/1/1 1.0
define/local y0/r/1/1 0.0
define/local x0/d/1/1 0.0
define/local dx/d/1/1 0.0
define/local sw/i/1/1 1
define/local xtit/c/1/48 " " ALL
define/local ytit/c/1/48 " " ALL

show/disp >Null					!Design the plot
if outputi(12) .lt. 0 then
  mess/out 1 plt_uves "NO plot device open!"
  return -1
endif
XSIZE = UVES_GXSZ({outputi(12)})		!Get the X-dim in pixel
YSIZE = UVES_GYSZ({outputi(12)})		!.. Y-dim in pixel
XLL = XLL * XSIZE / 625.
YLL = YLL * YSIZE / 425.			!coordinates in mm
if XLL .lt. 30 XLL = 30
if XLL .gt. 50 XLL = 50
if YLL .lt. 12 YLL = 12
if YLL .gt. 30 YLL = 30
XSZ = m$nint(XSIZE / PXMM - 0.5) - XLL		!X-size and Y-size of
YSZ = m$nint(YSIZE / PXMM - 0.5) - YLL * 1.5	!the plot in mm
x0 = m$value({p1},start(1))			!first wavelength
dx = m$value({p1},step(1))			!step size on wavelength
XLOW = x0					!max. X axis range
XUPP = x0 + m$value({p1},npix(1)) * dx		![wavel.units] or [PIXEL]


get/pathid {p1}					!Get instrument path id
if {q1} .ne. 0 return
if PATHID .lt. 3 then				!RED arm
  PCOL = 2					!plot colour: red
else						!BLUE arm
  PCOL = 4					!plot colour: blue
endif

copy/dk {p1} CUNIT/c/1/16  ytit			!Get the X-axis and Y-axis
copy/dk {p1} CUNIT/c/17/32 xtit			!titles
if xtit(1:1) .eq. "P" .OR. xtit(1:1) .eq. "p" -
  sw = 0



nax = m$value({p1},NAXIS)			!Dimensions of input
@p uves_plt_uves,XY_RANGE {p1} {p2} {p3} {x0} {dx}
if {q1} .ne. 0 goto ERR1

set/graph default				!Configure plot
set/graph pmode=1 lwidth=1 tsize=1.0 font=1
set/graph xformat=AUTO
if ymax .gt. 999 then				!Prepare Y-title and Y-scale
  ysca = 1000.
  ytit = "{ytit} (\times 10\{\^\{+3\}\})"
else if ymax .lt. 0.001 then
  ysca = 0.001
  ytit = "{ytit} (\times 10\{\^\{-3\}\})"
else
  ysca = 1.
endif
ymin = ymin/ysca
ymax = ymax/ysca

						!Plot the coordinate system
plot/axes {xmin},{xmax} {ymin},{ymax} -{XSZ},-{YSZ},{XLL},{YLL} -
  "{xtit}" "{ytit}"

if ysca .ne. 1.0 then				!Adjust the Y-scaling if
  ymin = ymin*ysca				!necessary
  ymax = ymax*ysca
  plrgrap(5) = plrgrap(5)*ysca
  plrgrap(6) = plrgrap(6)*ysca
  plrgrap(7) = plrgrap(7)*ysca
  plrgrap(8) = plrgrap(8)*ysca
endif

@p uves_plt_uves,DISPL_TITL {p1} {p4}		!Show the title of the plot
set/format F6.2
out_b = "y_min, dy: {ymin}, {yoff}"		!Bottom label of the plot
@p uves_plt_uves,PLT_IDENT 0.0 {YLL} {XSZ} "{out_b}"



BRANCH nax 1,2 ONEDIM,TWODIM
 
ONEDIM:
  set/format I1

  set/graph colour={PCOL}			!Spectra will be plotted
  overplot/row {ifrm} >Null			!in BLUE or RED
  if ny .lt. 2 goto ENDBRA
  set/format
  x0 = xmin
  y0 = ymin
  do i1 = 2 {ny} 1
    {ifrm},start(1) = m$value({ifrm},START(1)) - xwid
    x0 = x0 + xwid
    y0 = y0 + yoff
    overplot/row {ifrm} ? ? {y0} >Null
    @p uves_plt_uves,XAX {x0} {xmin} {y0} {csz} 1 1 {sw}
    set/grap colour={PCOL}
  enddo
  goto ENDBRA


TWODIM:
  define/local yz/r/1/1 0.0
  sw = 0
  x0 = x0 + dx * nx * (sp1 - 1)			!Start in X
  ymax = ymea
  set/graph colour={PCOL}
  overplot/row {ifrm} @{sp1} ? 0.0 >Null
!  if sp1 .gt. 1 -
!    @p uves_plt_uves,XAX {x0} {xmin} {ymin} {csz} 1 1 {sw} !Insert an x-axis
  set/format I1
  set/graph colour=1
  label/graph "#{sp1}" {xmax},{ymax} 0 {csz} 1
  set/format

  y0 = 0.0 + ymin
  do i1 = {ista} {iend}
    if i1 .le. {iend} then
      x0 = x0 + dx * nx
      y0 = y0 + yoff
      yz = y0 - ymin
      ymax = ymea + yz
      set/format I1 F14.6
      set/graph colour={PCOL}
      overplot/row {ifrm} @{i1} ? {yz} >Null
      @p uves_plt_uves,XAX {x0} {xmin} {y0} {csz} 1 1 {sw}
      label/graph "#{i1}" {xmax},{ymax} 0 {csz} 1
    endif
  enddo

ENDBRA:

if p5(1:1) .ne. "+" hardcopy/plot N,G {p5}

set/graph default
set/graph pmode=1


return 0

ERR1:
mess/out 1 {pid} "Bad x/y-range specification !"
return -1





!=== SUBROUTINES ==============================================================

ENTRY DISPL_TITL

define/param p1 ? c "Enter name of echelle spectrum:"
define/param p2 + c "Enter title:"


define/local i1/i/1/1 0
define/local i2/i/1/1 0
define/local x0/r/1/1 0.0
define/local y0/r/1/1 0.0
define/local ystp/r/1/1 0.0
define/local instrum/c/1/8 " " ALL
define/local title/c/1/200 " " ALL
define/local datobs/c/1/25 " " ALL
define/local filt/c/1/20 " " ALL
define/local fname/c/1/100 {p1}

filt = uves_filti(2)				!filter used

						!Get date of observ.
@p uves_jd2date {p1}				!used for subtitle
i1 = m$len(outputc)
datobs = outputc(1:{i1})

if m$existd(p1,h_origname) .eq. 1 then		!Take the original
  fname = "{{p1},{h_origname}}"			!filename for the title
endif
if p2(1:1) .eq. "+" then
  instrum = "{{p1},{h_instrume}}"		!Name of instrument
  title = "Echelle data ({instrum}): {fname}"
else
  if p2(1:1) .eq. """ then			!Strip leading and pending
    i1 = 2					!quotes of title parameter
    i2 = m$strlen(p2) - 1
  else
    i1 = 1
    i2 = 100
  endif

  title(1:100) = p2({i1}:{i2})
endif



if mid$sess(16:25) .eq. "99NOVpl1.1" then

    ystp = YLL / 6					!Display title
    x0 = XSZ / 2
    y0 = YSZ + 4 * ystp
    label/graph "{title}" {x0},{y0},mm 0 0.7 0

    y0 = YSZ + 2 * ystp				!Display sub-title (1)
    set/format F5.0	
    outputr(1) = {{p1},{h_cwlen({PATHID})}} * 10.0
    outputc = "{XLOW},{XUPP} A "
    title = "Cent.wavelength: {outputr(1)} A, \lambda \lambda : {outputc}"
    set/format
    title = "{title}, Filter: {filt}, CCD: {h_nameccd({PATHID})}"
    label/graph "{title}" {x0},{y0},mm 0 0.4 0

    y0 = y0 - ystp					!Display sub-title (2)
    title = "Obs.date: {datobs}"
    label/graph "{title}" {x0},{y0},mm 0 0.4 0
else
    !Display title
    label/graph "{title}" 0.5,0.99,no 0 0.7 0

    !Display sub-title (1)
    set/format F5.0	
    outputr(1) = {{p1},{h_cwlen({PATHID})}} * 10.0
    outputc = "{XLOW},{XUPP} A "
    title = "Cent.wavelength: {outputr(1)} A, \lambda \lambda : {outputc}"
    set/format
    title = "{title}, Filter: {filt}, CCD: {h_nameccd({PATHID})}"
    label/graph "{title}" 0.5,0.96,no 0 0.4 0

    !Display sub-title (2)
    title = "Obs.date: {datobs}"
    label/graph "{title}" 0.5,0.93,no 0 0.4 0

endif

return 0


!..............................................................................
ENTRY PLT_IDENT
define/param p1 ? n "Enter lower left X offset of diagram [mm]: "
define/param p2 ? n "Enter lower left Y offset of diagram [mm]: "
define/param p3 + n "Enter X size of diagram: "
define/param p4 + c "Enter bottom right label: "
define/param p5 0.2 n "Enter character size: "

inputc = m$symbol("USER")
out_a = m$time()



if mid$sess(16:25) .eq. "99NOVpl1.1" then
    outputr(2) = {p2} * 0.9
    label/grap "{out_a}; USER: {inputc}" -{p1},-{outputr(2)},mm 0 {p5} 1

    if p3(1:1) .ne. "+" .AND. p4(1:1) .ne. "+" -
      label/grap {p4} {p3},-{outputr(2)},mm 0 {p5} 2
else
    label/grap "{out_a}; USER: {inputc}" 0.02,0.02,no 0 {p5} 1

    if p3(1:1) .ne. "+" .AND. p4(1:1) .ne. "+" -
      label/grap {p4} 0.99,0.02,no 0 {p5} 2
endif
return 0


!..............................................................................
ENTRY XAX
define/param p1 ? n "Enter start value of x-axis (first datapoint):"
define/param p2 ? n "Enter x-offset of x-axis:"
define/param p3 ? n "Enter y-offset of x-axis:"
define/param p4 ? n "Enter character size:"
define/param p5 ? n "Enter line style:"
define/param p6 ? n "Enter colour:"
define/param p7 ? n "Switch xaxis numbering ON/OFF:"

define/local i1/i/1/1 0
define/local ntck/i/1/1 0
define/local stsz/r/1/1 0.0
define/local ypos/r/1/1 0.0
define/local xpos/r/1/1 0.0
define/local xval/r/1/1 0.0
define/local xlab/c/1/30 " " ALL

set/format
set/graph colour={p6}


ntck = (plrgrap(2) - plrgrap(1)) / plrgrap(3) - 1
xval = {p1} + plrgrap(3)
xpos = plrgrap(1) + plrgrap(3)
ypos = {p3} - plrgrap(8) / 2.0
do i1 = 1 {ntck}
  set/format F6.0
  xlab = "{xval}"
  set/format
  overplot/line {p5} {p2},{p3} {plrgrap(2)},{p3}
  if {p7} .eq. 1 label/graph {xlab} {xpos},{ypos} 0 {p4} 0
  set/format F14.5
  label/graph "|" {xpos},{p3} 0 {p4} 0
  xpos = xpos + plrgrap(3)
  xval = xval + plrgrap(3)
enddo


!..............................................................................
ENTRY XY_RANGE
define/param p1 ? c "Enter name of reduced echelle spectrum: "
define/param p2 + n "Enter X-range (xstart,xend): "
define/param p3 + n "Enter Y-range (xstart,xend): "
define/param p4 + n "Enter start in X:"
define/param p5 + n "Enter step in X:"

define/local i1/i/1/1 0
define/local i2/i/1/1 0
define/local dd/r/1/1 0.0

if NAX .eq. 1 then
  sp1 = 1					!Onedimensional spectrum
  nx = m$value({p1},NPIX(1))

  i1 = m$parse(p2,"xr")				!Get X-range
  if i1 .lt. 1 return -1			!at minimum: ? specified
  if i1 .eq. 3 then				!X-range: [XLOW,XUPP]
    xwid = {xr03}				!and xaxis width specif.
  endif
  if i1 .ge. 2 then				!At minimum [XLOW,XUPP]
    xmin = {xr01}				!has been passed to
    xmax = {xr02}				!the procedure? CHECK!
    XLOW = xmin					!lowest wavelength.
    if xmin .lt. 1000 return -1
    if xmax .lt. 0 then
      xwid = -xmax				!Negative xr02 is the
      xmax = xmin + xwid			!width of x-axis.
    else					!Total wavelength range
      if xmax .gt. XUPP xmax = XUPP		!to be plotted specified
      XUPP = xmax				!Upper limit check
      if i1 .eq. 2 xwid = 60.0
      xmax = xmin + xwid
    endif
  else if p2(1:1) .eq. "?" .OR. p2(1:1) .eq. "+"  then	!p3 assumed to be ?
    xwid = 60.0					!-> take as default the
    xmin = m$nint({p4}/10+0.5)*10.0		!60.0 Angstroems width
    xmax = xmin + xwid				!for x-axis range.
    XLOW = xmin
  else if {p2} .le. 100	then			!p3 assumed to be an order
    xwid = 60.0					!number.
    xmin = m$value({p1},WSTART({p2}))		!
    xmax = xmin + xwid				!
    XLOW = xmin
    XUPP = xmax
  else if {p2} .gt. 100 then			!starting wavelength specified
    xwid = 60.0					!-> take as default the
    xmin = m$nint({p2}/10+0.5)*10.0		!60.0 Angstroems width
    xmax = xmin + xwid				!for x-axis range.
    XLOW = xmin
  endif

  
  ny = (XUPP - XLOW) / xwid			!number of sub-spectra
  dd = (XUPP - XLOW) / xwid - ny
  if dd .gt. {p5} ny = ny+1

  ifrm = "cp{p1}"				!Create a copy to work on
  -copy {p1} {ifrm}

else						!TWO DIMENSIONS


  nx = m$value({p1},NPIX(1))			!X and Y dimension of
  ny = m$value({p1},NPIX(2))			!input frame

  i1 = m$parse(p2,"xr")				!Get the subspectra
  if i1 .lt. 1 return -1			!to be plotted.
  if xr01(1:1) .eq. "+" then
    sp1 = 1
  else
    sp1 = {xr01}
  endif
  ista = sp1 + 1
  if i1 .eq. 1 then				!Plot only one subspec.
    if sp1 .lt. 0 then				!Range is specified
      iend = ny / 2 - sp1			!relative to central
      sp1 = ny / 2 + sp1			!subspectrum
      if sp1 .lt. 0 sp1 = 1
      if iend .gt. ny iend = ny
      ista = sp1 + 1
      ny = iend - sp1 + 1
    else
      iend = sp1
      ny = 1
    endif
  else						!Plot a range of subsp.
    iend = {xr02}
    if sp1 .lt. 0 then				!Range is specified
      iend = ny / 2 - sp1			!relative to central
      sp1 = ny / 2 + sp1			!subspectrum
      if sp1 .lt. 0 sp1 = 1
      if iend .gt. ny iend = ny
      ista = sp1 + 1
    else
      if iend .lt. 0 iend = ny + iend		!Upper limit specified
    endif					!rel. to last subspec.
    ny = iend - sp1 + 1
  endif 
  xmin = {p4}					!X-axis: always full range
  xmax = {p4} + nx * {p5}

  ifrm = p1

endif

i1 = m$parse(p3,"yr")				!Get Y-range per sub-spectrum
if i1 .lt. 1 return -1				!at minimum: ? specified
if i1 .eq. 2 then				!Y-range: [ymin,ymax]
  ymin = {yr01}					!has been passed to
  ymax = {yr02}					!the procedure.
  yoff = ymax - ymin
  ymea = ( ymax + ymin ) / 2.0
  ymax = ny * yoff + ymin
  if ny .gt. 5 csz = 0.4
else						!p3 assumed to be ?
  statistic/ima {p1} ? ? ? SN			!Get y-range from the
  yoff = outputr(4) * 3				!statistic of the
  ymax = yoff * (ny+1)				!input frame [default].
  ymin = -yoff / ny
  ymea = ( yoff + ymin ) / 2.0
  if ny .gt. 5 then
    csz = 0.4
    ymin = -yoff
    ymea = yoff / 2.0
  endif
endif

mess/out 4 {pid} "Plot range: [{xmin},{ymin}:{xmax},{ymax}]"

return 0

