! $Id: uves_qc1_log.prg,v 1.13 2002/10/25 09:56:32 amodigli Exp $   
!+++ UVES_QC1_LOG ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 2000  European Southern Observatory
!
!.IDENT         $RCSfile: uves_qc1_log.prg,v $
!
!.KEYWORDS      Write keyword or comment into QC1 Log file
!
!.USAGE         qc1log/out mode frame keyword value comment
!
!.INPUT         mode    (P1): 0: dump contents of the cache to the real qc1log.
!                             1: open and add entry to log cache. The entries 
!                                in the cache are first dumped to the real log 
!                                as soon as qc1log/out 0 is called.
!                             2: start a new group.
!                             3: end a group.
!               frame   (P2): Name of frame referred to.
!               keyword (P3): Name of keyword to be written.
!               value   (P4): Value of keyword 'keyword' to be written.
!               comment (P5): Optional comment string.
!               dict    (P6): Dictionary name (required, if not QC dictionary).
!
!.OUTPUT        none.
!
!.RETURN        Q1:   0: successful return
!                     1: failed
!                    -1: warning, could not create log
!
!.DEFAULT
!
!.PURPOSE       Writes into QC1 Log file
!
!.ALGORITHM     qc1log/out creates an entry in a QC1 Log File using the
!               UNIX command 'QC1LogWriter'. For details of QC1LogWriter
!               please refer to the documentation of the command. It also
!               creates the same keyword in the header of concerned frame.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 01FEB
!
!.AUTHOR        Andrea Modigliani as derived from fors_qc1+log.prg from Stefan Bogun, ESO-DMO
!
!.VERSION       $Log: uves_qc1_log.prg,v $
!.VERSION       Revision 1.13  2002/10/25 09:56:32  amodigli
!.VERSION       Extended size of tmp qctable
!.VERSION
!.VERSION       Revision 1.12  2002/06/03 16:19:21  amodigli
!.VERSION       Now QC table has 500 rows to support FIBER mode
!.VERSION
!.VERSION       Revision 1.11  2002/03/12 15:38:37  amodigli
!.VERSION       moved path file as last on command line call
!.VERSION
!.VERSION       Revision 1.10  2002/03/04 09:21:38  amodigli
!.VERSION       Adding QC-KEY in table only if QC-LOG Env variables are defined
!.VERSION
!.VERSION       Revision 1.9  2002/01/31 14:27:17  amodigli
!.VERSION       cleaned (commented) lines of code not
!.VERSION       necessary using paf files
!.VERSION
!.VERSION       Revision 1.8  2002/01/22 15:16:05  amodigli
!.VERSION       1st implementation of paf files
!.VERSION
!.VERSION       Revision 1.7  2001/08/10 13:48:53  amodigli
!.VERSION       Cleaner way to check for code
!.VERSION
!.VERSION       Revision 1.6  2001/08/08 13:14:43  amodigli
!.VERSION       Added again prefix ESO to QC keywords
!.VERSION
!.VERSION       Revision 1.5  2001/07/12 20:50:27  amodigli
!.VERSION       Commented extra log messages
!.VERSION
!.VERSION       Revision 1.4  2001/07/04 07:59:31  amodigli
!.VERSION       New version supporting new QC1 Naming conventions
!.VERSION
!.VERSION       Revision 1.8  2001/06/20 12:42:52  sbogun
!.VERSION       QC1LogWriter called for every single keyword to dump.
!.VERSION       QC1LogWriter call implemented in shell script.
!.VERSION
!.VERSION       Revision 1.7  2001/06/15 15:53:22  sbogun
!.VERSION       Complete revision:
!.VERSION       Can handle groups now
!.VERSION       supports extra dictionaries
!.VERSION       ...
!.VERSION
!
!.COMMENT
!------------------------------------------------------------------------------


crossref mode frame keyword value comment

define/param P1 ? c "Enter mode:"
define/param P2 + c "Enter name of frame:"
define/param P3 + c "Enter name of keyword:"
define/param P4 + c "Enter value of keyword:"
define/param P5 + c "Enter comment string:"
define/param P6 + c "Enter dictionary name:"

define/local procid/c/1/20       "qc1_log"
define/local dict_prefix/c/1/20  "ESO-DFS-DIC."
define/local instrument/c/1/20   " " all

define/local mode/i/1/1          {p1}
define/local frame/c/1/100       {p2}
define/local keyword/c/1/100     {p3}
define/local value/c/1/100       " " all
define/local comment/c/1/100     " " all
define/local dict/c/1/100        " " all 
define/local dict_path/c/1/100   " " all 
define/local opt/c/1/255         " " all
define/local command/c/1/255     " " all 
define/local i/i/1/1             0
define/local n/i/1/1             0
define/local frame_fits/c/1/100  " " all
define/local quote/c/1/1         "
define/local singlequote/c/1/1   '
define/local hyphen/c/1/3 "~"  all
define/local pfile/c/1/32     "qc1_uves.paf" 
define/local char_buf/c/1/80 " " all
define/local filecode/i/1/2   0    all
write/keyword value              {p4}
write/keyword comment            {p5}
write/keyword dict               {p6}
instrument = m$upper(rbinstrument)

! Check whether cache table exists already, otherwise create it
if m$exist(qc1log_table) .eq. 0 .or. m$existk("qc1log_id") .eq. 0 then
  write/key QC1LOG_ID/i/1/1 0
  mess/out 4 {procid} " "
  mess/out 4 {procid} "Creating qc1 log table {qc1log_table}"
  create/table {qc1log_table} 5 800 null
  create/column {qc1log_table} :frame   " " A100 C*100
  create/column {qc1log_table} :keyword " " A100 C*100
  create/column {qc1log_table} :value   " " A100 C*100
  create/column {qc1log_table} :comment " " A100 C*100
  create/column {qc1log_table} :dict    " " A100 C*100
  QC1LOG_ID = 1
endif

! Check whether group key exists
if m$existk("qc1log_group") .eq. 0 then
  write/key QC1LOG_GROUP/i/1/1 0
endif


branch mode 0,1,2,3 DMP,ADD,START,STOP

mess/out 1 {procid} "Wrong mode selected (0,1,2,3 are allowed) !"
goto FIN


DMP:
  if QC1LOG_ID .le. 1 then
       mess/out 3  {procid} "There's nothing to log..."
    return -1
  endif

! Check whether system variables DFS_QC1_LOG_DIR and DFS_QC1_DIC_DIR exist
  out_a = m$symbol("DFS_QC1_LOG_DIR")
  if out_a .eq. "DFS_QC1_LOG_DIR" then
    append/out "Shell variable DFS_QC1_LOG_DIR not defined. Cannot\n"
    append/out "create QC1 Log!"
    mess/out 2 {procid}
    return -1
  endif

  out_a = m$symbol("DFS_QC1_DIC_DIR")
  if out_a .eq. "DFS_QC1_DIC_DIR" then
    append/out "Shell variable DFS_QC1_DIC_DIR not defined. Cannot\n"
    append/out "create QC1 Log!"
    mess/out 2 {procid}
    return -1
  endif


! Check whether dictionary exists
  out_a = m$symbol("DFS_QC1_DIC_DIR") // "/" // dict_prefix // instrument -
          // "_QC"
  if m$exist(out_a) .eq. 0 then
    append/out "Dictionary {out_a} does not exist.\n"
    append/out "Cannot create QC1 Log!"
    mess/out 2 {procid}
    return -1
  endif


! Check whether last group ended
  if qc1log_group .eq. 1 then
    qc1log/out 1 "stop_group" 0 0
  endif
  qc1log_group = 0

! this to not have problems with single quotes  
  set/midas squote=no
! Loop over all entries in QC1Log Cache
  mess/out 4 {procid} " "
  n = qc1log_id - 1
  do i = 1 n

    opt = " "

    frame   = "{{qc1log_table},:frame,@{i}}"
    keyword = "{{qc1log_table},:keyword,@{i}}"
    value   = "{{qc1log_table},:value,@{i}}"
    comment = "{{qc1log_table},:comment,@{i}}"  !superfluos
    dict    = "{{qc1log_table},:dict,@{i}}"     !superfluos
 
! Start group keyword
    if frame .eq. "start_group" then
      !start and stop are anyway added when dumping paf file
 
      open/file {pfile} write filecode
      if filecode(1) .eq. -1 then
         mess/out 3 {pid} "Something went wrong"
      endif
      goto ENDLOOP
    endif

! Stop group keyword
    if frame .eq. "stop_group" then
      !start and stop are anyway added when dumping paf file
      
      !search path
      dict_path = m$symbol("INS_ROOT") // "/SYSTEM/Dictionary"
      !search dictionary: UVES specific
      dict  = "UVES_ICS,CCDDCS,DPR,OBS,PRIMARY-FITS,PRO,TPL,UVES_ICS" 
      close/file {filecode(1)}
      $QC1LogWriter -s UVES  -I {dict_path} -D {dict} -p {pfile}
      !delete file to be sure in case of more than one test to dump the proper one
      -delete {pfile}
      goto ENDLOOP
    endif

! Replace MIDAS file name by FITS file name
    if m$exist(frame) .eq. 1 then
      if m$existd(frame,H_PIPEFILE) .eq. 1 then
        frame_fits = "{{frame},{H_PIPEFILE}}"
        if keyword .eq. "{H_PIPEFILE}" then
          value = frame_fits
        endif
      else if m$existd(frame,H_FILENAME) .eq. 1 then
        frame_fits = "{{frame},{H_FILENAME}}"
        outputi = m$indexb(frame_fits,"/")+1
        frame_fits = frame_fits({outputi(1)}:)
      endif
      frame = frame_fits      
    endif

! Create QC keyword name
    out_a = m$upper(keyword)
    outputi = m$index(out_a,"ESO.")
    if outputi .eq. 1 then
      keyword = keyword(5:)
    endif

! Check for VLT dictionaries
    if dict .ne. "+" then

! Check whether system variable INS_ROOT exists
      out_a = m$symbol("INS_ROOT")
      if out_a .eq. "INS_ROOT" then
        append/out "Shell variable INS_ROOT not defined. Cannot\n"
        append/out "qc1log keywords defined in VLT dictionaries!"
        mess/out 2 {procid}
        goto ENDLOOP
      endif
    endif

 
! Create log entry
    !w/o "{keyword}"
    if keyword .ne. "+" then
      @p uves_qc1_log,get_type {value}
      if {q1} .eq. 3 then        
        !store line to dump in buffer
        char_buf = keyword // hyphen // singlequote // value // singlequote 
      else   
        !store line to dump in buffer
        char_buf = keyword // hyphen // "{value}"  
      endif
    endif
 
    !if comment .ne. "+" then
    !  opt = opt // "~-c~" // quote // comment // quote
    !endif
 
! Build the required system call and execute it.
! This will be done in a clean way with a later MIDAS release.
    !replace hyphen with a small space
    char_buf = m$repla(char_buf,"~"," ")
    !dump
    write/file {filecode(1)} {char_buf}     
    mess/out 4 {procid} "Entry added to QC1 Log file."

ENDLOOP:
  enddo
  !reset single quote midas normal behaviour back
  set/midas squote=yes

  QC1LOG_ID = 0
  goto FIN


ADD:
! Check whether system variables DFS_QC1_LOG_DIR and DFS_QC1_DIC_DIR exist
  out_a = m$symbol("DFS_QC1_LOG_DIR")
  if out_a .eq. "DFS_QC1_LOG_DIR" then
    !If the user has not properly installed QC-LOG system 
    !silently exit
    return -1
  endif
 
  out_a = m$symbol("DFS_QC1_DIC_DIR")
  if out_a .eq. "DFS_QC1_DIC_DIR" then
    !If the user has not properly installed QC-LOG system 
    !silently exit
    return -1
  endif
 
 
! Check whether dictionary exists
  out_a = m$symbol("DFS_QC1_DIC_DIR") // "/" // dict_prefix // instrument -
          // "_QC"
  if m$exist(out_a) .eq. 0 then
    !If the user has not properly installed QC-LOG system 
    !silently exit
    return -1
  endif
 
 
! Check whether cache table exists already, otherwise create it
  if qc1log_id .eq. 0 then
    mess/out 4 {procid} " "
    mess/out 4 {procid} "Creating qc1 log table {qc1log_table}"
    create/table {qc1log_table} 5 800 null
    create/column {qc1log_table} :frame   " " A100 C*100
    create/column {qc1log_table} :keyword " " A100 C*100
    create/column {qc1log_table} :value   " " A100 C*100
    create/column {qc1log_table} :comment " " A100 C*100
    create/column {qc1log_table} :dict    " " A100 C*100
    QC1LOG_ID = 1
  endif

  if frame .eq. "+" then
    mess/out 1 {procid} "Please specify a frame name (P2)!"
    return 1
  endif

  if keyword .ne. "+" .and. value .eq. "+" then
    mess/out 1 {procid} "Please specify a value (P4) for keyword {keyword}!"
    return 1
  endif

  if comment .eq. "+" .and. keyword .eq. "+" then
    append/out "Please specify either keyword (P3) and value (P4)\n"
    append/out "or comment (P5)!"
    mess/out 1 {procid}
    return 1
  endif


! Create entry into table
  {qc1log_table},:frame,@{QC1LOG_ID}   = "{frame}"
  {qc1log_table},:keyword,@{QC1LOG_ID} = "{keyword}"
  {qc1log_table},:value,@{QC1LOG_ID}   = "{value}"
  {qc1log_table},:comment,@{QC1LOG_ID} = "{comment}"
  {qc1log_table},:dict,@{QC1LOG_ID}    = "{dict}"
  

! Write keyword into concerned frame
  if m$exist(frame) .eq. 1 .and. dict .eq. "+" then
    keyword = "ESO.{keyword}"
    @p uves_qc1_log,get_type {value}
    if {q1} .eq. 1 then
      write/desc {frame} {keyword}/i/1/1 {value}
    else if {q1} .eq. 2 then
      write/desc {frame} {keyword}/d/1/1 {value}
    else
      outputi(1) = m$strlen(value)
      write/desc {frame} {keyword}/c/1/{outputi(1)} "{value}"
    endif
    if comment .ne. "+" then
      write/dhelp {frame} {keyword} "{comment}"
    endif
  endif

  !mess/out 3 {procid} "f={frame} k={keyword} v={value} c={comment} d={dict}"
  !mess/out 3 {procid} "Entry {QC1LOG_ID} added to QC1 log table {qc1log_table}."
  QC1LOG_ID = QC1LOG_ID + 1
  goto FIN


START:
  if qc1log_group .eq. 1 then
    qc1log/out 1 "stop_group" 0 0
  endif

  qc1log/out 1 "start_group" 0 0
  qc1log_group = 1

  goto FIN


STOP:
  if qc1log_group .eq. 0 then
    mess/out 2 {procid} "There's no group to stop! Ignoring command."
    goto FIN
  endif

  qc1log/out 1 "stop_group" 0 0
  qc1log_group = 0

  goto FIN


FIN:
return 0




entry get_type
! Entry to check type of a keyword
! Returns Q1=1 if type = integer
!         Q1=2 if type = float
!         Q1=3 if type = string

define/parameter P1 ? c "Enter parameter to check:"

! Check for comma (m$tstno doesn't catch this)
outputi = m$index(p1,",")
if outputi(1) .ne. 0 then
  write/keyword q1 3
  return {q1}
endif

! Check if number
outputi = m$tstno(p1)

if outputi(1) .eq. 0 then
  write/keyword q1 3
else
  outputi = m$index(p1,".")
  if outputi(1) .eq. 0 then
    write/keyword q1 1
  else
    write/keyword q1 2
  endif
endif

return {q1}




