! $Id: uves_reduce.prg,v 1.41 2002/10/25 12:29:47 amodigli Exp $
!+++ UVES_REDUCE ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_reduce.prg 
!
!.KEYWORDS      spectroscopy, echelle, science reduction
!
!.USAGE         @p (IDENT) incat [outcat] refcat [ffmet] [extract] [bmeasur] [off_slit_skw_sw]
!
!.INPUT         incat   (P1): set of input frames (ascii list)
!               refcat  (P3): set of reference frames (ascii list)
!				. DRS setup table
!				. order table
!				. background table
!				. line tables for each slit window (s,o,s)
!				. master flatfield
!				. [master bias]
!               ffmet   (P4): method to be used for flatfielding:
!				P: pixel-to-pixel flatfielding
!				E: extracted data flatfielding
!				N: no flatfielding
!               extract (P5): extraction method:
!				OPTIMAL or AVERAGE.
!		bmeasur	(P6): background measurement method to use:
!				MINIMUM: take the minimu in the box BKGRAD
!				  This may be of a big advantage if the
!				  background positions are not well defined.
!				MEDIAN:  take the median in the box BKGRAD
!		off_slit_skyw_sw	(P7): set to N,N,N: to use manually defined OFFSET,SLIT,SKYWINDOW parameters
!				set to Y,Y,Y it will automatically determine the correspondent parameters.
!               Default Y,Y,Y
!
!.OUTPUT        outcat	(P2): path + basename of output files:
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!		Q2:  the classification of the main product
!
!.DEFAULT	P2	+
!		P4	E
!		P5	OPTIMAL
!		P6	MEDIAN
!		P7	YES,YES,YES
!
!		BKGMTD=SPLINE BKGSTEP=100 BKGRAD=10,2 BKGVISU=YES
!		SKYOPT=YES MGOPT=YES MRGMTD=AVER
!		REBMTD=NONLINEAR EXTSIGMA=3
!		RESPOPT=NO CROPT=YES
!
!.PURPOSE       Perform the science data reduction of UVES observations.
!
!.ALGORITHM     The data reduction is based on commands from the ECHELLE
!		context: REDUCE/ECHELLE, EXTRACT/OPTIMAL, EXTRACT/SKY ...
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/03/09  Created.
!		1.0 1999/06/07  Extracted data flatfielding implemented.
!		1.0 1999/06/08  Pixel-to-pixel flatfielding implemented.
!		    1999/06/17  Extraction method switch implemented (P5).
!		    1999/06/28  Background measur. method parameter implem.
!		    1999/07/05	Intermediate products are official, now.
!		    1999/07/12	Sky subtraction on MERGED data!
!                   1999/07/13	Crossref command implemented.
!		    1999/07/14	DRS table controlls ECHELLE global key settings
!				OR the user before calling this procedure.
!		    1999/07/20	Get the sample size at central wavelength of
!				first order (GET/SAMPLE).
!		    1999/07/23	Check if background is already subtracted from
!				the flatfield!
!                   1999/09/15	Calculate variances.
!                   1999/09/16	Parameters for classify/product 'properly' set.
!                   1999/09/17	Flatfielding option P4=N implemented.
!                   1999/09/22	FORCE_DRS implemented.
!                   1999/09/23	Take the inverse of the std.deviation for the
!				weights during optimal merging.
!                   1999/10/05	New version of optimal merging implemented,
!				use MERGE/ECHELLE with MRGMTD=OPTIMAL instead
!				of MERGE/OPTIMAL.
!                   1999/10/13	COPY/DD removed after SUBTRACT/BACK as it is
!				already done by SUBTRACT/BACK.
!                   		Variance calc. for S1/S2 corrected: (calc_v1v2)
!                   1999/10/15	Switch CROPT to NO in case of EXTMTD=AVERAGE.
!                   1999/11/03	Use OFFSET/ECHE to get order offset, adjust
!				sky windows (DET_SKYWIND entry).
!				Reset OFFSET after sky extraction.
!                   1999/11/12	Take the sky line table as object line table
!				as soon as abs(OFFSET)>SLIT/2
!                   1999/11/15	LINTAB_OFFSET entry: get the OFFSET descriptor
!				from sky window line table.
!                   1999/11/17	DET_SKYWIND must be called AFTER the selection
!				of the order table!
!                   1999/11/18	OFFSET may be passed as parameter.
!				SKYWIND now takes account for binning.
!                   1999/11/29	bug in DET_SKYWIND entry fixed.
!                   1999/12/10	return the main product as Q2
!                   1999/12/19	OBJSET keyword added, used by necopt.exe
!				representing the offset of the object within
!				order
!                   1999/12/20	Disabled the final statistics calculation for
!				some products (see CLASSIFY/PROD)
!
!.COMMENT	(1) RESTRICTION: The global keywords:
!			EXTOPT, BKGOPT, REBOPT, FFOPT, MGOPT
!			SKYWIND, NSKY, [OFFSET]
!		are always set by the program.
!
!		(2) The SAMPLE keyword will be set to the MINIMUM dispersion
!		which is found at the lowest order (blue part of the echelle)
!		as soon as it is previously set to undefined (SAMPLE=+++),
!		otherwise the specified value will be used!
!
!		(3) WARNING: During testing phase RON and GAIN will be set to
!		a 'proper' constant as soon as 0.0 are read from the header.
!		This is ONLY for testing functionality and should be REMOVED
!		for the release !!!
!
!		(4) WARNING: Due to the fact that the wavelength calibration
!		in WAVECA/UVES is done on the standard setting (offset=0) the
!		RESAMPLING may be not absolutely right as soon as DET_SKYWIND
!		detects an offset of the orders !!!
!		This has to be checked!
!		
!==============================================================================
! Who  when      Where       Why
! AM   10/05/01  main        to issue an error in case bad value of DATA
!                            return -1 for ERR4
! AM   31/05/01  all proc    DFS code rev step 1: mainly pseudocode 
!                                                 and a few more comments 
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.41 $
! $Log: uves_reduce.prg,v $
! Revision 1.41  2002/10/25 12:29:47  amodigli
! Fixed bug on graphics
!
! Revision 1.40  2002/10/25 10:01:08  amodigli
! Fixed problem with graphycs display
!
! Revision 1.39  2002/06/03 16:19:53  amodigli
! Some cosmetic changes: WARNING->INFO
!
! Revision 1.38  2002/05/29 06:05:37  amodigli
! information aboaut wav sampling from WARNING to INFO
!
! Revision 1.37  2002/02/11 11:00:55  amodigli
! Renamed sky_opt ==> opt_sky to properly use
! the variance produced from necopt.for
!
! Revision 1.36  2002/02/08 16:17:52  amodigli
! Factorized products to short length of comp/ima
! and make code more portable and fast(?)
!
! Revision 1.35  2002/01/30 10:37:54  amodigli
! we delete opt_sky_x(flat_field) as it has
! no meaning
!
! Revision 1.34  2002/01/23 14:40:42  amodigli
! Fixed some misbehaviour with opt_sky_ ima
! reduction.
!
! Revision 1.33  2001/12/21 13:39:58  amodigli
! Added sky image optimally extracted,
! fixed a portability problem to OSF with **,
! added support of MASTER_DFLAT, MASTER_IFLAT
!
! Revision 1.32  2001/10/04 12:01:01  amodigli
! Added raw frame descriptors to CRMASK
!
! Revision 1.31  2001/10/02 15:57:57  amodigli
! Fixed bug on COMP/BARY (now we use sessagesimal
! units) and copied raw frame on raw_frm_{p1}
! As delivered to DFO
!
! Revision 1.30  2001/08/08 13:14:42  amodigli
! Added again prefix ESO to QC keywords
!
! Revision 1.29  2001/07/14 06:54:42  amodigli
! Corrected the logic of issuing messages for
! user defined OBJSET value
!
! Revision 1.28  2001/07/13 15:15:04  amodigli
! Added info that OBJSET should be less than
! SLIT/2
!
! Revision 1.27  2001/07/12 15:28:28  amodigli
! Now RON and CONAD are PORT number dependent
! Allowed to set user defined value of OBJSET
! to properly support optimal extraction of
! multi-object sources
!
! Revision 1.26  2001/07/10 09:48:47  amodigli
! to issue an error in case bad value of DATA
! return -1 for ERR4
!
! Revision 1.25  2001/07/04 15:53:35  amodigli
! Taking in count of sky slit width to weight
! their contribute to merged sky
!
! Revision 1.24  2001/06/07 07:59:02  amodigli
! Documented Meaning of DWIN
! Added a missing dummy product in case nsky=1
!
! Revision 1.23  2001/05/31 08:23:34  amodigli
! Was missing the Who/When in some entries
!
! Revision 1.22  2001/05/31 07:52:00  amodigli
! DFS Code review step1: mainly pseudocode
! and a few more comments.
!
! Revision 1.21  2001/05/18 06:01:27  amodigli
! Added dummy product in case nsky=1 to keep
! order numbering and number of pipe products
!
! Revision 1.20  2001/05/04 14:05:05  amodigli
! New bkg extraction par
!
! Revision 1.19  2001/01/25 16:13:19  amodigli
! Added checks on skywind.
! Ready for Paranal
!
! Revision 1.18  2001/01/25 08:07:17  amodigli
! Cleaned an inappropriate comment in .TODO
!
! Revision 1.17  2001/01/24 17:58:29  amodigli
! It should extract BOTH sky window with
! MEDIAN (better in cosmic removal respect
! to AVERAGE) method.
!
! Revision 1.16  2001/01/19 12:33:36  amodigli
! Fixed a typo in the DO classification
! for order and xorder tables
!
! Revision 1.15  2001/01/18 14:37:38  amodigli
! As delivered to DFO
!
! Revision 1.14  2001/01/18 14:19:28  amodigli
! As delivered to DFO
!
! Revision 1.13  2001/01/18 11:56:43  amodigli
! Version with P7 splitted in 3: Y,Y,Y
! and some w/o to monitor behaviour
!
! Revision 1.12  2001/01/08 12:57:24  amodigli
! Renamed debug.tbl order_trace.tbl
! debug2.tbl xorder_trace.tbl
!
! Revision 1.11  2001/01/04 10:39:32  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!Performs science reduction of point-like sources
!------------------------------------------------------------------------------
! Pseudocode:
!  initial definitions
!  if p1 is catalog 
!     do loop over entries 
!        CONFIG: read/set instrument configuration and reference catalog frames
!        REDUCE: do science reduction
!        DO_CLASS: classify data products
!     enddo
!  else
!        CONFIG
!        REDUCE
!        DO_CLASS
!  endif
!==============================================================================
crossref input catalog refer ffmet extract bmeasur offset
!
define/param p1 ? c "Enter name of input catalog:"
define/param p2 + c "Enter name of output catalog:"
define/param p3 ? c "Enter name of reference catalog:"
define/param p4 ? c "Enter flatfielding method (P,E,...):"
define/param p5 ? c "Enter extraction method (AVERAGED, OPTIMAL):"
define/param p6 MEDIAN c "Enter background measur. method ([MEDIAN], MINIMUM):"
define/param p7 Y,Y,Y c "Automatically set offset [Y]/N, slit [Y]/N, skywindows [Y]/N :"

define/local pid/c/1/10 "reduce" ? +lower_levels
!
! Global variables
!
define/local sky/c/1/100 " " ? +lower_levels
define/local sky1/c/1/100 " " ? +lower_levels
define/local sky2/c/1/100 " " ? +lower_levels
define/local correct1/c/1/100 " " ? +lower_levels
define/local correct2/c/1/100 " " ? +lower_levels
define/local modext/c/1/25 " " ALL +lower_levels	!set by ENTRY CONFIG
define/local LINTAB_1/c/1/100 " " ALL +lower_levels
define/local LINTAB_2/c/1/100 " " ALL +lower_levels
define/local FFMTD/c/1/10 " " ALL +lower_levels
define/local uves_debug/c/1/80 " " ALL +lower_levels
define/local uves_plot_debug/c/1/80 " " ALL +lower_levels

! This setting is for internal use: 
! allows to add an extra table (xtrace_order_x.tbl, X=BLUE,REDL,REDU) 
! to the pipe data producs. Usually is not set
uves_debug = m$symbol("UVES_DEBUG") 
uves_debug = m$upper(uves_debug)
 
! This setting is for internal use:
! If set to YES allows to automatically display
! multi plots for order trace on pos and fwhm
! to do QC
uves_plot_debug = m$symbol("UVES_PLOT_DEBUG") 
uves_plot_debug = m$upper(uves_plot_debug)
 
!
! Local variables
!
define/local ifrm/c/1/100 " " all
define/local ext/c/1/4 " " all
define/local num/i/1/1 0
define/local catal/i/1/1 0
define/local main_prod/c/1/25 " " all


set/graph default
set/graph pmode=1
if NICE_CREA .eq. "Y" assign/disp d,1 >Null	!Select display
						!Check output catalog option
if m$exist(p2) .eq. 0 .AND. p2(1:1) .ne. "+" -
  create/icat {p2} NULL {UVES_IDSTRNG} >Null
						!Check input type.
parse/path {p1}					!Get file extension.
ext = m$upper(q3)
if ext(1:4) .eq. ".CAT" then			!Check if we got a catalog.
  show/icat {p1} >Null
  num = outputi(1)				!Check number of entries.
  if num .lt. 1 goto ERR1			!No entries in the catalog.
  goto CATALOG
else if ext(1:4) .eq. ".BDF" then
  ifrm = p1
  goto SINGLE
else
  goto ERR2					!Bad P1 specification
endif


set/format I1					!Now loop through all
CATALOG:					!input files.
  store/frame ifrm {p1} 1 FINI
SINGLE:
  write/out " "
  mess/out 3 {pid} "Do the science reduction for {ifrm}..."

  @p uves_reduce,CONFIG {ifrm} {p3} {p4} {p5} {p6} {p7}
  if {q1} .lt. 0 goto CHECK

  @p uves_reduce,REDUCE {ifrm}
  if {q1} .lt. 0 goto CHECK

  mess/out 3 {pid} "Classify products ..."
  @p uves_reduce,DO_CLASS {ifrm} {p2} {FFMTD} {modext}
  if m$tstno(q1) .eq. 1 then  !q1 is a number only if problems on data frm are present
    if {q1} .eq. -1 goto ERR4 !in such a case we check if the number is indeed -1 (as in case of problems) 
  endif                       !and we exit issuing an error message
  main_prod = q1

  if NICE_CREA .eq. "Y" then
    assign/graph g,3 >Null
    modify/graph W >Null
    plot/uves r_{ifrm} + + "Reduced science data (UVES): r_{ifrm}" r_{ifrm}.ps
  endif

  mess/out 4 {pid} "Save session."
  save/echelle {modext(2:)} >Null		!save session for debug purpose

  mess/out 3 {pid} "Science reduction done, output: r_{ifrm}."

CHECK:
  if ext(1:4) .eq. ".BDF" goto FINI		!single frame -> finished.
  goto CATALOG
FINI:
return 0 {main_prod}

						!Error handling
ERR1:
  mess/out 1 {pid} "Empty input catalog !"
  return -1
ERR2:
  mess/out 1 {pid} "You have to specify full name of input frame (.cat, .bdf)!"
  return -1
ERR3:
  mess/out 1 {pid} "Input file `{ifrm}` does not exist !"
  return -1
ERR4:
  mess/out 1 {pid} "Data with bad value! Abort."
  return -1


!=============================================================================
!=============================================================================

!=== SUBROUTINES =============================================================
ENTRY CONFIG
define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter name of reference image catalog:"
define/param p3 ? c "Enter flatfielding method (P,E,...):"
define/param p4 ? c "Enter extraction method (AVERAGED, OPTIMAL):"
define/param p5 ? c "Enter background measur. method (MEDIAN, MINIMUM):"
define/param p6 ? c "Automatically set offset [Y]/N, slit [Y]/N, skywindows [Y]/N :"
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   31/05/01  this entry    DFS code rev step 1:
!-----------------------------------------------------------------------------
! Configures the instrument: define part of the extraction parameters and set
! all the master calibration frames.
! See also pseudocode.
!-----------------------------------------------------------------------------
!pseudocode:
!  if FORCE_DRS="Y" read DRS setup table
!  read order table
!  read bkg table
!  determines skywind parameters: 
!   object offset, object extraction slit, skywind(1-4) (sky1-2 extraction slit)
!  read lower (sky1)=LINE_1, upper (sky2)=LINE_2, median (obj)=LINE tables
!       if obj offset > +offset(LINTAB_2)/2 LINTAB=LINTAB_2 (obj in upper slit part)
!  else if obj offset < -offset(LINTAB_2)/2 LINTAB=LINTAB_1 (obj in lower slit part)
!  else LINTAB is the right one (obj at center slit)
!  read bias frame, if raw frame not bias subtracted => subtract bias
!  if bkgr is not subtracted subtracts background display it
!  read master flat
!  if master flat not bkgr subtracted subtracts from it bkgr
!  if FFMTD="P" divides input frames by FF and sets FFOPT to NO
!  else set FFOPT to YES (will flat field the frame later on during extraction)
!  defines some names for extracted sky1,2 and FFs
!  if EXTMTD="A" set cosmic ray rejection to NO
!  defines resampling constant to 2/3 lower sampling step of line table
!  displays info on extraction 
!=============================================================================


define/local i1/i/1/1 0
define/local ron/r/1/1 0.0
define/local gain/r/1/1 0.0
define/local do_class/c/1/25 " " all

config/instr {p1}
modext = q2					!set global keyword MODEXT

search/icat {p2} {DRSRC_ID({PATHID})} NB
if m$strlen(out_a) .eq. 0 then
  append/out "No appropriate DRS setup table found!"
  if FORCE_DRS(1:1) .eq. "Y" goto SKIP
  mess/out 2 {pid}
  mess/out 3 {pid} "Use global keyword settings (SHOW/ECHELLE)!"
else
  savini/echelle {out_a} READ
endif

search/icat {p2} {ORDER_ID({PATHID})} NB	!order table
if m$strlen(out_a) .eq. 0 then
  mess/out 1 {pid} "No appropriate order table found!"
  goto SKIP
else
  set/echelle ORDTAB={out_a} EXTOPT=YES
endif

search/icat {p2} {BACK_ID({PATHID})} NB		!interorder background table
if m$strlen(out_a) .eq. 0 then
  set/echelle BKGOPT=NO
  mess/out 1 {pid} "No appropriate background table found!"
  goto SKIP
else
  set/echelle BAKTAB={out_a} BKGOPT=YES		!BKGOPT never used in echelle!
!  @p uves_updtbl R {BAKTAB}			!Update STEP descriptor
!  if {q1} .lt. 0 goto SKIP
endif

@p uves_reduce,DET_SKYWIND {p1} 1 {p6} {p4}	!determine order OFFSET
if {q1} .lt. 0 goto SKIP			!and SKYWIND keyword

search/icat {p2} {LINTAB_ID({PATHID})}1 NB	!line table
if m$strlen(out_a) .eq. 0 then			!of 1. sky-window
  set/echelle REBOPT=NO
  append/out "No appropriate line table (sky1) found!\n"
  goto SKIP
else
  LINTAB_1 = out_a
endif

search/icat {p2} {LINTAB_ID({PATHID})}3 NB	!line table
if m$strlen(out_a) .eq. 0 then			!of 2. sky-window
  set/echelle REBOPT=NO
  append/out "No appropriate line table (sky2) found!\n"
  goto SKIP
else
  LINTAB_2 = out_a
endif

search/icat {p2} {LINTAB_ID({PATHID})}2 NB	!line table
if m$strlen(out_a) .eq. 0 then			!of object-window
  set/echelle REBOPT=NO
  append/out "No appropriate line table found!\n"
  goto SKIP
else
  @p uves_reduce,LINTAB_OFFSET {LINTAB_2}	!Get OFFSET of sky window line
  outputr(1) = outputr(1) / 2.0			!table, return as outputr(1)
  if OFFSET .gt. outputr(1) then		!Choose appropriate line table
    set/echelle LINTAB={LINTAB_2} REBOPT=YES	!.. object shifted in upper sky
  else if OFFSET .lt. -{outputr(1)} then
    set/echelle LINTAB={LINTAB_1} REBOPT=YES	!.. object shifted in lower sky
  else
    set/echelle LINTAB={out_a} REBOPT=YES
  endif
  echord(2) = m$value({LINTAB},ORDER(2))	!Set the echelle order keyword:
  echord(3) = m$value({LINTAB},ORDER(1))	!1st and last absolut order.
  echord(6) = echord(3) + 1			!conversion constant: rel->abs.
endif

!saves raw frame on a temporary file
!This was asked from DFO to speed up its QC checks
-copy {p1} raw_frm_{p1}
set/echelle FLAT=+++				!Search for calibration frames:
search/icat {p2} {MBIAS_ID({PATHID})} NB	!MASTER_BIAS
if m$strlen(out_a) .eq. 0 then
  mess/out 2 {pid} "No appropriate master bias found!\nNo bias subtraction!"
else
  if m$existd(p1,"BIAS_SUBTR") .ne. 0 then
    mess/out 3 {pid} "Bias already subtracted !"
  else
    mess/out 3 {pid} "Subtract bias ..."
    compute/image &b = 1.0*{p1} - {out_a}
    -rename middummb.bdf {p1}
    write/desc {p1} BIAS_SUBTR/c/1/3 YES	!indicates BIAS subtraction
  endif						
endif

select/grap {pathid}				!select graphics window

clear/channel overlay
if m$existd(p1,"BACKGROUND") .eq. 0 then	!BACKGROUND descrip. indicates
  mess/out 3 {pid} "Subtract background from `{p1}' ..."
  set/grap yaxis=-50,450


!am: beging new bkg extraction way.
!Backroud subtraction: method SPLINE with BKGSMO=0,BKGSTEP depending 
! detector X size and binning, BKGRAD=2,radY (depending on binning)
! BKGDEG=1. After we do a post smoothing of the resulting bkg image
! with smoothing constant depending on detector sizes and binning
! and wav setting (to prevent from structure noticeable for R860).
!
   -copy {p1} pre_bg_sub_{p1}
   subtract/back {p1} tmp_bg_{p1} &b  P6={p5} >Null !Subtract background f. science
   define/local sizeX/i/1/1  0
   define/local sizeY/i/1/1  0

   sizeX = m$value({p1},NPIX(1))
   sizeY = m$value({p1},NPIX(2))
   !w/o "sizeX={sizeX}"
   !w/o "sizeY={sizeY}"
   define/local radSmoX/i/1/1 25
   define/local radSmoY/i/1/1 10
   radSmoX = 300 * {sizeX}/4096    
   define/local wlen/i/1/1 0
   wlen = m$value({p1},{h_cwlen({PATHID})})
   if wlen .eq. 860 then
     radSmoY = 500 * {sizeY}/2048  
   else
     radSmoY = 200 * {sizeY}/2048  
   endif
   mess/out 3 {pid} "radSmoX={radSmoX} radSmoY={radSmoY}"
   filter/smooth tmp_bg_{p1}  bg_{p1} {radSmoX},{radSmoY} >Null
   compute/image {p1} = pre_bg_sub_{p1} - bg_{p1}
!am: end new code

!=============================================
! 17/3/2000 AM: Added to display resulting Bkg
!=============================================

  mess/out 3 {pid} "INFO: Display the resulting Background image"
  clear/channel overlay
  load bg_{p1}        
!==============================================

  -rename middummb.bdf {p1}			!and overwrite input frame.
  if BKGVISU(1:1) .eq. "Y" then
    if m$exist("bg_{p1}") .eq. 1 then
      copy/grap postscript
      -rename postscript.ps bg_{p1}.ps
      assign/disp 3
      clear/channel overlay
      load/ima bg_{p1} sca=-8,-5 cuts=f,3sig
    endif
  endif
else						!the background subtraction!
  mess/out 2 {pid} "Background from `{p1}' already subtracted !"
endif

assign/disp 1
clear/channel overlay

!1st try in case the users specify a standard Flat
search/icat {p2} {MFLAT_ID({PATHID})} NB	!MASTER_FLAT

!2nd try again in case the users specify a Iodine Cell Flat
if m$strlen(out_a) .eq. 0 then
search/icat {p2} {MIFLAT_ID({PATHID})} NB	!MASTER_IFLAT
endif

!3rd try again in case the users specify a Deuterium Flat
if m$strlen(out_a) .eq. 0 then
search/icat {p2} {MDFLAT_ID({PATHID})} NB	!MASTER_DFLAT
endif

!finally...do final check
if m$strlen(out_a) .eq. 0 .OR. p3(1:1) .eq. "N" then
  if p3(1:1) .eq. "N" then
    mess/out 2 {pid} "Flatfield disabled!"
  else
    mess/out 2 {pid} "No appropriate flatfield found!\nNo flatfielding !"
  endif
  set/echelle FFOPT=NO CORRECT="no_ff"
  FFMTD = "NO"
else

!am: 3 March 2000
!Check if really necessary to do Background subtraction from Flat Filed
!
  set/echelle FLAT={out_a}
  if m$existd(flat,"BACKGROUND") .gt. 0 then
    mess/out 3 {pid} "Background from flatfield already subtracted!"
    set/echelle CORRECT={FLAT}
  else
    mess/out 3 {pid} "Subtract background from flatfield ..."
    set/echelle CORRECT=b_{flat} BLAZE=z_{flat}

!am: old code
!    subtract/back {flat} bg{modext} {correct} P6={p5} !>Null
!am: new code 06/04/01
      -copy {flat} pre_bg_sub_{flat}
      subtract/back {flat} tmp_bg_{flat} &b >Null
      define/local sizeXff/i/1/1  0
      define/local sizeYff/i/1/1  0
      sizeXff = m$value({flat},NPIX(1))
      sizeYff = m$value({flat},NPIX(2))
      define/local radSmoXff/i/1/1 25
      define/local radSmoYff/i/1/1 10
      do_class = "{{flat},{UVES_IDSTRNG}}"
      if "{do_class}" .eq. "FLAT_BLUE" then
         radSmoX = 25 * {sizeX}/4096
      else
         radSmoX = 50 * {sizeX}/4096
      endif
      if wlen .eq. 860 then
         radSmoYff = 300 * {sizeY}/2048
      else
         radSmoYff = 100 * {sizeY}/2048
      endif
      mess/out 3 {pid} "radSmoXff={radSmoXff} radSmoYff={radSmoYff}"
      filter/smooth tmp_bg_{p1}  bg{modext} {radSmoXff},{radSmoYff} >Null
      compute/image {correct} = pre_bg_sub_{flat} - bg{modext}
!am: end new code     

    
!    copy/dd {flat} *,3 {correct}		!subtract background f. {flat}
    delete/desc {correct} {h_origname} N	!and store in {correct}.
    if BKGVISU(1:1) .eq. "Y" then	
      if m$exist("bg_{p1}") .eq. 1 then
	copy/grap postscript
	-rename postscript.ps bg{modext}.ps
	assign/disp 2
	clear/channel overlay
	load/ima {correct} sca=-8,-5 cuts=f,3sig
	assign/disp 3
	clear/channel overlay
	load/ima bg{modext} sca=-8,-5 cuts=f,3sig
      endif
      assign/disp 1
      clear/channel overlay
      assign/grap g,3
    endif
  endif

  if p3(1:1) .eq. "P" then
    if m$existd(p1,"FLATMETHOD") .ne. 0 then
      mess/out 3 {pid} "Flatfielding already done."
    else
      mess/out 3 {pid} "Do the pixel-to-pixel flatfielding."
      statistic/ima {correct} ? ? ? SN		!do the normalization
      compute/ima {p1} = {p1} / {correct} * {outputr(3)}
      write/desc {p1} CUNIT/c/1/16 "RELATIVE FLUX"
      write/desc {p1} FLATMETHOD/c/1/11 PIXEL
    endif
    set/echelle FFOPT=DONE
    FFMTD = "PIXEL"
  else
    mess/out 3 {pid} "Do the flatfielding during extraction."
    write/desc {flat} FLATMETHOD/c/1/11 EXTRACT
    set/echelle FFOPT=YES
    FFMTD = "EXTRACT"
  endif
endif

define/local pn/i/1/1 0
set/format i1       
pn = {uves_portid({PATHID})}
set/format 
ron = m$value({p1},{h_ron({pn})})			!port number dependent!
if ron .le. 0.0 then
  ron = 5.0
  mess/out 2 {pid} "Bad RON value detected, take default: {ron} !"
endif
gain = m$value({p1},{h_confac({pn})})		!port number debendent!
if gain .le. 0.0 then				![GAIN]=ADU/e-, [CONAD]=e-/ADU
  gain = 2.1
  mess/out 2 {pid} "Bad GAIN value detected, take default: {gain} !"
endif	  					!Here the keyword GAIN is
						!meant to be the inverse gain,
						!i.e. CONAD !!!
						!
sky  = "_sky{modext}.bdf"			!Rootname of extracted sky
sky1 = "_sky{modext}1.bdf"			!Rootname of extracted sky (1)
sky2 = "_sky{modext}2.bdf"			!Rootname of extracted sky (2)
correct1 = "b1_{flat}"				!Rootnames of correction FFs
correct2 = "b2_{flat}"				!in the sky windows 1 and 2.
set/echelle EXTMTD={p4} GAIN={gain} RON={ron}
if EXTMTD(1:1) .eq. "A" then
  EXTMTD = "AVERAGE"				!Average extraction cannot
  CROPT = "NO "					!do a cosmic ray rejection.
else if EXTMTD(1:1) .eq. "O" then
  EXTMTD = "OPTIMAL"
endif

if EXTSIGMA .lt. 0 set/echelle CROPT=NO
if CROPT(1:1) .eq. "N" EXTSIGMA = -1




!=======================================
!Begin to define Sample
!=======================================
set/format F7.5

!17/03/200 AM:
!Commented out as GET/SAMPLE is not reliable enough
!
!if SAMPLE(1:3) .eq. "+++" then			!Get the wavelength binsize
!  get/sample 					!at the centre of the first
!  SAMPLE = "{outputd(1)}"			!order.
!else
!  mess/out 2 {pid} "SAMPLE = {SAMPLE} read from DRS-table !"
!endif
!if {SAMPLE} .lt. 0 then
!   mess/out 2 {pid} "Warning: The GET/SAMPLE gives SAMPLE = {SAMPLE} "  
!   SAMPLE = "{{LINTAB},PIXEL(1)}" 
!   mess/out 2 {pid} "Warning: SAMPLE set from LINTAB-PIXEL(1) to {SAMPLE} !"
!endif
!
!We use this setting instead:
!
define/local rsample/R/1/1 0.
RSAMPLE = 2.*{{LINTAB},PIXEL(1)}/3.
SET/ECHELLE SAMPLE = {RSAMPLE}

mess/out 3 {pid} "SAMPLE set from 2/3*(LINTAB-PIXEL(1)) to {SAMPLE}"

!=======================================
!End Old Way to define SAMPLE
!=======================================




set/format

image/section {p1} Y				!specify scan region
scan/echelle {p1} {outputi(2)},{outputi(4)} >Null

set/format F2.1
append/out "Reduction Process Steps:\n"
append/out "(1) Background subtraction: YES\n"
append/out "(2) Cosmic ray rejection:   {CROPT}, ({EXTSIGMA}*sigma)\n"
append/out "(3) Flatfield correction:   {FFOPT}, {correct}\n"
append/out "(4) Order extraction:       {EXTOPT}, (method: {EXTMTD})\n"
append/out "(5) Sky subtraction:        {SKYOPT}\n"
append/out "    Object-window:          {slit} centered at {OFFSET}\n"
if nsky .eq. 0 then
  append/out "    IS inserted:          No Sky subtraction! \n"
elseif nsky .gt. 1 then
  append/out "    Sky-windows:            {skywind(1)},{skywind(2)};{skywind(3)},{skywind(4)}\n"
else
  append/out "    Sky-window:             {skywind(1)},{skywind(2)}\n"
endif
append/out "(6) Resample:               {REBOPT}, (method: {REBMTD})\n"
append/out "    Wavelength sampling:    {sample} Angstroem\n"
append/out "(7) Response correction:    {RESPOPT}\n"
append/out "(8) Order merging:          {MGOPT}, (method: {MRGMTD})\n\n"
append/out "Use CCD-region:             [{scan(1)},{scan(2)}] ({CCDBIN(1)}x{CCDBIN(2)})"
mess/out 3 {pid}

set/format
return 0
SKIP:
  append/out "--> skip input frame: `{p1}'"
  mess/out 1 {pid}
  return -1
!=============================================================================
!==============================================================================
ENTRY REDUCE
define/param p1 ? c "Enter input frame:"

define/local pf/c/1/4 " " ALL
define/local rwbin/c/1/100 " " ALL
define/local exptime/d/1/1 1.0
define/local skyw/i/1/1 1
define/local slicer_name/c/1/25 " "
define/local time/i/1/1 0
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   31/05/01  this entry    DFS code rev step 1:
!----------------------------------------------------------------------------- 
!.............................................................................
!Do the real extraction: extracts, flat fields (if appropriate), resamples,
! merges the object and if appropriate do the same with the sky and subtract
! from merged object the merged sky.
! see also pseudocode below
!-----------------------------------------------------------------------------
!pseudocode:
!  if FFMTD="E" FFOPT="NO" (no flat fielding)
!  extract the object with the proper method (Average or Optimal)
!                =>get xb_frm (extracted bkgr subtracted frame)
!  Determines Earth to Helio centric cohordinate corr. and store it in header
!  If EXTMTD="O" and If plot debug option is set plots the pos/fwhm order traces
!  displays extracted object
!  if EXTMTD="O" renames and resample weight image=>rwbin
!  if FFMTD="E" extracts the FF
!     calculates variances
!     flat field the extracted object=>get fxb_frm
!     rebin fxb and xb => wfxb_frm, w_xb_frm
!     calculates error associated to all operations=>err_frm
!     calculates weights
!     merges the orders using optimal merging with calculated weights=>m_frm
!     if SKYMTD="Y" extracts FF in two sky windows:
!        extract/sky on first FF window, renormlizes it, calculates variance, 
!        rebins it (wavelength resampling)
!        if nsky=2 do the same as before for second sky extraction window
!     endif
!  else if FFMTD="P" (pix-to-pix flat fielding)=> Flat Fielding was done before
!     rebin xb => wfxb_frm
!     if EXTMTD="A" calculates variances 
!     else (it has done during extraction) renames it
!     calculates error image
!     rebin variance, error
!     calculates weight
!     merges wxb=>m_frm
!  else => FFMTD="NO"=>No flat Fielding at all
!     calculates variance
!     calculates error
!     rebin error, variance
!     calculates weights
!     merge orders => m_frm
!  endif
!  if SKYOPT="Y"
!     extracts the two sky windows
!     if FFMTD="E"
!        normalizes the sky1 to its FF (1)
!        if nsky=2 normalizes the sky2 to its FF (2)
!        rebin sky1
!        calculates variance of sky1
!        calculates error of sky1
!        if nsky=2 
!           rebin sky2
!           calculates variance of sky2
!           calculates error of sky2
!        endif
!     else if FFMTD="P" (flat field was done before)
!        rebin sky1
!        calculates variance of sky1
!        calculates error of sky1
!        if nsky=2 
!           rebin sky2
!           calculates variance of sky2
!           calculates error of sky2
!        endif
!     else
!        rebin sky1
!        calculates variance of sky1
!        calculates error of sky1
!        if ssky=2 
!           rebin sky2
!           calculates variance of sky2
!           calculates error of sky2
!        endif
!     endif
!     calculates weight for sky1
!     merges sky1
!     calculates weight for sky2
!     merges sky2
!     if EXTMTD="O"
!        merge weights rwbin=>mrwbin
!        calculates merged sky,variance
!     else
!        calculates merged sky,variance
!     endif
!     rebins xb_sky1 and if nsky=2 xb_sky2
!     calculates r_frm=m_frm-m_sky
!     calculates errm_frm
!  else  ==> No sky subtraction (it was done during opt extraction)
!     calculates r_frm=m_frm
!     calculates errm_frm
!  endif
!  END!!
!=============================================================================



exptime = m$value({p1},O_TIME(7))

mess/out 3 {pid} "Prepare the  OBJECT  window:"
mess/out 3 {pid} "Do the {EXTMTD} extraction ..."
if FFMTD(1:1) .EQ. "E" FFOPT = "NO"
						!Do the order extraction:
						
!time = m$secs(); mess/out 3 {pid} "start={time}"

extract/echelle {p1} xb_{p1} !{SESSOUTV}		!OPTIMAL or AVERAGE (EXTMTD)
!time = m$secs()-time; mess/out 3 {pid} "time={time}"

!compute/imag xb_{p1} = xb_{p1} / {exptime}	!and a time normalization
copy/dd {p1} *,3 xb_{p1}			!??? shall we reset O_TIME(7)
 
define/local qc_obs_time/d/1/1 1.0
define/local qc_ra/d/1/1 1.0
define/local qc_dec/d/1/1 1.0
define/local qc_geolat/d/1/1 1.0
define/local qc_geolon/d/1/1 1.0

!Earth to Helio-centric coordinate correction
set/format F12.8
qc_ra       = m$value({p1},O_POS(1))
qc_dec      = m$value({p1},O_POS(2))
qc_geolat   = m$value({p1},{h_geolat})
qc_geolon   = m$value({p1},{h_geolon})
qc_obs_time = m$value({p1},O_TIME(7))  !using an image as input it take the
                                       !date from the descriptor O_TIME(1,2,3)
                                       !and the UT from O_TIME(5)

define/local hms_ra/c/1/80     " " all
define/local dms_dec/c/1/80    " " all
define/local dms_geolat/c/1/80 " " all
define/local dms_geolon/c/1/80 " " all

@p uves_reduce,deg2hms {qc_ra}
hms_ra = q1
@p uves_reduce,deg2dms {qc_dec}
dms_dec = q1
@p uves_reduce,deg2dms {qc_geolat}
dms_geolat = q1
@p uves_reduce,deg2dms {qc_geolon}
dms_geolon = q1


 
COMPUTE/BARYCORR {p1} {hms_ra} {dms_dec} {dms_geolon} {dms_geolat} >Null
write/descriptor xb_{p1} ESO.QC.VRAD.BARYCOR/d/1/1 {outputr(1)}
write/descriptor xb_{p1} ESO.QC.VRAD.HELICOR/d/1/1 {outputr(2)}

set/format
 
define/local frm_id/c/1/80 " "
 
if EXTMTD(1:1) .eq. "O" then
    copy/dd xb_{p1} *,3 order_trace.tbl
    copy/dd xb_{p1} *,3 xorder_trace.tbl
    if NICE_CREA .eq. "Y" then 
        if uves_plot_debug .eq. "YES" then
            assign/grap g,3
            MPLOT/CHUN "order_trace.tbl" 1.5 pos
            MPLOT/CHUN "order_trace.tbl" 1.5 fwhm
        endif
    endif
    -rename order_trace.tbl order_trace{modext}.tbl
    -rename xorder_trace.tbl xorder_trace{modext}.tbl
endif
 
 
 
 
!
clear/channel overlay
load/ima xb_{p1} sca=-7,10
!
! Now we got the optimal extracted object as well as the weights used for
! the optimal extraction process (valid for EXTMTD=OPTIMAL).
! Next we have to extract the lower and upper sky-windows in simple AVERAGE
! mode. Rebin these extracted sky-windows in the same way as the object.
! Due to fringes, in the red wavelength range we have to flatfield each sky
! window, separately before averaging and rebinning the extracted sky-windows.
! Finally, subtract the rebinned, flatfielded and weighted sky from the
! rebinned, flatfielded and weighted object.
!
set/echelle FFOPT=NO				!This has to be set otherwise
						!we always get the warning when
						!doing the rebinning!
						
!=================================================================
! AMO 26/07/00 (from suggestion of AKA)
! This check on Image Slicer is now done in entry DET_SKYWIND	
!=================================================================			
!slicer_name = m$value({p1},{h_slicer_name}) 					
!if slicer_name(1:4) .eq. "FREE" then
!  mess/out 3 {pid} "No image slicer inserted" 
!else 
!  mess/out 3 {pid} "Number of slices: {{p1},{h_slicer_no}}"
!  set/echelle SKYOPT=NO				!Do not perform Sky subtraction 
!endif				
!=================================================================
						

if EXTMTD(1:1) .eq. "O" then
  -rename wbin.bdf wbin{modext}.bdf		!The weights of the OPTIMAL
  copy/dd {p1} *,3 wbin{modext}			!extract. have to be resampled
  define/local index/i/1/1 0
  define/local crmask/c/1/80 " " all
  index = m$index(p1,".bdf")
  index = index -1
  crmask = "crm_{p1(:{index})}.tbl" 
  copy/dd {p1} *,3 {crmask}
  copy/dd {p1} *,3 "opt_sky_xb_{p1}"  
  delete/desc wbin{modext} {h_origname} N	!Remove original filename desc.
  rwbin = "r_wbin{modext}.bdf"			!Name of the resampled weights
  rebin/echelle wbin{modext}.bdf {rwbin} {SAMPLE} NONL {LINTAB} {SESSOUTV}
						!
  set/echelle SKYOPT=NO				!Sky has been subtracted
						!during optimal extraction.
						
!  AKA var frame has NaNs which stops the pipeline
!  workaround var set to one ... ?!
!  stat/ima var_xb_{p1}
!  comp/ima var_xb_{p1} = 1 
  						
endif

if FFMTD(1:1) .eq. "E" then			!flatfielding on the extracted
  if EXTMTD(1:1) .eq. "A" then			!orders

    mess/out 3 {pid} "EXTRACT averaged flatfield in the obj.window ..."
    write/desc {correct} FLATMETHOD/c/1/16 AVERAGED_EXTRACT
    extract/echelle {correct} x{correct}  {SESSOUTV}
						!compute the variances
    mess/out 3 {pid} "Compute VARIANCES of extracted flatfield ..."
    @p uves_reduce,CALC_VARI x{correct} var_x{correct} p4={slit}

  else						!OPTIMAL extraction

    mess/out 3 {pid} "EXTRACT weighted flatfield in the obj.window ..."
    write/desc {correct} FLATMETHOD/c/1/16 WEIGHTED_EXTRACT
    extract/optimal -
      {correct} x{correct} {slit},{echord(4)},{echord(5)} p7=NO {SESSOUTV}
      -delete opt_sky_x{correct}  
				      		!variances are automatically
  endif						!determined

  copy/dd {correct} *,3 x{correct}		!copy the ESO-header.
  delete/desc x{correct} {h_origname} N		!Remove original filename desc.

  statistic/imag x{correct} ? ? ? SN
  compute/imag x{correct} = x{correct} / {outputr(3)}
  compute/imag var_x{correct} = var_x{correct} / ({outputr(3)}*{outputr(3)})

  mess/out 3 {pid} "FLATFIELD and RESAMPLE the extracted object ..."
  compute/ima fxb_{p1} = xb_{p1} / x{correct} >Null
  write/desc fxb_{p1} CUNIT/c/1/16 "RELATIVE FLUX"

  rebin/echelle fxb_{p1} wfxb_{p1}  {SAMPLE} NONL {LINTAB} {SESSOUTV}
  write/desc wfxb_{p1} CUNIT/c/1/16 "Relative Flux"
  

  !Additionally do the resampling of the non-FF data:
  rebin/echelle xb_{p1}  w_xb_{p1}  {SAMPLE} NONL {LINTAB} >Null

  mess/out 3 {pid} "Compute VARIANCE of flatfielded object ..."
  @p uves_reduce,CALC_VARI xb_{p1} var_xb_{p1} p4={slit}
  @p uves_reduce,CALC_V1V2 -
     xb_{p1} x{correct} var_xb_{p1} var_x{correct} var_{p1}
  compute/imag err_{p1} = SQRT( var_{p1} )
  copy/dd xb_{p1} *,3 var_{p1}
  copy/dd xb_{p1} *,3 err_{p1}
  delete/desc var_{p1} {h_origname} N		!Remove original filename desc.
  delete/desc err_{p1} {h_origname} N		!Remove original filename desc.



  !Additionally do the resampling of the non-FF data:
  rebin/echelle x{correct}  w_x{correct}  {SAMPLE} NONL {LINTAB} >Null

  rebin/echelle err_{p1} errw{p1}  {SAMPLE} NONL {LINTAB} >Null
  rebin/echelle var_{p1} varw{p1}  {SAMPLE} NONL {LINTAB} >Null
  
  mess/out 3 {pid} "MERGE the orders of the object window ...\n"
  compute/ima wght = 1.0 / varw{p1} >Null	!Merge science frame
  merge/eche wfxb_{p1} m_{p1} ? OPTIMAL wght.bdf varm{p1} {SESSOUTV}
  
  if EXTMTD(1:1) .eq. "O" then	  
     compute/ima fopt_sky_xb_{p1} = opt_sky_xb_{p1} / x{correct} >Null
     write/desc fopt_sky_xb_{p1} CUNIT/c/1/16 "RELATIVE FLUX"
     rebin/echelle fopt_sky_xb_{p1} wfopt_sky_xb_{p1}  {SAMPLE} NONL {LINTAB} {SESSOUTV}
     write/desc wfopt_sky_xb_{p1} CUNIT/c/1/16 "Relative Flux"
     rebin/echelle opt_sky_xb_{p1} w_opt_sky_xb_{p1}  {SAMPLE} NONL {LINTAB} {SESSOUTV}
     @p uves_reduce,CALC_VARI opt_sky_xb_{p1} var_opt_sky_xb_{p1} p4={slit}
     @p uves_reduce,CALC_V1V2 -
     opt_sky_xb_{p1} x{correct} var_opt_sky_xb_{p1} var_x{correct} var_opt_sky_{p1}
     rebin/echelle var_opt_sky_{p1} varwopt_sky_{p1}  {SAMPLE} NONL {LINTAB} >Null
     compute/ima wght_opt_sky = 1.0 / varwopt_sky_{p1} >Null	!Merge opt extr sky frame
     merge/eche wfopt_sky_xb_{p1} m_opt_sky_{p1} ? OPTIMAL wght_opt_sky.bdf varmopt_sky_{p1} {SESSOUTV}  
  endif  
  MRGMTD = "AVERAGE"				!reset

  if SKYOPT(1:1) .eq. "Y" then

    mess/out 3 {pid} "Prepare the  SKY  windows:"
    mess/out 3 {pid} "EXTRACT averaged flatfield in the sky-windows ..."
    extract/sky {correct} x12_{correct} AVERAGE {SESSOUTV}
    -rename middumma.bdf x{correct1}
    statistic/imag x{correct1} ? ? ? SN
    skyw = (skywind(2) - skywind(1)) * outputr(3)
    @p uves_reduce,CALC_VARI x{correct1} var_x{correct1} p4={skyw}
    compute/imag x{correct1} = x{correct1} / {outputr(3)} >Null
    compute/imag var_x{correct1} = var_x{correct1} / ({outputr(3)}*{outputr(3)}) >Null
    !Additionally do the resampling of the non-FF data and variances:
    rebin/eche x{correct1} w_x{correct1} {SAMPLE} NONL {LINTAB_1} >Null

    if nsky .gt. 1 then
      -rename middummb.bdf x{correct2}
      statistic/imag x{correct2} ? ? ? SN
      skyw = (skywind(4) - skywind(3)) * outputr(3)
      @p uves_reduce,CALC_VARI x{correct2} var_x{correct2} p4={skyw}
      compute/imag x{correct2} = x{correct2} / {outputr(3)} >Null
      compute/imag var_x{correct2} = var_x{correct2} / ({outputr(3)}*{outputr(3)}) >Null
      rebin/eche x{correct2} w_x{correct2} {SAMPLE} NONL {LINTAB_2} >Null
    endif

  endif

else if FFMTD(1:1) .eq. "P" then	!pixel-to-pixel flatfielding
					!already done (see CONFIG)
  mess/out 3 {pid} "RESAMPLE the {EXTMTD} extracted object ..."
  rebin/echelle xb_{p1} wfxb_{p1}  {SAMPLE} NONL {LINTAB} {SESSOUTV}
  write/desc wfxb_{p1} CUNIT/c/1/16 "RELATIVE FLUX"
  if EXTMTD(1:1) .eq. "O" then
     rebin/echelle opt_sky_xb_{p1} wfopt_sky_xb_{p1}  {SAMPLE} NONL {LINTAB} {SESSOUTV}
     write/desc wfopt_sky_xb_{p1} CUNIT/c/1/16 "RELATIVE FLUX"
  endif
  !Additionally do the resampling of the non-FF data:
  rebin/echelle xb_{p1}  w_xb_{p1}  {SAMPLE} NONL {LINTAB} >Null

  mess/out 3 {pid} "Compute VARIANCES of flatfielded object ..."
  if EXTMTD(1:1) .eq. "A" then
    @p uves_reduce,CALC_VARI xb_{p1} var_{p1} p4={slit}
  else
    -rename var_xb_{p1} var_{p1}
    @p uves_reduce,CALC_VARI opt_sky_xb_{p1} var_opt_sky_{p1} p4={slit}
  endif
  
  compute/image err_{p1} = SQRT( var_{p1} )
  copy/dd xb_{p1} *,3 var_{p1}
  copy/dd xb_{p1} *,3 err_{p1}


    
  delete/desc var_{p1} {h_origname} N		!Remove original filename desc.
  delete/desc err_{p1} {h_origname} N		!Remove original filename desc.

 
  rebin/echelle err_{p1} errw{p1}  {SAMPLE} NONL {LINTAB} >Null
  rebin/echelle var_{p1} varw{p1}  {SAMPLE} NONL {LINTAB} >Null
  

  mess/out 3 {pid} "MERGE the orders of the object window ...\n"
  compute/ima wght = 1.0 / varw{p1} >Null	!Merge the science frame
  merge/eche wfxb_{p1} m_{p1} ? OPTIMAL wght.bdf varm{p1} {SESSOUTV}
  
  if EXTMTD(1:1) .eq. "O" then
     copy/dd opt_sky_xb_{p1} *,3 var_opt_sky_{p1}
     delete/desc var_opt_sky_{p1} {h_origname} N		!Remove original filename desc.    
     rebin/echelle var_opt_sky_{p1} varwopt_sky_{p1}  {SAMPLE} NONL {LINTAB} >Null 
     compute/ima wght_opt_sky = 1.0 / varwopt_sky_{p1} >Null	!Merge the science frame
     merge/eche wfopt_sky_xb_{p1} m_opt_sky_{p1} ? OPTIMAL wght_opt_sky.bdf varmopt_sky_{p1} {SESSOUTV} 
  endif
    
  MRGMTD = "AVERAGE"				!reset

else						!NO flatfielding

  mess/out 3 {pid} "Compute VARIANCE of extracted object ..."
  @p uves_reduce,CALC_VARI xb_{p1} var_{p1} p4={slit}
  compute/imag err_{p1} = SQRT( var_{p1} )

  rebin/echelle err_{p1} errw{p1}  {SAMPLE} NONL {LINTAB} >Null
  rebin/echelle var_{p1} varw{p1}  {SAMPLE} NONL {LINTAB} >Null

  mess/out 3 {pid} "RESAMPLE extracted orders ..."
  rebin/echelle xb_{p1} w_xb_{p1}  {SAMPLE} NONL {LINTAB} {SESSOUTV}
  write/desc w_xb_{p1} CUNIT/c/1/16 "Relative Flux"
 

  mess/out 3 {pid} "MERGE the orders of the object window ...\n"
  compute/ima wght = 1.0 / varw{p1} >Null	!Merge the science frame
  merge/eche w_xb_{p1} m_{p1} ? OPTIMAL wght.bdf varm{p1} {SESSOUTV}
  if EXTMTD(1:1) .eq. "O" then
     @p uves_reduce,CALC_VARI opt_sky_xb_{p1} var_opt_sky_{p1} p4={slit}  
     rebin/echelle var_opt_sky_{p1} varwopt_sky_{p1}  {SAMPLE} NONL {LINTAB} >Null 
     rebin/echelle opt_sky_xb_{p1} wopt_sky_xb_{p1}  {SAMPLE} NONL {LINTAB} {SESSOUTV}
     write/desc wopt_sky_xb_{p1} CUNIT/c/1/16 "Relative Flux"
     compute/ima wght_opt_sky = 1.0 / varwopt_sky_{p1} >Null	!Merge the science frame
     merge/eche wopt_sky_xb_{p1} m_opt_sky_{p1} ? OPTIMAL wght_opt_sky.bdf varm{p1} {SESSOUTV}  
  endif
 
  MRGMTD = "AVERAGE"				!reset

endif
write/desc m_{p1} CUNIT " " ALL
write/desc m_{p1} CUNIT/c/1/16 "RELATIVE FLUX"
write/desc m_{p1} CUNIT/c/17/32 "WAVELENGTH [A]"

if EXTMTD(1:1) .eq. "O" then
   write/desc m_opt_sky_{p1} CUNIT " " ALL
   write/desc m_opt_sky_{p1} CUNIT/c/1/16 "RELATIVE FLUX"
   write/desc m_opt_sky_{p1} CUNIT/c/17/32 "WAVELENGTH [A]"
endif
if SKYOPT(1:1) .eq. "Y" then
  mess/out 3 {pid} "EXTRACT sky-windows ..."
  @p uves_reduce,EXTRACT_SKY {p1} xb{sky1} xb{sky2}

  if FFMTD(1:1) .eq. "E" then
    pf = "wfxb"					!name prefix
    mess/out 3 {pid} "FLATFIELD and RESAMPLE the sky-windows ..."
    compute/imag n{sky} = xb{sky1}/x{correct1} >Null
    rebin/echelle n{sky} wfxb{sky1}  {SAMPLE} NONL {LINTAB_1} {SESSOUTV}
    if nsky .gt. 1 then
      compute/imag n{sky} = xb{sky2}/x{correct2} >Null
      rebin/echelle n{sky} wfxb{sky2}  {SAMPLE} NONL {LINTAB_2} {SESSOUTV}
    endif

    mess/out 3 {pid} "Compute VARIANCE of flatfielded sky ..."
    skyw = skywind(2) - skywind(1)
    @p uves_reduce,CALC_VARI xb{sky1} var_xb{sky1} p4={skyw}
    @p uves_reduce,CALC_V1V2 -
       xb{sky1} x{correct1} var_xb{sky1} var_x{correct1} var_{sky1}
    compute/imag err_{sky1} = SQRT( var_{sky1} )
    rebin/echell err_{sky1} errw{sky1} {SAMPLE} NONL {LINTAB_1} >Null
    rebin/echell var_{sky1} varw{sky1} {SAMPLE} NONL {LINTAB_1} >Null
    if nsky .gt. 1 then
      skyw = skywind(4) - skywind(3)
      @p uves_reduce,CALC_VARI xb{sky2} var_xb{sky2} p4={skyw}
      @p uves_reduce,CALC_V1V2 -
         xb{sky2} x{correct2} var_xb{sky2} var_x{correct2} var_{sky2}
      compute/imag err_{sky2} = SQRT( var_{sky2} )
      rebin/echell err_{sky2} errw{sky2} {SAMPLE} NONL {LINTAB_2} >Null
      rebin/echell var_{sky2} varw{sky2} {SAMPLE} NONL {LINTAB_2} >Null
    endif

  else if FFMTD(1:1) .eq. "P" then

    pf = "wfxb"					!name prefix
    mess/out 3 {pid} "RESAMPLE the flatfielded sky ..."
    rebin/echelle xb{sky1} wfxb{sky1}  {SAMPLE} NONL {LINTAB_1} {SESSOUTV}
    mess/out 3 {pid} "Compute VARIANCE of flatfielded sky ..."
    skyw = skywind(2) - skywind(1)
    @p uves_reduce,CALC_VARI xb{sky1} var_{sky1} p4={skyw}
    compute/imag err_{sky1} = SQRT( var_{sky1} )
    rebin/echell err_{sky1} errw{sky1} {SAMPLE} NONL {LINTAB_1} >Null
    rebin/echell var_{sky1} varw{sky1} {SAMPLE} NONL {LINTAB_1} >Null

    if nsky .gt. 1 then
      rebin/echelle xb{sky2} wfxb{sky2}  {SAMPLE} NONL {LINTAB_2} {SESSOUTV}
      skyw = skywind(4) - skywind(3)
      @p uves_reduce,CALC_VARI xb{sky2} var_{sky2} p4={skyw}
      compute/imag err_{sky2} = SQRT( var_{sky2} )
      rebin/echell err_{sky2} errw{sky2} {SAMPLE} NONL {LINTAB_2} >Null
      rebin/echell var_{sky2} varw{sky2} {SAMPLE} NONL {LINTAB_2} >Null
    endif
  else						!NO Flatfielding

    pf = "w_xb"					!name prefix
    mess/out 3 {pid} "RESAMPLE the extracted sky ..."
    rebin/echelle xb{sky1} w_xb{sky1}  {SAMPLE} NONL {LINTAB_1} {SESSOUTV}
    mess/out 3 {pid} "Compute VARIANCE of extracted sky ..."
    skyw = skywind(2) - skywind(1)
    @p uves_reduce,CALC_VARI xb{sky1} var_{sky1} p4={skyw}
    compute/imag err_{sky1} = SQRT( var_{sky1} )
    rebin/echell err_{sky1} errw{sky1} {SAMPLE} NONL {LINTAB_1} >Null
    rebin/echell var_{sky1} varw{sky1} {SAMPLE} NONL {LINTAB_1} >Null
    if nsky .gt. 1 then
      rebin/echelle xb{sky2} w_xb{sky2}  {SAMPLE} NONL {LINTAB_2} {SESSOUTV}
      skyw = skywind(4) - skywind(3)
      @p uves_reduce,CALC_VARI xb{sky2} var_xb{sky2} p4={skyw}
      -rename var_xb{sky2} var_{sky2}
      compute/imag err_{sky2} = SQRT( var_{sky2} )
      rebin/echell err_{sky2} errw{sky2} {SAMPLE} NONL {LINTAB_2} >Null
      rebin/echell var_{sky2} varw{sky2} {SAMPLE} NONL {LINTAB_2} >Null
    endif
  endif

  mess/out 3 {pid} "MERGE the orders of the sky windows ..."
  compute/ima wght = 1.0 / varw{sky1} >Null
  merge/eche {pf}{sky1} m{sky1} ? OPTIMAL wght.bdf varm{sky1} {SESSOUTV}
  if nsky .gt. 1 then
    compute/ima wght = 1.0 / varw{sky2} >Null
    merge/eche {pf}{sky2} m{sky2} ? OPTIMAL wght.bdf varm{sky2} {SESSOUTV}
  endif
  MRGMTD = "AVERAGE"				!reset
  if EXTMTD(1:1) .eq. "O" then			!Merge the sky
    mess/out 3 {pid} "... apply weights (optimal extract.) before ..."
    merge/eche {rwbin} m{rwbin}
    if nsky .gt. 1 then
      mess/out 3 {pid} "AVERAGE the merged sky ...\n"
      compute/ima m{sky} = m{rwbin} * (m{sky1} + m{sky2}) / 2.0
      compute/ima varm{sky} = m{rwbin}*m{rwbin} * (varm{sky1} + varm{sky2}) / 4.0
    else
      compute/ima m{sky} = m{rwbin} * m{sky1}
      compute/ima varm{sky} = m{rwbin}*m{rwbin} * varm{sky1}
    endif
  else
    if nsky .gt. 1 then
      mess/out 3 {pid} "AVERAGE the merged sky ...\n"
      define/local sky_slit1/r/1/1 0
      define/local sky_slit2/r/1/1 0
      sky_slit1 = skywind(2)-skywind(1)
      sky_slit2 = skywind(4)-skywind(3)
    
      compute/ima m{sky} = (m{sky1}*{sky_slit1} + m{sky2}*{sky_slit2}) / ({sky_slit1}+{sky_slit2})
      compute/ima varm{sky} = (varm{sky1}*{sky_slit1}*{sky_slit1} + varm{sky2}*{sky_slit2}*{sky_slit2})/({sky_slit1}+{sky_slit2})*({sky_slit1}+{sky_slit2})
    else
      -copy m{sky1} m{sky}
      -copy varm{sky1} varm{sky}
    endif
  endif

  !Additionally do the resampling of the non-FF data:
  rebin/echelle xb{sky1} w_xb{sky1} {SAMPLE} NONL {LINTAB_1} >Null
  if nsky .gt. 1 then
    rebin/echelle xb{sky2} w_xb{sky2} {SAMPLE} NONL {LINTAB_2} >Null
  endif

  mess/out 3 {pid} "Do the SKY SUBTRACTION ...\n"
  compute/imag r_{p1} = m_{p1} - m{sky} >Null	!Sky subtraction in wave. space
  compute/imag errm{p1} = SQRT( varm{p1} + varm{sky} )
  copy/dd r_{p1} *,3 errm{p1}
  delete/desc varm{p1} {h_origname} N		!Remove original filename desc.
  delete/desc errm{p1} {h_origname} N		!Remove original filename desc.
else

  -copy m_{p1} r_{p1}
  compute/imag errm{p1} = SQRT( varm{p1} )

endif

if FFMTD(1:2) .ne. "NO" FFOPT = "YES"		!Reset for the user.

!remove unused data
!-delete varw*
-delete vari*
!-delete var_x*
-delete var__*

return 0
!=============================================================================
!=============================================================================
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   10/07/01  this entry    abs({p1}) sobstituted by abs({p1})
! AM   31/05/01  this entry    DFS code rev step 1:
!-----------------------------------------------------------------------------
ENTRY CALC_VARI
define/param p1 ? c "Enter input frame:"
define/param p2 ? c "Enter output frame:"
define/param p3 {ron},{gain} c "Enter extraction pars (RON,CONAD):"
define/param p4 1 n "Enter number of averaged pixels (for averaged inputs):"
!-----------------------------------------------------------------------------
!Calculates the variance of an image
!=============================================================================
if EXTMTD(1:1) .eq. "O" then
  if "{p1(1:3)}" .ne. "sky" then
     !the variance is calculated from the optimal extraction code
     !frm name conventions coherent either for var(obj) and var(sky)
     -rename var_{p1} {p2}
     return 1
  else
     !we need to calculate the variance
  endif
endif
write/keyw INPUTR/r/1/2 {p3}
define/local RON/r/1/1 0.0
!
! Compute the variance frame. The input frame is assumed to be only background
! subtracted (de-biased)
! The variance frame is in units of [ADU^2]:
!   V_ij = (RON / CONAD)^2 + |IN_ij|/CONAD ;
!  [RON] = e-, [CONAD] = e-/ADU, [GAIN] = ADU/e-
!  Better to take 5the max than the absolute value.
 
if inputr(2) .gt. 0.0 then
  ron = inputr(1) / inputr(2)		!RON in ADU
  ron = ron * ron			!RON^2 in ADU
  compute/image {p2} = ( {RON} + max(0,{p1}) / {inputr(2)} ) / {p4}
else
  mess/out 1 {pid} "Bad CONAD to determine variance frame!"
  return -1
endif
return 0

!=============================================================================
!=============================================================================
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   31/05/01  this entry    DFS code rev step 1:
!-----------------------------------------------------------------------------
ENTRY CALC_V1V2
define/param p1 ? c "Enter input 1 frame:"
define/param p2 ? c "Enter input 2 frame:"
define/param p3 ? c "Enter variance 1 frame:"
define/param p4 ? c "Enter variance 2 frame:"
define/param p5 ? c "Enter output variance frame:"
!
!.............................................................................
!Calculates the total variance of a ratio of variances
!=============================================================================

! Determine the variance of F = S1/S2 which is:
! dF^2 = F^2*[ (dS1/S1)^2 + (dS2/S2)^2 ] = [V1/S2^2 + V2*S1^2/S2^4]
! V1 = dS1^2, V2 = dS2^2
!
!we write **4 as a multiple product to be portable to OSF1
!we factor **2 to limitate number of multiplications and lengh of command line
!to overcome a MIDAS limit
 
compute/ima den2 = {p2}*{p2}
compute/ima num2 = {p1}*{p1}
compute/ima {p5} = {p3} / (den2) >Null
compute/ima {p5} = {p5} + {p4} * num2 / (den2*den2) >Null 
!4 times * as ** is not portable to ALPHA OSF1 

return 0


!=============================================================================
!=============================================================================
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   31/05/01  this entry    DFS code rev step 1:
!-----------------------------------------------------------------------------
ENTRY LINTAB_OFFSET
define/param p1 ? c "Enter name of sky window line table (lower or upper):"
!.............................................................................
!Determines offset of given LINE table
!=============================================================================

if m$existd(p1,"OFFSET") .eq. 1 then
  outputr(1) = m$value({p1},OFFSET)
  outputr(1) = m$abs( outputr(1) )
else
  append/out "No OFFSET descriptor in `{p1}` found!\n"
  append/out "--> take the default: {slit}"
  mess/out 2 {pid}
  outputr(1) = m$abs(slit)
endif

!==============================================================================
!=============================================================================
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   31/05/01  this entry    DFS code rev step 1:
!-----------------------------------------------------------------------------
ENTRY DET_SKYWIND
define/param p1 ? c "Enter input frame:"
define/param p2 ? c "Enter offset between sky and object window:"
define/param p3 Y,Y,Y c "Automatically set offset [Y]/N, slit [Y]/N, skywindows [Y]/N :"
define/param p4 ? c "Extraction method (AVERAGE, OPTIMAL):"
!The correct order table has to be selected BEFORE!

!.............................................................................
! Determines extraction parameters: object offset, object extraction slit in pix,
! sky extraction parameters to later set the sky extraction slit(s)
! In case of Image Slicers proper extraction slit is set, extraction method is 
! average, sky subtraction is set to NO.
! In case of automatic setting/determination of extraction parameters a check
! on the skywind parameters is done to eventually limitate to 1 the sky extraction
! window.
! In case of manual setting of extraction parameters the previous check is disabled.
!=============================================================================

!=========================================================================
! Interactive data analysis extraction setting parameter (P7)
!=========================================================================
define/local changeOffs/c/1/1 " " 
define/local changeSlit/c/1/1 " " 
define/local changeSkyW/c/1/1 " " 
define/local k/i/1/1 0
define/local slitLength/r/1/1 0

if "{modext}" .eq. "_BLUE" then
   slitLength = {{p1},ESO.INS.SLIT2.LEN}
else 
   slitLength = {{p1},ESO.INS.SLIT3.LEN}
endif
  

k = m$parse(p3,"switch")
     if k .eq. 3 then
  changeOffs = "{switch01}"
  changeSlit = "{switch02}"
  changeSkyW = "{switch03}"
else if k .eq. 2 then
  changeOffs = "{switch01}"
  changeSlit = "{switch02}"
  changeSkyW = "Y"
else if k .eq. 1 then
  changeOffs = "{switch01}"
  changeSlit = "Y"
  changeSkyW = "Y"
else if k .eq. 0 then
  changeOffs = "Y"
  changeSlit = "Y"
  changeSkyW = "Y" 
else
 mess/out 3 {pid} "Too many "interactive extraction" sub-pars, max 3: [N]/Y, [N]/Y, [N]/Y"
 return -1
endif

changeOffs = m$upper(changeOffs)
changeSlit = m$upper(changeSlit)
changeSkyW = m$upper(changeSkyW)

!============================================================================
 
 


 
define/local dwin/r/1/1 {p2}
define/local slicer_name/c/1/25 " "

!mess/out 3 {pid} "SLIT={slit}"
 
if changeOffs(1:1) .eq. "Y" then
  stat/table {ORDTAB} :ORDER >Null
  offset/echelle {p1} {slit} {outputr(2)} ? SILENT >Null
  mess/out 3 {pid} "Using OFFSET/ECHELLE"
else
  outputr(1) = OFFSET
  mess/out 3 {pid} "Order offset read: {OFFSET} !"
endif
mess/out 3 {pid} "Offset={OFFSET}, outputr(1)={outputr(1)}"
outputr(4) = m$value({p1},{h_pixscl})			!Pixelscale ["/pixel]

!The check on Image Slicer is now done here
!AKA limit extraction slit to slicer length (moved here the check on IS: 
! before was in ENTRY REDUCE)
slicer_name = m$value({p1},{h_slicer_name}) 				
if slicer_name(1:4) .ne. "FREE" then
   mess/out 3 {pid} "Image Slicer {slicer_name} inserted"
   mess/out 3 {pid} "N. of slicer pos. on the IS slide: {{p1},{h_slicer_no}}"
   if slicer_name(1:6) .eq. "SLIC#1" outputr(2) =  8.0
   if slicer_name(1:6) .eq. "SLIC#2" outputr(2) =  8.0
   if slicer_name(1:6) .eq. "SLIC#3" outputr(2) = 10.0
   offset = 0.0
   outputr(1) = 0.0
   mess/out 3 {pid} "Slicer slit length in arcsec: {outputr(2)}"
   outputr(2) = outputr(2)/outputr(4)/{CCDBIN(2)} - 2.0
   mess/out 3 {pid} "Extraction slit length in pixel: {outputr(2)}"
   slit = outputr(2)	
   outputr(3) = 0.0
   set/echelle SKYOPT=NO
   nsky = 0
   return 0
else
   mess/out 3 {pid} "No image slicer inserted" 
   outputr(2) = m$value({p1},{h_sllen({PATHID})})
   outputr(2) = outputr(2)/outputr(4)/{CCDBIN(2)} - 2.0
   mess/out 3 {pid} "Full slit length in pixel: {outputr(2)}"
   outputr(3) = outputr(2) - slit/2.0
endif
!AKA

 
!write/key objset/r/1/1 0	!this keyword is used by the new version of
if changeOffs(1:1) .eq. "Y" then
   write/key objset/r/1/1 0
   objset = offset			!necopt.exe (introduced by S.Cristiani)
else
   if m$existk("OBJSET") .eq. 1 then
      if  {OBJSET} .lt. {SLIT}/2 then
          mess/out 3 {pid} "Using user-defined OBJSET {objset}"
      else
          mess/out 3 {pid} "OBJSET = {objset} MUST be < SLIT/2 = {SLIT}/2."
      endif          
   else
      mess/out 3 {pid} "OBJSET set to default value 0."
      write/key objset/r/1/1 0
   endif
endif
mess/out 3 {pid} "OBJSET={objset}"
!if p4(1:1) .eq. "XX" then
if p4(1:1) .eq. "O" then				!Use full slit for
  mess/out 3 {pid} "OPTIMAL extraction:\n"
  if changeOffs(1:1) .eq. "Y" offset = 0.0	!optimal extration
  if changeSlit(1:1) .eq. "Y" then
     slit = outputr(2)
     mess/out 3 {pid} "Use full slit as extraction slit\nset offset to {offset}"
  else
     slitLength = slitLength/outputr(4)/{CCDBIN(2)}   !NB Binned as we suppose the user give manual values in binned data.
                                                      !here we do not take out 2 pix to allow in principle the user to 
                                                      !choose full slit size (but slit borders may have problems).
     if slit .ge. 0 .and. slit .le. slitLength then
        mess/out 3 {pid} "Use user slit as extraction slit\nset offset to {offset}"
     else
        mess/out 1 {pid} "Extraction slit must be in the [0,{slitLength}] range!"
        return -1
     endif 
  endif
  return 0			!the rest is of no interest in that case!
endif
    
set/format F3.1
if m$abs(outputr(1)) .gt. outputr(3) then
  mess/out 1 {pid} "Bad offset determined: {outputr(1)} !"
  return -1
endif
if changeOffs(1:1) .eq. "Y" OFFSET = outputr(1)
 
if changeOffs(1:1) .eq. "Y" .AND. OFFSET .ne. 0 -
  mess/out 3 {pid} "Order offset determined: {OFFSET} !"
set/format

mess/out 3 {pid} "OFFSET={offset}"
mess/out 3 {pid} "OBJSET={objset}"

if changeSkyW(1:1) .eq. "Y" then
   mess/out 3 {pid} "Automatic setting of Sky Windows"
   SKYWIND(4) = outputr(2) / 2.0
   SKYWIND(1) = -SKYWIND(4)
   !DWIN(=1) is to leave some space between obj and sky extraction window
   !we may loose some obj flux but we are "sure" to extract only the sky.
   
   SKYWIND(2) = OFFSET - DWIN - SLIT/2  
   SKYWIND(3) = OFFSET + DWIN + SLIT/2
else
   mess/out 3 {pid} "User defined setting of Sky Windows"
   if SKYWIND(1) .ge. SKYWIND(2) then
      mess/out 1 {pid} "Skywind setting: SKYWIND(1)={SKYWIND(1)}, SKYWIND(2)={SKYWIND(2)}!"
      mess/out 1 {pid} "You must choose SKYWIND(1) < SKYWIND(2)!"
      return -1
   endif
   if SKYWIND(3) .ge. SKYWIND(4) then
      mess/out 1 {pid} "Skywind setting: SKYWIND(3)={SKYWIND(3)}, SKYWIND(4)={SKYWIND(4)}!"
      mess/out 1 {pid} "You must choose SKYWIND(3) < SKYWIND(4)!"
      return -1
   endif
endif
OUTPUTR(1) = SKYWIND(2) - SKYWIND(1)
OUTPUTR(2) = SKYWIND(4) - SKYWIND(3)

if changeSkyW(1:1) .eq. "Y" then 
    nsky = 2
    if outputr(1) .lt. 4 then			!Use only second sky window
      mess/out 3 {pid} "Use only second sky window!"
      skywind(1) = skywind(3)
      skywind(2) = skywind(4)
      nsky = 1
      LINTAB_1 = LINTAB_2				!rename line table !!
    endif
    if outputr(2) .lt. 4 then			!Use only first sky window
      mess/out 3 {pid} "Use only first sky window!"
      skywind(3) = skywind(1)
      skywind(4) = skywind(2)
      nsky = 1
    endif
endif  !else it is used the user defined nsky

return 0


!=============================================================================
!=============================================================================
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   31/05/01  this entry    DFS code rev step 1:
!-----------------------------------------------------------------------------
ENTRY EXTRACT_SKY
define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter name of extracted sky (1.window):"
define/param p3 ? c "Enter name of extracted sky (2.window):"
!.............................................................................
!Extract both given sky frames
!=============================================================================
define/local cnt/i/1/1 0
define/local num/i/1/1 0
define/local oslit/r/1/1 0

oslit = slit
slit   = skywind(2) - skywind(1)		!Size of 1st sky window
offset = skywind(1) + slit/2.			!Offset to center of sky window
extract/echelle {p1} {p2} p4=MEDIAN		!Do an MEDIAN extraction
slit = oslit					!Reset slit size
offset = 0

if nsky .lt. 2 return 0				!Only one sky window used!

slit   = skywind(4) - skywind(3)		!Do the same for the second
offset = skywind(3) + slit/2.
extract/echelle {P1} {p3} p4=MEDIAN
slit = oslit					!Reset slit size
offset = 0

return 0

!OLD:
!oslit = slit
!slit   = skywind(2) - skywind(1)		!Extract each row of the
!num = m$nint(slit)				!sky window and
!slit = 1					!finally do a MEDIAN-average
!do cnt = 1 num					!over all extracted rows
!   offset = skywind(1) + slit/2. + (cnt-1.)*slit
!   extract/echelle {P1} ss{cnt} p4=AVERAGE	!in order to remove cosmic
!enddo						!rays.
!create/icat ss.cat ss*.bdf >Null
!average/image {p2} = ss.cat ? ? median,1,0,index
!delete/icat ss.cat N >Null
!-delete ss.cat
!slit = oslit					!Reset slit size
!offset = 0

!if nsky .lt. 2 return 0			!Only one sky window used!

!slit   = skywind(4) - skywind(3)		!Do the same for the second
!num = m$nint(slit)				!sky window
!slit = 1
!do cnt = 1 num
!   offset = skywind(3) + slit/2. + (cnt-1.)*slit
!   extract/echelle {P1} tt{cnt} p4=AVERAGE
!enddo
!create/icat tt.cat tt*.bdf >Null
!average/image {p3} = tt.cat ? ? median,1,0,index
!delete/icat tt.cat N >Null
!-delete tt.cat
!slit = oslit					!Reset slit size
!offset = 0
!return 0

!=============================================================================
!=============================================================================
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   31/05/01  this entry    DFS code rev step 1:
!-----------------------------------------------------------------------------

ENTRY DO_CLASS
define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter name of output catalog:"
define/param p3 ? c "Enter flatfielding method (P,E,...):"
define/param p4 ? c "Enter UVES mode extension (_BLUE, _REDL, _REDU):"
!.............................................................................
! Classify all the pipe data products
!=============================================================================

define/local main_prod/c/1/25 " " all
define/local do_clas/c/1/25 " " all
define/local do_clas_in/c/1/25 " " all
define/local check_data/i/1/1 0

do_clas_in = "{{p1},{UVES_IDSTRNG}}"

do_clas = "{REDU_PX}{do_clas_in}"
main_prod = "{REDU_PX}{do_clas_in}"

classify/product r_{p1} {do_clas} {d_red} {d_pho},{d_geo},{d_det} OK y y

if {q1} .eq. -1 then 
   mess/out 1 {pid} "Error in main product classification!"
   return -1
endif

@p uves_reduce,ADD_CAT {p2} r_{p1}		!MAIN product: OBJECT-SKY
do_clas = "{MRGD_PX}{do_clas_in}"
classify/product m_{p1} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y y
@p uves_reduce,ADD_CAT {p2} m_{p1}
if P3(1:1) .ne. "P" then
  do_clas = "{WCAL_PX}{do_clas_in}"
  classify/product w_xb_{p1} {do_clas} {d_pre} {d_geo},{d_det} OK y y
  @p uves_reduce,ADD_CAT {p2} w_xb_{p1}
endif

if P3(1:1) .ne. "N" then
  do_clas = "{WCFF_PX}{do_clas_in}"
  classify/product wfxb_{p1} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y y
  @p uves_reduce,ADD_CAT {p2} wfxb_{p1}
endif
!
! Sky:
if SKYOPT(1:1) .eq. "Y" then
  do_clas = "{WCAL_PX}SKY1{p4}"
  classify/product w_xb{sky1} {do_clas} {d_pre} {d_geo},{d_det} OK y y
  @p uves_reduce,ADD_CAT {p2} w_xb{sky1}
  if P3(1:1) .ne. "N" then
    do_clas = "{WCFF_PX}SKY1{p4}"
    classify/produc wfxb{sky1} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y y
    @p uves_reduce,ADD_CAT {p2} wfxb{sky1}
  endif
  do_clas = "{MRGD_PX}SKY1{p4}"
  classify/product m{sky1} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y y
  @p uves_reduce,ADD_CAT {p2} m{sky1}
  if nsky .gt. 1 then
    do_clas = "{WCAL_PX}SKY2{p4}"
    classify/product w_xb{sky2} {do_clas} {d_pre} {d_geo},{d_det} OK y y
    @p uves_reduce,ADD_CAT {p2} w_xb{sky2}
    do_clas = "{WCFF_PX}SKY2{p4}"
    if P3(1:1) .ne. "N" then
      classify/prod wfxb{sky2} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y y
      @p uves_reduce,ADD_CAT {p2} wfxb{sky2}
    endif
    do_clas = "{MRGD_PX}SKY2{p4}"
    classify/product m{sky2} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y y
    @p uves_reduce,ADD_CAT {p2} m{sky2}
  else
    !creates dummy images 2 axis 1x1 bin
    crea/ima dummy_w_xb{sky2} 2,1,1
    if P3(1:1) .ne. "N" then
       crea/ima dummy_wfxb{sky2} 2,1,1
    endif
    crea/ima    dummy_m{sky2} 2,1,1
    
    !copies descriptors from good images in order to classify them
    copy/dd w_xb{sky1} *,3 dummy_w_xb{sky2}
    if P3(1:1) .ne. "N" then
       copy/dd wfxb{sky1} *,3 dummy_wfxb{sky2}
    endif
    copy/dd    m{sky1} *,3    dummy_m{sky2}
    
    !do "dummy" classification as before
    do_clas = "dummy_{WCAL_PX}SKY2{p4}"
    classify/product dummy_w_xb{sky2} {do_clas} {d_pre} {d_geo},{d_det} OK y y
    @p uves_reduce,ADD_CAT {p2} dummy_w_xb{sky2}
    do_clas = "dummy_{WCFF_PX}SKY2{p4}"
    if P3(1:1) .ne. "N" then
      classify/prod dummy_wfxb{sky2} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y y
      @p uves_reduce,ADD_CAT {p2} dummy_wfxb{sky2}
    endif
    do_clas = "dummy_{MRGD_PX}SKY2{p4}"
    classify/product dummy_m{sky2} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y y
    @p uves_reduce,ADD_CAT {p2} dummy_m{sky2}
    
  endif
  do_clas = "{MRGD_PX}AV_SKY{p4}"
  classify/product m{sky} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y y
  @p uves_reduce,ADD_CAT {p2} m{sky}
!
! Flatfields:
  if P3(1:1) .eq. "E" then
    do_clas = "{WCAL_PX}FLAT_SKY1{p4}"
    classify/product w_x{correct1} {do_clas} {d_pre} {d_geo},{d_det} OK y n
    @p uves_reduce,ADD_CAT {p2} w_x{correct1}
    if nsky .gt. 1 then
      do_clas = "{WCAL_PX}FLAT_SKY2{p4}"
      classify/product w_x{correct2} {do_clas} {d_pre} {d_geo},{d_det} OK y n
      @p uves_reduce,ADD_CAT {p2} w_x{correct2}
    else  
      !creates dummy images 2 axis 1x1 bin
      crea/ima dummy_w_x{correct2} 2,1,1
    
      !copies descriptors from good images in order to classify them
      copy/dd w_x{correct1} *,3 dummy_w_x{correct2}
  
      !do "dummy" classification as before
      do_clas = "dummy_{WCAL_PX}FLAT_SKY2{p4}"
      classify/product dummy_w_x{correct2} {do_clas} {d_pre} {d_geo},{d_det} OK y y
      @p uves_reduce,ADD_CAT {p2} dummy_w_x{correct2}
    endif
  endif
endif
if P3(1:1) .eq. "E" then
  do_clas = "{WCAL_PX}FLAT_OBJ{p4}"
  classify/product w_x{correct} {do_clas} {d_pre} {d_geo},{d_det} OK y n
  @p uves_reduce,ADD_CAT {p2} w_x{correct}
endif

do_clas = "ERRORBAR_{do_clas_in}"
classify/product errm{p1} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y n
@p uves_reduce,ADD_CAT {p2} errm{p1}
do_clas = "VARIANCE_{do_clas_in}"
classify/product var_{p1} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y n
@p uves_reduce,ADD_CAT {p2} var_{p1}
! am: adding SCIENCE bkg (if exist, usually yes)
if m$exist("bg_{p1}") .eq. 1 then
   do_clas = "BKG_SCI{p4}"
   classify/product bg_{p1} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y n
   @p uves_reduce,ADD_CAT {p2} bg_{p1}
endif
! am: adding FF bkg (if exist, usually not), p4=modext
if m$exist("bg{p4}") .eq. 1 then
   do_clas = "BKG_FF_{p4}"
   classify/product bg{p4} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y n
   @p uves_reduce,ADD_CAT {p2} bg{p4}
endif
if EXTMTD(1:1) .eq. "O" then
    do_clas = "ORDER_TRACE{p4}"
    classify/product order_trace{p4}.tbl {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y n
    @p uves_reduce,ADD_CAT {p2} order_trace{p4}.tbl
    if uves_debug .eq. "YES" then
        do_clas = "XORDER_TRACE{p4}"
        classify/product xorder_trace{p4}.tbl {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y n
        @p uves_reduce,ADD_CAT {p2} xorder_trace{p4}.tbl
    endif
    do_clas = "CRMASK{p4}"
    define/local index/i/1/1 0
    define/local crmask/c/1/80 " " all
    index = m$index(p1,".bdf")
    index = index -1
    crmask = "crm_{p1(:{index})}.tbl" 
    classify/product {crmask} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y n
    @p uves_reduce,ADD_CAT {p2} {crmask}
    
    if m$exist("m_opt_sky_{p1}") .gt. 0 then

        do_clas = "{MRGD_PX}SKY{p4}"
        classify/product "m_opt_sky_{p1}" {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y n
        @p uves_reduce,ADD_CAT {p2} "m_opt_sky_{p1}"
        
    endif 
endif



if EXTMTD(1:1) .eq. "O" SKYOPT = "YES"		!Reset for the user.

return {main_prod}
!=============================================================================
!=============================================================================
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   31/05/01  this entry    DFS code rev step 1:
!-----------------------------------------------------------------------------
ENTRY ADD_CAT
define/param p1 ? c "Enter catalog name:"
define/param p2 ? c "Enter filename to add to catalog:"
!.............................................................................
!Add a given frame entry to a given catalog
!=============================================================================
if p1(1:1) .eq. "+" return -1
add/icat {p1} {p2} >Null
return 0
!=============================================================================
!=============================================================================
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   31/05/01  this entry    DFS code rev step 1:
!-----------------------------------------------------------------------------

ENTRY INFO
define/param p1 ? c "Enter input frame:"
define/param p2 ? c "Enter flatfielding method (P,E,...):"
!.............................................................................
!Displays info
!=============================================================================
append/out "Intermediate frames produced:\n"
append/out "Flatfield method used:            {p2}\n"
append/out "Extr.Flatfield (1. sky window):   x{correct1}\n"
append/out "Extr.Flatfield (2. sky window):   x{correct2}\n"
append/out "Extr.Flatfield (obj.  window):    x{correct}\n\n"
append/out "Extracted sky (1. sky window):    xb{sky1}\n"
append/out "Extracted sky (2. sky window):    xb{sky2}\n"
append/out "FF&Resampled sky (1. sky window): wfxb{sky1}\n"
append/out "FF&Resampled sky (2. sky window): wfxb{sky2}\n\n"
append/out "Sky (FF&resampled):               wfxb{sky}\n"
append/out "Object ({EXTMTD} extracted):       xb_{p1}\n"
append/out "Object (FF&resampled):            wfxb_{p1}\n"
append/out "(Object - Sky):                   r_{p1}"

mess/out 3 {pid}


!=============================================================================
!=============================================================================
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   31/05/01  this entry    DFS code rev step 1:
!-----------------------------------------------------------------------------

ENTRY PLOTSET

define/param p1 ? c "Enter science input frame:"
define/param p2 + c "Enter name of correction flatfield:"
!.............................................................................
!Do several plots
!=============================================================================
define/local ofrm/c/1/100 " " ALL
define/local ff_corr/c/1/100 " " ALL
define/local lub/c/1/3 "lub"
define/local modxt/c*4/1/3 " " ALL
define/local nord/i/1/1 0

modxt(1) = "REDL"
modxt(2) = "REDU"
modxt(3) = "BLUE"


if p2(1:1) .ne. "+" then
  get/pathid {p1}
  ff_corr = p2
else
  ff_corr = correct
endif
parse/path {ff_corr}
ff_corr = q2
nord = m$value(x{ff_corr}.bdf,NPIX(2))

ofrm = "wbin_{modxt({pathid})}"


if m$exist("{ofrm}.bdf") .eq. 1 then
  mess/out 3 PLOTSET "Plot the weights"
  plot/uves {ofrm}.bdf 1,{nord} "Weights: {ofrm}.bdf"
  copy/grap postscript
  -rename postscript.ps {ofrm}.ps
endif

mess/out 3 PLOTSET "Plot the extracted correction flatfield"
ofrm = "x{ff_corr}"
plot/uves {ofrm}.bdf 1,{nord} "Extracted Flatfield (obj): {ofrm}.bdf"
copy/grap postscript
-rename postscript.ps x{ff_corr}.ps

mess/out 3 PLOTSET "Plot the {EXTMTD} extracted OBJECT"
plot/uves xb_{p1} 1,{nord} "{EXTMTD} extracted OBJECT: xb_{p1}"
copy/grap postscript
-rename postscript.ps xb_{p1}.ps

mess/out 3 PLOTSET "Plot the extracted SKY"
ofrm  = "wfxb_sky_{modxt({pathid})}"
plot/uves {ofrm}.bdf 1,{nord},0,50 "Mean extr.&ff SKY: {ofrm}.bdf"
copy/grap postscript
-rename postscript.ps {ofrm}.ps

mess/out 3 PLOTSET "Plot the re-binned OBJECT"
ofrm = "wfxb_{p1}"
plot/uves {ofrm} 1,{nord},0,500 "Flatfielded OBJECT: {ofrm}"
copy/grap postscript
-rename postscript.ps {ofrm}.ps

mess/out 3 PLOTSET "Plot the sky subtracted OBJECT: (obj-sky)"
ofrm = "r_{p1}"
plot/uves {ofrm} 1,{nord},0,500 !"Flatfielded OBJECT - SKY: {ofrm}"
copy/grap postscript
-rename postscript.ps {ofrm}.ps
!=============================================================================
ENTRY deg2hms
define/param p1 ? num "Enter value in deg units"
define/local in_val/d/1/1 {p1}
define/local out_val/c/1/80 " " all
define/local hours/i/1/1 0
define/local minutes/i/1/1 0
define/local seconds/d/1/1 0
define/local tmp/d/1/1 0
define/local hold/c/1/80 " " all
define/local sign/c/1/1 " "


hold = "{in_val}"
if m$index(hold,"-") .gt. 0 then
   in_val = m$abs(in_val)
   sign = "-"
else
   sign = "+"
endif      

set/format i1
!360 deg corresponds to 24 h=> 15 deg corresponds to 1 h
tmp     = in_val / 15
hours   = tmp           !takes the integer part = hours
tmp     = tmp - hours   !takes the mantissa 
tmp     = tmp * 60      !converts the mantissa in minutes
minutes = tmp           !takes the integer part = minutes
tmp     = tmp - minutes !takes the mantissa
seconds = tmp * 60      !converts the mantissa in seconds = seconds (with decimal)

out_val = "{sign}{hours},{minutes},{seconds}"
set/format 

return {out_val}
!=============================================================================
ENTRY deg2dms
define/param p1 ? num "Enter value in deg units"
define/local in_val/d/1/1 {p1}
define/local out_val/c/1/80 " " all
define/local degrees/i/1/1 0
define/local minutes/i/1/1 0
define/local seconds/d/1/1 0
define/local tmp/d/1/1 0
define/local hold/c/1/80 " " all
define/local sign/c/1/1 " "


hold = "{in_val}"
if m$index(hold,"-") .gt. 0 then
   in_val = m$abs(in_val)
   sign = "-"
else
   sign = "+"
endif      
set/format i1
tmp     = in_val
degrees = tmp           !takes the integer part = degrees
tmp     = tmp - degrees !takes the mantissa 
tmp     = tmp * 60      !converts the mantissa in minutes
minutes = tmp           !takes the integer part = minutes
tmp     = tmp - minutes !takes the mantissa
seconds = tmp * 60      !converts the mantissa in seconds = seconds (with decimal)

out_val = "{sign}{degrees},{minutes},{seconds}"
set/format

return {out_val}
!=============================================================================
