! $Id: uves_savcfg.prg,v 1.3 2002/06/03 16:20:56 amodigli Exp $
!+++ UVES_SAVCFG +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_savcfg.prg
!
!.KEYWORDS      keywords
!
!.USAGE         @p (IDENT) setup order type catg
!
!.INPUT         setup	(P1): name of setup table
!               order	(P2): name of order table
!               type	(P3): output type: FITS
!				default saves only in MIDAS internal format.
!               catg	(P4): name of order table
!
!.OUTPUT        
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT	P2	{ORDTAB}
!		P3	+
!		P4	+
!
!.PURPOSE       Save all global keywords belonging to the echelle context
!		in an extra setup table. This table may be re-loaded by
!		other reduction recipes in order to have a well defined
!		keyword-environment.
!		Set P1 to "+" in order to save tables (order-, background-,
!		line-tables) in FITS format having the product catagory passed
!		by parameter P4.
!		Use an appropriate image frame ind order to produce a FITS
!		format file having the correct primary header for proper
!		identification in the pipeline-context (the pipeline needs
!		various other FITS header keywords to find matching reference
!		frames except for the product catagory (ESO.PRO.CATG)!).
!		Add as a 4th argument the product classification if necessary.
!		The product classification will be stored in the hierarchical
!		header keyword ESO.PRO.CATG.
!
!		Examples:
!		---------
!		1) Create an order table in FITS format adding product catag.
!		   SAVE/DRS + order.tbl FITS ORDER_TABLE_BLUE
!		2) Same as before but with a pipeline-conform FITS header.
!		   rbf_0_b.bdf should have been the input frame from which
!		   the order table had been produced -- an order flatfield.
!		   SAVE/DRS rbf_0_b.bdf order.tbl FITS ORDER_TABLE_BLUE
!
!.ALGORITHM     Takes a copy of the order table in order to guarantee a
!		proper header (ESO-DICB-conform). Removes all table rows
!		from the copy. Use SAVINI/ECHELLE to save all keywords as
!		descriptors. Finally, the copy represents the setup table.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/03/22  Created.
!		    1999/04/14	FITS mode added.
!		    1999/05/05	Set P1=+ in order to create FITS tables
!		    1999/05/20	Set P1=image in order to create pipeline-
!				conform FITS files (primary header having
!				all FITS keywords necessary for identification)
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.3 $
! $Log: uves_savcfg.prg,v $
! Revision 1.3  2002/06/03 16:20:56  amodigli
! Some changes to support FIBER mode DRS tab
!
! Revision 1.2  2001/01/04 10:39:31  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------


define/param p1 ? c "Enter name of setup table:"
define/param p2 {ORDTAB} c "Enter name of order table:"
define/param p3 + c "Enter FITS in order to save as a FITS file, too:"
define/param p4 + c "Enter ESO.PRO.CATG entry for specified table:"

define/local ofrm/c/1/100 " " ALL
define/local mode/c/1/100 " " ALL
define/local ftyp/i/1/1 0

if p1(1:1) .ne. "+" then

  if m$exist(p1) .eq. 0 then			!Create the DRS setup table
    copy/tab {p2} {p1}

    show/table {p1} >Null
    delete/row {p1} @1..{outputi(2)}

    savini/echelle {p1} WRITE
    ofrm = p1
  else						!Check if it's an image
    ftyp = m$filtyp(p1,".bdf")
    if ftyp .eq. 2 then				!It's a table
      if m$existd(p1,h_procatg) .gt. 0 then	!Check first if it is a DRS tbl
	ofrm = "{{p1},{h_procatg}}"
        !Added support of FIBRE MODE
        if m$existd(p1,h_insmode) .gt. 0 then
           mode = "{{p1},{h_insmode}}"
           if mode .eq. "FIBRE" then
              !w/o {ofrm}
	      if ofrm(4:6) .ne. "DRS" goto ERROR
           else
	      if ofrm(1:3) .ne. "DRS" goto ERROR
           endif
	   savini/echelle {p1} WRITE
	   ofrm = p1
        endif
      endif
    else
      ofrm = p2
      copy/dd {p1} *,3 {ofrm}			!Copy the header of p1
      if p4(1:1) .ne. "+" classify/product {ofrm} {p4} + +
    endif					!-> necessary for pipeline
  endif						!   identification.
	
else						!Just create a FITS-file
  ofrm = p2					!of the input table: {p2}
endif

if m$existd(ofrm,h_procatg) .eq. 0 then
  if p4(1:1) .ne. "+" classify/product {ofrm} {p4} + +
endif

if P3(1:1) .eq. "F" then
  create/image empty 0 ? no			!Yes.
  copy/dd {ofrm} eso.* empty			!copy all ESO.xyz desc
  copy/dd {ofrm} instrume empty			!copy desc INSTRUME
  copy/dd {ofrm} exptime empty			!copy desc EXPTIME
  @p uves_parsepath {ofrm}
  outdisk/sfits empty.bdf,{ofrm} {q2}.tfits	!convert to single FITS
  -delete empty.bdf
endif

return 0

ERROR:
  mess/out 1 save "Only DRS setup tables may be used for update !"
  return -1
