! $Id: uves_selthar.prg,v 1.2 2001/01/04 10:39:31 amodigli Exp $
!+++ UVES_SELTHAR +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_selthar.prg
!
!.KEYWORDS      THAR arc lamp catalogue, uves physical model, spectroscopy, 
!		echelle,
!
!.USAGE         @p (IDENT) wlc LABCAT
!
!.INPUT         wlc     (P1): ThAr format check frame
!				These are ThAr exposures obtained with a small
!				slit length.
!		labcat	(P2): Line catalog: laboratory list with at least: 
!				:WAVE,:INT,:ION
!			(P3): OrderO.cat to save product
!			(P4): Measurement box (X,Y)
!			(P5): Translation of the ccd (X,Y)
!			(P6): Angle offsets (echelle, CD, CCD-rot)
!
!.OUTPUT        Table with selected lines, name thar{ord}sel  ({ord} = order nr)
!		TAKE CARE IF THE SAME ORDER IS OBSERVED ON DIFFERENT DETECTORS!
!		:select = 1  line completely unblended
!		:select = 2  line blending negligible (posit. error < 0.005 pix)
!
!		Table with blended lines, name thar{ord}all    TAKE CARE ....
!		:select = -1 heavily blended line, should not be used 
!		:nblend   # blending lines within [-8,8] unit: 
!			  sigma_gauss-linewidth
!		:iblmax   relat. intens. strongest blending line
!			  relat. intens. = I(bl.line)/I(considered line)
!		:nblend3  # blending lines within [-3,3]
!		:iblend3  sum of relat. intens. lines contrib. to :nblend3
!		:nblend6  # blending lines within [-8,-3]U[3,8]
!		:iblend6  sum of relat. intens. lines contrib. to :nblend6
!
!
!.RETURN        Q1:  0: successful return
! 		    -1: a successful return failed
!
!.DEFAULT	P1	{wlc}
!		P2	{LABCAT}
!		P3	+
!		P4	0.,0.
!		P5	0.,0.
!		P6	0.,0.,0.
!
!.PURPOSE: 	Selection of a TH and AR arc lamp lines for wavelength 
!		calibration taking into account the current configuration.
!
!.ALGORITHM	. Main routine:
!		Define global echelle keywords before calling 
!		the subroutines: "uves_psf.prg,", "uves_phymod.prg", 
!		"uves_selthar,selthar" 
!		do classification of the products and store them in
!		DB catalogue. 
!		. The "ENTRY selthar" is based on the algorithm of Herman 
!		HENSBERGE of Royal Observatory of Belgium.
!		. REFERENCE: Wavelength calibration at moderately high 
!		resolution, J.-P. De Cuyper and H. Hensberge, 
!		Astronomy & Astrophysics,   
!		Supplement series 128, 409-416 (1998).
! 
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        O.Boitquin, ESO/DMD
!
!.VERSION       1999/10/04  Created (OB).
!
!.COMMENT	. Parameters P5 and P6 are the parameters used by the
!		physical model routine uves_physmod.prg
!		. Comment of the purpose:
!		Selection of lines that are completely unblended or 
!		negligibly blended, taken into account the specific, 
!		local spectral format on the detector.
!		
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_selthar.prg,v $
! Revision 1.2  2001/01/04 10:39:31  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!
define/local   LABCAT/C/1/30   tharH.tbl ? +lower_levels
define/local   THARSEL/C/1/30  tharsel.tbl ? +lower_levels
define/param P1 {WLC} 	    IMA    "Input frame:"
define/param P2 {LABCAT}    CHAR   "Enter name of laboratory Catalogue"
define/param P3 +           CHAR   "OrderO.cat to save product"
define/param P4 0,0         NUMB   "Measurement box (X,Y): "
define/param P5 0,0         NUMB   "Translation of the ccd (X,Y): "
define/param P6 0,0,0       NUMB   "Angle offsets (echelle, CD, CCD-rot): "

define/local MBOX/R/1/2   {P4}
define/local trans/D/1/2  {p5}
!define/local angoff/D/1/3 {P6}

define/local wcen/R/1/1   0.0
define/local wmin/R/1/1   0.0
define/local wmax/R/1/1   0.0
define/local modext/c/1/25 " " ALL

define/local K/I/1/1  0
define/local disp/C/1/3 " " ALL
define/local fitreg/C/1/3 " " ALL
define/local loop/i/1/1 0
define/local ord/i/1/1 0

! For messout

define/local pid/C/1/20 "uves_selthar"

config/instr {p1}  ! to define PATHID and UVES_ARM
modext = q2

set/format F4.0

wcen = WCENTER/10.0
set/echelle wlc={P1} !lincat={p2}
set/echelle ordtab=o{wcen}{modext(2:)}.tbl baktab=b{wcen}{modext(2:)}.tbl
set/echelle lintab=l{wcen}{modext(2:)}.tbl
mess/out 3 {pid} "Start prediction of {p1} ..."
mess/out 3 {pid} "Use line reference list: {LABCAT}"

set/format F5.3
assign/disp 1
!CREATE/grap 0 1000,707
load/lut heat
!disp/lut
load/image {wlc} scale=f cen=c cuts=f,ihap
!load {wlc} scale=-8,-4  cuts=180,300

@p uves_psf
wmin = outputd(12) * 10.0 ! in Angstroem
wmax = outputd(13) * 10.0 ! in Angstroem
SELECT/TABL {LABCAT} :WAVE.GT.{WMIN}.AND.:WAVE.LT.{WMAX}
COPY/TABL {LABCAT} tharpart

! Runs the physical model and creates the line table
@p uves_phymod.prg -
   {wlc} tharpart {mbox(1)},{mbox(2)} {trans(1)},{trans(2)} {P6}

if Q1 .eq. "1" then
  mess/out 1 {pid} "Error occuring in the physical model. Abort..."
  return 1
endif

delete/column pline :LINELENPX :RESOL
join/tab tharpart :WAVE pline :ident {THARSEL}
name/column {THARSEL} :INT_1 :INT
name/column {THARSEL} :ION_1 :ION
name/column {THARSEL} :WAVE_2 :WAVE
name/column {THARSEL} :ORDER_2 :ORDER
name/column {THARSEL} :XMOD_2 :XMOD
name/column {THARSEL} :YMOD_2 :YMOD
name/column {THARSEL} :BIN_SIZE_2 :BIN_SIZE
name/column {THARSEL} :LINEWIDPX_2 :LINEWIDPX
name/column {THARSEL} :LINEWIDTH_2 :LINEWIDTH 
name/column {THARSEL} :BLAZE_2 :BLAZE
name/column {THARSEL} :IDENT_2 :IDENT 

!@@ ../proc/uves_selthar,selthar {THARSEL} modfree_sp_rg.tbl 23
do loop = 1 {modfree_sp_rg.tbl,TBLCONTR(4)} 1
  !@@ ../proc/uves_selthar,selthar {THARSEL} modfree_sp_rg.tbl {loop}
  set/format I3 F4.0
  ord = {modfree_sp_rg.tbl,:ORDER,@{loop}}
  
  sel/tab thar{ord}all :SELECT.gt.0.and.:SIGCEN.lt.0.05
  COPY/TAB thar{ord}all thar{ord}sel
  
  if loop .eq. 1 then
    copy/tab thar{ord}sel thar{wcen}_{ccd}
  else
    copy/tab  thar{wcen}_{ccd} thartmp
    merge/tab thartmp thar{ord}sel thar{wcen}_{ccd}
  endif
  
enddo

sort/tab thar{wcen}_{ccd} :wave

@@ ../proc/uves_selthar,reldiff thar{wcen}_{ccd}.tbl

!..............................................................................

ENTRY selthar

DEFINE/PARAM P1 {THARSEL}    	   TAB   "Enter name REFERENCE Catalogue"
DEFINE/PARAM P2  modfree_sp_rg.tbl TAB   "Enter name free spectral range tab"
DEFINE/PARAM P3  ?		   NUMB "Relative Order Number ?"
!DEFINE/PARAM P4  ?	   TAB   "Enter name selected line output Catalogue"

set/format I3
!THARSEL = {p1}
!!! pline.tbl must contains :WAVE (in nm), :IDENT (in A), :INT, :ION !!!
define/local k/i/1/1 0        			! cycle counter 
define/local relord/i/1/1 {p3}   		! relative order
define/local ord/i/1/1 {{P2},:ORDER,@{relord}}	! order nr., see keys p* ... 
						!!! FORMAT I3 !!!
define/local lam1/d/1/1 {{P2},:START,@{relord}}	! start wavelength (nm)
define/local lam2/d/1/1 {{P2},:END,@{relord}}	! end wavelength (nm)
define/local lams1/d/1/1 0.0  ! selection limit related to lam1 (A)
define/local lams2/d/1/1 0.0  ! selection limit related to lam2 (A)
define/local wpsf/r/1/1 2.5   ! PSF width (sigma) in PIXEL units (or linewidth)
define/local binsize/r/1/1 0.025 ! bin_size in Angstroem per pixel
define/local wth/r/1/1 0.     ! rough natural width Th lines (sigma)
define/local war/r/1/1 0.     ! rough natural width Ar lines (sigma)
define/local arth/r/1/1 1.0   ! multiplication factor for laboratory 
			! Ar intensities (relative to laboratory Th intensities)
			! DEFAULT=1.0
define/local photon/r/1/1 500 ! how many photons for int=1 ?

!                             ! used to cut-off faint lines
! Natural line widths: 
!  compute very rough natural width ratio (following lambda*v/c)
!  Should be computed for lamp temperature and pressure, presently the 
!  width for Th lines is pre-set at 0.4 km/s / c and only the 
!  ratio war/wth is computed in accord with the mass ratio between the 
!  two elements (assumption that thermal broadening dominates)
!  (Strictly speaking, the width should be computed line by line from 
!  their broadening parameters)
wth = 0.4/300000.  ! without unit
war = 2.41*{wth}   ! without unit (2.41 = sqrt(mass[Th]/mass[Ar])

! Select wavelength region corresponding to order
lams1 = {lam1}*10.-3   ! safety to include blending lines
lams2 = {lam2}*10.+3   ! safety to include blending lines
SEL/TAB {THARSEL} :IDENT.gt.{lams1}.and.:IDENT.lt.{lams2}
if {outputi(1)} .eq. 0 goto fini  ! Check if selection is empty
COPY/TAB {THARSEL} tharwork
SORT/TAB tharwork :WAVE

! Adapt Ar to Th strengths by factor arth (default = 1)
! width of PSF+natural line width (at present wpsf is taken from physical model):
! (Total width)**2 = (PSF width)**2 + (natural width)**2. 
SEL/TAB tharwork :ION.eq."TH*" {SESSOUTV}
COPY/TAB tharwork tharworkth
CRE/COL tharworkth :ID I*4 I5
COMP/TAB tharworkth :ID = 1   ! for Th lines
COMP/TAB tharworkth :WIDTH = 1000.0*{wth}*:IDENT/:BIN_SIZE
COMP/TAB tharworkth :WIDTH = sqrt(:LINEWIDPX*:LINEWIDPX+:WIDTH*:WIDTH)

SEL/TAB tharwork :ION.eq."AR*" {SESSOUTV}
COPY/TAB tharwork tharworkar
COMP/TAB tharworkar :INT = :INT*{arth}
CRE/COL tharworkar :ID I*4 I5
COMP/TAB tharworkar :ID = -1  ! for Ar lines
! rem. Th -- +1, Ar -- -1 allows to 'compute' which blends are mixed if 
! we would like later to do so
COMP/TAB tharworkar :WIDTH = 1000.0*{war}*:IDENT/:BIN_SIZE
COMP/TAB tharworkar :WIDTH = sqrt(:LINEWIDPX*:LINEWIDPX+:WIDTH*:WIDTH)

MERG/TAB tharworkth tharworkar tharwork
SORT/TAB tharwork :WAVE
DEL/TAB tharworkth no
DEL/TAB tharworkar no

arth:
! Provide necessary structure for tharwork.tbl
CRE/COL tharwork :SELECT I*4 I3
COMP/TAB tharwork :SELECT = -1     ! -1 not sel., >0 sel.
!cre/col tharwork :ORDER I*4 I3    ! Yet created by physical model
!comp/tab tharwork :ORDER = {ord}

! Blending lines within [-8,8] unit: sigma_gauss-linewidth
CRE/COL tharwork :NBLEND I*4 I6     ! nr. blending lines within 8 sigma
COMP/TAB tharwork :NBLEND = 0
COMP/TAB tharwork :IBLMAX = 0.      ! relative intensity of strongest blending line
                                    ! relat. intens. = I(bl.line)/I(considered line)

! Blending lines within [-3,3]
CRE/COL tharwork :NBLEND3 I*4 I7    ! nr. blending lines within 3 sigma
COMP/TAB tharwork :NBLEND3 = 0
COMP/TAB tharwork :IBLEND3 = 0.     ! sum of relat. intens. lines contrib. to :nblend3 lines

! Blending lines within [-8,-3]U[3,8]
CRE/COL tharwork :NBLEND6 I*4 I7    ! nr. blending lines within 3-6 sigma
COMP/TAB tharwork :NBLEND6 = 0
COMP/TAB tharwork :IBLEND6 = 0.     ! sum of relat. intens. lines contrib. to :nblend6 lines

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! CYCLE OVER ALL LINES IN THE CHOSEN WAVELENGTH RANGE
DO k = 1 {tharwork.tbl,tblcontr(4)}
  IF {tharwork,:WAVE,@{k}} .lt. {lam1} GOTO next  ! start at lam1
  IF {tharwork,:WAVE,@{k}} .gt. {lam2} GOTO done  ! list finished
! Compute separation between lines, relative to considered line and normalized 
! to larger involved line width, in pixel units
  !!comp/tab tharwork :DLINE = (:IDENT-{tharwork,:IDENT,@{k}})/{tharwork,:BIN_SIZE,@{k}}
  !comp/tab tharwork :DLINE = {tharwork,:XMOD,@{k}}-:XMOD ! OR with predicted position X
  !comp/tab tharwork :DLINE = :DLINE/max(:WIDTH,{tharwork,:WIDTH,@{k}})
  COMP/TAB tharwork :DLINE = ({tharwork,:XMOD,@{k}}-:XMOD)/max(:WIDTH,{tharwork,:WIDTH,@{k}})
  SEL/TAB tharwork abs(:DLINE).le.8.and.SEQ.ne.{k} {SESSOUTV} ! lines within 8 sigma
! Check if SELECTION IS EMPTY, then we have a perfectly ISOLATED LINE
  IF {tharwork.tbl,tblcontr(10)} .ne. 0 GOTO test
! ISOLATED LINE, select put to +1
  tharwork,:SELECT,@{k} = 1       ! Perfectly ISOLATED LINE
 GOTO next

 test:
! Check if all blending lines are sufficiently faint 
! to neglect their influence on position of considered line
  COPY/TAB tharwork thartest
  SEL/TAB tharwork all
! Check whether blending lines are near & strong; compute intensity ratio
! to the considered line
  COMP/TAB thartest :INTRATIO = :INT/{tharwork,:INT,@{k}}
! decision whether blending is acceptably small is based on a relation 
! derived from the above test with 2 gaussian lines  
  COMP/TAB thartest :TEST = 5.5+log10(2.*:INTRATIO)-:DLINE !!! rem.: 5.5 sigma
! Expressing that the acceptability limit for the intensity ratio depends 
! on the separation from the considered line.  
! :TEST < 0 in the case of QUASI-ISOLATION 
  STAT/TAB thartest :TEST  {SESSOUTV}! determines the maximum of :TEST
  IF {outputr(2)} .gt. 0.0 GOTO test2 ! stronger blending :TEST >= 0 for 
! at least one blending line 
  tharwork,:SELECT,@{k} = 2       ! QUASI-ISOLATED LINE or weak blending line
! Accept the line, but put :select = 2 that the line is not completely 
! unblended 

 test2:
! For possible later more stringent tests, 
! save information on badly blended lines
! (a) all within 8 SIGMA
  stat/tab thartest :INTRATIO {SESSOUTV}
  tharwork,:NBLEND,@{k} = {outputi(2)}
  tharwork,:IBLMAX,@{k} = {outputr(2)}
! (b) nearer than 3 SIGMA
  sel/tab thartest abs(:DLINE).lt.3
  stat/tab thartest :INTRATIO {SESSOUTV}
  tharwork,:NBLEND3,@{k} = {outputi(2)}
  tharwork,:IBLEND3,@{k} = {outputr(3)}  ! rem.: intensity summed out of DO
! (c) 3 to 6 SIGMA
  sel/tab thartest abs(:DLINE).ge.3. 
  stat/tab thartest :INTRATIO {SESSOUTV}
  tharwork,:NBLEND6,@{k} = {outputi(2)}
  tharwork,:IBLEND6,@{k} = {outputr(3)}  ! rem.: intensity summed out of DO
 next:
ENDDO

done:

! change average intensity to summed intensity
COMP/TAB tharwork :IBLEND3 = :IBLEND3*:NBLEND3
COMP/TAB tharwork :IBLEND6 = :IBLEND6*:NBLEND6

! Check usefullness with regard to line strength
! Using conversion factor "{photon}" the accuracy on the line center 
! is estimated: 
! VARIANCE = 0.17*FWHM_in_pix_units/I_center_in_photon_units.
! Formula which is valid for well-sampled lines.
! Presently, line intensities are converted to photons.
! BLAZE FUNCTION IS TAKEN INTO ACCOUNT, (instrument model dependent)
! to introduce at least approximative with a kind of mean normalized blaze 
! function (essentially a sinc-square dependency).
! :BLAZE gives the relation in the centre of the order and multiplication with a 
! model for the sensitivity along the order. 
COMP/TAB tharwork :SIGCEN = sqrt(0.17*2.35482*:WIDTH/({photon}*:INT*:BLAZE)) 

! Rename to output tables

! (a) full info
rena/tab tharwork thar{ord}all
! provide better output format (some of these columns could be deleted...)
name/col thar{ord}all :WIDTH F7.4
name/col thar{ord}all :IBLMAX F8.3
name/col thar{ord}all :IBLEND3 F8.3
name/col thar{ord}all :IBLEND6 F8.3
name/col thar{ord}all :SIGCEN F6.4
!del/col thar{ord}all :DLINE

! (b) useful & sufficiently strong lines
! (if not sufficient lines remain, the "0.03" pix might be relaxed, but 
! some weighting might be necessary in the wavelength calibration fitting 
! in order not to deteriorate the data from precise strong lines with 
! data from very weak lines (in the experience, up to 0.03 pix, weighting 
! is unimportant because not all systematic effects on this level are 
! eliminated)
SEL/TAB thar{ord}all :SELECT.gt.0.and.:SIGCEN.lt.0.05
! Copy work-table to the 2 output tables (all lines / not-too-weak tolerably 
! blended lines)  
COPY/TAB thar{ord}all thar{ord}sel 
DEL/TAB thartest no


fini:

return

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 

ENTRY reldiff

def/para P1 ? TAB  "Line catalog"

def/loc loop/i/1/1 0
def/loc previous/i/1/1 0
def/loc val1/R/1/1 0.
def/loc val2/R/1/1 0.
def/loc diff/R/1/1 0.

creat/col {P1} :diff R*8 F7.4
!show/table {P1}

do loop = 2 {{p1},tblcontr(4)}  !{outputi(2)}
 previous = loop -1
 val1 = {{P1},:wave,@{previous}}
 val2 = {{P1},:wave,@{loop}}
 diff = val2-val1
 {P1},:diff,@{loop} = {diff}
enddo

select/table {P1} :diff.gt.0.001
COPY/TABL {p1} {p1}_1


!..............................................................................
