! $Id: uves_shortlog.prg,v 1.2 2001/01/04 10:39:30 amodigli Exp $
!+++ UVES_SHORTLOG ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT	(C) 1998  European Southern Observatory
!
!.IDENT		uves_shortlog.prg
!
!.KEYWORDS	Create a short log file of reduction recipes
!
!.USAGE		@p (IDENT) mode logentry
!
!.INPUT		mode  	(P1): 1: open and add entry to log
!			      0: close and put log on terminal (for superrec.)
!			     -1: close and put log on terminal (for norm.rec.)
!				 If a normal recipe is called from a super
!				 recipe then the output and the closing of the
!				 log-file will be suppressed.
!			     -2: Puts the log of the previously closed log file
!				 on the terminal (in any case).
!				 
!		entry   (P2): entry to add
!
!.OUTPUT	
!
!.RETURN	Q1:  0: successful return
!		    -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE	Create a short log file of reduction steps
!
!.ALGORITHM	Short log file means, that for each reduction step only ONE
!               text line should be entered.
!
!.ENVIRON	UVES context
!   
!.LANGUAGE	MIDAS 98NOV
!
!.AUTHOR	Sebastian Wolf, ESO-DMO
!
!.VERSION	1.0 1998/12/17	Creation
!		    1999/03/10	Write shortlog to HISTORY buffer to frame
!				specified in P2 when P1=-1.
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_shortlog.prg,v $
! Revision 1.2  2001/01/04 10:39:30  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

define/param P1 ? i "Enter mode:"
define/param P2 + c "Enter log entry (filename when P1=-1):"
!read/key p1,p2,pipeline
define/local procid/c/1/20 "shortlog"

define/local mode/i/1/1 -1
define/local date/c/1/30 " " ALL

define/local i1/i/1/1 0
define/local fid/i/1/2 0 ALL
define/local SKIP/i/1/1 0
define/local line/c/1/100 {P2}

mode = {P1}
if line(1:1) .eq. "+" line(1:1) = " "	!assume P2 holds filename of main prod.

info/key SLOG_ID
if mid$info(1) .eq. 0 write/key SLOG_ID/i/1/2 0 ALL
info/key SLOG_NAME
if mid$info(1) .eq. 0 write/key SLOG_NAME/c/1/100 " " ALL

					!pipeline(6)=1 -> SUPER-recipe
SKIP = pipeline(6) * m$abs(mode)	!controls terminal output of log.
!if mode .eq. 0 .AND. pipeline(6) .eq. 0 SKIP = 1

branch mode -2,-1,0,1 OUT,CLO,CLO,ADD

mess/out 1 {procid} "Wrong mode selected (-2,-1,0,1 are allowed) !"
goto FIN
     
CLO:
     if SLOG_ID(1) .le. 0 then
       mess/out 1 {procid} -
         "No log file had been opened or\nan error occured while opening !"
       return -1
     else
       if SKIP .eq. 1 goto FIN
       mess/out 4 {procid} "Close log file: {SLOG_NAME}"
       write/key outputc "." ALL
       write/file {SLOG_ID(1)} {outputc(1:50)}
       close/file {SLOG_ID(1)}
       write/key SLOG_ID/i/1/2 0 ALL
       @p uves_shortlog,TODESCR {procid} {line}
     endif
     goto FIN
!     if pipeline(6) .eq. 1 goto FIN	!in super-recipe mode do no output.
					!call extra using mode=-2
OUT:
     open/file {SLOG_NAME} R fid
     write/out " "
!     append/out "Reduction recipe log:\n"
     set/format I1
     i1 = 0
loop:
     read/file {fid(1)} inputc
     if fid(2) .eq. -1 then
       close/file {fid(1)}
       mess/out 3 {procid}
       goto FIN
     endif
     if m$len(inputc) .eq. 0 goto loop		!skip empty lines
     if inputc(1:5) .eq. "....." goto loop	!.. and: ----- (last line)
     if i1 .eq. 0 then
       append/out "{inputc}\n    "
     else if i1 .lt. 10 then
       append/out " {i1}: {inputc}\n    "
     else
       append/out "{i1}: {inputc}\n    "
     endif
     i1 = i1 + 1
     goto loop
     
ADD: 
     if SLOG_ID(1) .eq. 0 then
       date = m$time()
       @p uves_shortlog,REPL_CH blank "{date}" _
       @p uves_shortlog,REPL_CH , {outputc} _
       SLOG_NAME = {outputc}//".log"
       mess/out 4 {procid} "Open log file: {SLOG_NAME}"
!       open/file {SLOG_NAME} WRITE SLOG_ID
       open/file {SLOG_NAME} WRITE fid
       SLOG_ID(1) = fid(1)
       SLOG_ID(2) = fid(2)
       write/file {SLOG_ID(1)} "Recipe log ({date}):"

     endif
     write/file {SLOG_ID(1)} {line}

     goto FIN

FIN:
return




!==============================================================================
!=== SUBROUTINES ==============================================================
!==============================================================================

ENTRY REPL_CH

define/param P1 ? c "Enter character to replace: "
define/param P2 ? c "Enter string where character should be replaced: "
define/param P3 ? c "Enter new character: "


define/local i1/i/1/1 0
define/local len/i/1/1 0
define/local ch/c/1/4 " " ALL


len = m$strlen(P2)			!get length of string

i1 = m$index(P1,"blank")
if i1 .le. 0 ch = P1(1:1)		!if P1 not " " get replace character
					!from P1, ch is preset to blanks!

outputc = P2
do i1 = 1 len
  if outputc({i1}:{i1}) .eq. ch(1:1) outputc({i1}:{i1}) = P3(1:1)
enddo

return {len}


ENTRY TODESCR

define/local line/c/1/100 {P2}
define/local ext/c/1/100 " " ALL
define/local catal/i/1/1 0
@p uves_parsepath {line}
ext = m$upper(q3)
if ext(1:4) .ne. ".CAT" goto SINGLE

CATALOG:
  store/frame line {p2} 1 FINI

SINGLE:
if m$strlen(line) .ne. 0 then	!assume P2 holds filename of main prod.
  mess/out 4 {p1} "Write shortlog to HISTORY of `{line}`"
  delete/descr {line} HISTORY N
  copy/ad {SLOG_NAME} {line} SHORTLOG -1
endif
if ext(1:4) .ne. ".CAT" goto FINI
goto CATALOG
FINI:
