! $Id: uves_split.prg,v 1.11 2002/10/25 09:58:26 amodigli Exp $
!+++ UVES_SPLIT +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_split.prg
!
!.KEYWORDS      environment, I/O
!
!.USAGE         @p (IDENT) input catalog trim
!
!.INPUT         input   (P1): input frames may be specified in different ways:
!				a) as name.bdf MIDAS frame
!				b) as name.fits FITS file
!				c) as name.cat image catalog (FITS or BDF)
!		catalog	(P2): output catalog holding all processed frames.
!		trim	(P3): trim flag. Set to 1 to switch trimming ON, i.e.
!				all pre- and postscan regions will be cut off.r
!
!.OUTPUT        
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT	P2	split.cat
!		P3	1
!
!.PURPOSE       Converts FITS files to BDF files if necessary and splits the
!		BDF file into two equal sized parts. The input file will be
!		removed from disk and from the catalog. The catalog will be
!		updated with the two new frames.
!		Checks if there is a BLUE UVES exposure which has only one
!		detector chip so we do not have to split. In that case we
!		only transpose and flip the frame!
!
!.ALGORITHM     Identify the data type BLUE- or RED-arm exposures from UVES
!		In case of RED exposures the data has to be splitted into
!		two separate frames (for each CCD of the mosaic).
!		Moreover, the splitted frames have to be rotated so that the
!		echelle orders have a standard orientation:
!		increasing wavelength from bottom left to top right.
!		The BLUE exposures has no CCD mosaic. Thus, these only have to
!		be transformed to the standard orientation.
!		Finally, the classification of the transformed frames will be
!		done filling the {UVES_IDSTRNG} (= DO_CLASSIFICATION) using
!		the original classification followed by the suffix _BLUE,
!		_REDL, or _REDU, respectively. The classification is only done
!		if the {UVES_IDSTRNG} is missing otherwise it is assumed that
!		the data orgranizer (DO) has done its job properly.
!
!.ENVIRON       UVES, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1999/03/09  Created.
!		1.0 1999/04/12	Allow trimming
!               1.0 1999/06/25	Do the identification of the UVES arms due
!				to the number of chips ({h_nchips}).
!               1.0 1999/07/06	Replace . and - in output files. MIDAS has
!				problems to handle with such names!
!               1.0 1999/07/07	Character + is not allowed in input files!
!				Replace by _ !
!               1.0 1999/07/13	Crossref command implemented.
!               1.0 1999/10/12	Order flat and format check classification add.
!               1.0 1999/11/08	Science and standard star classification added.
!               1.0 1999/12/18	Save product-type frames in prod{p2}, i.e. all
!				the frames which have not to be transformed.
!               1.0 1999/12/19	SCREEN_FLAT added to DOCLASS.
!
!.COMMENT	This procedure assumes that the input frames are
!		NO window readouts perpendicular to the dispersion direction.
!		So, for UVES there should be no window readouts in X.
!		Especially important for the RED arm, as here the separation
!		of the 2 chips are allways assumed to be at the CENTER !
!
!		The ENTRY DOCLASS has to be adjusted according to the uves.bdd 
!		configuration file!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.11 $
! $Log: uves_split.prg,v $
! Revision 1.11  2002/10/25 09:58:26  amodigli
! Use MIDAS names for ext table if present.
! Better support of SimCal mode
!
! Revision 1.9  2002/06/03 16:24:12  amodigli
! Some changes in support of FIBRE mode
!
! Revision 1.8  2002/04/08 13:40:04  amodigli
! Fixed a bug presenting with odd/even/all fibreff
!
! Revision 1.6  2002/03/05 12:27:29  amodigli
! uves_tech should be visible to the whole proc
!
! Revision 1.5  2002/03/04 16:55:33  amodigli
! Supporting new FIBER mode frames (but not activated)
!
! Revision 1.4  2002/02/08 15:55:49  amodigli
! Added support of binary table and some FIBRE
! mode frames
!
! Revision 1.3  2001/01/04 10:39:30  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

crossref input catalog trim
!
define/param p1 ? c "Enter the name of input frame (BDF or FITS) or catalog: "
define/param p2 split.cat c "Enter the name of output catalog:"
define/param p3 1 n "Enter trimming flag: 1=ON, 0=OFF:"
!
! Constants:
!
define/local pid/c/1/30 "uves_split" ? +lower_levels
!
! Variables:
!
define/local inspath/c/1/25 " " ALL
define/local ifrm/c/1/100 " " ALL
define/local base/c/1/100 " " ALL
define/local ext/c/1/100 " " ALL
define/local detname/c/1/25 " " ALL
define/local do_class/c/1/25 " " ALL
define/local catal/i/1/1 0
define/local nchip/i/1/1 0			!number of detector chips
define/local num/i/1/1 0			!number of entries in catalog
define/local trim/i/1/1 {p3}
define/local x1/i/1/1 1
define/local y1/i/1/1 1
define/local x2/i/1/1 0
define/local y2/i/1/1 0
define/local nx/i/1/1 0
define/local ny/i/1/1 0
define/local off/i/1/1 0
define/local nx2/i/1/1 0
define/local len/i/1/1 0
define/local i1/i/1/1 0
define/local fib_sw/i/1/1 0
define/local tim/i/1/1 0 ALL
define/local uves_tech/c/1/10 "ECHELLE" ? +lower_levels
tim = m$secs()


create/icat {p2} NULL {UVES_IDSTRNG} >Null
create/icat prod{p2} NULL {UVES_IDSTRNG} >Null
@p uves_parsepath {p1}				!get basename and extension
base = q2
ext  = m$upper(q3)
@p uves_split,REPL_CH . {base} _
base = outputc
@p uves_split,REPL_CH - {base} _
base = outputc
@p uves_split,REPL_CH + {base} _
base = outputc

if ext(1:5) .eq. ".FITS" then			!check if frame is a FITS-file
  mess/out 3 {pid} "Convert {p1}\nto {base}.bdf ..."
  mess/out 3 {pid} "and may separate the extention..."
  indisk/fits {p1} {base} !>Null		!... then convert to BDF
  if m$exist("{base}.tbl") .eq. 1 then
     mess/out 3 {pid} "and separate the extention..."
     !-rename "{base}.tbl" "fpt_{base}.tbl" 
  endif
  goto SINGLE
else if ext(1:4) .eq. ".BDF" then		!check if we already have BDF
  goto SINGLE
else if ext(1:4) .eq. ".CAT" then		!check if we got a catalog
  show/icat {p1} >Null
  num = outputi(1)
  if num .lt. 1 return -1			!no entries in the catalog
  goto CATALOG
else
  mess/out 1 {pid} "Unknown filetype: {ext}"
  return -1
endif

CATALOG:
  store/frame ifrm {p1} 1 FINI

  @p uves_parsepath {ifrm}
  base = q2
  ext  = m$upper(q3)
  @p uves_split,REPL_CH . {base} _
  base = outputc
  @p uves_split,REPL_CH - {base} _
  base = outputc
  @p uves_split,REPL_CH + {base} _
  base = outputc
  if ext(1:5) .eq. ".FITS" then
    mess/out 3 {pid} "Convert {ifrm}\nto {base}.bdf ..."
    indisk/fits {ifrm} {base} >Null
    if m$exist("{base}.tbl") .eq. 1 then
       mess/out 3 {pid} "and separate the extention..."
       !-rename "{base}.tbl" "fpt_{base}.tbl" 
    endif
  else if ext(1:4) .eq. ".TBL" then		!skip table catalog entries
    if m$existd(ifrm,h_procatg) .gt. 0 add/icat prod{p2} {ifrm} >Null
    num = num - 1
    if num .gt. 0 goto CATALOG
    goto FINI
  endif

SINGLE:
  if m$existd(base,h_procatg) .eq. 0 then	!only RAW frames have to be
						!processed !!!
    if NICE_CREA .eq. "Y" then
      mess/out 4 {pid}  "Display entire echelle spectrum"
      assign/disp d,1 >Null
      clear/channel overlay
      load/image {base} scale=f cuts=f,ihap >Null
    endif

    nx  = m$value("{base}.bdf",NPIX(1))			
    ny  = m$value("{base}.bdf",NPIX(2))
    if trim .eq. 1 then				!  P  CCD  O P  CCD  O
      configure/port {base}.bdf			! | |#####| | |#####| | RED-CCD
      image/section {base}.bdf			! | |#####| |           BLU-CCD
      x1 = outputi(1)				!image size:
      x2 = outputi(3)				![x1,x2] x [y1,y2]
      y1 = outputi(2)				!
      y2 = outputi(4)				!
      off = outputi(5)				!prescan offset
    else					!Do NO trimming
      x2 = nx					!--> take the full frame
      y2 = ny					!    for each chip
      off = 0					!    --> ignore prescan offset
    endif
    nx2 = nx / 2 + off + 1			!half of full frame + prescan
						!first image part.
    nchip = m$value("{base}.bdf",{h_nchips})	!Number of chips
    if nchip .eq. 2 then			!to identify RED or BLUE arm.
						!UVES RED arm

      mess/out 3 {pid} "Extract lower part of `{base}.bdf` ..."
      extract/frame  {base}.bdf {base}_l.bdf [@{nx2},@{y1}:@{x2},@{y2}] 4
      write/descr {base}_l.bdf start/D/1/2, 1,1
      write/descr {base}_l.bdf step/D/1/2, 1,1
      @p uves_split,DOCLASS {base}_l.bdf
      do_class = "{{base}_l,{UVES_IDSTRNG}}"
      len = m$strlen(do_class)
      i1  = m$indexb(do_class,"_") - 1
      if i1 .le. 0 i1 = len
      do_class = "{do_class(1:{i1})}{EXREDL}"
      {base}_l,{UVES_IDSTRNG} = "{do_class}"

      outputc = "lower part"
      if NICE_CREA .eq. "Y" -
        @p uves_split,PLOT {pid} {base}_l 2

      if trim .eq. 1 @p uves_split,RMSCAN {base}_l
      classify/product {base}_l {do_class} {d_pre} {d_empty} OK
  
						!second image part
      nx2 = nx2 - 1 - 2 * off
      mess/out 3 {pid} "Extract upper part of `{base}.bdf` ..."
      extract/frame {base}.bdf {base}_u.bdf [@{x1},@{y1}:@{nx2},@{y2}] 4
      write/descr {base}_u.bdf start/D/1/2, 1,1
      write/descr {base}_u.bdf step/D/1/2, 1,1
      @p uves_split,DOCLASS {base}_u.bdf
      do_class = "{{base}_u,{UVES_IDSTRNG}}"
      len = m$strlen(do_class)
      i1  = m$indexb(do_class,"_") - 1
      if i1 .le. 0 i1 = len
      do_class = "{do_class(1:{i1})}{EXREDU}"
      {base}_u,{UVES_IDSTRNG} = "{do_class}"

      outputc = "upper part"
      if NICE_CREA .eq. "Y" -
        @p uves_split,PLOT {pid} {base}_u 3

      if trim .eq. 1 @p uves_split,RMSCAN {base}_u
      classify/product {base}_u {do_class} {d_pre} {d_empty} OK

      if num .gt. 0 then			!we work in a catalog
	subtract/icat {p1} {base}.bdf >Null	!update catalog
	add/icat {p1} {base}_l.bdf >Null
	add/icat {p1} {base}_u.bdf >Null
      endif
      add/icat {p2} {base}_l.bdf >Null
      add/icat {p2} {base}_u.bdf >Null
      append/out "Frame `{base}.bdf` split into\n"
      append/out "`{base}_l.bdf` and `{base}_u.bdf`"

      if "{uves_tech(1:3)}" .eq. "MOS" then
         fib_sw = 1
      else  if m$existd("{base}_l.bdf","{h_dprtype}") .gt. 0 then
              if "{{base}_l.bdf,{h_dprtype}}" .eq. "LAMP,SFLAT" then
                  fib_sw = 1
         else if "{{base}_l.bdf,{h_dprtype}}" .eq. "LAMP,FLAT" then
                  fib_sw = 0      !to work with UVES-ECHELLE pipeline only!
         else
                  fib_sw = 0

         endif
      else

                  fib_sw = 0
      endif
      if fib_sw .eq. 1 then
            if m$exist("{base}.tbl") .gt. 0 then
               FIBMSK/FILL  {base}_l.bdf {base}.tbl 
               FIBMSK/FILL  {base}_u.bdf {base}.tbl
            else
               FIBMSK/FILL  {base}_l.bdf noext 
               FIBMSK/FILL  {base}_u.bdf noext
            endif
            write/des {base}_l.bdf CHIPCHOICE/c/1/1 "l"
            write/des {base}_u.bdf CHIPCHOICE/c/1/1 "u"
      endif
      mess/out 3 {pid} 


    else if nchip .eq. 1 then
						!BLUE part

      mess/out 3 {pid} "Transpose and flip frame `{base}.bdf`"
      extract/frame {base}.bdf {base}_b.bdf [@{x1},@{y1}:@{x2},@{y2}] 3
      flip/image {base}_b.bdf X
      write/descr {base}_b.bdf start/D/1/2, 1,1
      write/descr {base}_b.bdf step/D/1/2, 1,1
      @p uves_split,DOCLASS {base}_b.bdf
      do_class = "{{base}_b,{UVES_IDSTRNG}}"
      len = m$strlen(do_class)
      i1  = m$indexb(do_class,"_") - 1
      if i1 .le. 0 i1 = len
      do_class = "{do_class(1:{i1})}{EXBLUE}"
      {base}_b,{UVES_IDSTRNG} = "{do_class}"

      outputc = "blue part"
      if NICE_CREA .eq. "Y" -
        @p uves_split,PLOT {pid} {base}_b 2

      if trim .eq. 1 @p uves_split,RMSCAN {base}_b
      classify/product {base}_b {do_class} {d_pre} {d_empty} OK

      if num .gt. 0 then			!we work in a catalog
	subtract/icat {p1} {base}.bdf >Null	!update catalog
	add/icat {p1} {base}_b.bdf >Null
      endif
      add/icat {p2} {base}_b.bdf >Null
      append/out "Frame `{base}.bdf` transposed and rotated,\n"
      append/out "output to `{base}_b.bdf`"
      mess/out 3 {pid}

    endif

    -delete {base}.bdf				!remove original input frame
  else
    if m$index("{{base},{h_ident}}","MASTER_FORM") .gt. 0 then
       {base},{UVES_IDSTRNG} = "{{base},{h_ident}}   " 
    endif
    add/icat prod{p2} {base} >Null		!save the products in an extra
  endif						!catalog
  if num .eq. 0 goto FINI			!there was NO catalog

  num = num - 1					!count remaining catal. entries
  if num .gt. 0 goto CATALOG
FINI:

tim = m$secs() - tim 
set/format I1
mess/out 4 {pid} "Time used: {tim} s"



!=== SUBROUTINES =============================================================
ENTRY PLOT
define/param p1 ? c "Enter name of calling procedure:"
define/param p2 ? c "Enter name of frame to be displayed:"
define/param p3 ? n "Enter display number:"

mess/out 4 {p1} "Display {outputc}"
assign/disp d,{p3} >Null
clear/channel overlay
load/image {p2} scale=f cen=c cuts=f,ihap >Null
outputc = m$upper(outputc)
label/disp "{outputc}"  0.95,0.01 ? ? 2


ENTRY RMSCAN
define/param p1 ? c "Enter input frame:"

define/local i/i/1/1
define/local j/i/1/1

do i = 1 uves_nout
  j = uves_outi({i})
  delete/descr {p1} {h_pscan({j})} N
  delete/descr {p1} {h_oscan({j})} N
enddo


ENTRY REPL_CH

define/param P1 ? c "Enter character to replace: "
define/param P2 ? c "Enter string where character should be replaced: "
define/param P3 ? c "Enter new character: "


define/local i1/i/1/1 0
define/local len/i/1/1 0
define/local ch/c/1/4 " " ALL

len = m$strlen(P2)			!get length of string

i1 = m$index(P1,"blank")
if i1 .le. 0 ch = P1(1:1)		!if P1 not " " get replace character
					!from P1, ch is preset to blanks!

outputc = P2
do i1 = 1 len
  if outputc({i1}:{i1}) .eq. ch(1:1) outputc({i1}:{i1}) = P3(1:1)
enddo

return {len}


ENTRY DOCLASS
!This ENTRY has to be adjusted according to the uves.bdd configuration file!
!
define/param p1 ? c "Enter name of input file:"

define/local dprtype/c/1/25 " " ALL
define/local dprtech/c/1/25 " " ALL
define/local dpr_catg/c/1/25 " " ALL
define/local ins_mode/c/1/25 " " ALL

if m$existd(p1,UVES_IDSTRNG) .ne. 0 then
   mess/out 3 {pid} "DO_CLASS {{p1},{UVES_IDSTRNG}} present"
   if m$existd(p1,h_dprtype) .ne. 0 then
      dprtype = "{{p1},{h_dprtype}}"
         if m$index(dprtype,"OzPoz") .gt. 1 then 
                 !w/o "go to classify"
                 GOTO CLASSIFY
         endif
         if m$index(dprtype,"SimCal") .gt. 1 then  
                 !w/o "go to classify"
                 GOTO CLASSIFY
         endif
         if m$index(dprtype,"COMBINED") .gt. 1 then  
                 !w/o "go to classify"
                 GOTO CLASSIFY
         endif
         if m$index(dprtype,"LAMP,SFLAT") .gt. 1 then  
                 !w/o "go to classify"
                 GOTO CLASSIFY
         endif
   endif
endif
 
if m$existd(p1,UVES_IDSTRNG) .ne. 0 return {{p1},{UVES_IDSTRNG}}
					!valid DO-classification found, return.

CLASSIFY: 
if m$existd(p1,h_dprtech) .ne. 0 then
   if m$existd(p1,h_dprtype) .ne. 0 .and. "{{p1},{h_dprtype}}" .ne. "BIAS" then
      if m$existd(p1,h_insmode) .ne. 0 then
         ins_mode = "{{p1},{h_insmode}}"
      endif
  endif
endif
 
if m$existd(p1,h_dprtech) .ne. 0 then
  dprtech = "{{p1},{h_dprtech}}"
endif

if m$existd(p1,h_dprcatg) .ne. 0 then
  dpr_catg = "{{p1},{h_dprcatg}}"
endif
 
if m$index(dprtech,"MOS") .gt. 0 then
 
else
    if m$existd(p1,h_dprcatg) .ne. 0 then
      if m$index("{{p1},{h_dprcatg}}","SCIENCE") .gt. 0 then
        write/desc {p1} {UVES_IDSTRNG}/c/1/25 SCIENCE_
        return SCIENCE_
      endif
    endif
endif


if m$existd(p1,h_dprtype) .ne. 0 then
  dprtype = "{{p1},{h_dprtype}}"
else
  write/desc {p1} {UVES_IDSTRNG}/c/1/25 UVES
  return UVES
endif

if ins_mode(1:3) .eq. "BLU" then
   goto TECH_ECH  
else if ins_mode(1:3) .eq. "   " then
   goto TECH_ECH  
else if ins_mode(1:3) .eq. "RED" then
   goto TECH_ECH
else if ins_mode(1:3) .eq. "DIC"  then 
   goto TECH_ECH
else if m$index(dprtype,"BIAS") .gt. 0  then 
   goto TECH_ECH
else
   goto TECH_FIB
endif
 
 
TECH_ECH:
uves_tech = "ECHELLE"
if m$index(dprtype,"STD") .gt. 0 then
  write/desc {p1} {UVES_IDSTRNG}/c/1/25 STANDARD_
else if m$index(dprtype,"BIAS") .gt. 0 then
  write/desc {p1} {UVES_IDSTRNG}/c/1/25 BIAS_
else if m$index(dprtype,"DARK") .gt. 0 then
  write/desc {p1} {UVES_IDSTRNG}/c/1/25 DARK_
else if m$index(dprtype,"PARASITIC") .gt. 0 then
  write/desc {p1} {UVES_IDSTRNG}/c/1/25 PDARK_  
else if m$index(dprtype,"FLAT") .gt. 0 then
 if m$index(dprtype,"LAMP") .gt. 0 .AND. m$index(dprtech,"ECHELLE") .gt. 0 then
    if "{dprtype}" .eq. "LAMP,FLAT" then
       write/desc {p1} {UVES_IDSTRNG}/c/1/25 FLAT_
    else if "{dprtype}" .eq. "LAMP,IFLAT" then
       write/desc {p1} {UVES_IDSTRNG}/c/1/25 IFLAT_
    else if "{dprtype}" .eq. "LAMP,DFLAT" then
       write/desc {p1} {UVES_IDSTRNG}/c/1/25 DFLAT_
    else if "{dprtype}" .eq. "LAMP,SFLAT" then
       write/desc {p1} {UVES_IDSTRNG}/c/1/25 SFLAT_       
    endif  
  else if  m$index(dprtype,"LAMP") .gt. 0 .AND. m$index(dprtech,"IMAGE") .gt. 0 then
    write/desc {p1} {UVES_IDSTRNG}/c/1/25 SCREEN_FLAT_
  else
    write/desc {p1} {UVES_IDSTRNG}/c/1/25 UVES
  endif
else if m$index(dprtype,"LAMP") .gt. 0 then
  if m$index(dprtype,"WAVE") .gt. 0 then
    write/desc {p1} {UVES_IDSTRNG}/c/1/25 ARC_LAMP_
  else if m$index(dprtype,"ORDER") .gt. 0 then
    write/desc {p1} {UVES_IDSTRNG}/c/1/25 ORDER_FLAT_
  else if m$index(dprtype,"FMTCHK") .gt. 0 then
    write/desc {p1} {UVES_IDSTRNG}/c/1/25 ARC_LAMP_FORM_
  else
    write/desc {p1} {UVES_IDSTRNG}/c/1/25 UVES
  endif
else
  write/desc {p1} {UVES_IDSTRNG}/c/1/25 UVES
endif
return
 
TECH_FIB:
!w/o "dpr_tech {dprtech}"
uves_tech = "MOS"
 
if m$index(dprtech,"MOS") .gt. 0 then
!w/o "fiber-mode"
   if m$index(dprtype,"LAMP") .gt. 0 then
      if m$index(dprtype,"FLAT") .gt. 0 then
              if m$index(dprtype,"ALL") .gt. 0 then
            write/desc {p1} {UVES_IDSTRNG}/c/1/25 FIB_FF_ALL_
         else if m$index(dprtype,"ODD") .gt. 0 then 
            write/desc {p1} {UVES_IDSTRNG}/c/1/25 FIB_FF_ODD_
         else if m$index(dprtype,"EVEN") .gt. 0 then 
            write/desc {p1} {UVES_IDSTRNG}/c/1/25 FIB_FF_EVEN_
         else if m$index(dprtype,"ATTACH") .gt. 0 then
            write/desc {p1} {UVES_IDSTRNG}/c/1/25 FIB_FF_ATTACH_
         else if m$index(dprtype,"NASMYTH") .gt. 0 then
            write/desc {p1} {UVES_IDSTRNG}/c/1/25 FIB_FF_NASMYTH_            
         endif
      else if m$index(dprtype,"ORDERDEF") .gt. 0 then
         write/desc {p1} {UVES_IDSTRNG}/c/1/25 FIB_ORDEF_
      else if m$index(dprtype,"FMTCHK") .gt. 0 then
         write/desc {p1} {UVES_IDSTRNG}/c/1/25 ARC_LAMP_FORM_
      else if m$index(dprtype,"WAVE") .gt. 0 then
         write/desc {p1} {UVES_IDSTRNG}/c/1/25 FIB_ARC_LAMP_         
      endif
   else if m$index(dprtype,"OBJECT,SimCal") .gt. 0 then
      write/desc {p1} {UVES_IDSTRNG}/c/1/25 FIB_SCI_SIM_
   else if m$index(dprtype,"OBJECT,OzPoz") .gt. 0 then
      write/desc {p1} {UVES_IDSTRNG}/c/1/25 FIB_SCI_
   else if m$index(dprtype,"OBJECT,COMBINED") .gt. 0 then
      write/desc {p1} {UVES_IDSTRNG}/c/1/25 FIB_SCI_COM_      
   endif
else
  write/desc {p1} {UVES_IDSTRNG}/c/1/25 UVES
endif
 
return
