! $Id: uves_trans_cr.prg,v 1.2 2001/01/04 10:39:58 amodigli Exp $
!+++ UVES_TRANS_CR ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_trans_cr.prg
!
!.KEYWORDS      cosmic ray rejection, spectroscopy, echelle
!
!.USAGE         @p (IDENT) crmtab rawima [ordtab]
!
!.INPUT         crmtab  (P1): name of cosmic ray table
!               rawima  (P2): name of raw image where cosmics were detected.
!               ordtab  (P3): name of order table
!
!.OUTPUT        out_a: name of the transformed table (= x_{p1})
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT	P3	{ORDTAB}
!
!.PURPOSE       Transform cosmic ray rejection table into PIXEL-ORDER space.
!
!.ALGORITHM     Convert the cosmic ray rejection table (output of EXTRACT/OPTI
!		and REDUCE/UVES) into an image. Do an average order extraction
!		using EXTRACT/ORDER. Detect all pixel intensities greater than
!		0.0 which indicates a cosmic ray detection and store these in
!		the output table.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1999/11/12  Created.
!
!.COMMENT	The SLIT and OFFSET keyword have to be properly set!
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_trans_cr.prg,v $
! Revision 1.2  2001/01/04 10:39:58  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

crossref crmtab rawima
!
define/param p1 ? c "Enter name of bad pixel mask:"
define/param p2 ? c "Enter name of raw frame:"
define/param p3 {ORDTAB} c "Enter name of order table:"


show/table {p1} >Null
if outputi(2) .le. 0 goto ERR


-copy {p1} crmsk.tbl

create/colu crmsk.tbl :Z PIXEL >Null
comput/table crmsk.tbl :Z = 1.0
convert/table crmsk.bdf = crmsk.tbl :X,:Y :Z {p2} PLOT

extract/order crmsk.bdf x_crmsk.bdf {SLIT},0,{OFFSET} AVER {p3} COEFF
find/pixe x_crmsk.bdf <,0 OUT ALL x_{p1} >Null
comput/tabl x_{p1} :VALUE = 1.0

-delete crmsk.bdf crmsk.tbl x_crmsk.bdf

append/out "Bad pixel table {p1} transformed\n"
append/out "into PIXEL-ORDER space, Output: x_{p1}"
mess/out 3 trans

out_a = "x_{p1}"
return 0

ERR:
mess/out 1 trans "Bad pixel table `{p1}` is empty !"
return -1
