/* $Id: uves_corr_trap.c,v 1.2 2001/03/05 13:22:38 amodigli Exp $
/*+++ FORS_EXTRACT ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		uves_corr_trap
 
 .KEYWORDS	Corrects an image for trap column as specified in a table
 
 .USAGE		run PIPE_EXE:(IDENT)
 
 .INPUT		IN_A	   (char)  name of input frame 
                                   (biases,flats,science,std.star)
		
		IN_B       (char)  name of bad pixel table
                METH       (char)  name of correction method 
                                   L(inear) or R(random)

		MESS_LEVEL (int)   message level (??)

 .OUTPUT        OUT_A      (char)  name of output frame: cleaned frame.

 .RETURN	

 .PURPOSE	X
 
 .ALGORITHM	X
 
 .ENVIRON	MIDAS
    
 .LANGUAGE	C
 
 .AUTHOR	Andrea Modigliani, ESO-DMO
 
 .VERSION	1.0 2001/02/23	SW Creation.
 
 .COMMENT       

-----------------------------------------------------------------------------*/
/* ===========================================================================
 * $Name:  $
 * $Revision: 1.2 $
 * $Log: uves_corr_trap.c,v $
 * Revision 1.2  2001/03/05 13:22:38  amodigli
 * Small correction Ye>(Ymax-3) instead of
 * Ye>Ymax-3
 *
 * Revision 1.1  2001/03/05 07:00:57  amodigli
 * Created
 *
 * ===========================================================================
 */


/* include standard definitions */
#include <midas_def.h>      
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>



#define FILE_LEN 80                          /* max length of in files */
#define MESS_LEN 80                          /* max length of messages */
#define KEYW_LEN 80                          /* max length of keywords */
#define LABEL_LEN 20                         /* max length of a label  */
#define BUF_SIZE 12                          /* max size of ima buffer */

char prgName[FILE_LEN] = "uves_corr_trap";   /* program name */




#ifdef __STDC__          /* ANSI */

void 
main(int   argc, 
     char* argv[])

#else                    /* K&R */

void 
main(argc, argv)

int   argc;
char* argv[];

#endif
{



   char* bufData;
   char inIma[FILE_LEN];                     /* input File */
   char inTable[FILE_LEN];                   /* input Table */
   char cMethod[FILE_LEN];                   /* correction Method */
   char aLabel[LABEL_LEN];                   /* table label */
   char message[MESS_LEN];                   /* temp. string for outputs */


   int  messLevel = 0;                       /* message level */

  
   int  nPix[2];                             /* tmp variable for pixel data */
   int  nVal = 0;                            /* max no of returned chars */

   int  stat = 0;                            /* status */

   int  inImaId = 0;                         /* Id of input ima frame */
   int  inTabId = 0;                         /* Id of in ima frame */

   int  nCol = 0;                            /* No of columns */
   int  nRow = 0;                            /* No of rows */
   int  nSort = 0;                           /* sorted column */
   int  aCol = 0;                            /* No of all words per record */
   int  aRow = 0;                            /* No of allocated rows */

   int  valXs = 0;                           /* X start trap col value */
   int  valYs = 0;                           /* Y start trap col value */
   int  valXe = 0;                           /* X end trap col value */
   int  valYe = 0;                           /* Y end trap col value */

   int  colXs = 0;                           /* X start trap col value */
   int  colYs = 0;                           /* X start trap col value */
   int  colXe = 0;                           /* X start trap col value */
   int  colYe = 0;                           /* X start trap col value */

   int  unit = 0;                            /* address of unit pointer */
   int  null = 0;                            /* No of null values in data */


   int  imaSizeX = 0;                        /* size of ima along X in pix */
   int  imaSizeY = 0;                        /* size of ima along Y in pix */
   int  imaSizeT = 0;                        /* total ima size */

   float *pInImaF;                           /* image frame */
   float **mInImaF;                          /* matrix represent. */
   float *pOutImaF;                          /* image frame */
   float **mOutImaF;                         /* matrix represent. */   

   float **map2mat();                 /* map a matrix on a float vector */

   int SF = 0;                               /* alias for short float */
   int i  = 0;                               /* loop index */
   int j  = 0;                               /* loop index */

   SCSPRO(prgName);                    /* assign program name */
   SCKRDI("MESS_LEVEL",1,1,&nVal,&messLevel,&unit,&null); /* read keyword */
 
   stat = SCKGETC("in_a", 1,FILE_LEN,&nVal,inIma);   /* get input frame */
   stat = SCKGETC("in_b", 1,FILE_LEN,&nVal,inTable); /* get input table */
   stat = SCKGETC("method", 1,KEYW_LEN,&nVal,cMethod); /* get corr method */

  
   SCFOPN(inIma,D_R4_FORMAT,0,F_IMA_TYPE,&inImaId);   /* open input frame */
   SCDRDI(inImaId,"NPIX",1,2,&nVal,nPix,&unit,&null); /* get NPIX descript. */


   imaSizeX = nPix[0];
   imaSizeY = nPix[1];
   imaSizeT = imaSizeX*imaSizeY;
  
 
   /* check 
   sprintf(&message[0],"INFO: Method: %s",cMethod);  
   messout(messLevel,3,prgName,message);
   */



   /* check 
   sprintf(&message[0],"INFO: Ima par: %d %d %d",imaSizeX,imaSizeY,imaSizeT);  
   messout(messLevel,3,prgName,message);
   */
   SF = sizeof(float);

 
   /* do the mapping */
 
   pInImaF = (float*) malloc((size_t) imaSizeT*SF);
   if (pInImaF == NULL) no_memory("pInImaF",imaSizeT*SF);
   bufData = (char*) pInImaF;
   SCFGET(inImaId,1,imaSizeT,&nVal,bufData);    /* read the whole frame */
   /* map data for matrix access */
   mInImaF = map2mat(pInImaF,imaSizeX,imaSizeY,1,1); 
  
   /* get input table data */
   TCTOPN(inTable,F_I_MODE,&inTabId);
   TCIGET(inTabId, &nCol, &nRow, &nSort, &aCol, &aRow); 
   
   strcpy(aLabel,":XSTARTPIX");
   TCCSER(inTabId, aLabel, &colXs);
   strcpy(aLabel,":YSTARTPIX");
   TCCSER(inTabId, aLabel, &colYs);
   strcpy(aLabel,":XENDPIX");
   TCCSER(inTabId, aLabel, &colXe);
   strcpy(aLabel,":YENDPIX");
   TCCSER(inTabId, aLabel, &colYe);
 
   if(!strcmp(cMethod,"A")){   
     for (i=1; i<=nRow; i++) {
        TCERDI(inTabId, i, colXs, &valXs, &null);
        TCERDI(inTabId, i, colXe, &valXe, &null);
        TCERDI(inTabId, i, colYs, &valYs, &null);
        TCERDI(inTabId, i, colYe, &valYe, &null);
        corrAverage(mInImaF,valXs,valXe,valYs,valYe,imaSizeY);
     }
        sprintf(message,"Correction method: Average.");
        messout(messLevel,3,prgName,message);  
   
   }
   else if(!strcmp(cMethod,"L")){
     for (i=1; i<=nRow; i++) {
        TCERDI(inTabId, i, colXs, &valXs, &null);
        TCERDI(inTabId, i, colXe, &valXe, &null);
        TCERDI(inTabId, i, colYs, &valYs, &null);
        TCERDI(inTabId, i, colYe, &valYe, &null);
        corrLinear(mInImaF,valXs,valXe,valYs,valYe,imaSizeY);
     }
        sprintf(message,"Correction method: Linear.");
        messout(messLevel,3,prgName,message);  
   
   }
   else{
        sprintf(message,"Correction method not supported. No correction applied");
        messout(messLevel,2,prgName,message);  
   
   }

   /* release the corrected image on the input frame */
   SCFPUT(inImaId,1,imaSizeT,bufData);

   /* close */
   SCFCLO(inTabId);
   SCFCLO(inImaId);
   SCSEPI();


}

/*=========================================================================*/

#ifdef __STDC__         /* ANSI */
 
int 
corrAverage(float** ima,
            int     Xs,
            int     Xe,
            int     Ys,
            int     Ye,
            int     Ymax)

#else                   /* K&R */

int
corrAverage(ima,
            Xs,
            Xe,
            Ys,
            Ye,
            Ymax)

float** ima; 
int Xs;
int Xe;
int Ys;
int Ye;
int Ymax;

#endif

{
 
   int i=0;
   int j=0;
   int y1=0;
   int y2=0;
   /* check that the point is reasonably inside the image */
   findY1Y2(Ys,Ye,Ymax,&y1,&y2);
   /* printf("%d %d %d\n",y1,y2,Ymax); */
   for (j=Ys; j<=Ye; j++) {
      for (i=Xs; i<=Xe; i++) {
	 ima[j][i]=(ima[y1][i]+ima[y2][i])/2.;
         /*
         printf("%d %d %f %f %f \n",j,i,ima[j][i],ima[j-2][i],ima[j+2][i]);
	 */
      }
  
   }
   return(0);
}




#ifdef __STDC__         /* ANSI */
 
int 
corrLinear(float** ima,
           int     Xs,
           int     Xe,
           int     Ys,
           int     Ye,
           int     Ymax)

#else                   /* K&R */

int
corrLinear(ima,
           Xs,
           Xe,
           Ys,
           Ye,
           Ymax)

float** ima; 
int Xs;
int Xe;
int Ys;
int Ye;
int Ymax;

#endif

{
   int i=0;
   int j=0;
   int y1=0;
   int y2=0;
   float m =0;

   /* check that the point is reasonably inside the image */
   findY1Y2(Ys,Ye,Ymax,&y1,&y2);

   /*
   printf("Xs %d Xe %d Ys %d Ye %d\n",Xs,Xe,Ys,Ye);
   */
   for (j=Ys; j<=Ye; j++) {
      for (i=Xs; i<=Xe; i++) {
         m=(ima[y2][i]-ima[y1][i])/(y2-y1);
	 ima[j][i]=ima[y1][i]+m*(j-y1);
         /*
         printf("i=%d j=%d m=%f I=%f\n",i,j,m,ima[j][i]); 
	 */   
      }
  
   }
   return(0);
}

#ifdef __STDC__         /* ANSI */
int
findY1Y2(int Ys, 
         int Ye, 
         int Ymax, 
         int* y1, 
         int* y2)

#else                   /* K&R */
int
findY1Y2(Ys, 
         Ye, 
         Ymax, 
         y1, 
         y2)

int   Ys;
int   Ye;
int   Ymax;
int*  y1;
int*  y2;

#endif
{
   if (Ys<3){
     *y1=Ye+1;
     *y2=Ye+2;
   }
   else if (Ye>(Ymax-3)){
     *y1=Ys-2;
     *y2=Ys-1;
   }
   else{
     *y1=Ys-2;
     *y2=Ye+2;
   }

}
/* ============================= utilities ================================ */
#ifdef __STDC__             /* ANSI */

int 
no_memory(int   svar,
          char* num)      
#else                       /* K&R */

int 
no_memory(svar,
          num)

int   num;
char  *svar;

#endif

{


   char message[MESS_LEN];                  /* temp. string for outputs */
   int  messLevel;                             /* message level */
   int nbad, uni;

   nbad = 0;
   sprintf(message,"Cannot allocate memory for %s [%d]!\n",svar,num);
   messout(messLevel,1,prgName,message);
   SCETER(-1,"*** memory-error");

   SCSEPI();
   exit(1);
}

/*=========================================================================*/

#ifdef __STDC__             /* ANSI */

int 
warn(int i)

#else                       /* K&R */

int 
warn(i)

int i;

#endif

{

   char message[MESS_LEN];                  /* temp. string for outputs */
   int  messLevel;                             /* message level */
   int  imaDataType;
  

   if (i>0){
     sprintf(message,"Wrong image column number: %d; skip.",i);
     messout(messLevel,2,prgName,message);
   }else{
     switch(-i)
       {
       case 1: strncpy(message,"Max number of image columns: 19",MESS_LEN);
	 messout(messLevel,1,prgName,message);
	 SCETER(-2,"*** maximum-image-column-error");
	 SCSEPI();
	 exit(1);

       case 2: strncpy(message,"No valid image column found !",MESS_LEN);
	 messout(messLevel,1,prgName,message);
	 SCETER(-3,"*** valid-image-column-error");
	 SCSEPI();
	 exit(1);
	
       case 3: strncpy(message,"Image could not be saved !",MESS_LEN);
	 messout(messLevel,1,prgName,message);
	 SCETER(-4,"*** save-image-error");
	 SCSEPI();
	 exit(1);
	
       case 4: sprintf(message,"Dataformat %d not supported !",imaDataType);
	 messout(messLevel,1,prgName,message);
	 SCETER(-5,"*** dataformat-error");
	 SCSEPI();
	 exit(1);
       }
   }
}
/*=============================END=======================================*/
