/* $Id: uves_getwbin.c,v 1.2 2001/01/04 13:36:35 amodigli Exp $ */
/*+++ UVES_GETWBIN ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		uves_getwbin.exe
 
 .KEYWORDS	spectroscopy, echelle, wavelength calibration, wavelength bin
 
 .USAGE		run PIPE_EXE:(IDENT)
 
 .INPUT		IN_A	absolute order number
		IN_B	wavelength in Angstroems
 		 
 .OUTPUT	OUTPUTD(1) size of wavelength bin in Angstroems.
 
 .RETURN	Q1:  0: successful return
 		    -1: a successful return failed
 
 .PURPOSE	Determine the wavelength bin at a specified wavelength and
                absolute order.
 
 .ALGORITHM	The calculations are all based on the physical model routines
                stored in the C library libphysmod.a and done in SI units.
		The following global UVES context keywords have to be properly
		set using CONFIG/INSTR:
		    CCD, CROSS, UVES_ARM, WCENTER, UVES_AIR.

		These keywords will configure the physical model at the
		beginning.

 .ENVIRON	MIDAS, UVES context
    
 .LANGUAGE	C
 
 .AUTHOR	Sebastian Wolf, ESO-DMO
 
 .VERSION	1.0 1999/07/15	Creation
 
 .COMMENT       (1) All calculations of the physical model routines are made
                in nm. So, the input wavelength and results have to be trans-
		formed within this code.
		(2) Use CONFIG/INSTR before calling this program in order to
		guarantee proper global keyword settings!
-----------------------------------------------------------------------------*/
/* ===========================================================================
 * $Name:  $
 * $Revision: 1.2 $
 * $Log: uves_getwbin.c,v $
 * Revision 1.2  2001/01/04 13:36:35  amodigli
 * Added Id, Name, Revision, Log conf. control info
 *
 * ===========================================================================
 */

#include <tbldef.h>
#include <midas_def.h>
#include <stdlib.h>
#include <strings.h>
#include <ctype.h>
#include "physmod.h"

#define CD_IDnum_pos 3

#define MLEN 255
#define SLEN 60


char Mes[MLEN];                              /* message buffer */

main() {


  /* UVES configuration parameters */
  char   ArmId[SLEN+1], CCD[SLEN+1], CROSS[SLEN+1];
  int    CD_Id;                              /* Cross Disperser ID */
  int    cfgidx;                             /* Configuration Index */
  double wclen;                              /* Central Wavelength */
  double air[3];                             /* Air Characteristics */

  /* These configuration keywords have finally to be set in the physmod.c */
  double xtrans=0.0, ytrans=0.0, echoff=0.0, CDoff=0.0, ccdrotoff=0.0;


  int    num, unit, null;
  int    idx;
  char   ext[3];
  char   table[SLEN+1];                      /* wavelength table/array/a.ord */
  char   wlcol[SLEN+1];                      /* wavelength column/array/wlen */
  char   wbcol[SLEN+1];                      /* wavelength bin column/array */
  double dw;                                 /* Wavelength bin size */
  double wlen;                               /* Wavelength */
  double fsr_min, fsr_max;                   /* free spectral range */
  int    absord;                             /* absolute order number */


  SCSPRO("getwbin");
  

  /* Get the configuration keywords */
  SCKGETC("CCD",1,SLEN,&num,CCD);              /* 'e' or 'm'            */
  SCKGETC("CROSS",1,SLEN,&num,CROSS);          /* CD#1 .. CD#4          */
  SCKGETC("UVES_ARM",1,SLEN,&num,ArmId);       /*'b' or 'r'             */
  SCKRDD("WCENTER",1,1,&num,&wclen,&unit,&null); /* Central wavelength  */
  SCKRDD("UVES_AIR",1,3,&num,air,&unit,&null); /* Air characteristics:  */
  ArmId[0] = (int) tolower((int) ArmId[0]);    /* temperature,pressure  */
  CCD[0]   = (int) tolower((int) CCD[0]);      /* water vapor pressure. */
  CD_Id = atoi(&CROSS[ CD_IDnum_pos ]);        /* Crossdisperser ID     */
  wclen = wclen/10.0;                          /* now in nm */

  /* Initialiation of the UVES configuration */
  airConfig(air[0], air[1], air[3]);
  setccdrot(ccdrotoff, ccdrotoff);
  setIncidence(echoff, echoff, CDoff, CDoff);
  cfgidx = uvesConfig(ArmId[0],CCD[0],CD_Id,wclen);



  SCKGETC("IN_A",1,SLEN,&num,table);           /* table/array/single order */
  SCKGETC("IN_B",1,SLEN,&num,wlcol);           /* in-column/single wavel. */
  SCKGETC("OUT_A",1,SLEN,&num,wbcol);          /* output-column/array name */

  absord = atoi(table);
  if (absord != 0){

    wlen = atof(wlcol);                        /* selected wavelength */
    wlen /= 10.0;                              /* transformed to nm */
    if (wlen == 0.0){
      findFSR(absord, &wlen, &fsr_min, &fsr_max);
    }
    dw = waveBin(wlen,absord);                 /* get wavelength binsize */
    
  }else{
    idx = (int) rindex(".",(int) table);
    if (idx >= 0){

      sprintf(Mes,"Open table: `%s`\n",table);
      SCTPUT(Mes);
      sprintf(Mes,"*** this mode is not yet supported! ***\n");
      SCTPUT(Mes);

    }else{

      sprintf(Mes,"Read array: `%s`\n",table);
      SCTPUT(Mes);
      sprintf(Mes,"*** this mode is not yet supported! ***\n");
      SCTPUT(Mes);

    }
  }


  dw = dw * 10.0;                             /* transform to Angstroems */
  SCKWRD("OUTPUTD",&dw,1,1,&unit);
  SCSEPI();  
}


int errorhandling(ierr)
int ierr;
{
  sprintf(Mes,"Error `%d` occured !\n",ierr);
  SCTPUT(Mes);
  exit(1);
}
