/* $Id: uves_psf.c,v 1.4 2001/05/30 16:33:35 amodigli Exp $ */
/*+++ uves_psf.c +++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		uves_psf.exe
 
 .KEYWORDS	uves physical model, spectroscopy, echelle,
 
 .USAGE		run PIPE_EXE:(IDENT)
 
 .INPUT		FRSPRG:	free spectral range table name,
		ArmId:	Arm index,
		CHIP:	Chip index,
		CDID:	Cross-diperser index,
		wcent:	central wave,
		xtrans, ytrans:	   offset position (x,y),
		echoff, CDoff, ccdrotoff:    offset angle (echelle,CD,CCD-rot),
		pressure, tempcam, humid:  pressure, average camera temperature, 
		slitwidth: slit width.
		
 		 
 .OUTPUT	psf, 
 
 
 
 		modfree_sp_rg.tbl: free spectral range table
 
 .RETURN	Q1:  0: successful return
 		    -1: a successful return failed
 
 .PURPOSE	Running the UVES physical model and compute the PSF and other
 		relevant parameters. 
 
 .ALGORITHM	The calculations are all based on the physical model routines
                stored in the C library libphysmod.a and done in SI units.

		These input keywords will configure the physical model at the
		beginning.

 .ENVIRON	MIDAS, UVES context
    
 .LANGUAGE	C
 
 .AUTHOR	Olivier BOITQUIN, ESO-DMD
 
 .VERSION	1.0 1999/10/14-OB  Creation
 		    1999/10/27-OB 

 .COMMENT       All calculations of the physical model routines are made
                in nm. So, the input wavelength and results have to be trans-
		formed within this code.
----------------------------------------------------------------------------*/
/* ===========================================================================
 * $Name:  $
 * $Revision: 1.4 $
 * $Log: uves_psf.c,v $
 * Revision 1.4  2001/05/30 16:33:35  amodigli
 * Taken out error check disabling
 *
 * Revision 1.3  2001/01/04 13:36:29  amodigli
 * Added Id, Name, Revision, Log conf. control info
 *
 * ===========================================================================
 */

#ifdef __STDC__  /* ANSI */
  static void  error(void);
#else  /* K&R */
  static void  error();
#endif

#include <tbldef.h>
#include <midas_def.h>
#include <stdlib.h>
#include "physmod.h"
#define     PROID "PSF"
/* for messout function: no "static" (otherwise no messages from other files) */
int messlevel;

#ifdef __STDC__  /* ANSI */
  void main(int argc, char *argv[])
#else  /* K&R */
  void main(argc, argv)
  int  argc;
  char *argv[];
#endif
{

  int tid    = 0, outtid = 0 ;
  int ncol   = 0;
  int nsort  = 0;
  int allcol = 0, ordcol, wavcol, xcol,ycol;
  int allrow = 0;
  int actvals, kunit;
  int aRowNumber, col, null, i, imod = 0, mm;

/* for the computation of the physical model */  

  char    linetbl[200], modtbl[200], freemodtbl[200], aHeading[200], line[200];
  char    ArmId[2], CHIP[2] ;
  double  dblwav, wcent, xmod, ymod,xtrans,ytrans,echoff,CDoff,ccdrotoff;
  double  pressure, tempcam, humid;
  int     CDID, cfgindx, xpix, ypix, order;   
  float   CCDBIN[2];
  double  values[20];

/* for the computation of the free spectral range */

  char   fsr_opt[2];
  double x,y, x1, y1, l, dl, ls, le, fsr_min, fsr_max,xfsr_min,yfsr_min,xfsr_max,yfsr_max;
  int    m, m_min, m_max;
  double lmin,lmax, dxpix, dypix;
  int    mmin,mmax;
  int 	 startcol,endcol,ycentcol,fsrmincol,fsrmaxcol,xfsrmincol,xsrmaxcol,yfsrmincol,ysrmaxcol,bincol;


/* for messout */
  char  tmps[200];
  
/* For the computation of the width (in pixel and A) and resolution lines */  
  double   slitwidth, binsize, linewidth, linewidpx, resol, lcent; 
  int      binScol,lineWcol,lineWPXcol,lineLPXcol,resolcol,centcol,pixsclcol,pixsclCDcol;
  
/* For the computation of the true pixel-scale along echelle dispersion */
  double  scale, pixscale, pixscalCD, linelenpx, slitlength ;

/* For the computation of the blaze function */
  double   blaze, betacd, betaech, fc, rech, rcd;
  int      blzcol;

/*  int binx, biny; */

/* As usual */
  int   r, nval, unit;
  
  SCSPRO("uves_psf");
  SCKRDI("MESS_LEVEL",1,1,&nval,&messlevel,&unit,&null);
  
  SCKGETC("FRSPRG", 1, 60, &actvals, freemodtbl);
    
/* Get the configuration keywords */

  SCKRDC("ArmId",1,1,1,&actvals,ArmId,&kunit,&null);
  SCKRDC("CHIP",1,1,1,&actvals,CHIP,&kunit,&null);
  SCKRDR("CCDBIN",1,2,&actvals,CCDBIN,&kunit,&null);
  SCKRDI("CDID",1,1,&actvals,&CDID,&kunit,&null);
  SCKRDD("wcent",1,1,&actvals,&wcent,&kunit,&null);
  SCKRDD("xtrans",1,1,&actvals,&xtrans,&kunit,&null);
  SCKRDD("ytrans",1,1,&actvals,&ytrans,&kunit,&null);  
  SCKRDD("echoff",1,1,&actvals,&echoff,&kunit,&null);
  SCKRDD("CDoff",1,1,&actvals,&CDoff,&kunit,&null);
  SCKRDD("ccdrotoff",1,1,&actvals,&ccdrotoff,&kunit,&null);
  SCKRDD("pressure",1,1,&actvals,&pressure,&kunit,&null);
  SCKRDD("tempcam",1,1,&actvals,&tempcam,&kunit,&null);
  SCKRDD("humid",1,1,&actvals,&humid,&kunit,&null);
  SCKRDD("slitwidth",1,1,&actvals,&slitwidth,&kunit,&null);
  SCKRDD("slitlength",1,1,&actvals,&slitlength,&kunit,&null);
  SCKRDD("wavelength",1,1,&actvals,&dblwav,&kunit,&null);
  SCKRDC("fsr_opt",1,1,1,&actvals,fsr_opt,&kunit,&null);

  /* Initialiation of configuration */

  airConfig(pressure/1.33322, tempcam, humid);
  /*printf("Air Index = %f\n",airIndex(wcent)); */
  /*printf("Shift in x = %f, shift in y = %f\n",xtrans,ytrans); */
  /*printf("Offset of CCD rotation %f\n",ccdrotoff); */
  sprintf(&tmps[0],"Air Index = %f\nShift in x = %f, shift in y = %f",airIndex(wcent),xtrans,ytrans);
  messout(messlevel,3,PROID,tmps);
  sprintf(&tmps[0],"Offset angle of echelle, CD, CCD rotation :\n%f, %f, %f",echoff,CDoff,ccdrotoff);
  messout(messlevel,3,PROID,tmps);

  setccdrot(ccdrotoff, ccdrotoff);
  setIncidence(echoff, echoff, CDoff, CDoff);
/*  setBinning(binx, biny); */
  cfgindx = uvesConfig(ArmId[0],CHIP[0],CDID,wcent,CCDBIN[0],CCDBIN[1]);
  
/* Computation of the true pixel-scale along echelle dispersion */
/* scale = (K*FocalRatioNasmyth*PixelSize*ApertureCamera*1e-3)/FocalNasmyth
   K = 206265 : conversion radian to arcsec (1 radian = 206265 arcsec)*/
  scale = (206265.0*15.0*0.015*200*1e-3*CCDBIN[0])/120000;

/* Compute the PSF for the current wavelength */

  ccdSize(&xpix,&ypix); 
  /*
  sprintf(&tmps[0],"Arm = %s, CCD = %s, (xpix=%d,ypix=%d),\nCD=%d, Wcent= %f", ArmId, CHIP, xpix,ypix, CDID, wcent);
  messout(messlevel,3,PROID,tmps);
  */
  dblwav /= 10.0;    /* (10.0: conversion factor from nm to A)*/
  order = findM(dblwav);

  /*xyModel(dblwav, order, &xmod, &ymod);
  blaze = blz; */
  lm2beta(dblwav, order, &betaech, &betacd, &fc);
  beta2xy(betacd, betaech, fc, &xmod, &ymod);

/* Computes the anamorphic magnification and the blaze function */      
  photobeta(dblwav, betaech, betacd, &rech, &rcd, &blaze); 
  
  /*printf("xmod= %f , ymod = %f\n", xmod,ymod);*/ 

/* Translation */
  xmod += xtrans;
  ymod += ytrans;

/* Computes the width (in pixel and A) and resolution lines */
  pixscl(dblwav, order, rech, rcd, CCDBIN[0], CCDBIN[1], fc, slitwidth, slitlength, 
         &binsize, &pixscale, &pixscalCD, 
	 &linewidpx, &linelenpx, &linewidth, &resol);

/* Compute the minimum and maximun of order range */

  m_min = -1;
  m_max = -1;

  dxpix = (double) xpix;
  dypix = (double) ypix;
   
  lmModel(&l,&m_max,dxpix/2,1.0);
  lmModel(&l,&m_min,dxpix/2,dypix);
   
/*printf("m_min= %d,m_max= %d\n",m_min,m_max); */

/* Compute the minimum and maximun of the wavelength range on CCD */
 
  mmin = -1;
  mmax = -1;

  lmModel(&lmax,&mmin,1.0,dypix);
  lmModel(&lmin,&mmax,dxpix,1.0); 

  lmModel(&lmax,&mmin,dxpix,dypix);
  lmModel(&lmin,&mmax,1.0,1.0); 

/* Central wavelength of the CCD */
  lmModel(&l,&m,dxpix/2,dypix/2);  

/* output data */

  values[0] = l;
  values[1] = linewidpx;
  values[2] = linelenpx;
  values[3] = linewidth;
  values[4] = binsize;
  values[5] = pixscale;
  values[6] = pixscalCD;
  values[7] = resol;
  values[8] = order;
  values[9] = xmod;
  values[10] = ymod;
  values[11] = lmin;
  values[12] = lmax;
  values[13] = m_max;
  values[14] = m_min;
  values[15] = m_max - m_min +1;
  values[16] = mmax;
  values[17] = mmin;
  values[18] = mmax - mmin +1;
  values[19] = -1.0 * blaze;
  
/* If the current wavelength belong to the range */
  if (dblwav > lmin && dblwav < lmax) values[19] = blaze;  

/*
printf("Blaze function = %f \n",blaze);
printf("wave = %f nm, line width = %f \n",dblwav, linewidpx);
printf("wave = %f nm, line width = %f \n",values[0],values[1]);
for (i=0; i<20;i++) {
  printf("values[%d] = %f\n", i, values[i]);
}  
*/
  SCKWRD("OUTPUTD",values,1,20,&unit);

/* Compute the free spectral range */

if (fsr_opt[0] == 'Y') {

  sprintf (&tmps[0],"Determine the Free Spectral Range\n");
  messout(messlevel,3,PROID,tmps);
 
    
/* Creation of the modfree_sp_rg.tbl table */
  imod = 0;
  m_max = mmax;
  m_min = mmin;
 
  TCTINI(freemodtbl, F_TRANS, F_IO_MODE, 19,m_max-m_min+1, &outtid);
  
  sprintf(&tmps[0],"FSR-table created: %s ncol=%d, nrow=%d",freemodtbl,17,m_max-m_min+1);  
  messout(messlevel,3,PROID,tmps);
  
  TCCINI(outtid, D_R8_FORMAT, 1, "I6"   , " ", "ORDER", &ordcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "WVCENTCOL", &centcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "WAVECENT", &wavcol);  
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "START", &startcol);  
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "END", &endcol);  
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "YCENT", &ycentcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "FSRMIN", &fsrmincol);  
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "FSRMAX", &fsrmaxcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "XFSR_MIN", &xfsrmincol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "XFSR_MAX", &xsrmaxcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "YFSR_MIN", &yfsrmincol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "YFSR_MAX", &ysrmaxcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "PIXSCALE", &pixsclcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "PIXSCALCD", &pixsclCDcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "BIN_SIZE", &bincol);  
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "LINEWIDPX", &lineWPXcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "LINEWIDTH", &lineWcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "RESOL", &resolcol);
  TCCINI(outtid, D_R8_FORMAT, 1, "F12.4", " ", "LINELENPX", &lineLPXcol);

 for (m=m_max; m>=m_min; m--) {
     /*printf("%d ",m);*/
     findFSR(m,&l, &fsr_min, &fsr_max);
     
     xyModel(l,m,&x,&y);

/* Translation */
      x += xtrans;
      y += ytrans;
     
     xyModel(fsr_min,m,&xfsr_min,&yfsr_min);
     
/* Translation */
      xfsr_min += xtrans;
      yfsr_min += ytrans;

     xyModel(fsr_max,m,&xfsr_max,&yfsr_max);
     
/* Translation */
      xfsr_max += xtrans;
      yfsr_max += ytrans;

     dl = waveBin(l,m)*1e4; /* in mA */

/*     linewidpx = slitwidth /pixscale[cfgindx-1]; */
     pixscale = scale/(cameraFocal(l)*rech); /* in mA */
     pixscalCD = pixscale *(CCDBIN[1]/CCDBIN[0]) *rech/rcd; /* in arcsec/pixel */
     linewidpx = slitwidth /pixscale;  /* pixel */
     linelenpx = slitlength /pixscalCD;  /* pixel */
     linewidth = dl * linewidpx * 1e-3; /* in A (*pixel) */
     resol = l * 10.0 / linewidth;  /* without unit */
	                          /* (10.0: conversion factor from nm to A)*/

     
     mmin  = m;
     mmax = m;
     
     lmModel(&lcent,&m,dxpix/2,y);
     lmModel(&lmax,&mmax,dxpix,y);
     lmModel(&lmin,&mmin,1.0,y); 

/*     if (y>=0. && y <=dypix) {*/
       sprintf(&tmps[0],"m, waveCent (nm), Start, End, yCent, FSRmin, FSRmax, bin (mA) = %d %f  %f %f %f %f %f %f\n",m,l,lmin,lmax,y,fsr_min,fsr_max,dl);
/*     SCTPUT(tmps); */
       messout(messlevel,4,PROID,tmps);
       sprintf(&tmps[0],"pixel-scale = %f, anamorphic corrections = %f, %f\n", pixscale, rech, rcd);
       messout(messlevel,4,PROID,tmps);
       
       TCEWRI(outtid, ++imod, ordcol, &m);
       TCEWRD(outtid, imod, centcol, &lcent);
       TCEWRD(outtid, imod, wavcol, &l);
       TCEWRD(outtid, imod, startcol, &lmin);
       TCEWRD(outtid, imod, endcol, &lmax);
       TCEWRD(outtid, imod, ycentcol, &y);
       TCEWRD(outtid, imod, fsrmincol, &fsr_min);
       TCEWRD(outtid, imod, fsrmaxcol, &fsr_max);
       TCEWRD(outtid, imod, xfsrmincol, &xfsr_min);
       TCEWRD(outtid, imod, xsrmaxcol, &xfsr_max);
       TCEWRD(outtid, imod, yfsrmincol, &yfsr_min);
       TCEWRD(outtid, imod, ysrmaxcol, &yfsr_max);
       TCEWRD(outtid, imod, pixsclcol, &pixscale);
       TCEWRD(outtid, imod, pixsclCDcol, &pixscalCD);
       TCEWRD(outtid, imod, bincol, &dl);
       TCEWRD(outtid, imod, lineWPXcol, &linewidpx);
       TCEWRD(outtid, imod, lineWcol, &linewidth);
       TCEWRD(outtid, imod, resolcol, &resol);
       TCEWRD(outtid, imod, lineLPXcol, &linelenpx);
/*     }*/
  }
}
  SCKWRC("Q1",1,"0",1,60,&unit);
  SCSEPI();  
}

/*---------------------------------------------------------------------*/

#ifdef __STDC__  /* ANSI */
  static void error(void)
#else  /* K&R */
  static void error()
#endif
{
  int  r, unit;

  r = SCKWRC("Q1",1,"1",1,60,&unit);
  r = SCSEPI();
}

/*---------------------------------------------------------------------*/

