! $Id: uves_cal_mkmaster.prg,v 1.3 2001/05/04 14:09:46 amodigli Exp $
!+++ UVES_CAL_MKMASTER ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_cal_mkmaster.prg
!
!.KEYWORDS      spectroscopy, echelle, master creation
!
!.USAGE         @p (IDENT) sof_in out sof_ref
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!               sof_ref (P3): set of reference frames (ascii list)
!				order-, line-table, line reference table,
!				and drs-setup-table
!		bmode	(P4): bias subtraction mode:
!				M: take the master frame,
!				S: take a smoothed version of the master,
!				   append smoothing window: S,<x>,<y>
!				   (see FILTER/SMOOTH)
!				A: take the average of the master,
!				   subtract this constant.
!				<num>: Specify a number to be subtracted.
!		dmode	(P5): dark subtraction mode:
!				M: take the master frame,
!				S: take a smoothed version of the master
!				   append smoothing window: S,<x>,<y>
!				   (see FILTER/SMOOTH)
!				A: take the average of the master,
!				   subtract this constant.
!				<num>: Specify a number to be subtracted.
!
!.OUTPUT        out     (P2): path + basename of output files:
!				line table (LINTAB_ID)
!				data reduction setup table (DRSRC_ID)
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT	P4	M
!		P5	M
!
!.PURPOSE       Do an averaging of the calibration frames in pixel-pixel space.
!
!.ALGORITHM     Get the set of frames of the RB via INIT/PIPELINE. Call master
!		creation command MASTER/UVES from UVES context. Finally, save
!		all products as FITS files at the specified output location.
!
!.ENVIRON       UVES, CCDRED context
!
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/03/24  Created.
!		1.0 1999/07/01	Reset the defaults P4=M , P5=M.
!		    1999/07/21  Background measuring method parameter added.
!				Shortlog: bug (redflag) fixed and extended.
!		    1999/09/26	Check return value of FINISH/PIPE
!		    1999/12/10	main product was not sent to FINISH/PIPELINE!
!
!.COMMENT	No quality control recipes included!
!		The parameters P4 and P5 as numbers will be overriden by
!		master frame specifications in the set of reference frames!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.3 $
! $Log: uves_cal_mkmaster.prg,v $
! Revision 1.3  2001/05/04 14:09:46  amodigli
! Added RB info to FITS header
!
! Revision 1.2  2001/01/04 13:44:16  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
define/local line/c/1/79 "-" ALL
write/out {line}
LOG/REC uves_cal_mkmaster IN
!
! Check parameters
!
define/param p1 ? c "Enter name of ASCII filelist with INPUT frames:"
define/param p2 ? c "Enter name of OUTPUT frame:"
define/param p3 + c "Enter name of ASCII filelist with REFERENCE frames:"
define/param p4 M c "Enter bias subtraction mode (M,S,A,<number>):"
define/param p5 M c "Enter dark subtraction mode (M,S,A,<number>):"
define/param p6 MEDIAN c "Enter background measur. method (MEDIAN, MINIMUM):"
!
! local constant declaration
!
define/local pid/c/1/16 cal_mkmaster
!
! local variable declaration
!
define/local ifrm/c/1/100 " " ALL
define/local main_prod/c/1/25 " " all
define/local do_clas/c/1/25 " " ALL
define/local i1/i/1/1 1
define/local num/i/1/1 0
define/local catal/i/1/1 0

mess/out 3 {pid} -
  "Recipe UVES_CAL_MKMASTER will perform\na master frame creation."

@p uves_paragraph 1

INITIALIZE/PIPELIN {P1} {P3} mkmasterI.cat + + + reject.cat mkmasterO.cat
if {Q1} .gt. 0 return {Q1}

 
PROHEAD/INIT {P1} {P3} 1,"uves_cal_mkmaster" -
"BMODE,{P4}" "DMODE,{P5}" "BGMTD,{P6}"
 
@p uves_paragraph 2
init/echelle
clean/disp A					!clear displays and graph.win.


split/uves mkmasterI.cat			!transpose and flip frames
show/icat split.cat >Null			!and split for RED arm
num = outputi(1)				!number of input frames
						!split.cat hold ONLY RAW frames
						!Do the master
						creation:
define/local ncat/i/1/1 0
@p flames_find_same_flat.prg mkmasterI.cat
ncat = q1
set/format i1
do index = 1 ncat
   MASTER/UVES sflat{index}.cat mkmasterO.cat mkmasterI.cat {p4} {p5} {P6}
enddo
set/format
if {q1} .lt. 0 goto ABORT
main_prod = q2

show/icat mkmasterO.cat >Null
num = outputi(1)				!number of output frames
if num .le. 0 goto ABORT

catal = 0
ILOOP:
  store/frame ifrm mkmasterO.cat 1 ELOOP
       
  if {{ifrm},redflag(1)} .eq. 0 -
          shortlog/out 1 "Master Bias frame `{ifrm}' created."
        
  if {{ifrm},redflag(1)} .eq. 1 -
          shortlog/out 1 "Bias subtraction for `{ifrm}' done."
  if {{ifrm},redflag(2)} .eq. 1 -
          shortlog/out 1 "Dark subtraction for `{ifrm}' done."
  if {{ifrm},redflag(3)} .eq. 1 then
          shortlog/out 1 "Background subtraction for `{ifrm}' done."
          shortlog/out 1 "Master frame `{ifrm}' created."
  endif
  if {{ifrm},redflag(1)} .eq. 2 -
          shortlog/out 1 "Bkg frame `{ifrm}' created."
  
  goto ILOOP
ELOOP:
 
prohead/finish  mkmasterO.cat 1

@p uves_paragraph 3

FINISH/PIPELINE mkmasterO.cat {P2} + {main_prod}
if {q1} .eq. 1 goto ABORT

LOG/REC uves_cal_mkmaster OUT
@p uves_paragraph 4
return 0


ABORT:
mess/out 1 {pid} "Error in uves_mkmaster occured !"
write/error 1,APP,NO "Error in uves_mkmaster occured !"


