! $Id: uves_cal_predict.prg,v 1.6 2001/05/04 14:10:31 amodigli Exp $
!+++ UVES_CAL_PREDICT ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_cal_predict
!
!.KEYWORDS      Physical model, spectroscopy, echelle, order positions
!
!.USAGE         @p (IDENT) sof_in out sof_ref [optpar]
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!
!.OUTPUT        out     (P2): path + basename of output files:
!				order table (ORDER_ID),
!				background table (BACK_ID),
!				line table (LINTAB_ID),
!				setup table (DRSRC_ID).
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Runs the physical model to obtain a first guess calibration   
!		        solution: determine echelle order and line position table from 
!		        the ThAr arc lamp line catalogue.
!
!.ALGORITHM     Loads all the raw frames, splits them if necessary,
!		        runs the "uves_fmtstab.prg" command.
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Olivier BOITQUIN, ESO-DMD.
!
!.VERSION       1.0 1999/09/01  Created (OB).
!		        1999/09/26	Check return value of FINISH/PIPE (SW)
!		        1999/09/30	Model parameter added (SW)!
!		        1999/12/07	bad procedure header, fixed (SW)
!				            info on terminal partly wrong, fixed (SW)
!		        1999/12/10	main product was not sent to FINISH/PIPELINE!
!
!.COMMENT	
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.6 $
! $Log: uves_cal_predict.prg,v $
! Revision 1.6  2001/05/04 14:10:31  amodigli
! Added RB info to FITS header
!
! Revision 1.5  2001/01/04 13:44:15  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
define/local line/c/1/79 "-" ALL
write/out {line}
LOG/REC uves_cal_predict IN
!
! Check parameters
!
define/param p1 ? c "Enter name of ASCII filelist with INPUT frames:"
define/param p2 ? c "Enter name of OUTPUT frame:"
define/param p3 ? c "Enter name of ASCII filelist with REFERENCE frames:"
define/param P4 40,40       NUMB      "Dimension of measurement box (X,Y): "
define/param P5 0,0         NUMB      "Translation of the ccd (X,Y): "
define/param P6 0,0,0       NUMB      "Angle offsets (echelle, CD, CCD-rot):"
define/param P7 0,0         NUMB      "Measurement position offsets (X,Y): "  
!
! local constant declaration
!
define/local pid/c/1/16 cal_predict
!
! local variable declaration
!
define/local main_prod/c/1/25 " " all
define/local ifrm/c/1/100 " " ALL
define/local otbl/c/1/100 " " ALL
define/local btbl/c/1/100 " " ALL
define/local num/i/1/1 0
define/local catal/i/1/1 0 

mess/out 3 {pid} -
  "Recipe UVES_CAL_PREDICT will determine\nechelle order and line postions."

@p uves_paragraph 1

INITIALIZE/PIPELINE {P1} {P3} predicI.cat + + + reject.cat predicO.cat
if {Q1} .gt. 0 return {Q1}

PROHEAD/INIT {P1} {P3} 1,"uves_cal_predict" -
"XYDIM,{P4}" "XYTRN,{P5}" "ANGLE,{P6}" "XYOFF,{P7}"

@p uves_paragraph 2
clean/disp A					!clear displays and graph.win.

! the following lines have been moved into the uves_fmtstab.prg
! AKA 2000-03-19
!search/icat predicI.cat {LINCAT_ID} >Null
!if m$strlen(out_a) .gt. 0 then
!  set/echelle lincat={out_a}
!else
!  mess/out 1 {pid} "Line catalog is missing!"
!  return -1
!endif

split/uves predicI.cat				!transpose and flip frames

LOOP:

  store/frame ifrm split.cat 1 ENDLOOP

  @p uves_fmtstab.prg  {ifrm} predicI.cat predicO.cat {p4} {p5} {p6} {p7}
  if {q1} .ne. 0 goto ABORT			!An error had occured.
  main_prod = q2
  goto LOOP

ENDLOOP:

shortlog/out 1 "Order positions and line positions are predicted."
prohead/finish predicO.cat 1


@p uves_paragraph 3
FINISH/PIPELINE predicO.cat {P2} + {main_prod}
if {q1} .eq. 1 goto ABORT

LOG/REC uves_cal_predict OUT
@p uves_paragraph 4
return 0

ABORT:
mess/out 1 {pid} "Error in uves_fmtstab occured !"
write/error 1,APP,NO "Error in uves_fmtstab occured !"
