! $Id: uves_cal_wavecal.prg,v 1.3 2001/05/04 14:11:16 amodigli Exp $
!+++ UVES_CAL_WCAL ++ +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_cal_wavecal.prg
!
!.KEYWORDS      spectroscopy, echelle, order, wavelength calibration
!
!.USAGE         @p (IDENT) sof_in out sof_ref [optpar]
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!               sof_ref (P3): set of reference frames (ascii list)
!				order-, line-table, line reference table,
!				and drs-setup-table
!               mode    (P4): auto or interactive:
!				. auto: selects the WLCMTD=GUESS and looks for
!				  previously created line table (LINTAB_ID)
!				. interactive: selects the WLCMTD=TWO-D; needs
!				  no previous solution.
!
!.OUTPUT        out     (P2): path + basename of output files:
!				line table (LINTAB_ID)
!				data reduction setup table (DRSRC_ID)
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Determine the dispersion relation solution from arc lamp
!		exposures.
!
!.ALGORITHM     Extract the echelle orders by means of the order table, after
!		the transformation of the UVES arc lamp input files which
!		includes an optional splitting (red arm), rotation and flipping
!		of the frame. The line identification will be done with a
!		first guess solution -- a previously created line table either
!		from a physical model or through an interactive run.
!
!.ENVIRON       UVES, ECHELLE context
!
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/03/16  Created.
!		1.0 1999/05/20	P5 of uves_wavecal properly set.
!		    1999/09/26	Check return value of FINISH/PIPE
!		    1999/12/07	Parameters P5 and P6 added.
!!		    1999/12/10	main product was not sent to FINISH/PIPELINE!

!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.3 $
! $Log: uves_cal_wavecal.prg,v $
! Revision 1.3  2001/05/04 14:11:16  amodigli
! Added RB info to FITS header
!
! Revision 1.2  2001/01/04 13:44:15  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
define/local line/c/1/79 "-" ALL
write/out {line}
LOG/REC uves_cal_wavecal IN
!
! Check parameters
!
define/param p1 ? c "Enter name of ASCII filelist with INPUT frames:"
define/param p2 ? c "Enter name of OUTPUT frame:"
define/param p3 ? c "Enter name of ASCII filelist with REFERENCE frames:"
define/param p4 ? c "Enter wavelength calibration mode (auto or interactive):"
define/param p5 Y c "Enter YES to get spectral resolution plot, otherwise NO:"
define/param p6 N c "Enter YES to get wavelength calibration only at the centre, otherwise NO:"
!
! local constant declaration
!
define/local pid/c/1/16 cal_wavecal
!
! local variable declaration
!
define/local ifrm/c/1/100 " " ALL
define/local do_clas/c/1/25 " " ALL
define/local main_prod/c/1/25 " " all
define/local i1/i/1/1 1
define/local num/i/1/1 0

mess/out 3 {pid} -
  "Recipe UVES_CAL_WAVECAL will perform\nthe wavelength calibration."

@p uves_paragraph 1

INITIALIZE/PIPELINE {P1} {P3} wcalI.cat + + + reject.cat wcalO.cat
if {Q1} .gt. 0 return {Q1}

PROHEAD/INIT {P1} {P3} 1,"uves_cal_wavecal" -
"MODE,{P4}" "RPLT,{P5}" "CENT,{P6}"


@p uves_paragraph 2
init/echelle
clean/disp A					!clear displays and graph.win.


split/uves wcalI.cat				!transpose and flip frames
show/icat split.cat >Null			!and split for RED arm
num = outputi(1)				!number of input frames
						!split.cat hold ONLY RAW frames
						!Do the wavelength calibration:
@p uves_wavecal split.cat wcalO.cat wcalI.cat {p4} {p5} {p6} NO
if {q1} .lt. 0 goto ABORT
main_prod = q2

shortlog/out 1 "Order extraction done."
shortlog/out 1 "Wavelength calibration done."

prohead/finish  wcalO.cat 1

@p uves_paragraph 3

FINISH/PIPELINE wcalO.cat {P2} + {main_prod}
if {q1} .eq. 1 goto ABORT

LOG/REC uves_cal_wavecal OUT
@p uves_paragraph 4
return 0


ABORT:
mess/out 1 {pid} "Error in uves_wavecal occured !"
write/error 1,APP,NO "Error in uves_wavecal occured !"

