! $Id: uves_obs_spatred.prg,v 1.1 2002/05/30 10:06:40 amodigli Exp $
!+++ UVES_OBS_SPATRED ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_obs_spatred.prg
!
!.KEYWORDS      spectroscopy, echelle, order, wavelength calibration
!
!.USAGE         @p (IDENT) sof_in out sof_ref ffmeth exmeth  bkgmeth 2dExtPar
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!               sof_ref (P3): set of reference frames (ascii list)
!				order-, line-table, line reference table,
!				and drs-setup-table
!               ffmeth  (P4): flatfielding method
!				. P: pixel-to-pixel flatfielding
!				. E: flatfielding during extraction
!               exmeth  (P5): extraction method
!				. AVERAGED
!				. OPTIMAL
!               bkgmeth (P6): background extraction method
!               . MINIMUM
!               . MEDIAN
!               2dExtrPar (p8): 2d extraction parameter
!               . [Y]/N SWITCH
!                       Y: do 2d extraction
!                       N: don't do 2d extraction
!
!               . MIN : background method during 2d extraction
!               . [E]/P/N : flat fielding method 
!                                E: during extraction
!                                P: pixel-to-pixel flat fielding
!                                N: no flat fielding
!               . [A]/O/N merging method during 2d extraction
!   
!
!.OUTPUT        out     (P2): path + basename of output files:
!				reduced science frame (..._ID)
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Do a echelle data reduction of science frames.
!
!.ALGORITHM     
!
!.ENVIRON       UVES, ECHELLE context
!
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/03/16  Created.
!		1.0 1999/06/08  Pixel-to-pixel flatfielding implemented.
!		    1999/06/17  Extraction method switch implemented (P5).
!		    1999/07/21  Background measuring method parameter added.
!		    1999/09/26	Check return value of FINISH/PIPE
!		    1999/12/10	main product was not sent to FINISH/PIPELINE!
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.1 $
! $Log: uves_obs_spatred.prg,v $
! Revision 1.1  2002/05/30 10:06:40  amodigli
! *** empty log message ***
!
! Revision 1.8  2001/06/07 08:55:58  amodigli
! Added reference to parameter P7
!
! Revision 1.7  2001/05/14 13:19:33  amodigli
! Set default bkg method to median and switched off 2d extraction
!
! Revision 1.6  2001/05/04 14:11:53  amodigli
! Added RB info to FITS header
!
! Revision 1.5  2001/02/09 16:20:36  amodigli
! Improoved/updated header info
!
! Revision 1.4  2001/01/19 12:32:57  amodigli
! Fixed a small problem using P8 in selection
! of different options for 2d extraction
!
! Revision 1.3  2001/01/04 13:44:14  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
define/local line/c/1/79 "-" ALL
write/out {line}
LOG/REC uves_obs_spatred IN
!
! Check parameters
!
define/param p1 ? c "Enter name of ASCII filelist with INPUT frames:"
define/param p2 ? c "Enter name of ASCII filelist with REFERENCE frames:"
define/param p3 ? c "Enter name of OUTPUT frame:"
define/param p4 ? c "Enter flatfielding method (P,E,...):"
define/param p5 ? c "Enter extraction method (AVERAGED,OPTIMAL):"
define/param p6 MEDIAN c "Enter background measur. method (MEDIAN, MINIMUM):"
define/param p7 Y,Y,Y  c "Enter offset,slit,skyw. autom det. option ([Y]/N,[Y]/N,[Y]/N):"
define/param p8 N,MED,E,A c "Enter 2d extr. method Execute,Bkg,FF,Merg ([N]/Y,[MED]/MIN,[E]/P/N,[A]/O/N):"

!
! local constant declaration
!
define/local pid/c/1/16 obs_scired
!
! local variable declaration
!
define/local ifrm/c/1/100 " " ALL
define/local do_clas/c/1/25 " " ALL
define/local main_prod/c/1/25 " " all
define/local i1/i/1/1 1
define/local num/i/1/1 0



define/local parExt/c/1/10 " " all
define/local parBkg/c/1/10 " " all
define/local parFF/c/1/10 " " all
define/local parMer/c/1/10 " " all
define/local k/i/1/1 0
  
$date
k = m$parse(p8,"par")
     if k .eq. 4 then 
  parExt = "{par01}"
  parBkg = "{par02}"
  parFF  = "{par03}"
  parMer = "{par04}"
else if k .eq. 3 then
  parExt = "{par01}" 
  parBkg = "{par02}"
  parFF  = "{par03}"
  parMer = "Average"
else if k .eq. 2 then
  parExt = "{par01}"
  parBkg = "{par02}"
  parFF  = "E"
  parMer = "Average"
else if k .eq. 1 then
  parExt = "{par01}"  
  parBkg = "Minimum"
  parFF  = "E"
  parMer = "Average"
else if k .eq. 0 then
  parExt = "Y"
  parBkg = "Minimum"
  parFF = "E"
  parMer = "Average" 
else
 mess/out 3 {pid} "Too many sub-pars for p8, max 3: BkgMtd, FFMTD, MERMTD, Extraction"
 return -1
endif

parExt = m$upper(parExt)
parBkg = m$upper(parBkg)
parFF  = m$upper(parFF)
parMer = m$upper(parMer)

!========================================
!Check on reduction parameters values
!========================================

if "{parExt(1:1)}" .eq. "Y" then !OK
         if "{parBkg(1:2)}" .eq. "MI" then !OK
    else if "{parBkg(1:2)}" .eq. "ME" then !OK
    else 
       mess/out 3 {pid} "Bkg parameter value {parBkg} not allowed![MINIMUM]/MEDIAN"
    endif
         if "{parFF(1:1)}" .eq. "E" then !OK
    else if "{parFF(1:1)}" .eq. "P" then !OK
    else if "{parFF(1:1)}" .eq. "N" then !OK
    else 
       mess/out 3 {pid} "Flat Fielding parameter value {parFF} not allowed![E]/P/N"
    endif
         if "{parMer(1:1)}" .eq. "A" then  !OK
    else if "{parMer(1:1)}" .eq. "O" then  !OK
    else if "{parMer(1:1)}" .eq. "N" then  !OK
    else 
       mess/out 3 {pid} "Merging parameter value {parMer} not allowed![AVERAGE]/OPTIMAL/NOAPPEND"
    endif
else 
   mess/out 3 {pid} "2d extraction not performed!"
endif


mess/out 3 {pid} -
  "Recipe UVES_OBS_SPATRED will perform\nthe science reduction."

@p uves_paragraph 1

INITIALIZE/PIPELINE {P1} {P3} scredI.cat + + + reject.cat scredO.cat
if {Q1} .gt. 0 return {Q1}


PROHEAD/INIT {P1} {P3} 1,"uves_obs_spatred" -
"FFMTD,{P4}" "EXMTD,{P5}" "BGMTD,{P6}" "INTMTD,Y,Y,Y" "MTD2D,{P8}"


@p uves_paragraph 2
init/echelle
clean/disp A					!clear displays and graph.win.


split/uves scredI.cat				!transpose and flip frames
show/icat split.cat >Null			!and split for RED arm
num = outputi(1)				!number of input frames
						!split.cat hold ONLY RAW frames
						!Do the wavelength calibration:
!@p uves_reduce split.cat scredO.cat scredI.cat {p4} {p5} {p6}
!if {q1} .lt. 0 goto ABORT
!main_prod = q2

shortlog/out 1 "Science reduction done."

if "{parExt(1:1)}" .eq. "Y" then !OK
   @p uves_reduce_spat split.cat scredO.cat scredI.cat {parBkg},{parFF},{parMer}
   shortlog/out 1 "2d Science reduction done."
endif

prohead/finish  scredO.cat 1

@p uves_paragraph 3

FINISH/PIPELINE scredO.cat {P2} + {main_prod}
if {q1} .eq. 1 goto ABORT

LOG/REC uves_obs_spatred OUT
@p uves_paragraph 4
return 0


ABORT:
mess/out 1 {pid} "Error in uves_reduce occured !"
write/error 1,APP,NO "Error in uves_reduce occured !"
