#!/bin/sh
# $Id: uves_ccdtest.sh,v 1.8 2002/06/04 14:49:57 amodigli Exp $
#+++ UVES_CCDTEST +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         ccdtest.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#                       series of biases        [BIAS_x]
#                       series of screen flats pairs [SCREEN_FLAT_x]
#
#.OUTPUT        dispersion relation solution [LINE_TABLE_y]
#		ccdtestutin plot (POSTSCRIPT)
#		(y: REDL, REDU, BLUE)
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Creates the dispersion relation solution saved as a
#		line table and produces a resolution plot as a POSTSCRIPT
#		file.
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally calls UVES context command WAVECAL/UVES.
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Sebastian Wolf, ESO-DMO
#
#.VERSION       1.0 1999/12/17  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.8 $
# $Log: uves_ccdtest.sh,v $
# Revision 1.8  2002/06/04 14:49:57  amodigli
# Simplified (compacted) using common scripts.
# Now each main script need to be in the
# PIPE_HOME/uves/uves/scripts directory with the
# common other ones.
#
# Revision 1.7  2002/02/05 08:41:36  amodigli
# Made it portable to HP-UX,SunOS,Linux,OSF1
#
# Revision 1.6  2001/02/12 14:53:16  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.5  2001/01/04 14:16:38  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------
# Note:
#
# be sure to have in your path the commands: 
#    xdialog (HP-UX), or 
#    xmessage (SunOS,Linux,OSF1)
#-------------------------------------------------------------------------------
 
my_proc=uves_ccdtest.prg
my_script=uves_ccdtest.sh
my_test=ccdtest
my_cat=ccdtest.cat
export my_proc
export my_script
export my_test
export my_cat
if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 

script_dir=$PIPE_HOME/uves/uves/scripts
#common part to do some checks and settings
. $script_dir/uves_common.sh
 
#MIDAS opening procedure
. $script_dir/uves_midas_pre.sh
 
 
#MIDAS specific procedure
printf 'SPLIT/UVES ccdtest.cat \n \
    CCDTEST/UVES split.cat \n \
    write/out " " \n \
    write/out "To exit MIDAS session type  bye" \n \
' >> ${wrk}/${my_proc}

# Start the MIDAS session
. $script_dir/uves_midas_start.sh

exit
