#!/bin/sh
# $Id: uves_common.sh,v 1.2 2002/08/16 15:28:19 amodigli Exp $
#+++ UVES_FORMAT +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 2002  European Southern Observatory
#
#.IDENT         uves_common.sh
#
#.KEYWORDS      shell script
#
#.USAGE         call from UVES scripts
#
#.INPUT         list    list of selected input files:
#
#.OUTPUT        reference frames necessary for science reduction:
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       To do some basic checks and prepare input/output catalogues
#
#.ALGORITHM    
#
#.ENVIRON       UVES-UVES context
#   
#.LANGUAGE      shell script
#
#.AUTHOR        Andrea Modigliani, ESO-DMO
#
#.VERSION       1.0 2002/04/15  Common script
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.2 $
# $Log: uves_common.sh,v $
# Revision 1.2  2002/08/16 15:28:19  amodigli
# Check for existence of /tmp dir
#
# Revision 1.1  2002/06/03 16:28:09  amodigli
# 1st version
#
# Revision 1.1  2002/04/16 09:33:07  amodigli
#
# Derived from uves pipe, still to be tested.
# Added Files:
# uves_common.sh uves_format.sh uves_stability.sh
# uves_steps.sh uves_wavecal.sh
#
#------------------------------------------------------------------------------
# Note:
#
# be sure to have in your path the commands: 
#    xdialog (HP-UX), or 
#    xmessage (SunOS,Linux,OSF1)
#------------------------------------------------------------------------------

#
# Create a temprary working area
#
localdir=`pwd`
#
#check for /tmp dir
#
if [ -s /tmp ]
then
   cd /tmp
   rm -rf gasg2_${USER}
   mkdir gasg2_${USER}
   wrk=/tmp/gasg2_${USER}
   export wrk
   echo $wrk
else
   echo "Error. /tmp directory does not exist or have 0 size. Exit"
   exit -1
fi
#
#check for $MIDWORK dir
#
if [ -z "$MIDWORK" ] 
then 
   cd $HOME/midwork
else
   cd $MIDWORK
fi
#
#prepare temporary working directory
#
rm -rf tmpwrk
mkdir tmpwrk
mkdir tmpwrk/data
cd tmpwrk

if [ -s ${wrk}/frames.ascii ]
then 
      rm ${wrk}/frames.ascii
fi
if [ -s ${wrk}/${my_proc} ]
then 
      rm ${wrk}/${my_proc}
fi
#
# Get the set of input files
#
sof=`echo $*`
#
# Create symbolic links to the input data as original path may be too long!
#
par=0 #initialize parameter to 0 (default do all steps)
for frm in $sof
do
   num=$frm
   #echo $num
   res=`expr match "$num" "[0-9]*"` #check if numeric par is given
   if [ $res != "1" ] ;             #if not means is a FITS file
   then
      printf "%s\n" `basename $frm` >> ${wrk}/frames.ascii
      if [ ! -s $frm ] 
      then
         frm=$localdir/$frm
      fi

      if [ ! -s $frm ] 
      then 
         echo 'File not found: ' $frm 
      exit 111
      else
        ln -s $frm .
      fi
   else
     par=$frm                      #if num parameter is given update par
   fi
done
export par
#cat  ${wrk}/frames.ascii
#
#
#
# Do an error handling if set of input frames are empty
#
#
if [ ! -s ${wrk}/frames.ascii ]
then
system=`uname`
msg='
ERROR: Empty set of input files.
You have to select the following list of input files:
   ThAr line table    [LINE_REFER_x]
  Formatcheck frame [ARC_LAMP_FORM_x]
  (x: may be RED, or BLUE)'

case "$system" in
      HP-UX)   xdialog  -bg white  -fg red -m $msg;;
      SunOS)   xmessage -bg white  -fg red -m $msg;;
      Linux)   xmessage -bg white  -fg red -m $msg;;
      OSF1)    xmessage -bg white  -fg red -m $msg;;
      *)       echo "$system is not yet supported";;
esac
exit${HOME}/sw/demo_all/cdb_uves/thargood_2.tfits
fi
#
#check for PIPE_HOME env variable definition, if not defined assumes default value
#
if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 
















