#!/bin/sh
# $Id: uves_effic.sh,v 1.9 2002/10/25 09:27:47 amodigli Exp $
#+++ UVES_EFFIC +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         effic.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#                       standard star exposure [STANDARD_y]
#                       DRS table     [DRS_SETUP_x]
#                       order table   [ORDER_TABLE_x]
#                       background table [BACKGR_TABLE_x]
#                       line tables   [LINE_TABLE_xn]
#                       standard star flux table [FLUX_STD_TABLE]
#                       atmospheric extinction table [EXTCOEFF_TABLE]
#                       (y: RED, BLUE x:BLUE, REDL, REDU; n:1,2,3)
#
#
#
#.OUTPUT        dispersion relation solution [LINE_TABLE_y]
#		resolutin plot (POSTSCRIPT)
#		(y: REDL, REDU, BLUE)
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Creates the dispersion relation solution saved as a
#		line table and produces a resolution plot as a POSTSCRIPT
#		file.
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally calls UVES context command WAVECAL/UVES.
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Sebastian Wolf, ESO-DMO
#
#.VERSION       1.0 1999/12/17  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.9 $
# $Log: uves_effic.sh,v $
# Revision 1.9  2002/10/25 09:27:47  amodigli
# Bug fix
#
# Revision 1.8  2002/06/04 14:49:56  amodigli
# Simplified (compacted) using common scripts.
# Now each main script need to be in the
# PIPE_HOME/uves/uves/scripts directory with the
# common other ones.
#
# Revision 1.7  2002/02/07 16:02:41  amodigli
# Made it portable to HP-UX,SunOS,Linux
#
# Revision 1.6  2001/02/12 14:53:16  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.5  2001/01/04 14:16:38  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------
# Note:
#
# be sure to have in your path the commands: 
#    xdialog (HP-UX), or 
#    xmessage (SunOS,Linux,OSF1)
#-------------------------------------------------------------------------------
my_proc=uves_effic.prg
my_script=uves_effic.sh
export my_proc
export my_script
my_test=efficency
my_cat=efficiency.cat
export my_test
export my_cat
if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 

script_dir=$PIPE_HOME/uves/uves/scripts
#common part to do some checks and settings
. $script_dir/uves_common.sh

#MIDAS opening procedure
. $script_dir/uves_midas_pre.sh


#MIDAS specific procedure
printf '@p uves_extrref efficiency.cat \n \
        SPLIT/UVES data.cat \n \
        EFFICIEN/UVES split.cat out.cat refer.cat ' >> ${wrk}/${my_proc}

#MIDAS closing procedure
. $script_dir/uves_midas_post.sh

# Start the MIDAS session
. $script_dir/uves_midas_start.sh

exit

