#!/bin/sh
# $Id: uves_orderpos.sh,v 1.2 2002/06/04 14:49:57 amodigli Exp $
#+++ UVES_ORDERPOS +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         uves_orderpos.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#                       Order Definition frame [ORDER_FLAT_x]
#                       DRS Setup table   [DRS_SETUP_x]
#
#.OUTPUT        reference frames necessary for science reduction:
#			    order table, bkg table, drs setup table.
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Runs order definition  step
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally call UVES context commands:
#		ORDERP/UVES
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 02FEB
#
#.AUTHOR        Andrea Modigliani, ESO-DMO
#
#.VERSION       1.0 2002/02/07  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.2 $
# $Log: uves_orderpos.sh,v $
# Revision 1.2  2002/06/04 14:49:57  amodigli
# Simplified (compacted) using common scripts.
# Now each main script need to be in the
# PIPE_HOME/uves/uves/scripts directory with the
# common other ones.
#
# Revision 1.1  2002/02/08 08:54:11  amodigli
# added uves_orderpos.sh and uves_wavecal.sh
#
# Revision 1.6  2001/02/12 14:53:07  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.5  2001/01/04 14:16:35  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------
# Note:
#
# be sure to have in your path the commands: 
#    xdialog (HP-UX), or 
#    xmessage (SunOS,Linux,OSF1)
#-------------------------------------------------------------------------------
my_proc=uves_orderpos.prg
my_script=uves_orderpos.sh
my_test=orderpos
my_cat=orderpos.cat
export my_proc
export my_script
export my_test
export my_cat
if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 

script_dir=$PIPE_HOME/uves/uves/scripts
#common part to do some checks and settings
. $script_dir/uves_common.sh

#MIDAS opening procedure
. $script_dir/uves_midas_pre.sh

#MIDAS specific test procedure
printf '@p uves_extrref orderpos.cat \n \
        FORMAT/UVES data.cat refer.cat P5=NO P6=2 \n ' >>${wrk}/${my_proc}

#MIDAS closing procedure
. $script_dir/uves_midas_post.sh

# Start the MIDAS session
. $script_dir/uves_midas_start.sh

exit
