#!/bin/sh
# $Id: uves_plot.sh,v 1.8 2002/02/05 07:29:02 amodigli Exp $
#+++ UVES_PLOT +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         plot.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#                       Almost any of the result frames
#
#.OUTPUT        dispersion relation solution [LINE_TABLE_y]
#		resolutin plot (POSTSCRIPT)
#		(y: REDL, REDU, BLUE)
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Creates the dispersion relation solution saved as a
#		line table and produces a resolution plot as a POSTSCRIPT
#		file.
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally calls UVES context command WAVECAL/UVES.
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Sebastian Wolf, ESO-DMO
#
#.VERSION       1.0 1999/12/17  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.8 $
# $Log: uves_plot.sh,v $
# Revision 1.8  2002/02/05 07:29:02  amodigli
# added a note on portability
#
# Revision 1.7  2002/02/05 07:18:26  amodigli
# Made it portable to SunOS,Linux,OSF1
#
# Revision 1.6  2001/02/12 14:53:15  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.5  2001/01/04 14:16:37  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------
# Note:
#
# be sure to have in your path the commands: 
#    xdialog (HP-UX), or 
#    xmessage (SunOS,Linux,OSF1)
#-------------------------------------------------------------------------------


# Create a temprary working area
localdir=`pwd`
base_dir=/tmp
cd ${base_dir}


rm -rf gasg_${USER}
mkdir gasg_${USER}
wrk=/tmp/gasg_${USER}
echo $wrk
cd $HOME/midwork
rm -rf tmpwrk
mkdir tmpwrk
cd tmpwrk
if [ -s ${wrk}/frames.ascii ]
then 
      rm ${wrk}/frames.ascii
fi
if [ -s ${wrk}/plot.prg ]
then 
      rm ${wrk}/plot.prg
fi



# Get the set of input files
list=`echo $*`



rm -f tmpdat
mkdir tmpdat
cd tmpdat


#
# Create symbolic links to the input data as original path may be too long!
#


for frm in $list
do
  printf "%s\n" `basename $frm` >> ${wrk}/frames.ascii
  if [ ! -s $frm ]
  then
     frm=$localdir/$frm
  fi
  if [ ! -s $frm ] 
  then 
       echo 'File not found: ' $frm 
       exit 111
  else
       ln -s $frm .
  fi
done
#
#
# Do an error handling if set of input frames are empty
#
#
if [ ! -s ${wrk}/frames.ascii ]
then

system=`uname`
msg='
ERROR: Empty set of input files.
You have to select the following list of input files:
  Reduced frame [REDUCED_x]
  (x: may be RED, or BLUE)'
  
case "$system" in
      HP-UX)   xdialog  -bg white  -fg red -m $msg;;
      SunOS)   xmessage -bg white  -fg red -m $msg;;
      Linux)   xmessage -bg white  -fg red -m $msg;;
      OSF1)    xmessage -bg white  -fg red -m $msg;;
      *)       echo "$system is not yet supported";;
esac
exit
fi
ln -s $1 .


if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 

printf '@d pipeline.start >Null \n \
    @d pipeline.control d >Null \n \
    write/key pipe_home/c/1/100 " " ALL \n \
    pipe_home = m$symbol("PIPE_HOME") \n \
    set/context uves {pipe_home}/uves/context >Null\n  \    
    UVES_GXSZ(1) = 1000 \n \
    UVES_GYSZ(1) = 710 \n \
    write/key ifrm/c/1/100 " " ALL \n \
    write/key catal/i/1/1 0 \n \
    write/key wrk/c/1/100 " " ALL \n \
    wrk = m$symbol("USER") \n \
    wrk = "/tmp/gasg_{wrk}" \n \
    create/icat in.cat {wrk}/frames.ascii >Null\n \
    indisk/fits in.cat ? OC >Null\n \
    create/icat plot.cat *.bdf >Null\n \
    create/grap 1 {UVES_GXSZ(1)},{UVES_GYSZ(1)},200,400 \n \
    hardcop/plot ON \n \
    catal = 0 \n \
    store/frame ifrm plot.cat \n \
    config/instr {ifrm} \n \
    show/uves \n \
    if {{ifrm},NAXIS} .eq. 2 outputc = "1,{{ifrm},NPIX(2)}"
    if {{ifrm},NAXIS} .eq. 1 write/key outputc " " ALL
    plot/uves {ifrm} {outputc} \n \
    write/out " " \n \
    write/out " "\n \
    write/out "(1) To exit MIDAS session type  `bye`" \n \
    write/out "(2) Use `hardcop/plot` n,g PSfile to create" \n \
    write/out "    POSTSCRIPT hardcopies (see `help hardcop/plot`)" \n \
    write/out "(3) Use  `help plot/uves`  to get the options" \n \
    write/out "    redoing the plot" \n \
' > ${wrk}/plot.prg



xterm -geom 132x24+0-0 -fn fixed -T 'gasgano: plot.sh' -e inmidas -P -j '@@ '${wrk}'/plot.prg' &

exit

