#!/bin/sh
# $Id: uves_popul.sh,v 1.10 2002/06/04 14:49:56 amodigli Exp $
#+++ UVES_POPUL +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT      (C) 1998  European Southern Observatory
#
#.IDENT         popul.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#                       Formatcheck frame [ARC_LAMP_FORM_y]
#                       Orderdefinition frame [ORDER_FLAT_y]
#                       ThAr line table   [LINE_REFER_TABLE]
#                       ThAr exposure     [ARC_LAMP_y]
#                       biases [BIAS_y]  (optional)
#                       flats [FLAT_y]
#			and (optional) master bias for master flatfield
#			creation.
#			(y: may be RED, or BLUE)
#
#.OUTPUT        reference frames necessary for science reduction:
#			drs setup table, order table, background table,
#			line tables, and optionally a master bias and a
#			master flat.
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Creates all the reference frames necessary for the science
#		reduction.
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally call UVES context commands:
#		PREDICT/UVES, ORDERP/UVES, WAVECAL/UVES and MASTER/UVES
#		The master and dispersion relation creation will be skipped
#		as soon as no bias, flats or ThAr exposures specified.
#		Instead of biases you may specify a master bias for the
#		master flatfield creation.
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Sebastian Wolf, ESO-DMO
#
#.VERSION       1.0 1999/12/17  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.10 $
# $Log: uves_popul.sh,v $
# Revision 1.10  2002/06/04 14:49:56  amodigli
# Simplified (compacted) using common scripts.
# Now each main script need to be in the
# PIPE_HOME/uves/uves/scripts directory with the
# common other ones.
#
# Revision 1.9  2002/02/07 16:00:14  amodigli
# Made it portable to HP-UX,SunOS,Linux
#
# Revision 1.8  2002/01/28 13:36:40  amodigli
# This fix a portability problem to SunOs,Linux.
#
# Revision 1.7  2001/02/12 14:53:15  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.6  2001/01/04 14:16:37  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------
# Note:
#
# be sure to have in your path the commands: 
#    xdialog (HP-UX), or 
#    xmessage (SunOS,Linux,OSF1)
#-------------------------------------------------------------------------------

my_proc=uves_popul.prg
my_script=uves_popul.sh
my_test=popul
my_cat=popul.cat
export my_proc
export my_script
export my_test
export my_cat
if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 

script_dir=$PIPE_HOME/uves/uves/scripts
#common part to do some checks and settings
. $script_dir/uves_common.sh
 
#MIDAS opening procedure
. $script_dir/uves_midas_pre.sh
 
#MIDAS specific procedure
printf '@p uves_extrref popul.cat \n \
        PREPAR/CALD data.cat refer.cat\n '>> ${wrk}/${my_proc} 

#MIDAS closing procedure
. $script_dir/uves_midas_post.sh
 
#close MIDAS section
printf 'bye \n ' >> ${wrk}/${my_proc}
 
# Start the MIDAS session
. $script_dir/uves_midas_start.sh

exit
