#!/bin/sh
# $Id: uves_reducOPT.sh,v 1.6 2002/02/05 08:42:48 amodigli Exp $
#+++ UVES_REDUC +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         reduc.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#			science exposure [SCIENCE_BLUE, SCIENCE_RED]
#			DRS table     [DRS_SETUP_BLUE, DRS_SETUP_REDx]
#			order table   [ORDER_TABLE_BLUE, ORDER_TABLE_REDx]
#			background table [BACKGR_TABLE_BLUE, BACKGR_TABLE_REDx]
#			line tables   [LINE_TABLE_BLUEn, LINE_TABLE_REDxn]
#			master flat   [MASTER_FLAT_BLUE, MASTER_FLAT_REDx]
#			master bias   [MASTER_BIAS_BLUE, MASTER_BIAS_REDx]
#				      whih is optional.
#				      (x:L,U; n:1,2,3)
#
#.OUTPUT        reduced science spectra using optimal extraction
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Do a science reduction using optimal extraction
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally calls UVES context command REDUCE/UVES
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Sebastian Wolf, ESO-DMO
#
#.VERSION       1.0 1999/12/20  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.6 $
# $Log: uves_reducOPT.sh,v $
# Revision 1.6  2002/02/05 08:42:48  amodigli
# Made it portable to HP-UX,SunOS,Linux,OSF1
#
# Revision 1.5  2001/02/12 14:53:14  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.4  2001/01/04 14:16:36  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------
# Note:
#
# be sure to have in your path the commands: 
#    xdialog (HP-UX), or 
#    xmessage (SunOS,Linux,OSF1)
#-------------------------------------------------------------------------------

# Create a temprary working area
localdir=`pwd`
base_dir=/tmp
cd ${base_dir}


rm -rf gasg_${USER}
mkdir gasg_${USER}
wrk=/tmp/gasg_${USER}
echo $wrk

if [ -z "$MIDWORK" ] 
then 
   cd $HOME/midwork
else
   cd $MIDWORK
fi


rm -rf tmpwrk
mkdir tmpwrk
cd tmpwrk



if [ -s ${wrk}/frames.ascii ]
then 
      rm ${wrk}/frames.ascii
fi
if [ -s ${wrk}/reduc.prg ]
then 
      rm ${wrk}/reduc.prg
fi



# Get the set of input files
list=`echo $*`





rm -f tmpdat
mkdir tmpdat
cd tmpdat



#
# Create symbolic links to the input data as original path may be too long!
#

for frm in $list
do
  printf "%s\n" `basename $frm` >> ${wrk}/frames.ascii
  if [ ! -s $frm ]
  then
     frm=$localdir/$frm
  fi
  if [ ! -s $frm ] 
  then 
       echo 'File not found: ' $frm 
       exit 111
  else
       ln -s $frm .
  fi
done

#
#
# Do an error handling if set of input frames are empty
#
#

if [ ! -s ${wrk}/frames.ascii ]
then
system=`uname`
msg='
ERROR: Empty set of input files.
You have to select the following list of input files:
 science exposure [SCIENCE_BLUE, SCIENCE_RED]
 DRS table [DRS_SETUP_x]
 order table [ORDER_TABLE_x]
 backgr table [BACKR_TABLE_x]
 line tables [LINE_TABLE_xi]
 master bias [MASTER_BIAS_x]
 master flat [MASTER_FLAT_x]
 (x: may be RED, or BLUE, i=1,2,3)'
  
case "$system" in
      HP-UX)   xdialog  -bg white  -fg red -m $msg;;
      SunOS)   xmessage -bg white  -fg red -m $msg;;
      Linux)   xmessage -bg white  -fg red -m $msg;;
      OSF1)    xmessage -bg white  -fg red -m $msg;;
      *)       echo "$system is not yet supported";;
esac
exit
fi

if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 

printf '@d pipeline.start\n \
    @d pipeline.control d\n \
    write/key pipe_home/c/1/100 " " ALL \n \
    pipe_home = m$symbol("PIPE_HOME") \n \
    set/context uves {pipe_home}/uves/context \n  \    
    write/key wrk/c/1/100 " " ALL \n \
    wrk = m$symbol("USER") \n \
    wrk = "/tmp/gasg_{wrk}" \n \
    create/icat in.cat {wrk}/frames.ascii >Null\n \
    write/out "INFO    [reduc]:            Transfering the data ..."\n \
    indisk/fits in.cat ROOT=ref >Null\n \
    create/icat reduc.cat ref* >Null\n \
    write/out "INFO    [reduc]:            ... stored in catalog reduc.cat"\n \
    @p uves_extrref reduc.cat \n \
    CONFIG/DISP p3=0.8 \n \
    SPLIT/UVES data.cat \n \
    REDUCE/UVES split.cat out.cat refer.cat E OPTIMAL MEDIAN \n \
    ASSIGN/GRA g,3 \n \
    MODIF/GRA W \n \
    write/out " " \n \
    write/out "To exit MIDAS session type  bye" \n \
' > ${wrk}/reduc.prg



xterm -geom 132x24+0-0 -fn fixed -T 'gasgano: reduc.sh' -e inmidas -P -j '@@ '${wrk}'/reduc.prg' &

exit
