#!/bin/sh
# $Id: uves_reduce.sh,v 1.8 2002/06/04 14:49:58 amodigli Exp $
#+++ UVES_REDUC +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         reduc.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#                       science exposure [SCIENCE_y]
#                       DRS table     [DRS_SETUP_x]
#                       order table   [ORDER_TABLE_x]
#                       background table [BACKGR_TABLE_x]
#                       line tables   [LINE_TABLE_xn]
#                       master flat   [MASTER_FLAT_x]
#                       master bias   [MASTER_BIAS_x]
#                       (y: RED, BLUE, x:BLUE, REDL, REDU; n:1,2,3)
#
#.OUTPUT        reduced science spectra using average extraction
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Do a science reduction using average extraction
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally calls UVES context command REDUCE/UVES
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Sebastian Wolf, ESO-DMO
#
#.VERSION       1.0 1999/12/20  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.8 $
# $Log: uves_reduce.sh,v $
# Revision 1.8  2002/06/04 14:49:58  amodigli
# Simplified (compacted) using common scripts.
# Now each main script need to be in the
# PIPE_HOME/uves/uves/scripts directory with the
# common other ones.
#
# Revision 1.7  2002/02/05 08:42:15  amodigli
# Made it portable to HP-UX,SunOS,Linux,OSF1
#
# Revision 1.6  2001/02/12 14:53:14  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.5  2001/01/04 14:16:37  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------
# Note:
#
# be sure to have in your path the commands: 
#    xdialog (HP-UX), or 
#    xmessage (SunOS,Linux,OSF1)
#-------------------------------------------------------------------------------
my_proc=uves_reduce.prg
my_script=uves_reduce.sh
my_test=reduce
my_cat=reduce.cat
export my_proc
export my_script
export my_test
export my_cat
if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 

script_dir=$PIPE_HOME/uves/uves/scripts
#common part to do some checks and settings
. $script_dir/uves_common.sh

#MIDAS opening procedure
. $script_dir/uves_midas_pre.sh

#MIDAS specific procedure

#MIDAS specific test procedure
printf '@p uves_extrref reduce.cat \n \
        SPLIT/UVES data.cat \n \
        REDUCE/UVES split.cat out.cat refer.cat E AVERAGE MEDIAN \n' >> ${wrk}/${my_proc}

#MIDAS closing procedure
. $script_dir/uves_midas_post.sh



# Start the MIDAS session
. $script_dir/uves_midas_start.sh


exit

