#!/bin/sh
# $Id: uves_resol.sh,v 1.8 2002/06/04 14:49:56 amodigli Exp $
#+++ UVES_RESOL +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         resol.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#                       ThAr exposure [ARC_LAMP_x]
#                       DRS table     [DRS_SETUP_x]
#                       order table   [ORDER_TABLE_x]
#                       guess table   [LINE_TABLE_x]
#                       line reference catalog  [LINE_REFER_TABLE]
#
#.OUTPUT        dispersion relation solution [LINE_TABLE_y]
#		resolutin plot (POSTSCRIPT)
#		(y: REDL, REDU, BLUE)
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Creates the dispersion relation solution saved as a
#		line table and produces a resolution plot as a POSTSCRIPT
#		file.
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally calls UVES context command WAVECAL/UVES.
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Sebastian Wolf, ESO-DMO
#
#.VERSION       1.0 1999/12/17  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.8 $
# $Log: uves_resol.sh,v $
# Revision 1.8  2002/06/04 14:49:56  amodigli
# Simplified (compacted) using common scripts.
# Now each main script need to be in the
# PIPE_HOME/uves/uves/scripts directory with the
# common other ones.
#
# Revision 1.7  2002/02/07 16:00:46  amodigli
# Made it portable to HP-UX,SunOS,Linux
#
# Revision 1.6  2001/02/12 14:53:12  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.5  2001/01/04 14:16:36  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------
# Note:
#
# be sure to have in your path the commands: 
#    xdialog (HP-UX), or 
#    xmessage (SunOS,Linux,OSF1)
#-------------------------------------------------------------------------------
my_proc=uves_resol.prg
my_script=uves_resol.sh
export my_proc
export my_script
my_test=resol
my_cat=resol.cat
export my_test
export my_cat
if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 
script_dir=$PIPE_HOME/uves/uves/scripts
#common part to do some checks and settings
. $script_dir/uves_common.sh
 
 
#MIDAS opening procedure
. $script_dir/uves_midas_pre.sh
 
#MIDAS specific procedure
printf '@p uves_extrref resol.cat \n \
        SPLIT/UVES data.cat \n \
        WAVECA/UVES split.cat out.cat refer.cat AUTO YES YES \n' >> ${wrk}/${my_proc}
#MIDAS closing procedure
. $script_dir/uves_midas_post.sh
# Start the MIDAS session
. $script_dir/uves_midas_start.sh

exit
