#! /bin/sh
#
# script to setup pipeline installation and sample reduction blocks
# after unpacking of the distribution.
#
# "@(#) $Id: setup.sh,v 1.4 2002/02/06 09:28:19 amodigli Exp amodigli $"
#
# Usage:
#        setup.sh


HERE=`pwd`
HERE_=`echo $HERE | sed 's/\//\\\\\//g'`
TEMPFILE=temp


UVES_REL=1.4.0
PIPE_DIR=${HERE}/pipeline
UVES_HOME=${HERE}
INST_PIPE_DIR=${PIPE_DIR}/uves

REC_DIR=${INST_PIPE_DIR}/uves/calibDB/ech/rec/
RUL_DIR=${INST_PIPE_DIR}/uves/calibDB/rul/

REDBLOCK_DIR=${HERE}/ex/rb
MID_STARTUP=${REDBLOCK_DIR}/pipe.prg
TCSH_UVES=${HERE}/.tcsh_uves_env
BASH_UVES=${HERE}/.bash_uves_env

if [ -z "${MIDASHOME}" ]; then
   echo "Environment variable MIDASHOME is not defined. Abort."
   exit 1
fi 
if [ -z "${MIDVERS}" ]; then
   echo "Environment variable MIDVERS is not defined. Abort."
   exit 1
fi 
# Verify the MIDASHOME and MIDVERS variables are defined, otherwise exit

# in this release we check only the year
MIDASV1=`echo $MIDVERS | cut -c1-5`
MIDASV2=`echo $MIDVERS | cut -c1-2`
case  "${MIDASV2}" in 
      "02") ;;
      "03") ;;
      "04") ;;
      "05") ;;
      "06") ;;
      "07") ;;
      "08") ;;
      "09") ;;
      "te") ;;
      *)   echo "MIDAS version is older than 02SEP.";
           echo "Please install MIDAS release 02SEP or later. EXIT.";
           exit 1;;
esac

if [ ! -d "${MIDASHOME}" ]; then
  echo "${MIDASHOME}: not such directory."
  echo "Call your MIDAS operator. EXIT."
  exit 1
fi
if [ ! -d "${MIDASHOME}/${MIDVERS}" ] && \
   [ ! -l "${MIDASHOME}/${MIDVERS}" ]; then
  echo "${MIDASHOME}/${MIDVERS}: not such directory."
  echo "Call your MIDAS operator. EXIT."
  exit 1
fi

echo
echo 'This script installs the UVES pipeline and'
echo 'configures the sample reduction blocks for immediate use.'
echo
echo


echo '1. Installation of the UVES pipeline'
echo '-------------------------------------------'
echo
echo

echo ${PIPE_DIR}

( cd $PIPE_DIR
  tar xvf *.tar
  cd uves-$UVES_REL
  ./setup user
)

if [ $? != 0 ]
then
  echo 'Pipeline installation failed. Exiting'
  exit 2
fi


echo
echo done.
echo
echo
echo '2. Configuring the reduction blocks'
echo '-----------------------------------'
echo
echo

echo ${REDBLOCK_DIR}

( cd $REDBLOCK_DIR
  for file in uves*.rb
  do
    echo ${file}
    sed "s/@@PIPE_DIR@@/$HERE_/" $file > $TEMPFILE
    mv $TEMPFILE $file
  done
)


echo done.
echo
echo
echo '3. Creating a MIDAS startup file'
echo '--------------------------------'
echo
echo

cat > $MID_STARTUP <<EOF
!
! MIDAS startup file to define settings for the UVES pipeline.
! The RBS context is then enabled and the pipeline with displays
! started.
!
! 
! Usage:
!  - type '@@ pipe.prg' in a running MIDAS session
!    or
!  - type 'inmidas -P -j "@@ pipe.prg"' in a UNIX shell
!
!
! Available commands:
!  - start/pipeline Display/NoDisplay 
!                       to start the pipeline with/without Display
!  - execute/RB rbfile  to execute given RB
!  - RBS/help           to get help about this context
!
!    UVES CONTEXT:      standard UVES context commands 
!                       (please read the documentation)
! 
!  - TUTORI/UVES        UVES pipeline tutorial
!                       supposed to be executed in the dirrectory 
!                       ~/uves/tutorial/test
!

EOF


echo 'write/key CALIBDB_RUL/c/1/200  "'$RUL_DIR'"' >> $MID_STARTUP
echo 'write/key CALIBDB_REC/c/1/200  "'$REC_DIR'"' >> $MID_STARTUP

cat >> $MID_STARTUP <<EOF

set/context RBS >Null
start/pipeline Display >Null
@d  pipeline.control display,,80 overwrite
set/context uves $INST_PIPE_DIR/context/

WRITE/OUT " "
WRITE/OUT "Enable/disable displays using the command"
WRITE/OUT "  START/PIPELINE Display/Nodisplay (currently enabled)"
WRITE/OUT " "
WRITE/OUT "Enable/disable overwrite data products option"
WRITE/OUT "  SET/CONTEXT uves $PIPE_HOME/context/ (UVES context)"
WRITE/OUT " "
WRITE/OUT "Execute a reduction block using the command"
WRITE/OUT "  EXECUTE/RB <rbfile>"
WRITE/OUT " "
WRITE/OUT "Or Run the tutorial with the command"
WRITE/OUT "  CHANGE/DIR $UVES_HOME/tutorial/test/
WRITE/OUT "  TUTORI/UVES "
WRITE/OUT " "
WRITE/OUT "Or Use the UVES CONTEXT commands ...."
WRITE/OUT " "
EOF



echo done.

echo
echo
echo '4. Creating a TCSH environment setup file'
echo '-----------------------------------------'
echo
echo

cat > $TCSH_UVES <<EOF
#
# UVES setup file to define tcsh environment  for the UVES pipeline.
# 
# Usage:
#  - type 'source $TCSH_UVES_ENV_SET' if you use tcsh
#
#
EOF
echo '# - Define a system variable $PIPE_HOME:' >> $TCSH_UVES
echo '    setenv PIPE_HOME "'$PIPE_DIR'"'       >> $TCSH_UVES
echo '# - Define a system variable $UVES_HOME:' >> $TCSH_UVES
echo '    setenv UVES_HOME "'$UVES_HOME'"'      >> $TCSH_UVES
echo '# - Include in your PATH the directory '  >> $TCSH_UVES
echo '#  ' $INST_PIPE_DIR'/uves/scripts :'      >> $TCSH_UVES
echo '    set path = ( $path "'$INST_PIPE_DIR/uves/scripts'" )' >> $TCSH_UVES
echo '#   for the csh, tcsh you can also put'  >> $TCSH_UVES
echo '#   this file in your .cshrc, .profile, etc.' >> $TCSH_UVES

cat >> $TCSH_UVES <<EOF
#
#   It is useful to have set the following alias umidas
#   which loads the uves context from the proper directory
#
alias umidas 'inmidas -j "@d pipeline.start; @d pipeline.control D O; set/context uves $PIPE_DIR/uves/context; "' 
EOF

echo done

echo
echo
echo '5. Creating a BASH environment setup file'
echo '-----------------------------------------'
echo
echo

cat > $BASH_UVES <<EOF
#
# UVES setup file to define bash environment  for the UVES pipeline.
# 
# Usage:
#  - type 'source $BASH_UVES' if you use tcsh
#
EOF
echo '# - Define a system variable $PIPE_HOME:' >> $BASH_UVES
echo '    export PIPE_HOME="'$PIPE_DIR'"'       >> $BASH_UVES
echo '# - Define a system variable $UVES_HOME:' >> $BASH_UVES
echo '    export UVES_HOME="'$UVES_HOME'"'      >> $BASH_UVES
echo '# - Include in your PATH the directory '  >> $BASH_UVES
echo '#  ' $INST_PIPE_DIR'/uves/scripts :'      >> $BASH_UVES
echo '    export PATH=${PATH}:"'$INST_PIPE_DIR/uves/scripts'"' >> $BASH_UVES
echo '# - Include in your PATH the directory '  >> $BASH_UVES
echo '#   for the csh, tcsh you can also put'  >> $BASH_UVES
echo '#   this file in your .cshrc, .profile, etc.' >> $BASH_UVES

cat >> $BASH_UVES <<EOF
#
#   It is useful to have set the following alias umidas
#   which loads the uves context from the proper directory
#
alias umidas='inmidas -j "@d pipeline.start; @d pipeline.control D O; set/context uves $PIPE_DIR/uves/context; mid\\\$mode(3) = 0"' 
EOF


echo done




echo 
echo 
echo 'Configuration sucessfully finished.'
echo
echo 'To use the UVES pipeline now you have to do the following:'
echo
echo ' - If you have csh or tcsh:'
echo '   % source '$TCSH_UVES
echo ' - If you have bash:'
echo '   % source '$BASH_UVES
echo 
echo ' - To execute one of the UVES sample reduction blocks:'
echo '   % cd '$REDBLOCK_DIR
echo '   % inmidas -P -j "@@ pipe.prg"'
echo '   Midas 001> EXECUTE/RB <rbfile>'
echo
echo '   Midas NNN> bye'
echo
echo ' - To execute the UVES TUTORIAL:'
echo '   % cd '$UVES_HOME'/tutorial/test/'
echo '   % inmidas -P -j "@@ '$REDBLOCK_DIR'/pipe.prg"'
echo '   Midas 001> TUTORI/UVES'
echo
echo '   Midas 002> bye'
echo '   rm *     (remember to clean up after used the tutorial)'
echo
exit 0













